<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:tns="http://schema.solution.ioc.ibm.com/CognosContent" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:include href="../../common_content/CommonContent.xsl"/>
	<xsl:output method="text" indent="yes" />

	<xsl:template match="/">#!/bin/sh
###############################################################################
# Licensed Materials - Property of IBM
# Restricted Materials of IBM
# PID
# (C) Copyright IBM Corp. 2010, 2013
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp
###############################################################################
dos2unix $IOC_BASE_DIR/variables.properties
source ~/.baprofile
source $IOC_BASE_DIR/variables.properties

cd $IOC_BASE_DIR

export SED_SCRIPT_NAME=com.ibm.ioc.sed.script.txt
sed -e '/^#/d' $IOC_BASE_DIR/variables.properties | awk -F = '{print "s#~{" $1 "}#" substr($0,index($0,"=")+1) "#g"}' > $IOC_BASE_DIR/$SED_SCRIPT_NAME


TOOLBAR_FILE=$COGNOS_HOME/templates/ps/portal/viewer-toolbar.xml
chmod 775 $TOOLBAR_FILE
sed -i 's/systemCondition="RV_TOOLBAR_BUTTONS"/ /g' $TOOLBAR_FILE

		<xsl:for-each select="tns:cognosContent/*">
		<xsl:apply-templates select="."/><!-- Here to transform with common template first -->
		<xsl:variable name="nodeName" select="name(.)" />
			<xsl:choose>
					
				<xsl:when test="$nodeName='report'">
log.info "###############################################################################"
log.info "# Import cognos report "
log.info "#   ARCHIVE_NAME=<xsl:value-of select="@archivename" />"
log.info "###############################################################################"

JAVA=$COGNOS_WAS_HOME/java/bin/java
FILE_NAME=<xsl:value-of select="@file" />
ARCHIVE_NAME=<xsl:value-of select="@archivename" />
ARCHIVE_PWD=<xsl:value-of select="@archivepwd" />
DEPLOYMENT_NAME=<xsl:value-of select="@deployname" />
COGNOS_DEPLOYMENT_JAR=<xsl:value-of select="@cognosDeploymentJar" />
DISPATCH_URL=http://localhost:$COGNOS_DISP_PORT/p2pd/servlet/dispatch

log.info "Begin to copy $IOC_BASE_DIR/$FILE_NAME to $COGNOS_HOME/deployment/ directory"
cp -v -f $IOC_BASE_DIR/$FILE_NAME $COGNOS_HOME/deployment/
if [ $? -ne 0 ] ; 
then 
	log.error "Copy $IOC_BASE_DIR/$FILE_NAME failed. Please check if $FILE_NAME or $COGNOS_HOME/deployment/ dir exists";
	exit 1; 
fi
log.info "Copy $FILE_NAME to $COGNOS_HOME/deployment/ directory complete"
chmod -R 775 $COGNOS_HOME/deployment/$ARCHIVE_NAME*

# Build the CLASSPATH required
for jar in axis axisReportNetClient commons-discovery-0.2 commons-logging-1.1 log4j-1.2.8 jaxrpc wsdl4j-1.5.1; do
  CLASSPATH="$CLASSPATH:$IOC_BASE_DIR/lib/$jar.jar"
done

$JAVA -classpath "$CLASSPATH":$IOC_BASE_DIR/$COGNOS_DEPLOYMENT_JAR cognos.ImportReports -a $ARCHIVE_NAME -i $ARCHIVE_PWD -d $DEPLOYMENT_NAME -g $DISPATCH_URL -s $NAMESPACE_ID -u $COGNOS_ADMIN_NAME -p $COGNOS_ADMIN_PWD
if [ $? -ne 0 ] ; 
then 
	log.error "Import reports $ARCHIVE_NAME failed.";
	exit 1; 
fi
				</xsl:when>
						
				<xsl:when test="$nodeName='catalogdb'">
log.info "###############################################################################"
log.info "# Catalog remote db2 database"
log.info "###############################################################################"
su - $COGNOS_MIDDLEWARE_DB2_ACCOUNT -c "db2 uncatalog node <xsl:value-of select="@nodename" />"
su - $COGNOS_MIDDLEWARE_DB2_ACCOUNT -c "db2 uncatalog database <xsl:value-of select="@dbName" />"
su - $COGNOS_MIDDLEWARE_DB2_ACCOUNT -c "db2 catalog tcpip node <xsl:value-of select="@nodename" /> remote <xsl:value-of select="@remoteHostName" /> server <xsl:value-of select="@remoteDB2Port" />"
if [ $? -ne 0 ] ; 
then 
	log.error "Catalog remote db2 <xsl:value-of select="@remoteHostName" />:<xsl:value-of select="@remoteDB2Port" /> failed.";
	exit 1; 
fi
log.info "Catalog remote db2 <xsl:value-of select="@remoteHostName" />:<xsl:value-of select="@remoteDB2Port" /> success.";

su - $COGNOS_MIDDLEWARE_DB2_ACCOUNT -c "db2 catalog database <xsl:value-of select="@dbName" /> as <xsl:value-of select="@dbName" /> at node <xsl:value-of select="@nodename" />"
if [ $? -ne 0 ] ; 
then 
	log.error "Catalog remote db2 database <xsl:value-of select="@dbName" /> failed.";
	exit 1; 
fi
log.info "Catalog remote db2 database <xsl:value-of select="@dbName" /> success.";
				</xsl:when>
						
				<xsl:when test="$nodeName='datasource'">
log.info "###############################################################################"
log.info "# Create cognos datasource "
log.info "###############################################################################"

JAVA=$COGNOS_WAS_HOME/java/bin/java
DISPATCH_URL=http://$COGNOS_SERVER_HOSTNAME:$COGNOS_DISP_PORT/p2pd/servlet/dispatch/ext
COGNOS_DB2_URL=jdbc:db2://$COGNOS_DB2_REMOTE_HOSTNAME:$COGNOS_DB2_LOCAL_PORT
COGNOS_DEPLOYMENT_JAR=<xsl:value-of select="@cognosDeploymentJar" />

# Build the CLASSPATH required
for jar in axis axisReportNetClient commons-discovery-0.2 commons-logging-1.1 log4j-1.2.8 jaxrpc wsdl4j-1.5.1; do
  CLASSPATH="$CLASSPATH:$IOC_BASE_DIR/lib/$jar.jar"
done

$JAVA -classpath "$CLASSPATH":$IOC_BASE_DIR/$COGNOS_DEPLOYMENT_JAR cognos.Datasource -iD $NAMESPACE_ID -uN $COGNOS_ADMIN_NAME -pD $COGNOS_ADMIN_PWD -sDsN <xsl:value-of select="@dsname" /> -UID $COGNOS_APPLICATIONS_DB2_ACCOUNT -PWD $COGNOS_APPLICATIONS_DB2_PWD -CONN $COGNOS_DB2_URL -DNS <xsl:value-of select="@dbname" /> -g $DISPATCH_URL
if [ $? -ne 0 ] ; 
then 
	log.error "Create datasource <xsl:value-of select="@dsname" /> failed.";
	exit 1; 
fi
				</xsl:when>
						
				<xsl:when test="$nodeName='resource'">
###############################################################################
# Replace resource file <xsl:value-of select="@file" />
###############################################################################

echo "# Replace resource file <xsl:value-of select="@file" />"

sed -i -c -f $IOC_BASE_DIR/$SED_SCRIPT_NAME $IOC_BASE_DIR/<xsl:value-of select="@file" />

if [ $? -ne 0 ]
then 
  echo "Fail to replace resource file <xsl:value-of select="@file" />"
  exit 1
fi

echo "###############################################################################"
echo "# Configure WAS with resource file <xsl:value-of select="@file" />             "
echo "###############################################################################"

echo $PROFILE_PATH

$COGNOS_PROFILE_PATH/bin/wsadmin.sh -user $DMGR_ACCOUNT -password $DMGR_PASSWORD -lang jython -f lib/server_config.py $IOC_BASE_DIR/<xsl:value-of select="@file" />

if [ $? -ne 0 ]
then 
  echo "Fail to config WAS with resource file <xsl:value-of select="@file" />"
  exit 1
fi
				</xsl:when>

				<xsl:when test="$nodeName='checkdb2exists'">
log.info "###############################################################################"
log.info "# Checking for existence of db2"
log.info "###############################################################################"
\db2ls -a
if [ $? -ne 0 ] ; 
then 
	log.error "Check db2 exists failed.";
	exit 1; 
fi
log.info "Check db2 exists succeeded.";
				</xsl:when>
						
				<xsl:when test="$nodeName='copylib'">
log.info "###############################################################################"
log.info "# Copying lib <xsl:value-of select="@libFileName" />"
log.info "###############################################################################"

IS_FILE_EXISTS=0
IS_LINK_EXISTS=0
IS_LINK_BROKEN=0

LINK_NAME=<xsl:value-of select="@libLinkName" />
FILE_NAME=<xsl:value-of select="@libFileName" />
FILE_NAME_LOCAL=

# Check if lib exists
FILE_NAME_LOCAL=`ls /lib | grep $LINK_NAME.`
if [[ $FILE_NAME_LOCAL != "" ]]
then
	IS_FILE_EXISTS=1
fi

# Check if link exists or if it is broken
if [ -L $LINK_NAME ]
then 
	IS_LINK_EXISTS=1

	file $LINK_NAME | grep "broken" > /dev/null
	if [ $? -eq 0 ]
	then
		IS_LINK_BROKEN=1
	fi
fi

rc=0
if [ $IS_FILE_EXISTS -eq 0 ]
then
	# lib doesn't exist, so remove the link, copy the lib in, and create link
	log.info "lib not found so copying new lib over and creating link to it."
	rm -f /lib/$LINK_NAME
	cp -f lib/$FILE_NAME /lib
	ln -s /lib/$FILE_NAME /lib/$LINK_NAME > /dev/null
	rc=$?
else
	if [ $IS_LINK_EXISTS -eq 0 ]
	then
		if [ -e $LINK_NAME ]
		then
			rm -f /lib/$LINK_NAME
			ln -s /lib/$FILE_NAME_LOCAL /lib/$LINK_NAME > /dev/null
			rc=$?
		fi
	elif [ $IS_LINK_BROKEN -eq 1 ]
	then
		log.info "lib found but link to it is broken so creating new link."
		rm -f $LINK_NAME
		ln -s /lib/$FILE_NAME_LOCAL /lib/$LINK_NAME > /dev/null
		rc=$?
	fi
fi

if [ $rc -ne 0 ] ; 
then 
	log.error "Copy lib <xsl:value-of select="@libFileName" /> failed.";
	exit 1; 
fi
log.info "Copy lib <xsl:value-of select="@libFileName" /> succeeded.";
				</xsl:when>
						
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
</xsl:stylesheet>