# /*
#  * IBM Confidential OCO Source Material
#  *
#  * Licensed Materials - Property of IBM
#  * 
#  * (C) Copyright IBM Corp. 2011  All rights reserved.
#  *
#  * The source code for this program is not published or otherwise divested
#  * of its trade secrets, irrespective of what has been deposited with the
#  * U. S. Copyright Office.
#  *
#  * US Government Users Restricted Rights - Use, duplication or
#  * disclosure restricted by GSA ADP Schedule Contract with IBM
#  * Corp.
#  */

#******************************************************************************
# File Name:   common.py
# Description: This file contains a number of useful wsadmin procs that help
#              isolate the tedious coding for common operations.
# Fix record:
#******************************************************************************

import sys
import re
import sre
def regexp(pattern, string, flags=0):
        if(re.compile(pattern, flags).search(string)==None): return 0
        else: return 1
#endDef        
def regexpn(pattern, string, flags=0):
        r = re.compile(pattern, flags).subn("", string)
        return r[1]
#endDef
errorCode = ""
errorInfo = ""
traceLevel = "error"

#--------------------------------------------------------------------
# Procedure:   loadProperties
# Description: Loads a properties file from disk and returns a Java
#              Properties object.
#
# Parameters:  propFileName	Name of the properties file
# Returns:     Properties object representing contents of the file
#--------------------------------------------------------------------
def loadProperties ( propFileName ):
	global AdminControl
	from java.util import Properties
        fileprop = Properties() 
        from java.io import FileInputStream
        fileStream = FileInputStream( propFileName )
	fileprop.load(fileStream)
        # see if a value was set for cellName, if not, default to machine name
        cellName = fileprop.getProperty("cellName","")
        writemsg("cellname=" + cellName )
        if (cellName == "" or cellName == "<HOSTNAME>" or cellName == None):
                #allCells=AdminConfig.list('Cell')
                #import  java
                #lineSeparator = java.lang.System.getProperty('line.separator')
                #arrayAllCells = allCells.split(lineSeparator)
                #cell=arrayAllCells[0]
                #cellName=AdminConfig.showAttribute(cell, "name")
                cellName=AdminControl.getCell( )
                writemsg("Cell name was not set in properties file, set default =" + cellName )
                fileprop.setProperty("cellName",cellName)
        #endIf 
        nodeName = fileprop.getProperty("nodeName","")
        if (nodeName == "" or nodeName == "<HOSTNAME>" or nodeName == None):
                nodeName=AdminControl.getNode( )
                writemsg("Node name was not set in properties file, set default =" + nodeName )
                fileprop.setProperty("nodeName",nodeName)
        #endIf 
        return fileprop
#endDef 

#--------------------------------------------------------------------
# Procedure:   writemsg
# Description: Writes a message prepended by a timestamp
# Parameters:
#			msg	 	The message to write
#
# Returns:	None
# History:
# 2003/12/23	wdj	Created
#--------------------------------------------------------------------
def writemsg ( msg ):
	from java.util import Date
        date = Date()
        print "[" + date.toString() + "]" + msg
#endDef 

#--------------------------------------------------------------------
# Procedure:   trace
# Description: trace is used to emit trace messages depending on trace level.
# Parameters:
#		level	(trace level specified)
#		msg	(Message to output)
# Returns:	
# Example: trace info "Now do the xyz step"
# For now, trace does not emit a time stamp.  Probably need to add that.
#--------------------------------------------------------------------

def trace ( level, msg ):
        global traceLevel
	from java.util import Date
        javadate = Date()
        date = javadate.toString()
        if ( traceLevel == "none" ):
                return 
        elif ( traceLevel == "error" ):
                if (level == "error"):
                        print "<"+date+">: "+ msg
        elif ( traceLevel == "warn" ):
                if (regexp(["error"|"warn"], level) ):
                        print "<"+date+">: "+ msg
        elif ( traceLevel == "info" ):
                if (regexp(["error"|"warn" |"info"], level) ):
                        print "<"+date+">: "+ msg 
        elif ( traceLevel == "debug" ):
                print "<"+date+">: "+ msg
        elif ( traceLevel == "default" ):
             	print "Unknown traceLevel: "+ traceLevel
                print "<"+date+">: "+ msg 
        #endIf
#endDef 

def traceEntry ( procName ):
        trace("debug", ">>>> ENTRY "+procName+" " )
#endDef 

def traceExit ( procName ):
        trace("debug", "<<<< EXIT "+procName )
#endDef 

#--------------------------------------------------------------------
# Procedure:   setTraceLevel
# Description: setTraceLevel is used to change the trace level
# Parameters:
#		level	(trace level specified)
#		
# Returns:	
#--------------------------------------------------------------------
def setTraceLevel ( level ):
        global traceLevel
	import re
        trace("debug", "Setting trace level- tracelevel:" + level)
        #traceString = " 'none' | 'error' | 'warn' | 'info' | 'debug' "
        if ( level in ["none" , "error" , "warn", "info" , "debug" ] ):
        	traceLevel = level
        else:
                writemsg("unknown trace level: "+ level)
        #endElse 
#endDef 

#--------------------------------------------------------------------
# Procedure:   initTraceLevel
# Description: gets the tracelevel from the properties file and sets it
#
# Parameters:  props	Properties object
# Returns:     N/A
#--------------------------------------------------------------------
def initTracelevel ( trclevel ):

	setTraceLevel(trclevel)
#endDef 

#--------------------------------------------------------------------
# Procedure:   getTraceLevel
# Description: getTraceLevel is used to get the current trace level
# Parameters:
#		
# Returns:	
#--------------------------------------------------------------------
def getTraceLevel (  ):
        global traceLevel
        return traceLevel
#endDef 

#--------------------------------------------------------------------
# Procedure:   saveChanges 
# Description: saveChanges based on the value of finaloperation
# Parameters:	
#		finaloperation (save|reset)					
#			
# Returns:	N/A
#--------------------------------------------------------------------
def saveChanges ( finalOperation ):
        global AdminConfig
        trace("info", AdminConfig.queryChanges( ) )
        if ( finalOperation == "save" ):
                trace("info", "*** Saving changes to the master configuration ***" )
                AdminConfig.save( )
                # Synchronize changes to the nodes
                trace("info", "*** Synchronizing changes to nodes ***" )
                #syncAllNodesToMaster( )
        elif ( finalOperation == "reset" ):
                trace("info", "*** Resetting the configuration ***" )
                AdminConfig.reset( )
        elif ( finalOperation == "default" ):
                trace("error", "Unknown finaloperation: "+finalOperation )
        #endIf
#endDef 

#--------------------------------------------------------------------
# Procedure:   buildTargetList
# Description: Builds and returns a list of the specified target type
#              i.e. It will return all Cells (one), all Nodes, or 
#              all cluster members of the specified cluster.
#
# Parameters:  scope	The scope of targets to return (Cell, Node, 
#			Server)
#	       clusterName Name of cluster.  Only applicable for
#	                   scope=Server
# Returns:     List of cells, nodes, or servers
#--------------------------------------------------------------------
def buildTargetList ( scope, scopename ):
        global AdminConfig, AdminControl

        # Get the name of the cell and node
        cellName = getCellName()
        dmNode = getNodeName()
        targetList = []

        if (scope == "Node"):
                nodelist = AdminConfig.list("Node").split(java.lang.System.getProperty("line.separator"))
                for node in nodelist:
                        nodename = AdminConfig.showAttribute(node, "name" )
                        if (regexp("(DmgrNode)", nodename, 0) != 0):
                                writemsg("This is deployment manager node :" + nodename)
                       	#endIf 
                       	elif (regexp("webserver*?node", nodename, 0) != 0):
                                writemsg("This is webserver node :" + nodename)    
                        #endIf 
                        else:
                                writemsg("This is base node :" + nodename)
                                if ( scopename == nodename ):
                                	writemsg("Find target scope name :" + nodename)
                                	targetList.append(nodename)
                                #endIf 
                        #endElse 

                #endFor 

        elif (scope == "Cell"):
                targetList = [cellName]
        elif (scope == "ServerCluster"):
                clusterObj = AdminConfig.getid("/Cell:"+cellName+"/" )
                clusterlist = AdminConfig.list("ServerCluster", clusterObj ).split(java.lang.System.getProperty("line.separator"))              
                for cluster in clusterlist:
                        cluster_name = AdminConfig.showAttribute(cluster, "name" )
                        if ( scopename == cluster_name ):
                            writemsg("Find target scope name :" + cluster_name)
                       	    targetList.append(cluster_name)
                        #endIf
                #endFor 
	elif (scope == "Server"):
		serverlist = AdminConfig.list("Server").split(java.lang.System.getProperty("line.separator"))
                for server in serverlist:
                        servername = AdminConfig.showAttribute(server, "name" )
                       	if (regexp("webserver\w+", servername, 0) != 0):
                        	writemsg("This is a web server, will remove from target list :" + servername)    
                        #endIf 
                        else:
                        	writemsg("This is a base server, will add to target list :" + servername)
                        	if ( scopename == servername ):
                        		writemsg("Find target scope name :" + servername)
                        		targetList.append(servername)
                        	#endIf
                        #endElse
                #endFor 
        #endIf 
        #print targetList

        return targetList
#endDef 

#--------------------------------------------------------------------
# Procedure:   getCellName
# Description: returns the name of the Cell to which we're connected
# Parameters:  none
# Returns:     the name of this cell
#--------------------------------------------------------------------
def getCellName (  ):
        global AdminControl
        cellName = AdminControl.getCell( )
        return cellName
#endDef

#--------------------------------------------------------------------
# Procedure:   getNodeName
# Description: returns the name of the Node to which we're connected
# Parameters:  none
# Returns:     the name of this cell
#--------------------------------------------------------------------
def getNodeName (  ):
        global AdminControl
        nodeName = AdminControl.getNode()
        return nodeName
#endDef 

#--------------------------------------------------------------------
# Procedure:   findScopeEntry
# Description: Find cell or node entry
# Parameters:						
#		scope 		(Cell or Node) case sensitive!!!
#		scopeName	(cellName or nodeName)
# Returns:	targetName = success
#               0	   = entry does not exist
#--------------------------------------------------------------------
def findScopeEntry ( scope, scopeName ):
        global AdminApp, AdminConfig
        scopeEntry = 0
        if (scope == "Cell" ):
                scopeEntry = findCell(scopeName)
        elif (scope == "Node" ):
                scopeEntry = findNode(scopeName)
        elif (scope == "Server" ):
                scopeEntry = findServer(scopeName)
        elif (scope == "ServerCluster" ):
                scopeEntry = findServerCluster(scopeName)
        #endIf 	
        return scopeEntry
#endDef 

#--------------------------------------------------------------------
# Procedure:   findCluster
# Description: 
# Parameters:
#		clusterName 
# Returns:	serverName=success
#		0 =server does not exist
#--------------------------------------------------------------------
def findServerCluster ( clusterName ):
        oServerToUse = findConfigTarget(clusterName, "ServerCluster" )
        return oServerToUse
#endDef 

#--------------------------------------------------------------------
# Procedure:   findServer
# Description: 
# Parameters:
#		serverName 
# Returns:	serverName=success
#		0 =server does not exist
#--------------------------------------------------------------------
def findServer ( serverName ):
        oServerToUse = findConfigTarget(serverName, "Server" )
        return oServerToUse
#endDef

#--------------------------------------------------------------------
# Procedure:   findNode
# Description: 
# Parameters:
#		nodeName 
# Returns:	nodeName = success 
#		0 =node does not exist
#--------------------------------------------------------------------
def findNode ( nodeName ):
        oNodeToUse = findConfigTarget(nodeName, "Node" )
        return oNodeToUse
#endDef 

#--------------------------------------------------------------------
# Procedure:   findCell
# Description: 
# Parameters:
#		cellName 
# Returns:	cellName = success
#		h0 = cell does not exist
#--------------------------------------------------------------------
def findCell ( cellName ):
        oCellToUse = findConfigTarget(cellName, "Cell" )
        return oCellToUse
#endDef 

#--------------------------------------------------------------------
# Procedure:   findConfigTarget
# Description: 
# Parameters:
#		nameSearch	 
#		type		Cell, Node, Server   
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def findConfigTarget ( nameSearch, type ):
        global AdminConfig
	oNameToUse = 0
	olist=AdminConfig.list(type).split(java.lang.System.getProperty("line.separator"))
        for oI in olist:
		name = AdminConfig.showAttribute(oI, "name" )
		if (name == nameSearch ):
			#writemsg("Found target entry name:" + oI)
                       	oNameToUse = oI
                #endIf 

        return oNameToUse
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageWASVariable
# Description: Creates/Deletes a WebSphere variable at the desired scope
# Parameters:
#		scope		scope at which to create/delete WAS var
#		scopeName
#		varName		name of the WAS variable
#		varValue	value of the WAS variable
#		varDesc		description of the WAS var
#		create		if true, create the specified WAS var, otherwise, delete it
# Returns:	None
#--------------------------------------------------------------------
def manageWASVariable ( scope, scopeName, varName, varValue, varDesc, create ):
        global AdminConfig
        # Find the appropriate scope object
        scopeEntry = findScopeEntry(scope, scopeName )
        #print "scopeEntry: " + scopeEntry
        writemsg("scopeEntry: " + scopeEntry)
        if (scopeEntry == 0):
                writemsg("ERROR: " + scopeName + " not found at scope: "+scope)
                return 1
        #endIf 

        # See if the variable exists
        varSubstitutions  = AdminConfig.list("VariableSubstitutionEntry", scopeEntry ).split(java.lang.System.getProperty("line.separator"))
        found = "false"
        # Look at each variable entry and see if the name matches the one
        # that was passed in.
        #writemsg("varSubstitutions = " + varSubstitutions )
        # if ( varSubstitutions != [None] ):
        for varEntry in varSubstitutions:
        	if ( varEntry != "" ):
                	thisEntry = AdminConfig.showAttribute(varEntry, "symbolicName" )
	                if (thisEntry == varName):
	                        found = "true"
	                        if (create):
	                                writemsg("Updated WAS variable "+varName+" with value "+varValue+" and description "+varDesc )
	                                AdminConfig.modify(varEntry, [["value", varValue], ["description", varDesc]] )
	                        else:
	                                writemsg("Deleted WAS variable "+varName )
	                                AdminConfig.remove(varEntry )
	#                        #endElse 
	#                #endIf 
	
	        #endFor 

        # we didn't find the requested variable
        # create it if requested, otherwise warn that it wasn't found
        if (found == "false"):
                if (create):
                        parent= AdminConfig.getid("/"+ scope + ":" + scopeName+ "/VariableMap:/" )
                        var = AdminConfig.create("VariableSubstitutionEntry", parent, [["symbolicName", varName], ["value", varValue], ["description", varDesc]] )
                        writemsg("Created WAS variable "+varName+" with value "+varValue+" and description "+varDesc )
                else:
                        writemsg("WAS variable "+varName+" doesn't exist" )
                #endElse 
        #endIf 

        return 0
#endDef 

#--------------------------------------------------------------------
# Procedure:   findJ2CAuthAlias
# Description:
# Parameters:
#               cellName	(Name of cell)
#               authAliasName	(Authentication Alias Name)
# Returns:      targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def findJ2CAuthAlias ( cellName, authAliasName ):
        global AdminConfig
        authDataAlias = 0
        authEntries = AdminConfig.list("JAASAuthData" ).split(java.lang.System.getProperty("line.separator"))
        for authEntry in authEntries :
        	if ( len(authEntry)>0 ):
	                aliasEntry = AdminConfig.showAttribute(authEntry,"alias")
	                if (authAliasName == aliasEntry ):
	                	authDataAlias = authEntry
	                        break 
	        	#endIf
		#endIf     
        #endFor 
        if (authDataAlias != 0):
                return authDataAlias
        #endIf 
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:   manageJ2CAuthEntry
# Description: Creates/Deletes a J2C authorization alias
#		If the alias already exists the auth Alias will not be replaced
#
# Parameters:
#		cellName	(Name of Cell)
#		authAliasName	(Authentication Alias name) 
#		userid		(User ID)
#		password	(Password)
#		create		(Boolean if true, create entry, else delete it)
#		description	(Description of the Alias)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageJ2CAuthEntry ( cellName, authAliasName, userid, password, create, description = " " ):
        global AdminApp, AdminConfig
        authDataAlias = 0
        traceEntry("manageJ2CAuthEntry")
        # validate the J2CAuthEntry parameters
        if (cellName == "" or cellName == None):
                writemsg("Cell Name is required.")
        #endIf 
        if (authAliasName == "" or authAliasName == None):
                 writemsg("Authorization Alias Name is required.")
        #endIf 
        if (userid == "" or userid == None):
                writemsg("Userid is required.")
        #endIf 
        if (password == "" or password == None):
                writemsg("Password is required.")
        #endIf 

        # Check if alias already exists
        authDataAlias =findJ2CAuthAlias(cellName, authAliasName )

        if (authDataAlias != 0):
                if ( not create ):
                        trace("info", "Deleting J2C Auth Alias "+ authDataAlias )
                        AdminConfig.remove(authDataAlias )
                else:
                        trace("info", "J2C Auth Alias "+authAliasName+" already exists" )
                #endElse 

        else:
                if (create):
                        #-- The following makes the auth alias conform to WAS auth alias naming conventions
                        if ((authDataAlias != "") and (authDataAlias != None)):
                                authAlias = cellName+"/"+authAliasName
                                # always create auth aliases at the cell scope
                                # set attrs [subst {(cells/${cellName}:security.xml)} ]
                                cellEntry = AdminConfig.list("Security")
                                attrs0 = [["alias", authAliasName], ["userId", userid], ["password", password], ["description", description ]]
                                authDataAlias = AdminConfig.create("JAASAuthData", cellEntry, attrs0 )
                                trace("info", "Created JASS authorization aliases" )
                        #endIf 
                #endIf 
                else:
                        trace("info", "J2C Auth Alias "+authAliasName+" doesn't exist" )
                #endElse 
        #endElse 
        traceExit("manageJ2CAuthEntry" )
        return authDataAlias
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJDBCProviderWithScope
# Description: 
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#		providerName
#		providerDescription	(xyz jdbc driver)
#               driverClassPath		(i.e. /sqllib/java/db2java.zip
#		implementationClassName	(e.g. oracle.jdbc.pool.OracleConnectionPoolDataSource)
#		providerType		WAS provider type
#		create			(Boolean, if true create objects, else delete them)
# Returns:	The JDBC Provider object if successful.
#--------------------------------------------------------------------
def manageJDBCProviderWithScope ( scope, scopeName, providerName, providerDescription, driverClassPath, implementationClassName, providerType, create ):
        global AdminConfig
        traceEntry("manageJDBCProviderWithScope" )

        scopeEntry = findScopeEntry(scope, scopeName )
        trace("info", "scopeEntry "+ scopeEntry )
        if (scopeEntry == 0):
                return 1
        #endIf 
        provider = findJDBCProviderWithScope(scope, scopeName, providerName )
        if (provider != 0):
                if ( not create):
                        trace("info", "Deleting JDBC provider "+providerName )
                        AdminConfig.remove(provider )
                else:
                        trace("info", "JDBC provider "+provider+" already exists" )
                #endElse 

        else:
                if (create):
                        trace("info", "Creating JDBC provider "+providerName )
                        attrs1 = [["classpath", driverClassPath], ["implementationClassName", implementationClassName], ["providerType", providerType], ["name", providerName], ["description", providerDescription]]
                        provider = AdminConfig.create("JDBCProvider", scopeEntry, attrs1 )
                else:
                        trace("info", "JDBC Provider "+providerName+" doesn't exist" )
                #endElse 
        #endElse 
        traceExit("manageJDBCProviderWithScope" )
        return provider
#endDef

#--------------------------------------------------------------------
# Procedure:   createDataSourceWithScope
# Description: Create a generic datasource based on parameters
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#		providerName
#   		datasourceName
#		datasourceDescription
#		datasourceJNDIName	
#		datasourceHelperClassname
#		statementCacheSize
#		authDataAlias
#		attrs
#
# Returns:	0 = success
#		1 = fail ( no provider)
#--------------------------------------------------------------------  
def createDataSourceWithScope ( scope, scopeName, providerName, datasourceName, datasourceDescription, datasourceJNDIName, datasourceHelperClassname, statementCacheSize, authDataAlias, attrs ):
        global AdminConfig
        traceEntry("createDataSourceWithScope" )

        ds1 = findDataSourceWithScope(scope, scopeName, datasourceName )
        if (ds1 == 0):
                trace("info", "Creating Data Source: "+datasourceName )

                attrs2 = [["name", datasourceName], ["description", datasourceDescription], ["jndiName", datasourceJNDIName], ["datasourceHelperClassname", datasourceHelperClassname], ["statementCacheSize", statementCacheSize], ["authDataAlias", authDataAlias]]
                provider = findJDBCProviderWithScope(scope, scopeName, providerName )
                if (provider == 0):
                        return 1
                #endIf 
                datasource = AdminConfig.create("DataSource", provider, attrs2 )
                propSet1 = AdminConfig.create("J2EEResourcePropertySet", datasource, [] )

                trace("info", "create J2EEResourceProperty" )
                AdminConfig.create("J2EEResourceProperty", propSet1, attrs )

        else:
                trace("info", "Datasource "+datasourceName+" already exists" )
        #endElse 
        traceExit("createDataSourceWithScope" )
        return 0
#endDef 

#--------------------------------------------------------------------
# Procedure:   addCustomPropertyToDataSourceWithScope
# Description: 
# Parameters:
#		scope		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		dsName		(DataSource name)
#		custName	(Custom Property Name)
#		custValue	(Custom Property Value)
# return :
# 		None
#--------------------------------------------------------------------
def addCustomPropertyToDataSourceWithScope ( scope, scopeName, dsName, custName, custValue ):
        global AdminConfig
        traceEntry ("addCustomPropertyToDataSourceWithScope")
        ds = findDataSourceWithScope(scope, scopeName, dsName )
        propSet = AdminConfig.showAttribute(ds, "propertySet" )
        name = ["name", custName]
        value = ["value", custValue]
        attr = [name, value]
        AdminConfig.create("J2EEResourceProperty", propSet, attr )
        traceExit ("addCustomPropertyToDataSourceWithScope") 
#endDef 

#--------------------------------------------------------------------
# Procedure:   setConnectionPool
# Description: modify the settings that affect the max/min size of the DB connection pool
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#   		datasourceName
#		connectionPoolMax 	(Maximum connections)
#               connectionPoolMin	(Minimum connections)
# Returns:	0 = success
#		1 = fail ( datasource not found )
#--------------------------------------------------------------------  
def setConnectionPool ( scope, scopeName, datasourceName, connectionPoolMax, connectionPoolMin ):
        global AdminConfig
        traceEntry ("setConnectionPool")
        datasource = findDataSourceWithScope(scope, scopeName, datasourceName )
        if (datasource == 0):
                writemsg("Datasource not found")
                return 1
        #endIf 
        attrs = [["maxConnections", connectionPoolMax], ["minConnections", connectionPoolMin]]
        AdminConfig.create("ConnectionPool", datasource, attrs )
	traceExit ("setConnectionPool") 
        return 0        
#endDef 

#--------------------------------------------------------------------
# Procedure:   setMappingModule
# Description: 
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#   		datasourceName
#		authDataAlias		(container-managed Alias)
#               mappingConfigAlias	(mapping config alia)
# Returns:	0 = success
#		1 = fail ( Datasource not found )
#--------------------------------------------------------------------  
def setMappingModule ( scope, scopeName, datasourceName, authDataAlias, mappingConfigAlias ):
        global AdminConfig
        traceEntry ("setMappingModule")
        datasource = findDataSourceWithScope(scope, scopeName, datasourceName )
        if (datasource == 0):
                writemsg("Datasource not found")
                return 1
        #endIf 
        attrs = [["authDataAlias", authDataAlias], ["mappingConfigAlias", mappingConfigAlias]]
        AdminConfig.create("MappingModule", datasource, attrs )
	traceExit ("setMappingModule") 
        return 0
#endDef 

#--------------------------------------------------------------------
# Procedure:   setDataSourceCMP
# Description: 
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#   		datasourceName
#		authDataAlias		(container-managed Alias)
# Returns:	0 = success
#		1 = fail (Datasource not found)
#		2 = fail (J2CResourceAdapter not found)
#--------------------------------------------------------------------  
def setDataSourceCMP ( scope, scopeName, datasourceName, authDataAlias ):
        global AdminConfig
        traceEntry ("setDataSourceCMP")
        datasource = findDataSourceWithScope(scope, scopeName, datasourceName )
        if (datasource == 0):
                writemsg("Datasource not found")
                return 1
        #endIf 
        adapterattr = "relationalResourceAdapter"
        j2cresourceadapter = AdminConfig.showAttribute(datasource, adapterattr )
        if (j2cresourceadapter == 0):
                writemsg("J2CResourceAdapter not found")
                return 2
        #endIf 
        tmpname = "mycmp"
        name = ["name", tmpname]
        cmpDatasource = ["cmpDatasource", datasource]
        authDataAlias = ["authDataAlias", authDataAlias]
        attrs = [name, cmpDatasource, authDataAlias]
        AdminConfig.create("CMPConnectorFactory", j2cresourceadapter, attrs )
	traceExit ("setDataSourceCMP")
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:   getAttributeValue
# Description: 
# Parameters:
#		objectid
#		attributename
# Returns:	attrValue
#--------------------------------------------------------------------
def getAttributeValue ( objectid, attributename ):
        global AdminConfig
	traceEntry ("getOneAttributeValue")
        attrNameValue = AdminConfig.showAttribute(objectid, attributename )
        traceExit ("getOneAttributeValue")
        return attrValue
#endDef 
#--------------------------------------------------------------------
# Procedure:   findJDBCProviderWithScope
# Description: 
# Parameters:
#		scope		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		name		(JDBC provider name)
# Returns:	targetName=success
#		0 = does not exist
#--------------------------------------------------------------------
def findJDBCProviderWithScope ( scope, scopeName, name ):
        traceEntry ("findJDBCProviderWithScope")	
        oNameToUse = findConfigTargetWithScope(scope, scopeName, name, "JDBCProvider" )
        traceExit ("findJDBCProviderWithScope") 
        return oNameToUse
#endDef

#--------------------------------------------------------------------
# Procedure:   findDataSourceWithScope
# Description: 
# Parameters:						
#		scope 		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		datasourceName
# Returns:	targetName 	(success) 
#		0 		(does not exist)
#--------------------------------------------------------------------

def findDataSourceWithScope ( scope, scopeName, datasourceName ):
        traceEntry ("findDataSourceWithScope")
        datasource = findConfigTargetWithScope(scope, scopeName, datasourceName, "DataSource" )
        traceExit ("findDataSourceWithScope")  	
        return datasource
#endDef 

#--------------------------------------------------------------------
# Procedure:   findConfigTargetWithScope
# Description: 
# Parameters:						
#		scope 		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		nameSearch	(name string to search)
#		type 		(Cell, Node Server ...)
# Returns:	targetName 	(success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def findConfigTargetWithScope ( scope, scopeName, nameSearch, typename ):
        global AdminConfig
        traceEntry("findConfigTargetWithScope: " + typename )
        trace("info","-----> Start findConfigTargetWithScope "+scope+" "+scopeName+" "+nameSearch+" "+ typename)
        scopeToUse = 0
        newWAS5NamePattern = nameSearch
        newWAS6NamePattern = nameSearch
        stringToFindWAS6 = ""
        stringToFindWAS5 = ""

        # 08/02/2006 update
        if (scope == "Cell" ):
                scopeToUse = findCell(scopeName )
                stringToFindWAS6 = "cells/"+scopeName+"|"
                stringToFindWAS5 = "cells/"+scopeName+":"
                newWAS5NamePattern += "\(cells/"+scopeName+":*?"
                newWAS6NamePattern += "\(cells/"+scopeName+"\|*?"
        elif (scope == "ServerCluster" ):
                scopeToUse = findServerCluster(scopeName)
                stringToFindWAS6 = "clusters/"+scopeName+"|"
                stringToFindWAS5 = "clusters/"+scopeName+":"
                newWAS5NamePattern += "\(cells/*?/clusters/"+scopeName+":*?"
                newWAS6NamePattern += "\(cells/*?/clusters/"+scopeName+"\|*?"
        elif (scope == "Node" ):
                scopeToUse = findNode(scopeName )
                stringToFindWAS6 = "nodes/"+scopeName+"|"
                stringToFindWAS5 = "nodes/"+scopeName+":"
                newWAS5NamePattern += "\(cells/*?/nodes/"+scopeName+":*?"
                newWAS6NamePattern += "\(cells/*?/nodes/"+scopeName+"\|*?"
        elif (scope == "Server" ):
                scopeToUse = findServer(scopeName )
                stringToFindWAS6 = "servers/"+scopeName+"|"
                stringToFindWAS5 = "servers/"+scopeName+":"
                newWAS5NamePattern += "\(cells/*?/nodes/*?/servers/"+scopeName+":*?"
                newWAS6NamePattern += "\(cells/*?/nodes/*?/servers/"+scopeName+"\|*?"
        #endIf 
	writemsg("scopeToUse=" + scopeToUse )
        if (scopeToUse == 0):
                return 0
        #endIf
        writemsg("newWAS5NameSearch=" + newWAS5NamePattern )
        writemsg("newWAS6NameSearch=" + newWAS6NamePattern )
        oList = AdminConfig.list(typename,scopeToUse).split(java.lang.System.getProperty("line.separator"))
        for oI in oList :
	        if (regexp(newWAS6NamePattern, oI, 0) == 1 ) :
	                if (oI.find(stringToFindWAS6)!=-1):
	                        writemsg("Found target:" + nameSearch + " for type:" + typename )	              
	                        writemsg(oI)
	                        oNameToUse = oI
	                        break
	                else:
	                        # writemsg("Target:" + nameSearch + " for type:" + typename + " not found" )
	                        oNameToUse = 0
	        else:
	                if (regexp(newWAS5NamePattern, oI, 0) == 1 ):
	                        if (oI.find(stringToFindWAS5)!=-1):
	                               writemsg("Found target:" + nameSearch + " for type:" + typename )	              
	                               writemsg(oI)
	                               oNameToUse = oI
	                               break
	                        else:
	                               # writemsg("Target:" + nameSearch + " for type:" + typename + " not found" )
	                               oNameToUse = 0
	                else:
	                        # writemsg("Target:" + nameSearch + " for type:" + typename + " not found" )
	                        oNameToUse = 0
	                #endElse 
	        #endElse 
	#endFor	
        traceExit("findConfigTargetWithScope: " + typename )
        return oNameToUse
#endDef

#--------------------------------------------------------------------
# Procedure:	manageMQQueueDestination
# Description:  Creates/Deletes a WebSphere MQ Queue Destination
# Parameters:
#		providerName	(name of this JMS provider)
#		scope		(Cell, Node, Server)
#		scopeName	(Actual name of cell, node, or server)
#		name		(name of MQ Queue Destination)
#		jndiName	(JNDI used to bind connection factory to server)
#		baseQueueName	(name of base queue where messages are sent)
#		baseQueueMgr	(name of base queue manager)
#		targetClient	(receiving application is JMS-compliant or WebSphere MQ)
#		CCSID		( code page ) - added YKL 02/02/2006
#		create		(Boolean if true, create queue dest., else delete it)
#
# Returns:	1 (could not create)
# History:
# 2004/08/20	pty	Created
#--------------------------------------------------------------------
def manageMQQueueDestination ( providerName, scope, scopeName, name, jndiName, baseQueueName, baseQueueMgr, targetClient, CCSID, create ):
        global AdminConfig

        traceEntry("manageMQQueueDestination" )

        # does the queue destination exist?
        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
        #endIf 
        queueDest = findMQQueueDestinationWithScope(scope, scopeName, name )
        if (queueDest != 0):
                if ((  not create ) ):
                        AdminConfig.remove(queueDest )
                        trace("info", "Deleted MQ Queue Destination "+name )
                else:
                        trace("info", "MQ Queue Destination "+name+" already exists" )
                #endElse 

        else:
                if (( create ) ):
                        trace("info", "Creating MQ Queue Destination "+name )
                        jmsProvider = findJMSProviderWithScope(scope, scopeName, providerName )

                        nameAttr = ["name", name]
                        jndiAttr = ["jndiName", jndiName]
                        baseQueueNameAttr = ["baseQueueName", baseQueueName]
                        baseQueueMgrAttr = ["baseQueueManagerName", baseQueueMgr]
                        targetClientAttr = ["targetClient", targetClient]
                        CCSID = ["CCSID", CCSID]

                        mqQDestAttrs = [nameAttr, jndiAttr, baseQueueNameAttr, baseQueueMgrAttr, targetClientAttr, CCSID]

                        template = AdminConfig.listTemplates("MQQueue" )
                        template = template.split(java.lang.System.getProperty("line.separator"))
                        #?PROBLEM? (jacl 1030) previous LINDEX may need variable.split("xx")

                        AdminConfig.createUsingTemplate("MQQueue", jmsProvider, mqQDestAttrs, template[0] )
                else:
                        trace("info", "MQ Queue Destination "+name+" doesn't exist" )
                #endElse 
        #endElse 

        traceExit("manageMQQueueDestination" )
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:	findMQQueueDestinationWithScope
# Description:  Finds the MQ Queue Destination at the specified scope, if it exists.
# Parameters:
#		scope		(Cell, Node, Server)
#		scopeName	(Actual name of cell, node, or server)
#		name		(name of MQ Queue Destination to find)
#
# Returns:	qDest		(success), 0 (could not find)
# History:
# 2004/10/24	fwr	Created
#--------------------------------------------------------------------
def findMQQueueDestinationWithScope ( scope, scopeName, name ):
        qDest = findConfigTargetWithScope(scope, scopeName, name, "MQQueue" )
        return qDest
#endDef 

#--------------------------------------------------------------------
# Procedure:	findJMSProviderWithScope
# Description:  Finds the JMS provider at the specified scope, if it exists.
#               Note: This proc also works for the built-in WMQ providers.
# Parameters:
#		scope		(Cell, Node, Server)
#		scopeName	(Actual name of cell, node, or server)
#		jmsProviderName	(name of JMS provider to find)
#
# Returns:	provider (success), 1 (could not create)
# History:
# 2004/01/07	wdj	Created
#--------------------------------------------------------------------
def findJMSProviderWithScope ( scope, scopeName, jmsProviderName ):
        jmsProvider = findConfigTargetWithScope(scope, scopeName, jmsProviderName, "JMSProvider" )
        return jmsProvider
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageMessageListener
# Description: 
# Parameters:
#		serverName 	(name of server on which LP will exist)
#		lpName		(name of listener port to create)
#		cfJndiName	(connection factory JNDI name)
#		destJndiName	(JMS destination JNDI name)
#		maxSessions	(maximum concurrent sessions)
#		maxRetries	(max number of tries to deliver message)
#		maxMessages	(max messages processed in one session)
#		create		(Boolean if true, create listener, else delete it) 
# Returns:	serverName=success; 0 =server does not exist
# History:
# 2003/03/15	ccd	Created
# 2004/09/09	pty	Modified to include max sessions, retries and messages
#--------------------------------------------------------------------
def manageMessageListener ( serverName, lpName, cfJndiName, destJndiName, maxSessions, maxRetries, maxMessages, create ):
        global AdminConfig

        traceEntry("manageMessageListener" )
        # 09/14/2006 fix for cluster installation
        listenPort = findConfigTargetWithScope("Server", serverName, lpName, "ListenerPort" )
        trace("info", "lpName port "+lpName )
        trace("info", "Listener port " + str(listenPort) )
        if (listenPort != 0):
                if ((  not create ) ):
                        trace("info", "Deleting Listener Port" + lpName )
                        AdminConfig.remove(listenPort )
                else:
                        trace("info", "Listener Port "+lpName+" already exists" )
                #endElse 

        else:
                if (( create ) ):
                        trace("info", "Creating Listener port "+lpName )
                        nameAttr = ["name", lpName]
                        destJndiAttr = ["destinationJNDIName", destJndiName]
                        cfJndiAttr = ["connectionFactoryJNDIName", cfJndiName]
                        maxSessAttr = ["maxSessions", maxSessions]
                        maxRetrAttr = ["maxRetries", maxRetries]
                        maxMessAttr = ["maxMessages", maxMessages]
                        lpAttrs = [nameAttr, destJndiAttr, cfJndiAttr, maxSessAttr, maxRetrAttr, maxMessAttr]

                        appserver = findServer(serverName )
                        mls = AdminConfig.list("MessageListenerService", appserver )

                        lp = AdminConfig.create("ListenerPort", mls, lpAttrs )

                        # Apparently we have to create this object to make the listener
                        # port start automatically when the server starts.
                        AdminConfig.create("StateManageable", lp, [["initialState", "START"]] )
                else:
                        trace("info", "Listener port "+lpName+" doesn't exist" )
                #endElse 
        #endElse 
        traceExit("manageMessageListener" )
        return 0
#endDef 

#--------------------------------------------------------------------
# Procedure:   findJAASConfigurationEntry
# Description:
# Parameters:
#               configName
# Returns:      targetName (success) 0 (does not exist)
# History:
# 2005/04/14    SPR     Created
#--------------------------------------------------------------------
def findJAASConfigurationEntry ( aliasName ):
        global AdminApp, AdminConfig

        writemsg(aliasName )
        configName = 0
        jaasConfigList = AdminConfig.list("JAASConfigurationEntry" ).split(java.lang.System.getProperty("line.separator"))

        for configEntry in jaasConfigList:
                #?PROBLEM? (jacl 232) previous FOREACH_ITEM_IN_LIST may need variable.split("xx")
                if (AdminConfig.showAttribute(configEntry, "alias" ) == aliasName):
                        print aliasName+" found"
                        configName = configEntry
                        break 
                #endIf 
        #endFor 

        if (configName != 0):
                return configName
        else:
                return 0
        #endElse 
#endDef 

#--------------------------------------------------------------------
# Procedure:   getConfigObject
# Description: Gets the config ID for the specified object name and
#		   object type.  Provided as a convenience.
# Parameters:  objectName	The name of the config object
#		   objectType	The type of object
# Returns:	   Config ID of the specified object.
#--------------------------------------------------------------------
def getConfigObject ( objectName, objectType ):
        global AdminConfig

        try:
                _excp_ = 0
                objID = AdminConfig.getid("/"+objectType+":"+objectName+"/" )
                _excp_ = 0 #reset (in case of nested exceptions)
        except:
                _type_, _value_, _tbck_ = sys.exc_info()
                objID = str(_value_)
                _excp_ = 1
        #endTry 
        if (_excp_ ):
                print "Error getting object ID of Config Object"
                print "Error Message = "+objID
                return 
        #endIf 
        return objID
#endDef 

#--------------------------------------------------------------------
# Procedure:   assignSharedLibToEAR
# Description: Assigns a Shared Library to an Enterprise Application.
# Parameters:  eaName		Name of the enterprise application.
#		   sharedLib	Name of the shared library
# Returns:	   Config ID of the specified object.
#--------------------------------------------------------------------
def assignSharedLibToEAR ( eaName, sharedLib ):
        global AdminConfig
        deployment = AdminConfig.getid("/Deployment:"+eaName+"/" )
        appDeploy = AdminConfig.showAttribute(deployment, "deployedObject" )
        classLoader = AdminConfig.showAttribute(appDeploy, "classloader" )

        libattr = ["libraryName", sharedLib]
        sclattr = ["sharedClassloader", "true"]
        attrs = [libattr, sclattr]

        AdminConfig.create("LibraryRef", classLoader, attrs )
#endDef 

#--------------------------------------------------------------------
# Procedure:   findSharedLibrary
# Description:
# Parameters:
#               libName
# Returns:      targetName (success) 0 (does not exist)
# History:
# 2005/04/14    SPR     Created
#--------------------------------------------------------------------
def findSharedLibrary ( libName ):
        global AdminApp, AdminConfig
        
        if ((libName == "") or (libName == None)):
        	return 0

        writemsg("looking for: "+libName )
        libID = 0

        libName = libName.strip( )
        libList = AdminConfig.list("Library" ).split(java.lang.System.getProperty("line.separator"))
        for lib in libList:
                #?PROBLEM? (jacl 265) previous FOREACH_ITEM_IN_LIST may need variable.split("xx")

                name = AdminConfig.showAttribute(lib, "name" )
                writemsg("checking lib: "+lib+" with name "+name )

                if (cmp(name, libName) == 0):
                        print libName+" found: "+lib
                        libID = lib
                        break 
                #else:

                        # nop
                #endElse 
        #endFor 

        # writemsg stdout "returning $libID"

        if (libID != 0):
                return libID
        else:
                return 0
        #endElse 
#endDef 


#--------------------------------------------------------------------
# Procedure:	manageMQQueueConnectionFactory
# Description:  Creates/Deletes a WebSphere MQ Queue Connection Factory
# Parameters:
#		providerName	(name of this JMS provider)
#		scope		(Cell, Node, Server)
#		scopeName	(Actual name of cell, node, or server)
#		name		(name of Connection Factory)
#		jndiName	(JNDI used to bind connection factory to server)
#		queueManager	(name of queue manager for connection factory)
#		transportType   (specifies client connection or JNI bindings)
#		clientID	(JMS client identifier)
#		persistence	(message retention)
#		enableXA	(XA or non-XA QCF/TCF and coordination of messages)
#		CCSID		(code page )  - added YKL 02/02/2006
#		conPool		(connection pool settings) - added ktchu 10/7/2004
#		sessPool	(session pool settings) - added ktchu 10/7/2004
#		create		(Boolean, if true, create the QueueConnFactory, else
#				delete it)
#
#
# Returns:	1 (could not create)
# History:
# 2004/08/20	pty	Created
#--------------------------------------------------------------------
def manageMQQueueConnectionFactory ( providerName, scope, scopeName, name, jndiName, queueManager, transportType, clientID, msgRetention, XAEnabled, CCSID, conPool, sessPool, create ):
        global AdminConfig

        traceEntry("manageMQQueueConnectionFactory" )

        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
        #endIf 
        connFac = findMQQueueConnectionFactoryWithScope(scope, scopeName, name )
        if (connFac != 0):
                if ((  not create ) ):
                        trace("info", "Deleting MQ Queue Connection Factory "+name )
                        AdminConfig.remove(connFac )
                else:
                        trace("info", "MQ Queue Connection Factory "+name+" already exists" )
                #endElse 

        else:
                if (( create ) ):
                        trace("info", "MQ Queue Connection Factory "+name+" not found, creating it now." )
                        print "%%%scope"+scope
                        jmsProvider = findJMSProviderWithScope(scope, scopeName, providerName )

                        nameAttr = ["name", name]
                        jndiAttr = ["jndiName", jndiName]
                        queueMgrAttr = ["queueManager", queueManager]
                        transTypeAttr = ["transportType", transportType]
                        clientIDAttr = ["clientID", clientID]
                        msgRetentionAttr = ["msgRetention", msgRetention]
                        XAAttr = ["XAEnabled", XAEnabled]
                        CCSID = ["CCSID", CCSID]
                        conPoolAttr = ["connectionPool", conPool]
                        sessPoolAttr = ["sessionPool", sessPool]
                        authAttr = ["authMechanismPreference","BASIC_PASSWORD"]
			descAttr = ["description",""]
			
                        mqConnFacAttrs = [nameAttr, jndiAttr, queueMgrAttr, transTypeAttr, clientIDAttr, msgRetentionAttr, XAAttr, CCSID, conPoolAttr, sessPoolAttr, authAttr , descAttr ]

                        template = AdminConfig.listTemplates("MQQueueConnectionFactory" ).split(java.lang.System.getProperty("line.separator"))[0]
                        #?PROBLEM? (jacl 1170) previous LINDEX may need variable.split("xx")
                        print "jmsProvider=" + str(jmsProvider)
                        print "mqConnFacAttrs=" + str(mqConnFacAttrs)
                        print "template=" + str(template)

                        qmfactory=AdminConfig.createUsingTemplate("MQQueueConnectionFactory", jmsProvider, mqConnFacAttrs, template )
                        mappingAttrs=[["mappingConfigAlias","DefaultPrincipalMapping"],["authDataAlias",""]]
                        AdminConfig.create("MappingModule",qmfactory,mappingAttrs)
                        return 0
                else:
                        trace("info", "MQ Queue Connection Factory "+name+" doesn't exist" )
                #endElse 
                traceExit("manageMQQueueConnectionFactory" )
        #endElse 
#endDef 

#--------------------------------------------------------------------
# Procedure:	manageRemoteMQQueueConnectionFactory
# Description:  Creates/Deletes a WebSphere MQ Queue Connection Factory
# Parameters:
#		providerName	(name of this JMS provider)
#		scope		(Cell, Node, Server)
#		scopeName	(Actual name of cell, node, or server)
#		name		(name of Connection Factory)
#		jndiName	(JNDI used to bind connection factory to server)
#		queueManager	(name of queue manager for connection factory)
#		host		(specifies client connection host)
#		port		(specifies client connection port)
#		channel		(specifies client connection channel)
#		transportType   (specifies client connection or JNI bindings)
#		clientID	(JMS client identifier)
#		persistence	(message retention)
#		enableXA	(XA or non-XA QCF/TCF and coordination of messages)
#		CCSID		(code page )  - added YKL 02/02/2006
#		conPool		(connection pool settings) - added ktchu 10/7/2004
#		sessPool	(session pool settings) - added ktchu 10/7/2004
#		create		(Boolean, if true, create the QueueConnFactory, else
#				delete it)
#
#
# Returns:	1 (could not create)
# History:
# 2004/08/20	pty	Created
#--------------------------------------------------------------------
def manageRemoteMQQueueConnectionFactory ( providerName, scope, scopeName, name, jndiName, queueManager, host, port, channel, transportType, clientID, msgRetention, XAEnabled, CCSID, conPool, sessPool, componentAuthAlias, create ):
        global AdminConfig

        traceEntry("manageMQQueueConnectionFactory" )

        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
        #endIf 
        connFac = findMQQueueConnectionFactoryWithScope(scope, scopeName, name )
        if (connFac != 0):
                if ((  not create ) ):
                        trace("info", "Deleting MQ Queue Connection Factory "+name )
                        AdminConfig.remove(connFac )
                else:
                        trace("info", "MQ Queue Connection Factory "+name+" already exists" )
                #endElse 

        else:
                if (( create ) ):
                        trace("info", "MQ Queue Connection Factory "+name+" not found, creating it now." )
                        print "%%%scope"+scope
                        jmsProvider = findJMSProviderWithScope(scope, scopeName, providerName )

                        nameAttr = ["name", name]
                        jndiAttr = ["jndiName", jndiName]
                        queueMgrAttr = ["queueManager", queueManager]
                        hostAttr = ["host", host]
                        portAttr = ["port", port]
                        channelAttr = ["channel", channel]                        
                        transTypeAttr = ["transportType", transportType]
                        clientIDAttr = ["clientID", clientID]
                        msgRetentionAttr = ["msgRetention", msgRetention]
                        XAAttr = ["XAEnabled", XAEnabled]
                        CCSID = ["CCSID", CCSID]
                        conPoolAttr = ["connectionPool", conPool]
                        sessPoolAttr = ["sessionPool", sessPool]
                        authAttr = ["authMechanismPreference","BASIC_PASSWORD"]
                        descAttr = ["description",""]
                        
                        mqConnFacAttrs = [nameAttr, jndiAttr, queueMgrAttr, hostAttr, portAttr, channelAttr, transTypeAttr, clientIDAttr, msgRetentionAttr, XAAttr, CCSID, conPoolAttr, sessPoolAttr, authAttr , descAttr ]

                        template = AdminConfig.listTemplates("MQQueueConnectionFactory" ).split(java.lang.System.getProperty("line.separator"))[0]
                        #?PROBLEM? (jacl 1170) previous LINDEX may need variable.split("xx")
                        print "jmsProvider=" + str(jmsProvider)
                        print "mqConnFacAttrs=" + str(mqConnFacAttrs)
                        print "template=" + str(template)

                        qmfactory=AdminConfig.createUsingTemplate("MQQueueConnectionFactory", jmsProvider, mqConnFacAttrs, template )
                        mappingAttrs=[["mappingConfigAlias","DefaultPrincipalMapping"],["authDataAlias",componentAuthAlias]]
                        print "mappingAttrs=" + str(mappingAttrs)
                        AdminConfig.create("MappingModule",qmfactory,mappingAttrs)
                        return 0
                else:
                        trace("info", "MQ Queue Connection Factory "+name+" doesn't exist" )
                #endElse 
                traceExit("manageMQQueueConnectionFactory" )
        #endElse 
#endDef 

#--------------------------------------------------------------------
# Procedure:	findMQQueueConnectionFactoryWithScope
# Description:  Finds the MQ Queue Connection Factory at the specified scope, if it exists.
# Parameters:
#		scope		(Cell, Node, Server)
#		scopeName	(Actual name of cell, node, or server)
#		name		(name of MQ Queue Connection Factory to find)
#
# Returns:	connFac (success), 0 (could not find)
# History:
# 2004/10/24	fwr	Created
#--------------------------------------------------------------------
def findMQQueueConnectionFactoryWithScope ( scope, scopeName, name ):
        connFac = findConfigTargetWithScope(scope, scopeName, name, "MQQueueConnectionFactory" )
        return connFac
#endDef 

#--------------------------------------------------------------------
# Procedure:   assignSharedLibToApplication
# Description: Assigns a Shared Library to a WebModule.
# Parameters:  	appName		Name of the enterprise application.
#		sharedLib	Name of the shared library
# Returns:	None.
#--------------------------------------------------------------------
def assignSharedLibToApplication ( appName, sharedLib ):
        global AdminApp 
        option = "[ -MapSharedLibForMod [[ " + appName + " " + "META-INF/application.xml" + " " + sharedLib + " ]]]"
        print "assignSharedLibToApplication option = " + option
	AdminApp.edit( appName , option )
#endDef

#--------------------------------------------------------------------
# Procedure:   assignSharedLibToWebModule
# Description: Assigns a Shared Library to a WebModule.
# Parameters:  	appName		Name of the enterprise application.
#		moduleName	Name of the Web Module.
#		warfile		Name of the warfile.
#		descriptor	Name of descriptor.
#		sharedLib	Name of the shared library
# Returns:	None.
#--------------------------------------------------------------------
def assignSharedLibToWebModule ( appName, moduleName, warfile, descriptor, sharedLib ):
        global AdminApp
        warfiledescriptor =  warfile + "," + descriptor 
        option = "[ -MapSharedLibForMod [[ " + moduleName + " " + warfiledescriptor + " " + sharedLib + " ]]]"
        print "assignSharedLibToWebModule option = " + option
	AdminApp.edit( appName , option )
#endDef 

#--------------------------------------------------------------------
# Procedure:   assignClassloaderToWebModule
# Description: Assigns a classloader to a WebModule.
# Parameters:  	appName		Name of the enterprise application.
#		warfile		Name of the warfile.
#		mode		classloader mode. ( PARENT_LAST or PARENT_FIRST )
# Returns:	None.
#--------------------------------------------------------------------
def assignClassloaderToWebModule ( appName, warfile, mode ):
        global AdminConfig
        deployments  = AdminConfig.getid("/Deployment:"+appName+"/" )
        depObject = AdminConfig.showAttribute(deployments, "deployedObject" )
        modules = AdminConfig.showAttribute(depObject, "modules")
        arrayModules = modules[1:len(modules)-1].split(" ")
        for module in arrayModules:
        	uri = AdminConfig.showAttribute(module, "uri")        	
        	if (cmp(uri, warfile) == 0):
        		print "WebModule " + warfile + " found :" + module
        		currentloaderMode = AdminConfig.showAttribute(module, "classloaderMode")
        		print "Current classloaderMode = " + currentloaderMode
        		AdminConfig.modify(module, [["classloaderMode", mode]])
        		print "WebModule classloaderMode set to " + mode
        		break

#endDef 


#--------------------------------------------------------------------
# Procedure:   manageSIBus
# Description: Creates/Deletes SI bus
#		If the bus already exists the bus will not be replaced
#
# Parameters:
#		busName	(Name of Bus)
#		busDescription	(Description of bus) 
#		busSecurity	(User ID)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageSIBus(busName, busDescription, busSecurity, create):
        global AdminApp, AdminConfig
        
        traceEntry("manageSIBus" )
        # Check if bus already exists
        bus = findSIBus(busName)

        if (bus != 0):
                if ( not create ):
                        trace("info", "Deleting SI bus "+ bus )
                        AdminConfig.remove(bus)
                else:
                        trace("info", "SI bus "+busName+" already exists" )
                #endElse 

        else:
                if (create):
			cell = findCell(getCellName())
			attrs = [["name", busName],["description", busDescription],["secure", busSecurity]]
  
			bus = AdminConfig.create("SIBus", cell, attrs)
                #endIf 
                else:
                        trace("info", "SI bus "+busName+" doesn't exist" )
                #endElse 
        #endElse 
        traceExit("manageSIBus" )
        return bus
#endDef

#--------------------------------------------------------------------
# Procedure:   manageSIBusMember
# Description: Creates/Deletes SI bus member
#		If the bus member already exists the bus member will not be replaced
#
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#		busName	(Name of Bus)
#		severName	(Name of server) 
#		clusterName	(Name of cluster)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageSIBusMember(scope, scopeName, busName, serverName, clusterName, create):
        global AdminApp, AdminConfig
        
        traceEntry("manageSIBusMember" )
        
        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
                
        nodeName = scopeName
        
        bus = findSIBus(busName)
        
        # Check if bus member already exists
        busmember = findSIBusMember(busName,nodeName,serverName)

        if (busmember != 0):
                if ( not create ):
                        trace("info", "Deleting SI bus "+busName+" member - node:" + nodeName+" server:"+ serverName+" clusterName:"+clusterName)
                        
                        attrs = "[-bus %s -node %s -server %s]" % (busName, nodeName, serverName)
			#remove the member
			AdminTask.removeSIBusMember(attrs)
                else:
                        trace("info", "SI bus "+busName+" member - node:" + nodeName+" server:"+ serverName+" clusterName:"+clusterName+" already exists" )
                #endElse 

        else:
                if (bus != 0):
	                if (create):
				#Adding the member.
				attrs = "[-bus %s -node %s -server %s]" % (busName, nodeName, serverName)
				AdminTask.addSIBusMember(attrs)
	                #endIf 
	                else:
	                        trace("info", "SI bus "+busName+" member - node:" + nodeName+" server:"+ serverName+" clusterName:"+clusterName+" doesn't exist" )
	                #endElse 
	       	#endIf 
	       	else:
	       		trace("info", "SI bus "+busName+" doesn't exist" )
	       	#endElse
        #endElse 
        traceExit("manageSIBusMember" )
        return bus
#endDef

#--------------------------------------------------------------------
# Procedure:   manageSIBusDestination
# Description: Creates/Deletes SI bus destination
#		If the bus destination already exists the bus destination will not be replaced
#
# Parameters:
#		scope 		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		destName	(Name of destination)
#		busName		(Name of Bus)
#		severName	(Name of server)
#		destinationType	( Queue, TopicSpace, Alias, Foreign )
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageSIBusDestination(scope, scopeName, destName, busName, serverName, destinationType , create):
        global AdminApp, AdminConfig
        
        traceEntry("manageSIBusDestination" )
        
        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
                
        nodeName = scopeName
        
        bus = findSIBus(busName)
        
        # Check if bus destination already exists
        busDestination = findSIBusDestination(destName,busName,destinationType)

        if (busDestination != 0):
                if ( not create ):
                        trace("info", "Deleting SI destination:"+destName+" bus:"+busName+"  - node:" + nodeName+" server:"+ serverName)
                        
                        attrs = "[-bus %s -name %s ]" % (busName,destName)
			#remove the destination
			AdminTask.deleteSIBDestination(attrs)
                else:
                        trace("info", "SI destination:"+destName+" bus:"+busName+"  - node:" + nodeName+" server:"+ serverName+" already exists" )
                #endElse 

        else:
                if (bus != 0):
	                if (create):
				#Adding the destination.
				attrs = "[-bus %s -name %s -type %s -node %s -server %s]" % (busName,destName,destinationType,nodeName,serverName)
				AdminTask.createSIBDestination(attrs)
	                #endIf 
	                else:
	                        trace("info", "SI destination:"+destName+" bus:"+busName+"  - node:" + nodeName+" server:"+ serverName+" doesn't exist" )
	                #endElse 
	       	#endIf 
	       	else:
	       		trace("info", "SI bus "+busName+" doesn't exist" )
	       	#endElse
        #endElse 
        traceExit("manageSIBusDestination" )
        return bus
#endDef


#--------------------------------------------------------------------
# Procedure:   findSIBus
# Description:
# Parameters:
#               busName	(Name of bus)
#		
# Returns:      targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def findSIBus ( busName):
        global AdminConfig
        siBusEntries = AdminConfig.list("SIBus" ).split(java.lang.System.getProperty("line.separator"))
        for siBusEntry in siBusEntries :
        	if ( len(siBusEntry)>0 ): 
	                busNameEntry = AdminConfig.showAttribute(siBusEntry,"name")
	                if (busName == busNameEntry ):
	                	return siBusEntry
	                        break 
	        	#endIf
		#endIf     
        #endFor 
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:   findSIBusMember
# Description:
# Parameters:
#               bus	(bus)
#		nodeName (Name of node)
#		serverName (Name of server)
# Returns:      targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def findSIBusMember ( busName, nodeName,serverName):
        global AdminConfig
        attrs = "[-bus %s]" % (busName)
	siBusEntries = AdminTask.listSIBusMembers(attrs).split(java.lang.System.getProperty("line.separator")) 
        for siBusEntry in siBusEntries :
        	if ( len(siBusEntry)>0 ):
        		nodeNameEntry = AdminConfig.showAttribute(siBusEntry,"node")
        		serverNameEntry = AdminConfig.showAttribute(siBusEntry,"server")
        		if ((nodeName==nodeNameEntry) and (serverName==serverNameEntry)):
	                	return siBusEntry
	                #endIf 	
		#endIf     
        #endFor 
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:   findSIBusDestination
# Description:
# Parameters:
#		destName (Name of destination)
#               bus	(bus)
#		nodeName (Name of node)
#		serverName (Name of server)
#		desType	(Type of destination - Queue)
# Returns:      targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def findSIBusDestination (destName, busName,desType):
        global AdminConfig
        attrs = "[-bus %s -type %s]" % (busName,desType)
	siBusDestEntries = AdminTask.listSIBDestinations(attrs).split(java.lang.System.getProperty("line.separator")) 
        for siBusDestEntry in siBusDestEntries :
        	if ( len(siBusDestEntry)>0 ):
        		nameEntry = AdminConfig.showAttribute(siBusDestEntry,"identifier")
        		if (destName==nameEntry):
	                	return siBusDestEntry
	                #endIf 	
		#endIf     
        #endFor 
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:   findJMSConnectionFactory
# Description:
# Parameters:
#		scopeEntry (Scope entry)
#               cfName	(Connection factory name)
#		busName (Name of bus)
#		cfType (Type of connection factory)
# Returns:      targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def findJMSConnectionFactory(scopeEntry,cfName,busName,cfType):
        global AdminConfig
        attrs = "[-type %s]" % (cfType)
	cfEntries = AdminTask.listSIBJMSConnectionFactories(scopeEntry,attrs).split(java.lang.System.getProperty("line.separator")) 
        for cfEntry in cfEntries :
        	if ( len(cfEntry)>0 ):
        		nameEntry = AdminConfig.showAttribute(cfEntry,"name")
        		if (cfName==nameEntry):
	                	return cfEntry
	                #endIf 	
		#endIf     
        #endFor 
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:   manageJMSConnectionFactory
# Description: Creates/Deletes JMS connection factory
#		If the connection factory already exists the connection factory will not be replaced
#
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#		cfName			(Name of connection factory)
#		jndiName		(JNDI name of CF)
#		busName	(Name of Bus)
#		cfType	(Type of CF) 
#		authDataAlias	(authentication alias)
#		targetType	(target type)
#		xaAuthAlias	(xa authentication alias)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageJMSConnectionFactory(scope,scopeName,cfName,jndiName,busName,cfType,authDataAlias,targetType,xaAuthAlias,endpoints,create):
        global AdminApp, AdminConfig
        
        traceEntry("manageJMSConnectionFactory" )
        
        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
                
        nodeName = scopeName
        
        bus = findSIBus(busName)
        
        # Check if connection factory already exists
        connectionFactory = findJMSConnectionFactory(scopeEntry,cfName,busName,cfType)

        if (connectionFactory != 0):
                if ( not create ):
                        trace("info", "Deleting JMS connection factory "+cfName+"  - node:" + nodeName)
                        
			#remove connection factory
			AdminTask.deleteSIBJMSConnectionFactory(connectionFactory)
                else:
                        trace("info", "JMS connection factory "+cfName+"  - node:" + nodeName+" already exists" )
                #endElse 

        else:
                if (bus != 0):
	                if (create):
				#create connection factory
				attrs = "[-name %s -jndiName %s -busName %s -type %s -targetType %s -authDataAlias %s -xaRecoveryAuthAlias %s -providerEndPoints %s]" % (cfName,jndiName,busName,cfType,targetType,authDataAlias,xaAuthAlias,endpoints)
	        		AdminTask.createSIBJMSConnectionFactory(scopeEntry,attrs)
	                #endIf 
	                else:
	                        trace("info", "JMS connection factory "+cfName+"  - node:" + nodeName+" doesn't exist" )
	                #endElse 
	       	#endIf 
	       	else:
	       		trace("info", "SI bus "+busName+" doesn't exist" )
	       	#endElse
        #endElse 
        traceExit("manageJMSConnectionFactory" )
        return bus
#endDef


#--------------------------------------------------------------------
# Procedure:   findJMSqueue
# Description:
# Parameters:
#		scopeEntry (Scope entry)
#               jmsQName	(JMS queue name)
#		busName (Name of bus)
# Returns:      targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def findJMSqueue(scopeEntry,jmsQName,busName):
        global AdminConfig
	jmsqEntries = AdminTask.listSIBJMSQueues(scopeEntry).split(java.lang.System.getProperty("line.separator")) 
        for jmsqEntry in jmsqEntries :
        	if ( len(jmsqEntry)>0 ):
        		nameEntry = AdminConfig.showAttribute(jmsqEntry,"name")
        		if (jmsQName==nameEntry):
	                	return jmsqEntry
	                #endIf 	
		#endIf     
        #endFor 
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:   manageJMSQueue
# Description: Creates/Deletes JMS queue
#		If the JMS queue already exists the JMS queue will not be replaced
#
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#		jmsQName			(Name of JMS queue)
#		jndiName		(JNDI name of JMS queue)
#		busName	(Name of Bus)
#		queueName	(Name of destination queue) 
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageJMSQueue(scope,scopeName,jmsQName,jndiName,busName, queueName, create):
        global AdminApp, AdminConfig
        
        traceEntry("manageJMSQueue" )
        
        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
                
        nodeName = scopeName
        
        bus = findSIBus(busName)
        
        # Check if jms queue already exists
        jmsqueue = findJMSqueue(scopeEntry,jmsQName,busName)

        if (jmsqueue != 0):
                if ( not create ):
                        trace("info", "Deleting JMS queue "+jmsQName+"  - node:" + nodeName)
                        
			#remove  JMS queue
			AdminTask.deleteSIBJMSQueue(jmsqueue)
                else:
                        trace("info", "JMS queue "+jmsQName+"  - node:" + nodeName+" already exists" )
                #endElse 

        else:
                if (bus != 0):
	                if (create):
				#create JMS queue
	        		attrs = "[-name %s -jndiName %s -queueName %s -busName %s]" % (jmsQName,jndiName,queueName,busName)
	        		AdminTask.createSIBJMSQueue(scopeEntry,attrs)
	                #endIf 
	                else:
	                        trace("info", "JMS queue "+jmsQName+"  - node:" + nodeName+" doesn't exist" )
	                #endElse 
	       	#endIf 
	       	else:
	       		trace("info", "SI bus "+busName+" doesn't exist" )
	       	#endElse
        #endElse 
        traceExit("manageJMSQueue" )
        return bus
#endDef

#--------------------------------------------------------------------
# Procedure:   findJMSTopic
# Description:
# Parameters:
#		scopeEntry 	(Scope entry)
#               jmsTopicName	(JMS Topic name)
#		busName 	(Name of bus)
# Returns:      targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def findJMSTopic (scopeEntry, jmsTopicName, busName):
        global AdminConfig
	jmsqEntries = AdminTask.listSIBJMSTopics(scopeEntry).split(java.lang.System.getProperty("line.separator")) 
        for jmsqEntry in jmsqEntries :
        	if ( len(jmsqEntry)>0 ):
        		nameEntry = AdminConfig.showAttribute(jmsqEntry,"name")
        		if (jmsTopicName==nameEntry):
	                	return jmsqEntry
	                #endIf 	
		#endIf     
        #endFor 
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:   manageJMSTopic
# Description: Creates/Deletes JMS Topic
#		If the JMS Topic already exists the JMS Topic will not be replaced
#
# Parameters:
#		scope 		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		name		(Name of JMS Topic)
#		jndiName	(JNDI name of JMS Topic)
#		busName		(Name of Bus)
#		topicName	(Name of destination topic) 
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageJMSTopic(scope,scopeName,name,jndiName,busName,topicName,topicSpace, create):
        global AdminApp, AdminConfig
        
        traceEntry("manageJMSTopic" )
        
        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
                
        nodeName = scopeName
        
        bus = findSIBus(busName)
        
        # Check if jms topic already exists
        jmstopic = findJMSTopic(scopeEntry,name,busName)

        if (jmstopic != 0):
                if ( not create ):
                        trace("info", "Deleting JMS topic "+ name +"  - node:" + nodeName)
                        
			#remove  JMS topic
			AdminTask.deleteSIBJMSTopic(jmstopic)
                else:
                        trace("info", "JMS topic "+ name +"  - node:" + nodeName+" already exists" )
                #endElse 

        else:
                if (bus != 0):
	                if (create):
				#create JMS topic
	        		attrs = "[-name %s -jndiName %s -topicSpace %s -topicName %s -busName %s]" % (name,jndiName,topicSpace,topicName,busName)
	        		AdminTask.createSIBJMSTopic(scopeEntry,attrs)
	                #endIf 
	                else:
	                        trace("info", "JMS topic "+name+"  - node:" + nodeName+" doesn't exist" )
	                #endElse 
	       	#endIf 
	       	else:
	       		trace("info", "SI bus "+busName+" doesn't exist" )
	       	#endElse
        #endElse 
        traceExit("manageJMSTopic" )
        return bus
#endDef


#--------------------------------------------------------------------
# Procedure:   findJMSActSpec
# Description:
# Parameters:
#		scopeEntry (Scope entry)
#               jmsActSpecName	(JMS activation spec name)
#		busName (Name of bus)
# Returns:      targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def findJMSActSpec(scopeEntry,jmsActSpecName,busName):
        global AdminConfig
	jmsactspecEntries = AdminTask.listSIBJMSActivationSpecs(scopeEntry).split(java.lang.System.getProperty("line.separator")) 
        for jmsactspecEntry in jmsactspecEntries :
        	if ( len(jmsactspecEntry)>0 ):
        		nameEntry = AdminConfig.showAttribute(jmsactspecEntry,"name")
        		if (jmsActSpecName==nameEntry):
	                	return jmsactspecEntry
	                #endIf 	
		#endIf     
        #endFor 
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:   findJ2CResourceAdapter
# Description:
# Parameters:
#		scopeEntry (Scope entry)
#               jmsActSpecName	(JMS activation spec name)
#		busName (Name of bus)
# Returns:      targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def findJ2CResourceAdapter(scopeEntry, adapterName):
        global AdminConfig
	j2cResourceAdapters = AdminConfig.list("J2CResourceAdapter",scopeEntry).split(java.lang.System.getProperty("line.separator")) 
        for j2cResourceAdapter in j2cResourceAdapters :
        	if ( len(j2cResourceAdapter)>0 ):
        		nameEntry = AdminConfig.showAttribute(j2cResourceAdapter,"name")
        		if (adapterName==nameEntry):
	                	return j2cResourceAdapter
	                #endIf 	
		#endIf     
        #endFor 
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:   manageJMSActivationSpec
# Description: Creates/Deletes JMS activation specification
#		If the JMS activation spec already exists the JMS activation spec will not be replaced
#
# Parameters:
#		scope 		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		jmsActSpecName	(Name of JMS activation specification)
#		jndiName	(JNDI name of JMS queueor topic)
#		busName		(Name of Bus)
#		jndiDest	(JNDI Name of destination queue) 
#		authAlias	(Authentication alias)
#		messageSelector (Message selector)
#		durability 	(durability )
#		subscriptionName(subscriptionname)
#		clientID 	(clientid)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageJMSActivationSpec(scope, scopeName, jmsActSpecName, jndiName, busName, jndiDest, destType, authAlias, messageSelector, durability, subscriptionName, clientID, create):
        global AdminApp, AdminConfig
        
        traceEntry("manageJMSActivationSpec" )
        
        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
                
        nodeName = scopeName
        durableSubscriptionHome = getNodeName() + "." + scopeName + "-" + busName 
        bus = findSIBus(busName)
        
        # Check if jms activation specification already exists
        jmsactspec = findJMSActSpec(scopeEntry,jmsActSpecName,busName)

        if (jmsactspec != 0):
                if ( not create ):
                        trace("info", "Deleting JMS activation spec "+jmsActSpecName+"  - node:" + nodeName)
                        
			#remove  JMS activation spec
			AdminTask.deleteSIBJMSActivationSpec(jmsactspec)
                else:
                        trace("info", "JMS activation spec "+jmsActSpecName+"  - node:" + nodeName+" already exists" )
                #endElse 

        else:
                if (bus != 0):
	                if (create):
				#create JMS activation spec
				if ((durability == "Durable") or (durability == "durable") ) :
	        			attrs = "[-name %s -jndiName %s -destinationJndiName %s -busName %s -destinationType %s -authenticationAlias %s -messageSelector %s -subscriptionDurability Durable -subscriptionName %s -clientId %s -durableSubscriptionHome %s]" % (jmsActSpecName,jndiName,jndiDest,busName,destType,authAlias,messageSelector,subscriptionName,clientID,durableSubscriptionHome)
        			else :
	        			attrs = "[-name %s -jndiName %s -destinationJndiName %s -busName %s -destinationType %s -authenticationAlias %s -messageSelector %s]" % (jmsActSpecName,jndiName,jndiDest,busName,destType,authAlias,messageSelector)        				
        			AdminTask.createSIBJMSActivationSpec(scopeEntry,attrs)
	                #endIf 
	                else:
	                        trace("info", "JMS activation spec "+jmsActSpecName+"  - node:" + nodeName+" doesn't exist" )
	                #endElse 
	       	#endIf 
	       	else:
	       		trace("info", "SI bus "+busName+" doesn't exist" )
	       	#endElse
        #endElse 
        traceExit("manageJMSActivationSpec" )
        return bus
#endDef

#--------------------------------------------------------------------
# Procedure:   manageMQActivationSpec
# Description: Creates/Deletes MQ activation specification
#		If the MQ activation spec already exists the MQ activation spec will not be replaced
#
# Parameters:
#		scope 		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		jmsActSpecName	(Name of JMS activation specification)
#		jndiName	(JNDI name of JMS queueor topic)
#		busName		(Name of Bus)
#		jndiDest	(JNDI Name of destination queue) 
#		authAlias	(Authentication alias)
#		messageSelector (Message selector)
#		durability 	(durability )
#		subscriptionName(subscriptionname)
#		clientID 	(clientid)
#		queueManager 	(queueManager)
#		hostName 	(hostName)
#		port	 	(port)
#		channel 	(channel)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageMQActivationSpec(scope, scopeName, mqActSpecName, jndiName, jndiDest, destType, authAlias, messageSelector, queueManager, hostName, port, channel, durability, subscriptionName, clientID, create):
        global AdminApp, AdminConfig
        
        traceEntry("manageMQActivationSpec" )
        
        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
                
       	trace("info", "scopeEntry= " + scopeEntry)
                
        # Check if jms activation specification already exists
        IsActivationSpecExist = 0
        activationSpecs=AdminTask.listWMQActivationSpecs(scopeEntry).split(java.lang.System.getProperty("line.separator")) 
     	for activationSpec in activationSpecs :
    		trace("info", "activationSpec= " + activationSpec)
    		if( activationSpec != "" ) : 
        		nameEntry = AdminConfig.showAttribute(activationSpec,"name")
			trace("info", "nameEntry= " + nameEntry)
        		if (mqActSpecName==nameEntry):
        			IsActivationSpecExist = 1
	                	break
	                #endIf
	        #endIf	
	#endFor 
        #trace("info", "IsActivationSpecExist= " + IsActivationSpecExist)       
        if (IsActivationSpecExist != 0):
                if ( not create ):
                        trace("info", "Deleting JMS activation spec "+mqActSpecName+"  - scopeName:" + scopeName)
                        
			#remove  JMS activation spec
			AdminTask.deleteWMQActivationSpec(activationSpec)
                else:
                        trace("info", "JMS activation spec "+mqActSpecName+"  - scopeName:" + scopeName+" already exists" )
                #endElse 

        else:
 		if (create):
			#create MQ activation spec
			if ((durability == "Durable") or (durability == "durable") ) :
	        		attrs = "[-name %s -jndiName %s -destinationJndiName %s -destinationType %s  -messageSelector %s -qmgrName %s -qmgrHostname %s -qmgrPortNumber %s -qmgrSvrconnChannel %s -subscriptionDurability Durable -subscriptionName %s -clientId %s -durableSubscriptionHome %s -authAlias %s]" % (mqActSpecName,jndiName,jndiDest,destType,messageSelector,queueManager,hostName,port,channel,durability, subscriptionName, clientID, authAlias)        				
        		else :
	        		attrs = "[-name %s -jndiName %s -destinationJndiName %s -destinationType %s  -messageSelector %s -qmgrName %s -qmgrHostname %s -qmgrPortNumber %s -qmgrSvrconnChannel %s -authAlias %s]" % (mqActSpecName,jndiName,jndiDest,destType,messageSelector,queueManager,hostName,port,channel, authAlias)        				
	        	trace("info", "attribute= " + attrs )
        		activationSpec = AdminTask.createWMQActivationSpec(scopeEntry,attrs)
		#endIf 
		else:
	        	trace("info", "JMS activation spec "+mqActSpecName+"  - scopeName:" + scopeName +" doesn't exist" )
	        #endElse 

        #endElse
        trace("info", "activationSpec= " + activationSpec)
        traceExit("manageMQActivationSpec" )
        return activationSpec
#endDef

#--------------------------------------------------------------------
# Procedure:   manageStartupBeanService
# Description: Enable/Disable Startup bean services

# Parameters:
#		serverName 			(Server Name )
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageStartupBeanService(serverName, create):
        global AdminApp, AdminConfig
        
        traceEntry("manageStartupBeanService" )
        
        
        server = findServer(serverName)
	startupbean = 0;	
        if (server != 0):
        	startupbean = AdminConfig.list('StartupBeansService', server)
                if ( not create ):
                        trace("info", "Disabling startup bean services on server "+serverName)
			#disable  
			AdminConfig.modify(startupbean, [['enable', 'false']])
                else:
                        trace("info", "Enabling startup bean services on server "+serverName)
                        #eanble
                        AdminConfig.modify(startupbean, [['enable', 'true']])
                #endElse 

        else:
	       	trace("info", "Server "+serverName+" doesn't exist" )
	       	#endElse
        #endElse 
        traceExit("manageStartupBeanService" )
        return startupbean
#endDef


#--------------------------------------------------------------------
# Procedure:   findObjectCache
# Description:
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#               objCacheName	(Object cache name)
# Returns:      targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def findObjectCache(scope, scopeName,objCacheName):
        traceEntry ("findObjectCache")	
        oNameToUse = findConfigTargetWithScope(scope, scopeName, objCacheName, "ObjectCacheInstance" )
        traceExit ("findObjectCache") 
        return oNameToUse
#endDef

#--------------------------------------------------------------------
# Procedure:   manageObjectCacheResource
# Description: Creates/Deletes Object cache resource
#		If the Object cache resource already exists the Object cache resource will not be replaced
#
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#		objCacheName			(Name of Object cache)
#		jndiName		(JNDI name of Object cache)
#		cacheSize	(Cache size)
#		defaultPriority	(Default priority) 
#		disableDependencyId	(Disable dipendency ID support)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageObjectCacheResource(scope,scopeName,objCacheName,jndiName,cacheSize,defaultPriority,disableDependencyId, create):
        global AdminApp, AdminConfig
        
        traceEntry("manageObjectCacheResource" )

        cacheProv = findConfigTargetWithScope(scope, scopeName, "CacheProvider", "CacheProvider" )

        
        # Check if Object cache already exists
        objectCache = findObjectCache(scope, scopeName,objCacheName)

        if (objectCache != 0):
                if ( not create ):
                        trace("info", "Deleting object cache "+objCacheName+" - scope:" + scope)
			#remove object cache
			AdminConfig.remove(objectCache)
                else:
                        trace("info", "Object cache "+objCacheName+" - scope:" + scope +" already exists" )
                #endElse 

        else:
               	if cacheProv!=0:
	                if (create):
				#create object cache
				attributes = [["name", objCacheName],["jndiName", jndiName],["cacheSize", cacheSize],["defaultPriority", defaultPriority],["disableDependencyId", disableDependencyId]]
				objectCache = AdminConfig.create("ObjectCacheInstance", cacheProv, attributes) 
	                #endIf 
	                else:
	                        trace("info", "Object cache "+objCacheName+" - scope:" + scope+" doesn't exist" )
	                #endElse 
	       	#endIf 
	       	else:
	       		trace("info", "Object cache provider doesn't exist" )
	       	#endElse 
        #endElse 
        traceExit("manageObjectCacheResource" )
        return objectCache
#endDef


#--------------------------------------------------------------------
# Procedure:   manageResourceEnvironmentProvider
# Description: Creates/Deletes Resource Environment Provider
#
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#		providerName		(Name of provider Name)
#		description		(description)
#		factoryClassname	(factoryClassname)
#		classname		(classname)
#		create			(Boolean if true, create entry, else delete it)
# Returns:	Provider (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageResourceEnvironmentProvider ( scope, scopeName, providerName, description, factoryClassname, classname, create ):
	global AdminApp, AdminConfig
        traceEntry("manageResourceEnvironmentProvider" )
	scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
        reProvider = findConfigTargetWithScope(scope, scopeName, providerName, "ResourceEnvironmentProvider" )
	if (reProvider != 0):
		if ( not create ):
                        trace("info", "Deleting Resource Environment Provider "+ providerName +" - scope:" + scope)
			#remove resource provider
			AdminConfig.remove(reProvider)
                else:
                        trace("info", "Resource Environment Provider "+ providerName +" - scope:" + scope +" already exists" )
                #endElse 

        else:
                if (create):
			#create resource provider
			attributes = [["name", providerName],['description', description]]
			reProvider = AdminConfig.create("ResourceEnvironmentProvider", scopeEntry, attributes) 
     			if( reProvider != 0): 
     				attributes = [['factoryClassname', factoryClassname],['classname', classname]]
     				AdminConfig.create("Referenceable", reProvider, attributes)				
                #endIf 
                else:
                        trace("info", "Resource Environment Provider "+ providerName +" - scope:" + scope+" doesn't exist" )
                #endElse 
        #endElse 
        traceExit("manageResourceEnvironmentProvider" )	
	return reProvider	
#endDef


#--------------------------------------------------------------------
# Procedure:   manageResourceEnvironmentProviderProperty
# Description: Creates/Deletes Resource Environment Provider Property
#
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#		providerName		(Name of provider Name)
#		name			(name)
#		value			(value)
#		create			(Boolean if true, create entry, else delete it)
# Returns:	Provider (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageResourceEnvironmentProviderProperty ( scope, scopeName, providerName, propName, propValue, create ):
	global AdminApp, AdminConfig
        traceEntry("manageResourceEnvironmentProviderProperty" )
	scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
        reProvider = findConfigTargetWithScope(scope, scopeName, providerName, "ResourceEnvironmentProvider" )
	propSet = AdminConfig.showAttribute(reProvider, 'propertySet') 
	attributes = [['name', propName],['value', propValue]]
	if (create): 
		if(propSet == None or propSet == ""):
	               	trace("info", "Create Resource Environment Provider propertySet and property"+ propName +" - scope:" + scope)
	        	propSet = AdminConfig.create('J2EEResourcePropertySet',reProvider,[])
			AdminConfig.create('J2EEResourceProperty', propSet, attributes)
		else :
			
			for ob in AdminConfig.list('J2EEResourceProperty',propSet).split(java.lang.System.getProperty("line.separator")) :
				name = AdminConfig.showAttribute(ob, "name")
	        		if (name == propName):
	                		trace("info", "Modify Resource Environment Provider Property "+ propName +" - scope:" + scope)
					AdminConfig.modify(ob, attributes)
					return
		
                	trace("info", "Create Resource Environment Provider Property "+ propName +" - scope:" + scope)
			AdminConfig.create('J2EEResourceProperty', propSet, attributes)
			return
					
				
	else :
		for ob in AdminConfig.list('J2EEResourceProperty',propSet).split(java.lang.System.getProperty("line.separator")) :
			name = AdminConfig.showAttribute(ob, "name")
	        	if (name == propName):
	                	trace("info", "Delete Resource Environment Provider Property"+ propName +" - scope:" + scope)
				AdminConfig.remove(ob)
        #endElse 
        traceExit("manageResourceEnvironmentProviderProperty" )	
	return 0	
#endDef

#--------------------------------------------------------------------
# Procedure:   manageResourceEnvironmentEntry
# Description: Creates/Deletes Resource Environment Provider Entry
#
# Parameters:
#		scope 			(Cell or Node or Server)
#		scopeName		(cellName or nodeName)
#		entryName		(Name of Entry Name)
#		jndiName		(jndi name)
#		providerName		(factoryClassname)
#		referenceId		(reference id)
#		create			(Boolean if true, create entry, else delete it)
# Returns:	Environment Entry (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageResourceEnvironmentEntry ( scope, scopeName, entryName, jndiName, providerName, referenceId, create ):
	global AdminApp, AdminConfig
        traceEntry("manageResourceEnvironmentEntry" )
	scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
        reProvider = findConfigTargetWithScope(scope, scopeName, providerName, "ResourceEnvironmentProvider" )
    	for ob in AdminConfig.list('Referenceable',reProvider).split(java.lang.System.getProperty("line.separator")) :
        	name = AdminConfig.showAttribute(ob, "factoryClassname")
        	if (name == referenceId):
 	           	refid=ob
		else :
			return 0
	if (create): 
		envEntry= findConfigTargetWithScope(scope, scopeName, entryName, "ResourceEnvEntry" )
		if ( envEntry != 0 ) :
			trace("info", "Resource Environment entry "+ entryName +" already existed")		
		else :
			trace("info", "Create Resource Environment entry "+ entryName +" - scope:" + scope)
			attributes = [['name', entryName],['jndiName', jndiName], ['referenceable', refid]]
			envEntry=AdminConfig.create('ResourceEnvEntry', reProvider, attributes)
                	trace("info", "create EnvironmentEntry J2EEResourceProperty" )
               		propSet = AdminConfig.create("J2EEResourcePropertySet", envEntry, [] )
			
	else :
		trace("info", "Delete Resource Environment entry"+ entryName +" - scope:" + scope)
		envEntry=findConfigTargetWithScope(scope, scopeName, entryName, "ResourceEnvEntry" )
		AdminConfig.remove(envEntry)
        #endElse 
        traceExit("manageResourceEnvironmentEntry" )	
	return envEntry	
#endDef

#--------------------------------------------------------------------
# Procedure:   addCustomPropertyToEnvironmentEntryWithScope
# Description: 
# Parameters:
#		scope		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		entryName	(DataSource name)
#		custName	(Custom Property Name)
#		custValue	(Custom Property Value)
# return :
# 		None
#--------------------------------------------------------------------
def addCustomPropertyToEnvironmentEntryWithScope ( scope, scopeName, entryName, custName, custValue ):
        global AdminConfig
        traceEntry ("addCustomPropertyToEnvironmentEntryWithScope")
        envEntry= findConfigTargetWithScope(scope, scopeName, entryName, "ResourceEnvEntry" )
        propSet = AdminConfig.showAttribute(envEntry, "propertySet" )
        name = ["name", custName]
        value = ["value", custValue]
        attr = [name, value]
        AdminConfig.create("J2EEResourceProperty", propSet, attr )
        traceExit ("addCustomPropertyToEnvironmentEntryWithScope") 
#endDef 


#--------------------------------------------------------------------
# Procedure:   manageWorkManager
# Description: Creates/Deletes Work manager
#		If the Work manager already exists the Work manager will not be replaced
#
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#		workMName			(Name of work manager)
#		jndiName		(JNDI name of work manager)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageWorkManager(scope,scopeName,workMName,jndiName, create):
        global AdminApp, AdminConfig
        
        traceEntry("manageWorkManager" )

        wrkMgrProv = findConfigTargetWithScope(scope, scopeName, "WorkManagerProvider", "WorkManagerProvider" )

        # Check if work manager already exists
        workManager = findConfigTargetWithScope(scope, scopeName, workMName, "WorkManagerInfo" )

        if (workManager != 0):
                if ( not create ):
                        trace("info", "Deleting work manager "+workMName+"  - scope:" + scope)
			#remove work manager
			AdminConfig.remove(workManager)
                else:
                        trace("info", "Object cache "+workMName+"  - scope:" + scope +" already exists" )
                #endElse 

        else:
               	if wrkMgrProv!=0:
	                if (create):
				#create work manager
				attributes = [["name", workMName],["jndiName", jndiName]]
				workManager = AdminConfig.create("WorkManagerInfo", wrkMgrProv, attributes) 
	                #endIf 
	                else:
	                        trace("info", "Work manager "+workMName+"  - scope:" + scope+" doesn't exist" )
	                #endElse 
	       	#endIf 
	       	else:
	       		trace("info", "Work manager provider doesn't exist" )
	       	#endElse 
        #endElse 
        traceExit("manageWorkManager" )
        return workManager
#endDef

#--------------------------------------------------------------------
# Procedure:   manageTimerManager
# Description: Creates/Deletes Timer manager
#		If the Timer manager already exists the Timer manager will not be replaced
#
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#		workMName			(Name of work manager)
#		jndiName		(JNDI name of work manager)
#		numAlarmThreads number of alarm threads
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def manageTimerManager(scope,scopeName,timerMName,jndiName, numAlarmThreads, create):
        global AdminApp, AdminConfig
        
        traceEntry("manageTimerManager" )

        timerMgrProv = findConfigTargetWithScope(scope, scopeName, "TimerManagerProvider", "TimerManagerProvider" )

        # Check if timer manager already exists
        timerManager = findConfigTargetWithScope(scope, scopeName, timerMName, "TimerManagerInfo" )

        if (timerManager != 0):
                if ( not create ):
                        trace("info", "Deleting timer manager "+timerMName+"  - scope:" + scope)
			#remove work manager
			AdminConfig.remove(timerMName)
                else:
                        trace("info", "Object cache "+timerMName+"  - scope:" + scope +" already exists" )
                #endElse 

        else:
               	if timerMgrProv!=0:
	                if (create):
				#create timer manager
				attributes = [["name", timerMName],["jndiName", jndiName], ["numAlarmThreads", numAlarmThreads]]
				timerManager = AdminConfig.create("TimerManagerInfo", timerMgrProv, attributes) 
	                #endIf 
	                else:
	                        trace("info", "Timer manager "+timerMName+"  - scope:" + scope+" doesn't exist" )
	                #endElse 
	       	#endIf 
	       	else:
	       		trace("info", "timer manager provider doesn't exist" )
	       	#endElse 
        #endElse 
        traceExit("manageTimerManager" )
        return timerManager
#endDef


#--------------------------------------------------------------------
# Procedure:   manageServerProperty
# Description: Creates/Deletes server property
#
# Parameters:
#		serverName		(cellName or nodeName)
#		serverpropName		(name of property)
#		value			(value of property)
#		description		(descrption of property)
#		required		(true or false) 
#		create			(Boolean if true, create entry, else delete it)
# Returns:	serverpropName 		(success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageServerProperty(serverName,serverpropName,value,description,required,create):
        global AdminTask, AdminConfig      
        traceEntry("manageServerProperty" )
        for server in AdminTask.listServers('[-serverType APPLICATION_SERVER ]').split(java.lang.System.getProperty("line.separator")) :
        	trace("info", "server = " + server )
        	sname = AdminConfig.showAttribute(server, "name")
	        if (serverName == sname):
	        	appServer=AdminConfig.list("ApplicationServer", server ) 
			nameAttr = ["name", serverpropName]
	 		valueAttr = ["value", value]
	       		descAttr = ["description",description]
			requiredAttr = ["required", required]              
			attrs = [nameAttr, valueAttr, descAttr, requiredAttr]
	        	found = 0
	        	for serverProperty in AdminConfig.list('Property', appServer).split(java.lang.System.getProperty("line.separator")) :
	                	if(serverProperty == None or serverProperty == ""):
	                		trace("info", "No server property")
	                		break
	                	else :
		                	name = AdminConfig.showAttribute(serverProperty, "name")
		                	if ( serverpropName == name ):
		                		trace("info", "property name" + name + " existed")
		                		found = 1
		                		if( create ) :
		                			trace("info", "Modify property =" + name )
		                			AdminConfig.modify(serverProperty, attrs)
		                		else : 
		                			trace("info", "Delete property =" + name )
		                			AdminConfig.remove(serverProperty)
	                #endFor
	                if (found == 0):
	                	if( create ) :
	                		trace("info", "Create property =" + serverpropName )
	                		AdminConfig.create('Property', appServer, attrs )
	                		
	                	else:		
	 				trace("info", "property =" + name + " not found, No need to delete.")
	        #endIf
	#endFor
        traceExit("manageServerProperty" )
        return serverpropName
#endDef

#--------------------------------------------------------------------
# Procedure:   findMQTopic
# Description:
# Parameters:
#		scopeEntry 	(Scope entry)
#               mqTopicName	(MQ Topic name)#		
# Returns:      targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def findMQTopic (scopeEntry, mqTopicName):
        global AdminConfig
	jmsqEntries = AdminTask.listWMQTopics(scopeEntry).split(java.lang.System.getProperty("line.separator")) 
        for jmsqEntry in jmsqEntries :
        	if ( len(jmsqEntry)>0 ):
        		nameEntry = AdminConfig.showAttribute(jmsqEntry,"name")
        		if (mqTopicName==nameEntry):
	                	return jmsqEntry
	                #endIf 	
		#endIf     
        #endFor 
        return 0
#endDef


#--------------------------------------------------------------------
# Procedure:   manageMQTopic
# Description: Creates/Deletes MQ Topic
#		If the MQ Topic already exists the MQ Topic will not be replaced
#
# Parameters:
#		scope 		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		name		(Name of JMS Topic)
#		jndiName	(JNDI name of JMS Topic)
#		topicName	(Name of destination topic) 
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageMQTopic(scope,scopeName,name,jndiName,topicName,create):
        global AdminApp, AdminConfig        
        traceEntry("manageMQTopic" )        
        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
                
        
        
        # Check if jms topic already exists
        mqtopic = findMQTopic(scopeEntry,name)

        if (mqtopic != 0):
                if ( not create ):
                        trace("info", "Deleting MQ topic "+ name +"  - scope:" + scopeName)
                        
			#remove  JMS topic
			AdminTask.deleteWMQTopic(mqtopic)
                else:
                        trace("info", "MQ topic "+ name +"  - scope:" + scopeName+" already exists" )
                #endElse 

        #else:
               
	if (create):
		#create MQ topic
		attrs = "[-name %s -jndiName %s -topicName %s ]" % (name,jndiName,topicName)
	        AdminTask.createWMQTopic(scopeEntry,attrs)
	#endIf 
	else:
		trace("info", "MQ topic "+name+"  - scope:" + scopeName+" doesn't exist" )
	#endElse 
	 
        traceExit("manageMQTopic" )
        return 0
#endDef

