<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:tns="http://schema.solution.ioc.ibm.com/CommonContent" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

	<!--  <xsl:output method="text" indent="yes" />-->

	<xsl:template match="*">
		<xsl:variable name="nodeName" select="name(.)" />
		<xsl:choose>
		
			<xsl:when test="$nodeName='replace'">
echo "###############################################################################"
echo "# Replace resource file <xsl:value-of select="@file" />                        "
echo "###############################################################################"

sed -i -c -f $IOC_BASE_DIR/$SED_SCRIPT_NAME $IOC_BASE_DIR/<xsl:value-of select="@file" />
if [ $? -ne 0 ]
then 
  echo "Failed to replace resource file <xsl:value-of select="@file" />"
  exit 1
fi
			</xsl:when>

			<xsl:when test="$nodeName='copy'">
#####################################################################################################
# Copy file <xsl:value-of select="@sour" /> to <xsl:value-of select="@dest" /> 
#####################################################################################################

echo "Copy file <xsl:value-of select="@sour" /> to <xsl:value-of select="@dest" />" 
mkdir -p <xsl:value-of select="@dest" />
\cp -rf $IOC_BASE_DIR/<xsl:value-of select="@sour" /> <xsl:text> </xsl:text> <xsl:value-of select="@dest" />
if [ $? -ne 0 ]
then 
  echo "Failed to copy file <xsl:value-of select="@sour" /> to <xsl:value-of select="@dest" />"
  exit 1
fi
			</xsl:when>

			<xsl:when test="$nodeName='append'">

#####################################################################################################
# Append file <xsl:value-of select="@sour" /> to <xsl:value-of select="@dest" /> 
#####################################################################################################

echo "Appending file <xsl:value-of select="@sour" /> to <xsl:value-of select="@dest" />" 

cat $IOC_BASE_DIR/<xsl:value-of select="@sour" /> <xsl:text> >> </xsl:text> <xsl:value-of select="@dest" />
if [ $? -ne 0 ]
then 
  echo "Failed to append file <xsl:value-of select="@sour" /> to <xsl:value-of select="@dest" />"
  exit 1
fi
			</xsl:when>
			
			<xsl:when test="$nodeName='updateprop'">
#####################################################################################################
# Update props file <xsl:value-of select="@dest" /> with props from file <xsl:value-of select="@sour" />
#####################################################################################################

echo "Updating props file <xsl:value-of select="@dest" /> with props from file <xsl:value-of select="@sour" />"


props_file_sour=<xsl:value-of select="@sour" />
props_file_dest=<xsl:value-of select="@dest" />

SAVE_IFS=$IFS
IFS='='
cat "$props_file_sour" | while read -ra line
do
	key=${line[0]}
	value=${line[1]}
	
	value=$(echo $value | sed -e 's/\\/\\\\/g')
	sed -i -e "s/$key=.*/$key=$value/g" "$props_file_dest"
done
IFS=$SAVE_IFS

if [ $? -ne 0 ]
then 
  echo "Failed to update props file <xsl:value-of select="@dest" /> with props from file <xsl:value-of select="@sour" />"
  exit 1
fi
			</xsl:when>

			<xsl:when test="$nodeName='dos2unix'">
#####################################################################################################
# dos2unix file <xsl:value-of select="@sour" />  
#####################################################################################################

echo "dos2unix file <xsl:value-of select="@sour" /> " 

dos2unix <xsl:value-of select="@sour" /> 
if [ $? -ne 0 ]
then 
  echo "Failed to run dos2unix for file <xsl:value-of select="@sour" /> "
  exit 1
fi
			</xsl:when>

			<xsl:when test="$nodeName='unzip'">
#####################################################################################################
# unzip file <xsl:value-of select="@zip-file" />  to  <xsl:value-of select="@dest" />
#####################################################################################################

echo "Unzipping file <xsl:value-of select="@zip-file" />  to  <xsl:value-of select="@dest" />"

unzip <xsl:value-of select="@zip-file" />  -d <xsl:value-of select="@dest" />

if [ $? -ne 0 ]
then 
  echo "Failed to unzip file <xsl:value-of select="@zip-file" /> "
  exit 1
fi
			</xsl:when>					

			<xsl:when test="$nodeName='SYS'">
log.info "#################################################################################################"
log.info "# Execute linux operating system command <xsl:value-of select="@command" />"
log.info "#################################################################################################"

<xsl:value-of select="@command" />
if [ $? -ne 0 ]
then 
  log.error "Failed to execute operating system command:<xsl:value-of select="@command" />"
  exit 1
fi
			</xsl:when>
			
			<xsl:when test="$nodeName='SYS_RUNAS'">
log.info "#################################################################################################"
echo_command=`echo "<xsl:value-of select="@command" />" | cut -d\  -f1`
log.info "# Execute linux operating system command $echo_command"
log.info "#################################################################################################"

su <xsl:value-of select="@runas" /> -c "<xsl:value-of select="@command" />"
if [ $? -ne 0 ]
then 
  log.error "Fail to execute operating system command: $echo_command"
  exit 1
fi
			</xsl:when>
			
			<xsl:when test="$nodeName='SYS_RUNAS_RETRY_ONCE'">
log.info "#################################################################################################"
echo_command=`echo "<xsl:value-of select="@command" />" | cut -d\  -f1`
log.info "# Execute linux operating system command $echo_command"
log.info "#################################################################################################"

su <xsl:value-of select="@runas" /> -c "<xsl:value-of select="@command" />"
if [ $? -ne 0 ]
then
  sleep <xsl:value-of select="@sleep" /> 
  su <xsl:value-of select="@runas" /> -c "<xsl:value-of select="@command" />"
fi
if [ $? -ne 0 ]
then 
  log.error "Fail to execute operating system command: $echo_command"
  exit 1
fi
			</xsl:when>

			<xsl:when test="$nodeName='restartCluster'">
log.info "#################################################################################################"
log.info "# Stop cluster: <xsl:value-of select="@clusterName" /> on dmgr: <xsl:value-of select="@dmgrProfilePath" />"
log.info "#################################################################################################"
su $WAS_RUN_AS -c "<xsl:value-of select="@dmgrProfilePath" />/bin/wsadmin.sh -conntype SOAP  -user <xsl:value-of select="@adminAccount" /> -password <xsl:value-of select="@adminPassword" /> -lang jython -f lib/stopCluster.py <xsl:value-of select="@cellName" /><xsl:text> </xsl:text><xsl:value-of select="@clusterName" /><xsl:text> </xsl:text><xsl:value-of select="@waitTime" />"
if [ $? -ne 0 ]
then
  log.error "Failed to stop cluster <xsl:value-of select="@clusterName" />"
  exit 1
fi

log.info "#################################################################################################"
log.info "# Start cluster: <xsl:value-of select="@clusterName" /> on dmgr: <xsl:value-of select="@dmgrProfilePath" />"
log.info "#################################################################################################"
su $WAS_RUN_AS -c "<xsl:value-of select="@dmgrProfilePath" />/bin/wsadmin.sh -conntype SOAP  -user <xsl:value-of select="@adminAccount" /> -password <xsl:value-of select="@adminPassword" /> -lang jython -f lib/startCluster.py <xsl:value-of select="@cellName" /><xsl:text> </xsl:text><xsl:value-of select="@clusterName" /><xsl:text> </xsl:text><xsl:value-of select="@waitTime" />"
if [ $? -ne 0 ]
then
  log.error "Failed to start cluster <xsl:value-of select="@clusterName" />"
  exit 1
fi
			</xsl:when>

			<xsl:when test="$nodeName='restartDmgr'">
log.info "#################################################################################################"
log.info "# Stop dmgr at: <xsl:value-of select="@dmgrProfilePath" /> and sleep for <xsl:value-of select="@waitTimeStop" /> seconds"
log.info "#################################################################################################"
su $WAS_RUN_AS -c "<xsl:value-of select="@dmgrProfilePath" />/bin/stopManager.sh -user <xsl:value-of select="@adminAccount" /> -password <xsl:value-of select="@adminPassword" />"
if [ $? -ne 0 ]
then
  log.error "Failed to stop dmgr at: <xsl:value-of select="@dmgrProfilePath" />"
  exit 1
fi
sleep <xsl:value-of select="@waitTimeStop" />

log.info "#################################################################################################"
log.info "# Start dmgr at: <xsl:value-of select="@dmgrProfilePath" /> and sleep for <xsl:value-of select="@waitTimeStart" /> seconds"
log.info "#################################################################################################"
su $WAS_RUN_AS -c "<xsl:value-of select="@dmgrProfilePath" />/bin/startManager.sh"
if [ $? -ne 0 ]
then
  log.error "Failed to start dmgr at: <xsl:value-of select="@dmgrProfilePath" />"
  exit 1
fi
sleep <xsl:value-of select="@waitTimeStart" />
			</xsl:when>

			<xsl:when test="$nodeName='restartNodeAgent'">
log.info "#################################################################################################"
log.info "# Stop nodeagent at: <xsl:value-of select="@portalProfilePath" /> and sleep for <xsl:value-of select="@waitTimeStop" /> seconds"
log.info "#################################################################################################"
su $WAS_RUN_AS -c "<xsl:value-of select="@portalProfilePath" />/bin/stopNode.sh -user <xsl:value-of select="@adminAccount" /> -password <xsl:value-of select="@adminPassword" />"
if [ $? -ne 0 ]
then
  log.error "Failed to stop nodeagent at: <xsl:value-of select="@portalProfilePath" />"
  exit 1
fi
sleep <xsl:value-of select="@waitTimeStop" />

log.info "#################################################################################################"
log.info "# Start nodeagent at: <xsl:value-of select="@portalProfilePath" /> and sleep for <xsl:value-of select="@waitTimeStart" /> seconds"
log.info "#################################################################################################"
su $WAS_RUN_AS -c "<xsl:value-of select="@portalProfilePath" />/bin/startNode.sh"
if [ $? -ne 0 ]
then
  log.error "Failed to start nodeagent at: <xsl:value-of select="@portalProfilePath" />"
  exit 1
fi
sleep <xsl:value-of select="@waitTimeStart" />
			</xsl:when>
			
			<xsl:when test="$nodeName='restartPortalServer'">
log.info "#################################################################################################"
log.info "# Stop portal server <xsl:value-of select="@portalServerName" /> at: <xsl:value-of select="@portalProfilePath" /> and sleep for <xsl:value-of select="@waitTimeStop" /> seconds"
log.info "#################################################################################################"
su $WAS_RUN_AS -c "<xsl:value-of select="@portalProfilePath" />/bin/stopServer.sh  <xsl:value-of select="@portalServerName" /> -username <xsl:value-of select="@adminAccount" /> -password <xsl:value-of select="@adminPassword" />"
if [ $? -ne 0 ]
then
  log.error "Failed to stop portal server <xsl:value-of select="@portalServerName" /> at: <xsl:value-of select="@portalProfilePath" />"
  exit 1
fi
sleep <xsl:value-of select="@waitTimeStop" />

log.info "#################################################################################################"
log.info "# Start portal server <xsl:value-of select="@portalServerName" /> at: <xsl:value-of select="@portalProfilePath" /> and sleep for <xsl:value-of select="@waitTimeStart" /> seconds"
log.info "#################################################################################################"
su $WAS_RUN_AS -c "<xsl:value-of select="@portalProfilePath" />/bin/startServer.sh <xsl:value-of select="@portalServerName" />"
if [ $? -ne 0 ]
then
  log.error "Failed to start portal server <xsl:value-of select="@portalServerName" /> at: <xsl:value-of select="@portalProfilePath" />"
  exit 1
fi
sleep <xsl:value-of select="@waitTimeStart" />
			</xsl:when>
			
			<xsl:when test="$nodeName='stopPortalServer'">
log.info "#################################################################################################"
log.info "# Stop portal server <xsl:value-of select="@portalServerName" /> at: <xsl:value-of select="@portalProfilePath" /> and sleep for <xsl:value-of select="@waitTimeStop" /> seconds"
log.info "#################################################################################################"
su $WAS_RUN_AS -c "<xsl:value-of select="@portalProfilePath" />/bin/stopServer.sh  <xsl:value-of select="@portalServerName" /> -username <xsl:value-of select="@adminAccount" /> -password <xsl:value-of select="@adminPassword" />"
if [ $? -ne 0 ]
then
  log.error "Failed to stop portal server <xsl:value-of select="@portalServerName" /> at: <xsl:value-of select="@portalProfilePath" />"
  exit 1
fi
sleep <xsl:value-of select="@waitTimeStop" />
			</xsl:when>
			
			<xsl:when test="$nodeName='startPortalServer'">
log.info "#################################################################################################"
log.info "# Start portal server <xsl:value-of select="@portalServerName" /> at: <xsl:value-of select="@portalProfilePath" /> and sleep for <xsl:value-of select="@waitTimeStart" /> seconds"
log.info "#################################################################################################"
su $WAS_RUN_AS -c "<xsl:value-of select="@portalProfilePath" />/bin/startServer.sh <xsl:value-of select="@portalServerName" />"
if [ $? -ne 0 ]
then
  log.error "Failed to start portal server <xsl:value-of select="@portalServerName" /> at: <xsl:value-of select="@portalProfilePath" />"
  exit 1
fi
sleep <xsl:value-of select="@waitTimeStart" />
			</xsl:when>
			
			<xsl:when test="$nodeName='stopCluster'">
log.info "#################################################################################################"
log.info "# Stop cluster: <xsl:value-of select="@clusterName" /> on dmgr: <xsl:value-of select="@dmgrProfilePath" />"
log.info "#################################################################################################"
su $WAS_RUN_AS -c "<xsl:value-of select="@dmgrProfilePath" />/bin/wsadmin.sh -conntype SOAP  -user <xsl:value-of select="@adminAccount" /> -password <xsl:value-of select="@adminPassword" /> -lang jython -f lib/stopCluster.py <xsl:value-of select="@cellName" /><xsl:text> </xsl:text><xsl:value-of select="@clusterName" /><xsl:text> </xsl:text><xsl:value-of select="@waitTime" />"
if [ $? -ne 0 ]
then
  log.error "Failed to stop cluster <xsl:value-of select="@clusterName" />"
  exit 1
fi
			</xsl:when>
			
			<xsl:when test="$nodeName='startCluster'">
log.info "#################################################################################################"
log.info "# Start cluster: <xsl:value-of select="@clusterName" /> on dmgr: <xsl:value-of select="@dmgrProfilePath" />"
log.info "#################################################################################################"
su $WAS_RUN_AS -c "<xsl:value-of select="@dmgrProfilePath" />/bin/wsadmin.sh -conntype SOAP  -user <xsl:value-of select="@adminAccount" /> -password <xsl:value-of select="@adminPassword" /> -lang jython -f lib/startCluster.py <xsl:value-of select="@cellName" /><xsl:text> </xsl:text><xsl:value-of select="@clusterName" /><xsl:text> </xsl:text><xsl:value-of select="@waitTime" />"
if [ $? -ne 0 ]
then
  log.error "Failed to start cluster <xsl:value-of select="@clusterName" />"
  exit 1
fi
			</xsl:when>

		</xsl:choose>
	</xsl:template>
</xsl:stylesheet>
