<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:tns="http://schema.solution.ioc.ibm.com/MBContent" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:include href="../../common_content/CommonContent.xsl"/>
	<xsl:output method="text" indent="yes" />

	<xsl:template match="/">#!/bin/sh
###############################################################################
# Licensed Materials - Property of IBM
# Restricted Materials of IBM
# PID
# (C) Copyright IBM Corp. 2010, 2013
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp
###############################################################################
dos2unix $IOC_BASE_DIR/variables.properties
source ~/.baprofile
source $IOC_BASE_DIR/variables.properties

cd $IOC_BASE_DIR

export SED_SCRIPT_NAME=com.ibm.ioc.sed.script.txt
sed -e '/^#/d' $IOC_BASE_DIR/variables.properties | awk -F = '{print "s#~{" $1 "}#" substr($0,index($0,"=")+1) "#g"}' > $IOC_BASE_DIR/$SED_SCRIPT_NAME

				<xsl:for-each select="tns:mbContent/*">
					<xsl:apply-templates select="."/><!-- Here to transform with common template first -->
					<xsl:variable name="nodeName" select="name(.)" />
					<xsl:choose>
						
						<xsl:when test="$nodeName='MQSCfile'">
###############################################################################
# Replace MQS file <xsl:value-of select="@file" />
###############################################################################
echo "Replace MQS file <xsl:value-of select="@file" />"

dos2unix <xsl:value-of select="@file" />
sed -i -c -f $IOC_BASE_DIR/$SED_SCRIPT_NAME $IOC_BASE_DIR/<xsl:value-of select="@file" />
if [ $? -ne 0 ]
then 
	echo "Fail to replace MQS file <xsl:value-of select="@file" />"
	exit 1
fi
	
########################################################################
# Check if queue manager is started, if not, start it; 0 is started 1 is stopped
########################################################################
rc=1

ps -ef > /tmp/tmp_processes
cms_pid=`cat /tmp/tmp_processes | grep amqzmgr0 | grep <xsl:value-of select="@QMgrName" /> | awk '{print $2}' | head -n 1`
if [ ! -z $cms_pid ]
then
	rc=0
fi

if [ $rc = 1 ]; then
	echo "------ Starting MQ Queue Manager <xsl:value-of select="@QMgrName" /> ------"
	/opt/mqm/bin/strmqm <xsl:value-of select="@QMgrName" />
	if [ $? -ne 0 ]
	then 
		echo "Fail to start queue manager:<xsl:value-of select="@QMgrName" />,quit the installation"
		exit 1
	fi
	rm -f /tmp/tmp_processes
	rc=0
fi
					
echo "###############################################################################"
echo "# Execute MQS file <xsl:value-of select="@file" />"
echo "###############################################################################"

echo "------ Starting creation of Generic Event MQ objects and a server connection channel ------"
/opt/mqm/bin/runmqsc <xsl:value-of select="@QMgrName" />  &lt; $IOC_BASE_DIR/<xsl:value-of select="@file" />
if [ $? -ne 0 -o $? -ne 10 ]
then 
	echo "Fail to create local queue objects and a server connection channel in <xsl:value-of select="@file" />"
	exit 1
fi

echo "------ Finished creation of Generic Event MQ objects ------"
echo "------ Stopping MQ Queue Manager <xsl:value-of select="@QMgrName" /> ------"
/opt/mqm/bin/endmqm -i <xsl:value-of select="@QMgrName" />

echo "------ Waiting for 180 sec before starting MQ manager again ------"

sleep 180

echo "------ Starting MQ Queue Manager <xsl:value-of select="@QMgrName" /> ------"
/opt/mqm/bin/strmqm <xsl:value-of select="@QMgrName" />
						</xsl:when>
						
						<xsl:when test="$nodeName='MQSCcommand'">
###############################################################################
# Execute MQSC command '<xsl:value-of select="@command" />' with acceptable output '<xsl:value-of select="@acceptableOutput" />' on queue manager '<xsl:value-of select="@QMgrName" />'.
###############################################################################

QMGR="<xsl:value-of select="@QMgrName" />"
ACCEPTABLE_OUTPUT="<xsl:value-of select="@acceptableOutput" />"
CMD="<xsl:value-of select="@command" />"

echo "------ Execute MQSC command '<xsl:value-of select="@command" />' with acceptable output '<xsl:value-of select="@acceptableOutput" />' on queue manager '<xsl:value-of select="@QMgrName" />'. ------"

isOutputAcceptable=0

output=$(echo $CMD | runmqsc $QMGR)
rc=$?
while read -r line; do
	case ${line} in
		${ACCEPTABLE_OUTPUT})
			isOutputAcceptable=1;;
		AMQ8118*)
			echo "Queue manager $QMGR does not exist.";;
		AMQ8146*)
			echo "Queue manager $QMGR is not available.";;
		AMQ*)
			echo "Unexpected message: ${line}.";;
	esac
done &lt;&lt;&lt; "${output}"

if [ $rc -ne 0 -a $isOutputAcceptable -ne 1 ]
then
	echo "Error executing MQSC command '<xsl:value-of select="@command" />' with acceptable output '<xsl:value-of select="@acceptableOutput" />' on queue manager '<xsl:value-of select="@QMgrName" />' (rc = $rc)."
	exit 1
fi

echo "------ rc = $rc | isOutputAcceptable = $isOutputAcceptable ------"
						</xsl:when>
						
						<xsl:when test="$nodeName='startQMgr'">
###############################################################################
# Start WebSphere queue manager: <xsl:value-of select="@QMgrName" />
###############################################################################

echo "------ Starting MQ Queue Manager <xsl:value-of select="@QMgrName" /> ------"
/opt/mqm/bin/strmqm <xsl:value-of select="@QMgrName" />
rc=$?
if [ $rc -ne 0 ]
then
	if [ $rc -eq 16 ]
	then
		echo "Error starting WebSphere MQ queue manager <xsl:value-of select="@QMgrName" />: it does not exist (rc = $rc)."
		exit 1
	elif [ $rc -eq 5 ]
	then
		echo "------ WebSphere MQ queue manager <xsl:value-of select="@QMgrName" /> is already running, so continuing (rc = $rc)... ------"
	else
		echo "Error starting WebSphere MQ queue manager <xsl:value-of select="@QMgrName" />  (rc = $rc)."
		exit 1
	fi
fi
						</xsl:when>
						
						<xsl:when test="$nodeName='stopQMgr'">
###############################################################################
# Stop WebSphere queue manager: <xsl:value-of select="@QMgrName" />
###############################################################################

isWaitRequired=1

echo "------ Stopping MQ Queue Manager <xsl:value-of select="@QMgrName" /> ------"
/opt/mqm/bin/endmqm -i <xsl:value-of select="@QMgrName" />
rc=$?
if [ $rc -ne 0 ]
then
	if [ $rc -eq 40 ]
	then
		echo "------ WebSphere MQ queue manager <xsl:value-of select="@QMgrName" /> is not available (already stopped?), so continuing (rc = $rc)... ------"
		isWaitRequired=0
	elif [ $rc -eq 16 ]
	then
		echo "Error stopping WebSphere MQ queue manager <xsl:value-of select="@QMgrName" />: it does not exist (rc = $rc)."
		exit 1
	else
		echo "Error stopping WebSphere MQ queue manager <xsl:value-of select="@QMgrName" /> (rc = $rc)."
		exit 1
	fi
fi

if [ $isWaitRequired -eq 1 ]
then
	echo "------ Waiting for <xsl:value-of select="@WaitTime" /> second(s) ------"
	sleep <xsl:value-of select="@WaitTime" />
fi
						</xsl:when>
						
						<xsl:when test="$nodeName='BROKER'">
###############################################################################
# Create Message Broker 
###############################################################################
echo "------ Setting MB profile ------"
cd 
. $MB_HOME/bin/mqsiprofile

echo "------ Creating Broker: <xsl:value-of select="@BkrName" /> ------"

$MB_HOME/bin/mqsicreatebroker  <xsl:value-of select="@BkrName" /> -q <xsl:value-of select="@QMgrName" />
if [ $? -ne 0 ]
then 
	echo "Fail to create broker <xsl:value-of select="@BkrName" />"
	exit 1
fi 

echo "------ Finished creation of broker: <xsl:value-of select="@BkrName" /> ------"
echo "------ Starting Broker ------"

$MB_HOME/bin/mqsistart <xsl:value-of select="@BkrName" /> 
if [ $? -ne 0 ]
then 
	echo "Fail to start broker <xsl:value-of select="@BkrName" />"
	exit 1
fi

echo "------ Finished starting of broker: <xsl:value-of select="@BkrName" /> ------"
						</xsl:when>
						
						<xsl:when test="$nodeName='EXEGROUP'">
echo "------ Setting MB profile ------"
cd 
. $MB_HOME/bin/mqsiprofile

########################################################################
# Check if broker is started, if not, start it; 0 is started 1 is stopped
########################################################################
rc=1

ps -ef > /tmp/tmp_processes
cms_pid=`cat /tmp/tmp_processes | grep bipbroker | grep <xsl:value-of select="@BkrName" /> | awk '{print $2}' | head -n 1`
if [ ! -z $cms_pid ]
then
	echo "------ Broker is started --------"
	rc=0
fi

if [ $rc = 1 ]; then
	echo "------ Starting Broker <xsl:value-of select="@BkrName" /> ------"
	$MB_HOME/bin/mqsistart <xsl:value-of select="@BkrName" />
	rm -f /tmp/tmp_processes
	rc=0
fi

echo "------ Creating Execution Group ------" 

$MB_HOME/bin/mqsicreateexecutiongroup <xsl:value-of select="@BkrName" /> -e <xsl:value-of select="@ExeGroupName" />
if [ $? -ne 0 ]
then 
	echo "Fail to create execution group <xsl:value-of select="@ExeGroupName" />"
	exit 1
fi

echo "------ Finished creation of execution group: <xsl:value-of select="@ExeGroupName" /> ------"
						</xsl:when>
						
						<xsl:when test="$nodeName='MQSIDEPLOY'">
echo "------ Setting MB profile ------"
cd 
. $MB_HOME/bin/mqsiprofile

########################################################################
# Check if broker is started, if not, start it; 0 is started 1 is stopped
########################################################################
rc=1

ps -ef > /tmp/tmp_processes
cms_pid=`cat /tmp/tmp_processes | grep bipbroker | grep <xsl:value-of select="@BkrName" /> | awk '{print $2}' | head -n 1`
if [ ! -z $cms_pid ]
then
	rc=0
fi

if [ $rc = 1 ]; then
	echo "------ Starting Broker <xsl:value-of select="@BkrName" /> ------"
	$MB_HOME/bin/mqsistart <xsl:value-of select="@BkrName" />
	rm -f /tmp/tmp_processes
	rc=0
fi

$MB_HOME/bin/mqsideploy <xsl:value-of select="@BkrName" /> -e <xsl:value-of select="@ExeGroupName" /> -a $IOC_BASE_DIR/<xsl:value-of select="@BarFile" />
if [ $? -ne 0 ]
then 
	echo "Fail to execute mqsideploy command to deploy <xsl:value-of select="@barfile" /> on execution group: <xsl:value-of select="@ExeGroupName" /> on broker: <xsl:value-of select="@BkrName" />"
	exit 1
fi

echo "------ deploying bar file completed ------"
						</xsl:when>

					</xsl:choose>
					
				</xsl:for-each>

	</xsl:template>
	
</xsl:stylesheet>