# IBM Confidential
#
# OCO Source Material
#
# IBM Integrated Information Core PID 5724-Y59
#
# (C) Copyright IBM Corp. 2011
#
# The source code for this program is not published or otherwise divested
# of its trade secrets, irrespective of what has been deposited with the
# U. S. Copyright Office.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

#******************************************************************************
# File Name:   common.py
# Description: This file contains a number of useful wsadmin procs that help
#              isolate the tedious coding for common operations.
# Fix record:
#******************************************************************************

import sys
import re
import sre

lineSeparator = java.lang.System.getProperty('line.separator')
errorCode = ""
errorInfo = ""
traceLevel = "error"

#--------------------------------------------------------------------
# Procedure:   regexp
# Description: 
#
# Parameters:  
# Returns:     
#--------------------------------------------------------------------
def regexp(pattern, string, flags=0):
    if(re.compile(pattern, flags).search(string)==None): 
        return 0
    else: 
        return 1
#endDef 

#--------------------------------------------------------------------
# Procedure:   regexpn
# Description: 
#
# Parameters:  
# Returns:     
#--------------------------------------------------------------------
def regexpn(pattern, string, flags=0):
    r = re.compile(pattern, flags).subn("", string)
    return r[1]
#endDef

#--------------------------------------------------------------------
# Procedure:   loadProperties
# Description: Loads a properties file from disk and returns a Java
#              Properties object.
#
# Parameters:  propFileName	Name of the properties file
# Returns:     Properties object representing contents of the file
#--------------------------------------------------------------------
def loadProperties (propFileName):
    global AdminControl
    
    bis = java.io.BufferedInputStream(java.io.FileInputStream(propFileName))
    props = java.util.Properties()
    props.load(bis) 
    bis.close()
	
	# Trim all property values to empty
    for key in props.keySet().iterator():
        value = trimToEmpty(props.get(key))
        props.setProperty(key, value)
    #endFor
	
    # Resolving shared properties
    for key in props.keySet().iterator():
        if (key.startswith("sharedproperty.")):
            sharedPropertyName = key[key.index(".") + 1:]
            sharedPropertyValue = props.get(key)
            propertyReference = "~{" + sharedPropertyName + "}"
            for key2 in props.keySet().iterator():
                value2 = props.get(key2)
                if (value2.find(propertyReference) > -1):
                    props.setProperty(key2, value2.replace(propertyReference, sharedPropertyValue))
                #endIf
            #endFor
        #endIf
    #endFor

    return props
#endDef 

#--------------------------------------------------------------------
# Procedure:   writemsg
# Description: Writes a message prepended by a timestamp
# Parameters:
#			msg	 	The message to write
#
# Returns:	None
# History:
# 2003/12/23	wdj	Created
#--------------------------------------------------------------------
def writemsg ( msg ):
    from java.util import Date
    
    date = Date()
    print "[" + date.toString() + "]" + msg
#endDef 

#--------------------------------------------------------------------
# Procedure:   trace
# Description: trace is used to emit trace messages depending on trace level.
# Parameters:
#		level	(trace level specified)
#		msg	(Message to output)
# Returns:	
# Example: trace info "Now do the xyz step"
# For now, trace does not emit a time stamp.  Probably need to add that.
#--------------------------------------------------------------------

def trace ( level, msg ):
        global traceLevel
	from java.util import Date
        javadate = Date()
        date = javadate.toString()
        if ( traceLevel == "none" ):
                return 
        elif ( traceLevel == "error" ):
                if (level == "error"):
                        print "<"+date+">: "+ msg
        elif ( traceLevel == "warn" ):
                if (regexp(["error"|"warn"], level) ):
                        print "<"+date+">: "+ msg
        elif ( traceLevel == "info" ):
                if (regexp(["error"|"warn" |"info"], level) ):
                        print "<"+date+">: "+ msg 
        elif ( traceLevel == "debug" ):
                print "<"+date+">: "+ msg
        elif ( traceLevel == "default" ):
             	print "Unknown traceLevel: "+ traceLevel
                print "<"+date+">: "+ msg 
        #endIf
#endDef 

def traceEntry ( procName ):
        trace("debug", ">>>> ENTRY "+procName+" " )
#endDef 

def traceExit ( procName ):
        trace("debug", "<<<< EXIT "+procName )
#endDef 

#--------------------------------------------------------------------
# Procedure:   setTraceLevel
# Description: setTraceLevel is used to change the trace level
# Parameters:
#		level	(trace level specified)
#		
# Returns:	
#--------------------------------------------------------------------
def setTraceLevel ( level ):
        global traceLevel
	import re
        trace("debug", "Setting trace level- tracelevel:" + level)
        #traceString = " 'none' | 'error' | 'warn' | 'info' | 'debug' "
        if ( level in ["none" , "error" , "warn", "info" , "debug" ] ):
        	traceLevel = level
        else:
                writemsg("unknown trace level: "+ level)
        #endElse 
#endDef 

#--------------------------------------------------------------------
# Procedure:   initTraceLevel
# Description: gets the tracelevel from the properties file and sets it
#
# Parameters:  props	Properties object
# Returns:     N/A
#--------------------------------------------------------------------
def initTracelevel ( trclevel ):

	setTraceLevel(trclevel)
#endDef 

#--------------------------------------------------------------------
# Procedure:   getTraceLevel
# Description: getTraceLevel is used to get the current trace level
# Parameters:
#		
# Returns:	
#--------------------------------------------------------------------
def getTraceLevel (  ):
        global traceLevel
        return traceLevel
#endDef 

#--------------------------------------------------------------------
# Procedure:   saveChanges 
# Description: saveChanges based on the value of finaloperation
# Parameters:	
#		finaloperation (save|reset)					
#			
# Returns:	N/A
#--------------------------------------------------------------------
def saveChanges ( finalOperation ):
        global AdminConfig
        trace("info", AdminConfig.queryChanges( ) )
        if ( finalOperation == "save" ):
                trace("info", "*** Saving changes to the master configuration ***" )
                AdminConfig.save( )
                # Synchronize changes to the nodes
                trace("info", "*** Synchronizing changes to nodes ***" )
                #syncAllNodesToMaster( )
        elif ( finalOperation == "reset" ):
                trace("info", "*** Resetting the configuration ***" )
                AdminConfig.reset( )
        elif ( finalOperation == "default" ):
                trace("error", "Unknown finaloperation: "+finalOperation )
        #endIf
#endDef 

#--------------------------------------------------------------------
# Procedure:   buildTargetList
# Description: Builds and returns a list of the specified target type
#              i.e. It will return all Cells (one), all Nodes, or 
#              all cluster members of the specified cluster.
#
# Parameters:  scope	The scope of targets to return (Cell, Node, 
#			Server)
#	       clusterName Name of cluster.  Only applicable for
#	                   scope=Server
# Returns:     List of cells, nodes, or servers
#--------------------------------------------------------------------
def buildTargetList ( scope, clusterName = "" ):
    global AdminConfig, AdminControl

    # Get the name of the cell and node
    cellName = getCellName()
    dmNode = getNodeName()
    targetList = []

    if (scope == "Node"):
        nodelist = AdminConfig.list("Node").split(java.lang.System.getProperty("line.separator"))
        for node in nodelist:
            nodename = AdminConfig.showAttribute(node, "name")
            if (regexp("(Manager)", nodename, 0) != 0):
                writemsg("This is deployment manager node :" + nodename)
            #endIf 
            elif (regexp("webserver*?node", nodename, 0) != 0):
                writemsg("This is webserver node :" + nodename)    
            #endIf 
            else:
                writemsg("This is base node :" + nodename)
                targetList.append(nodename)
            #endif 
        #endFor 
    elif (scope == "Cell"):
        targetList = [cellName]
    elif (scope == "ServerCluster"):
        clusterObj = AdminConfig.getid("/Cell:"+cellName+"/" )
        clusterlist = AdminConfig.list("ServerCluster", clusterObj ).split(java.lang.System.getProperty("line.separator"))              
        for cluster in clusterlist:
            cluster_name = AdminConfig.showAttribute(cluster, "name" )
            targetList.append(cluster_name)
        #endFor 
    elif (scope == "Server"):
        serverlist = AdminConfig.list("Server").split(java.lang.System.getProperty("line.separator"))
        for server in serverlist:
            servername = AdminConfig.showAttribute(server, "name" )
            if (regexp("webserver\w+", servername, 0) != 0):
                writemsg("This is a web server, will remove from target list :" + servername)    
            #endIf 
            else:
                writemsg("This is a base server, will add to target list :" + servername)
                targetList.append(servername)
            #endif
        #endFor 
    #endIf 

    return targetList
#endDef 



#--------------------------------------------------------------------
# Procedure:   getCellName
# Description: returns the name of the Cell to which we're connected
# Parameters:  none
# Returns:     the name of connected cell
#--------------------------------------------------------------------
def getCellName():
    global AdminControl
    cellName = AdminControl.getCell( )
    return cellName
#endDef

#--------------------------------------------------------------------
# Procedure:   getNodeName
# Description: returns the name of the Node to which we're connected
# Parameters:  none
# Returns:     the name of connected node
#--------------------------------------------------------------------
def getNodeName (  ):
    global AdminControl
    nodeName = AdminControl.getNode()
    return nodeName
#endDef 

#--------------------------------------------------------------------
# Procedure:   findScopeEntry
# Description: Find cell or node entry
# Parameters:						
#		scope 		(Cell or Node) case sensitive!!!
#		scopeName	(cellName or nodeName)
# Returns:	targetName = success
#               0	   = entry does not exist
#--------------------------------------------------------------------
def findScopeEntry ( scope, scopeName ):
    global AdminApp, AdminConfig
    scopeEntry = 0
    if (scope == "Cell" ):
        scopeEntry = findCell(scopeName)
    elif (scope == "Node" ):
        scopeEntry = findNode(scopeName)
    elif (scope == "Server" ):
        scopeEntry = findServer(scopeName)
    elif (scope == "ServerCluster" ):
        scopeEntry = findServerCluster(scopeName)
    #endIf 	
    
    return scopeEntry
#endDef 

#--------------------------------------------------------------------
# Procedure:   findCluster
# Description: 
# Parameters:
#		clusterName 
# Returns:	serverName=success
#		0 =server does not exist
#--------------------------------------------------------------------
def findServerCluster ( clusterName ):
        oServerToUse = findConfigTarget(clusterName, "ServerCluster" )
        return oServerToUse
#endDef 

#--------------------------------------------------------------------
# Procedure:   findServer
# Description: 
# Parameters:
#		serverName 
# Returns:	first match server id (success)
#		0 (server does not exist)
#--------------------------------------------------------------------
def findServer ( serverName ):
    oServerToUse = findConfigTarget(serverName, "Server" )
    return oServerToUse
#endDef

#--------------------------------------------------------------------
# Procedure:   findNode
# Description: 
# Parameters:
#		nodeName 
# Returns:	first match node id (success) 
#		0 (node does not exist)
#--------------------------------------------------------------------
def findNode ( nodeName ):
    oNodeToUse = findConfigTarget(nodeName, "Node" )
    return oNodeToUse
#endDef 

#--------------------------------------------------------------------
# Procedure:   findCell
# Description: 
# Parameters:
#		cellName 
# Returns:	first match cell id (success)
#		0 (cell does not exist)
#--------------------------------------------------------------------
def findCell ( cellName ):
    oCellToUse = findConfigTarget(cellName, "Cell" )
    return oCellToUse
#endDef 

#--------------------------------------------------------------------
# Procedure:   findConfigTarget
# Description: 
# Parameters:
#		nameSearch	 
#		type		Cell, Node, Server   
# Returns:	first match target id (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def findConfigTarget (nameSearch, type):
    global AdminConfig
    
    oNameToUse = 0
    olist=AdminConfig.list(type).split(java.lang.System.getProperty("line.separator"))
    for oI in olist:
        name = AdminConfig.showAttribute(oI, "name")
        if (name == nameSearch):
			#writemsg("Found target entry name:" + oI)
            oNameToUse = oI
            break
        #endIf
    #endFor 

    return oNameToUse
#endDef 

#--------------------------------------------------------------------
# Procedure:   trimToEmpty
# Description: 
# Parameters:
# Returns:	
#--------------------------------------------------------------------
def trimToEmpty(string):
    if (string == None):
        return ""
    else:
        return string.strip()
    #endIf
#endDef

#--------------------------------------------------------------------
# Procedure:   getTargetContainmentPath
# Description: get a unique cell, node, cluster, or server containment path, to facilitate WAS resource scope restriction.
#    To get a unique cell, leave clusterName, nodeName and serverName as ""
#    To get a unique cluster, leave nodeName and serverName as ""
#    To get a unique node, leave clusterName and serverName as ""
#    To get a unique server, leave clusterName as "".
#        If nodeName is also left as "", containment path of the first server with the serverName will be returned.
# Parameters:
#       clusterName   exclusive with nodeName
#       nodeName      exclusive with clusterName
#       serverName    depends on nodeName's valid existence
# Returns:	a validated containment path without trailing /
#--------------------------------------------------------------------
def getTargetContainmentPath(clusterName, nodeName, serverName):
    cellName = getCellName()
    targetContPath = "/Cell:" + cellName
    
    if (clusterName != None and clusterName != ""):
        targetContPath += "/ServerCluster:" + clusterName
    elif (nodeName != None and nodeName != ""):
        targetContPath += "/Node:" + nodeName
        if (serverName != None and serverName != ""):
            targetContPath += "/Server:" + serverName
        #endIf
    elif (serverName != None and serverName != ""):
        serverId = findServer(serverName)
        nodeName = serverId[serverId.find("/nodes/") + 7 : serverId.find("/servers/")]
        targetContPath += "/Node:" + nodeName + "/Server:" + serverName
    #endIf
    
    targetId = AdminConfig.getid(targetContPath)
    if (targetId == "" or targetId == None):
        raise ValueError, "Configuration object(s) for" + targetContPath + " not found"
    #endIf

    return targetContPath
#endDef

#--------------------------------------------------------------------
# Procedure:   getTargetContainmentPathFromProperties
# Description: 
# Parameters:
# Returns:	a validated containment path
#--------------------------------------------------------------------
def getTargetContainmentPathFromProperties(props, prefixi):
    clusterName = props[prefixi + ".target.cluster"]
    nodeName = props[prefixi + ".target.node"]
    serverName = props[prefixi + ".target.server"]
    
    return getTargetContainmentPath(clusterName, nodeName, serverName)
#endDef

#--------------------------------------------------------------------
# Procedure:   findObjects
# Description: Find objects in WAS configuration repository. Use the two attributes (both match) to filter.
# Parameters:
#       type        object type
#       scope		scope, a WAS configuration id, in which to search for the objects.
#           The search will be conducted throughout the scope, that is, also in sub-scopes. 
#           For example, if scope is a cell, then search will be conducted in 
#           all the nodes, servers, and clusters in the cell, as well as in the cell itself.
#       attr1Name    name of the first attribute used to filter objects; can be ignored
#       attr1Value   value of the first attribute used to filter objects     
#       attr2Name    name of the second attribute used to filter objects; can be ignored
#       attr2Value   value of the second attribute used to filter objects 
# Returns:	a list of WAS configuration ids of found objects or one WAS configuration id if only one object found
#--------------------------------------------------------------------
def findObjects(type, scope, attr1Name = "", attr1Value = "", attr2Name = "", attr2Value = ""):
    # attribute names may be None
    attr1Name = trimToEmpty(attr1Name)
    attr2Name = trimToEmpty(attr2Name)
    
    resultList = []
    objectList = AdminConfig.list(type, scope).split(lineSeparator)
    
    for entry in objectList:
        # There are cases that "" entry is returned in the objectList, such as when listing sibus members of an sibus which has no member
        if (entry == ""):
            continue
        #endIf
            
        if (attr1Name == "" and attr2Name == ""):
            resultList.append(entry)
        elif (attr1Name == "" and attr2Name != ""):
            raise ValueError, "Use attribute 1 first"
        elif (attr1Name != "" and attr2Name == ""):
            if (AdminConfig.showAttribute(entry, attr1Name) == attr1Value):
                resultList.append(entry)
            #endIf
        else:
            if (AdminConfig.showAttribute(entry, attr1Name) == attr1Value and AdminConfig.showAttribute(entry, attr2Name) == attr2Value):
                resultList.append(entry)
            #endIf
        #endIf
    #endFor
    
    if (len(resultList) == 1):
        return resultList[0]
    else:
        return resultList
    #endIf
#endDef

#--------------------------------------------------------------------
# Procedure:   findObjectsByTarget
# Description: Find objects in WAS configuration repository. Use the containment path and two attributes (both match) to filter.
# Parameters:
#       type         object type
#       targetContPath	 WAS cell, node, server, or cluster containment path in which to search for the objects.
#           The search will be conducted without considering sub-scopes of the targetContPath.
#           For example, if targetContPath is a cell, then search will be conducted in the cell itself, but not in
#           any node, server, nor cluster in the cell.
#       attr1Name    name of the first attribute used to filter objects; can be ignored
#       attr1Value   value of the first attribute used to filter objects     
#       attr2Name    name of the second attribute used to filter objects; can be ignored
#       attr2Value   value of the second attribute used to filter objects 
# Returns:	a list of WAS configuration ids of found objects or one WAS configuration id if only one object found
#--------------------------------------------------------------------
def findObjectsByTarget(type, targetContPath, attr1Name = "", attr1Value = "", attr2Name = "", attr2Value = ""):
    import types

    # Conduct first search to narrow the result
    targetConfigId = AdminConfig.getid(targetContPath)
    objectList = findObjects(type, targetConfigId, attr1Name, attr1Value, attr2Name, attr2Value)
    if (not objectList):   # No object found
        return objectList
    #endIf

    if (not isinstance(objectList, types.ListType)):   # Single object returned
        objectList = [objectList]
    #endIf
    
    # Get the directory part of the target config id. 
    # The directory part is ended with '|' so that it can distinguish the specified config id from its sub config ids.
    # For example: with '(cells/Cell01|', you can distinguish 'ABC(cells/Cell01|resource...' from 'ABC(cells/Cell01/nodes/Node01|resource...' 
    targetConfigIdDirPart = targetConfigId[targetConfigId.find("(") : targetConfigId.find("|") + 1]
    
    # Use the directory part of the target config id to filter the objectList
    resultList = []    
    for entry in objectList:
        if (entry.find(targetConfigIdDirPart) > -1):
            resultList.append(entry)
        #endIf
    #endFor
    
    if (len(resultList) == 1):
        return resultList[0]
    else:
        return resultList
    #endIf
#endDef
        
#--------------------------------------------------------------------
# Procedure:   manageWASVariable
# Description: Creates/Deletes a WebSphere variable
# Parameters:
#		targetContPath	containment path which identifies the target in/from which to create/delete the WAS resource
#		varName		name of the WAS variable
#		varValue	value of the WAS variable
#		varDesc		description of the WAS var
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	1 if there is error; 0 if successful
#--------------------------------------------------------------------
def manageWASVariable(targetContPath, varName, varValue, varDesc, create):
    scope = AdminConfig.getid(targetContPath + "/VariableMap:/")
    varSubstitution = findObjects("VariableSubstitutionEntry", scope, "symbolicName", varName)
    
    # Use the java.lang.String to resolve the backslash issue
    if (varValue == None):
        varValue = ""
    #endIf
    if (varDesc == None):
        varDesc = ""
    #endIf
    valueAttr = java.lang.String("['value','" + varValue + "']")
    commonMsgPart = " WAS variable " + varName + " with value " + varValue + " and description " + varDesc
    if (varSubstitution):      # Existing variable found. Either update it or delete it according to the 'create' argument value.
        if (create):
            AdminConfig.modify(varSubstitution, [valueAttr, ["description", varDesc]])
            writemsg("Updated" + commonMsgPart)
        else:
            AdminConfig.remove(varSubstitution)
            writemsg("Deleted WAS variable " + varName)
        #endIf
    else:                     # Existing variable not found. Create it if requested, otherwise warn that it wasn't found.
        if (create):
            var = AdminConfig.create("VariableSubstitutionEntry", scope, [["symbolicName", varName], valueAttr, ["description", varDesc]])
            writemsg("Created" + commonMsgPart)
        else:
            writemsg("WAS variable " + varName + " doesn't exist")
        #endif         
    #endIf
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageURL
# Description: Creates/Deletes a URL
# Parameters:
#		targetContPath	containment path which identifies the target in/from which to create/delete the WAS resource
#		name		name of the URL
#       providerName
#		jndiName	jndiName of the URL
#		spec	
#       description	
#       category
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	url
#--------------------------------------------------------------------
def manageURL(targetContPath, name, providerName, jndiName, spec, description, category, create):
    url = findObjectsByTarget("URL", targetContPath, "name", name)
    
    attrs = [["jndiName", jndiName], ["spec", spec], ["description", description], ["category", category]]
    if (url):      # Existing url found. Either update it or delete it according to the 'create' argument value.
        if (create):
            AdminConfig.modify(url, attrs)
            writemsg("Updated URL " + name)
        else:
            AdminConfig.remove(url)
            writemsg("Deleted URL " + name)
        #endIf
    else:                     # Existing url not found. Create it if requested, otherwise warn that it wasn't found.
        if (create):
            provider = findObjectsByTarget("URLProvider", targetContPath, "name", providerName)
            if (not provider):
                raise ValueError, "URL provider " + providerName + " in target " + targetContPath + " not found" 
            #endIf
            print provider
            attrs.append(["name", name])
            url = AdminConfig.create("URL", provider, attrs)
            writemsg("Created URL " + name)
        else:
            writemsg("URL " + name + " doesn't exist")
        #endif         
    #endIf
    
    return url
#endDef

#--------------------------------------------------------------------
# Procedure:   manageSharedLibrary
# Description: Creates/Deletes a shared library
# Parameters:
#		targetContPath	containment path which identifies the target in/from which to create/delete the WAS resource
#		sharedlibname		
#		classpath	
#		nativepath	
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	sharedlib
#--------------------------------------------------------------------
def manageSharedLibrary(targetContPath, sharedlibname, classpath, nativepath, create):
    sharedlib = findObjectsByTarget("Library", targetContPath, "name", sharedlibname)
    commonMsgPart = " shared library " + sharedlibname
        
    attrs = []
    if (classpath != None):
        # Use the java.lang.String to resolve the backslash issue
        attrs.append(java.lang.String("['classPath','" + classpath + "']"))
    #endIf
    if (nativepath != None):
        # Use the java.lang.String to resolve the backslash issue
        attrs.append(java.lang.String("['nativePath','" + nativepath + "']"))
    #endIf
    
    if (sharedlib):
        if (create):
            AdminConfig.modify(sharedlib, attrs)
            writemsg("Updated" + commonMsgPart)
        else:
            AdminConfig.remove(sharedlib)
            writemsg("Deleted" + commonMsgPart)
        #endIf
    else:
        if (create):
            scope = AdminConfig.getid(targetContPath)
            attrs.append(['name', sharedlibname])
            sharedlib = AdminConfig.create('Library', scope, attrs)
            writemsg("Created" + commonMsgPart)
        else:
            writemsg("Shared library " + sharedlibname + " doesn't exist")
        #endif  
    #endIf
    
    return sharedlib
#endDef 


#--------------------------------------------------------------------
# Procedure:   manageAppServerClassloader
# Description: Create/Delete an application server classloader. Assume that there is at most one class loader for an application server
# Parameters:
#		targetContPath	containment path which identifies the target in which to create/delete the WAS resource
#		mode	
#		libRefNames	
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	classloader
#--------------------------------------------------------------------
def manageAppServerClassloader(targetContPath, mode, libRefNames, create):
    import types
    classloader = AdminConfig.getid(targetContPath + "/ApplicationServer:/Classloader:/")
    if (classloader):
        if (create):
            AdminConfig.modify(classloader, [["mode", mode]])
            existingLibrefs = findObjects("LibraryRef", classloader)
            
            if (isinstance(existingLibrefs, types.ListType)):   # Single object returned
            	for libref in existingLibrefs:
            		AdminConfig.remove(libref)
            	#endFor
            #endIf
            else: 
    			AdminConfig.remove(existingLibrefs)
            #endIf
            
            for libName in libRefNames:
                libName = trimToEmpty(libName)
                AdminConfig.create('LibraryRef', classloader, [['libraryName', libName]])
            #endFor
            writemsg("Updated classloader for server " + targetContPath)
        else:
            AdminConfig.remove(classloader)
            writemsg("Deleted classloader for server " + targetContPath)
        #endIf
    else:
        if (create):
            appServer = AdminConfig.getid(targetContPath + "/ApplicationServer:/")
            classloader = AdminConfig.create('Classloader', appServer, [['mode', mode]])
            for libName in libRefNames:
                AdminConfig.create('LibraryRef', classloader, [['libraryName', libName]])
            #endFor
            writemsg("Created classloader for server " + targetContPath)
        else:
            writemsg("Classloader for server " + targetContPath + " doesn't exist")
        #endIf   
    #endIf
    
    return classloader
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJ2CAuthEntry
# Description: Creates/Deletes a J2C authorization alias
#		If the alias already exists the auth Alias will not be replaced
#
# Parameters:
#		authAliasName	(Authentication Alias name) 
#		userid		(User ID)
#		password	(Password)
#		create		(Boolean if true, create entry, else delete it)
#		description	(Description of the Alias)
# Returns:	authAlias
#--------------------------------------------------------------------
def manageJ2CAuthEntry (authAliasName, userid, password, description, create):
    traceEntry("manageJ2CAuthEntry")

    scope = AdminConfig.getid('/Cell:' + getCellName() + '/Security:/')  
    authAlias = findObjects("JAASAuthData", scope, "alias", authAliasName)
    
    attrs = [['userId', userid], ['password', password], ['description', description]]
    if (authAlias):
        if (create):
            manageJ2CAuthEntry_ValidateArguments(authAliasName, userid, password)
            AdminConfig.modify(authAlias, attrs)            
            writemsg("Updated JAAS authentication alias " + authAliasName)
        else:
            AdminConfig.remove(authAlias)
            writemsg("Deleted JAAS authentication alias " + authAliasName)
        #endIf
    else:
        if (create):
            manageJ2CAuthEntry_ValidateArguments(authAliasName, userid, password)
            attrs.append(['alias', authAliasName])
            authAlias = AdminConfig.create('JAASAuthData', scope, attrs)
            writemsg("Created JAAS authentication alias " + authAliasName)
        else:
            writemsg("JAAS authentication alias " + authAliasName + " doesn't exist")
        #endif   
    #endIf
    
    traceExit("manageJ2CAuthEntry" )
    
    return authAlias
#endDef 

def manageJ2CAuthEntry_ValidateArguments(authAliasName, userid, password):
    # validate the J2CAuthEntry parameters
    if (authAliasName == "" or authAliasName == None):
        raise ValueError, "Authorization Alias Name is required."
    #endIf 
    if (userid == "" or userid == None):
        raise ValueError, "userid is required."
    #endIf 
    if (password == "" or password == None):
        raise ValueError, "password is required."
    #endIf 
#endDef 
#--------------------------------------------------------------------
# Procedure:   manageJDBCProvider
# Description: 
# Parameters:
#		targetContPath
#		providerName
#		providerDescription	(xyz jdbc driver)
#       driverClassPath		(i.e. /sqllib/java/db2java.zip
#       xa
#		implementationClassName	(e.g. oracle.jdbc.pool.OracleConnectionPoolDataSource)
#		providerType		WAS provider type
#		create			(Boolean, if true create objects, else delete them)
# Returns:	The JDBC Provider object if successful.
#--------------------------------------------------------------------
def manageJDBCProvider ( targetContPath, providerName, providerDescription, classpath, nativepath, xa, implementationClassName, providerType, create ):
    traceEntry("manageJDBCProvider" )

    provider = findObjectsByTarget("JDBCProvider", targetContPath, "name", providerName)
    
    attrs = [["classpath", classpath], ["nativepath", nativepath], ["implementationClassName", implementationClassName], ["description", providerDescription]]
    if (provider):      
        if (create):
            AdminConfig.modify(provider, attrs)
            writemsg("Updated JDBCProvider " + providerName)
        else:
            AdminConfig.remove(provider)
            writemsg("Deleted JDBCProvider " + providerName)
        #endIf
    else:                     
        if (create):
            scope = AdminConfig.getid(targetContPath)
            attrs.append(["name", providerName])
            attrs.append(["providerType", providerType])   # Readonly property
            if (xa != None):
                attrs.append(["xa", xa])
            #endIf
            provider = AdminConfig.create("JDBCProvider", scope, attrs)
            writemsg("Created JDBCProvider " + providerName)
        else:
            writemsg("JDBCProvider " + providerName + " doesn't exist")
        #endif         
    #endIf
     
    traceExit("manageJDBCProvider" )
    
    return provider
#endDef

#--------------------------------------------------------------------
# Procedure:   createDataSource
# Description: Create a generic datasource based on parameters
# Parameters:
#		targetContPath 			
#		providerName
#   		datasourceName
#		datasourceDescription
#		datasourceJNDIName	
#		datasourceHelperClassname
#		statementCacheSize
#		authDataAlias
#       xaRecoveryAuthAlias
# Returns: datasource
#--------------------------------------------------------------------  
def createDataSource ( targetContPath, providerName, datasourceName, datasourceDescription, datasourceJNDIName, datasourceHelperClassname, statementCacheSize, authDataAlias, xaRecoveryAuthAlias):
    traceEntry("createDataSource" )
    
    provider = findObjectsByTarget("JDBCProvider", targetContPath, "name", providerName)
    if (not provider):
        raise ValueError, "JDBC provider " + providerName + " in scope " + targetContPath + " not found" 
    #endIf
    
    dataSourceAttrs = [["name", datasourceName], ["description", datasourceDescription], ["jndiName", datasourceJNDIName], ["authDataAlias", authDataAlias]]
    if (xaRecoveryAuthAlias != None):
        dataSourceAttrs.append(["xaRecoveryAuthAlias", xaRecoveryAuthAlias])
    #endIf
    if (datasourceHelperClassname != None):
        dataSourceAttrs.append(["datasourceHelperClassname", datasourceHelperClassname])
    #endIf
    if (statementCacheSize != None):
        dataSourceAttrs.append(["statementCacheSize", statementCacheSize])
    #endIf
    datasource = AdminConfig.create("DataSource", provider, dataSourceAttrs )
    AdminConfig.create("J2EEResourcePropertySet", datasource, [] )
    writemsg("Created data source " + datasourceName)
    
    traceExit("createDataSource" )
    return datasource
#endDef 

#--------------------------------------------------------------------
# Procedure:   addCustomPropertyToDataSource
# Description: 
# Parameters:
#		datasource  configuration id of the data source
#		custName	(Custom Property Name)
#		custValue	(Custom Property Value)
# return :
# 		None
#--------------------------------------------------------------------
def addCustomPropertyToDataSource ( datasource, custName, custValue ):
    traceEntry ("addCustomPropertyToDataSource")
    
    dsName = datasource[0:datasource.find("(")]
    propSet = AdminConfig.showAttribute(datasource, "propertySet" )
    attr = [["name", custName], ["value", custValue]]
    AdminConfig.create("J2EEResourceProperty", propSet, attr )
    writemsg("Added custom property " + custName + " to data source " + dsName)

    traceExit ("addCustomPropertyToDataSource") 
#endDef 
#--------------------------------------------------------------------
def addCustomPropertyToEntry ( entry, custName, custValue):
    
    propSet = AdminConfig.showAttribute(entry, "propertySet" )
    attr = [["name", custName], ["value", custValue]]
    AdminConfig.create("J2EEResourceProperty", propSet, attr )
    writemsg("Added custom property " + custName + " to entry " + entry)

    traceExit ("addCustomPropertyToDataSource") 
#endDef 
#--------------------------------------------------------------------
# Procedure:   createConnectionPoolForDatasource
# Description: modify the settings that affect the max/min size of the DB connection pool
# Parameters:
#		datasource  configuration id of the data source
#		connectionPoolMax 	(Maximum connections)
#       connectionPoolMin	(Minimum connections)
# Returns:	
#--------------------------------------------------------------------  
def createConnectionPoolForDatasource ( datasource, connectionPoolMax, connectionPoolMin ):
    traceEntry ("createConnectionPoolForDatasource")
    
    dsName = datasource[0:datasource.find("(")]
    
    attrs = []
    if (connectionPoolMax != None):
        attrs.append(["maxConnections", connectionPoolMax])
    #endIf
    if (connectionPoolMax != None):
        attrs.append(["minConnections", connectionPoolMin])
    #endIf
    
    connpool = None
    if (len(attrs) == 0):
        writemsg("Connection pool for data source " + dsName + " not requested to be created")
    else:
        AdminConfig.create("ConnectionPool", datasource, attrs )
        writemsg("Configured connection pool for data source " + dsName)
    #endIf
    
	traceExit ("createConnectionPoolForDatasource") 
#endDef 

#--------------------------------------------------------------------
# Procedure:   setMappingModuleForDatasource
# Description: 
# Parameters:
#  		datasource  configuration id of the data source
#       mappingConfigAlias	(mapping config alia)
#		authDataAlias		(container-managed Alias)
# Returns:	
#--------------------------------------------------------------------  
def setMappingModuleForDatasource ( datasource, mappingConfigAlias, containerAuthAlias ):
    traceEntry ("setMappingModuleForDatasource")

    dsName = datasource[0:datasource.find("(")]
    
    attrs = []
    if (mappingConfigAlias != None):
        attrs.append(["mappingConfigAlias", mappingConfigAlias])
    #endIf
    if (containerAuthAlias != None):
        attrs.append(["authDataAlias", containerAuthAlias])
    #endIf
    
    if (len(attrs) == 0):
        writemsg("MappingModule for data source " + dsName + " not requested to be created")
    else:
        AdminConfig.create("MappingModule", datasource, attrs )
        writemsg("Created MappingModule for data source " + dsName)
    #endIf
	
	traceExit ("setMappingModuleForDatasource") 
#endDef 

#--------------------------------------------------------------------
# Procedure:   setDataSourceCMP
# Description: 
# Parameters:
#		datasource  configuration id of the data source
#		containerAuthAlias		(container-managed Alias)
# Returns:	
#--------------------------------------------------------------------  
def setDataSourceCMP ( datasource, containerAuthAlias ):
    traceEntry ("setDataSourceCMP")
    
    dsName = datasource[0:datasource.find("(")]
    
    j2cresourceadapter = AdminConfig.showAttribute(datasource, "relationalResourceAdapter")
    if (j2cresourceadapter == 0):
        raise ValueError, "J2CResourceAdapter not found for data source " + dsName
    #endIf 
    
    attrs = [["name", "mycmp"], ["cmpDatasource", datasource], ["authDataAlias", containerAuthAlias]]
    AdminConfig.create("CMPConnectorFactory", j2cresourceadapter, attrs )
    writemsg("Created CMPConnectorFactory for data source " + dsName)
    
    traceExit ("setDataSourceCMP")
#endDef

#--------------------------------------------------------------------
# Procedure:   updateJvmGeneralProperties
# Description: 
# Parameters:
#  		jvm         configuration id of a server jvm  
#       maxheapsize
#		initheapsize
#		genericJvmArguments
# Returns:	
#--------------------------------------------------------------------  
def updateJvmGeneralProperties( jvm, maxheapsize, initheapsize, genericJvmArguments ):
    attrs = []
    if (maxheapsize != None):
        attrs.append(["maximumHeapSize", maxheapsize])
    #endIf
    if (initheapsize != None):
        attrs.append(["initialHeapSize", initheapsize])
    #endIf
    if (genericJvmArguments != None):
        attrs.append(["genericJvmArguments", genericJvmArguments])
    #endIf
    
    if (len(attrs) == 0):
        writemsg("Nothing to be updated for general properties of JVM " + jvm)
    else:
        AdminConfig.modify(jvm, attrs )
        writemsg("Updated general properties of JVM " + jvm)
    #endIf
#endDef 

#--------------------------------------------------------------------
# Procedure:   updateTransactionServiceGeneralProperties
# Description: 
# Parameters:
#  		transService         configuration id of a transaction service  
#       totalTranLifetimeTimeout
#		propogatedOrBMTTranLifetimeTimeout
# Returns:	
#--------------------------------------------------------------------
def updateTransactionServiceGeneralProperties(transService, totalTranLifetimeTimeout,propogatedOrBMTTranLifetimeTimeout):
    attrs = []
    if (totalTranLifetimeTimeout != None):
        attrs.append(["totalTranLifetimeTimeout", totalTranLifetimeTimeout])
    #endIf
    if (propogatedOrBMTTranLifetimeTimeout !=None):
    	attrs.append(["propogatedOrBMTTranLifetimeTimeout", propogatedOrBMTTranLifetimeTimeout])
    #endIf
    
    if (len(attrs) == 0):
        writemsg("Nothing to be updated for general properties of TransactionService " + transService)
    else:
        AdminConfig.modify(transService, attrs)
        writemsg("Updated general properties of TransactionService " + transService)
    #endIf
#endDef

#--------------------------------------------------------------------
# Procedure:   updateTransactionServiceGeneralProperties
# Description: 
# Parameters:
#  		traceService         configuration id of the traceService
#       startupTraceSpecificationPart
#       create
# Returns:	
#--------------------------------------------------------------------
def updateAppServerTraceServiceSettings(traceService, startupTraceSpecificationPart, create):

    oldSpec = AdminConfig.showAttribute(traceService, "startupTraceSpecification")
    writemsg("Current trace specification is " + oldSpec) 
    index = oldSpec.find(startupTraceSpecificationPart)
    if (create):
        if (index > -1):
            writemsg(startupTraceSpecificationPart + " already exists in the trace specification")
        else:
            newSpec = oldSpec + ":" + startupTraceSpecificationPart
            AdminConfig.modify(traceService, [["startupTraceSpecification", newSpec]])
            writemsg("Appended " + startupTraceSpecificationPart + " to the trace specification")
        #endIf
    else:
        if (index > -1):
            newSpec = oldSpec.replace(":" + startupTraceSpecificationPart, "")
            AdminConfig.modify(traceService, [["startupTraceSpecification", newSpec]])
            writemsg("Deducted " + startupTraceSpecificationPart + " from the trace specification")
        else:
            writemsg(startupTraceSpecificationPart + " does not exist in the trace specification")
        #endIf
    #endIf
    
#endDef

#--------------------------------------------------------------------
# Procedure:   updateTransactionServiceGeneralProperties
# Description: 
# Parameters:
#  		ejbContainer         configuration id of an ejb container  
#       totalTranLifetimeTimeout
# Returns:	
#--------------------------------------------------------------------
def updateEJBContainerGeneralProperties(ejbContainer, inactivePoolCleanupInterval):
    attrs = []
    if (inactivePoolCleanupInterval != None):
        attrs.append(["inactivePoolCleanupInterval", inactivePoolCleanupInterval])
    #endIf
    
    if (len(attrs) == 0):
        writemsg("Nothing to be updated for general properties of EJB Container " + ejbContainer)
    else:
        AdminConfig.modify(ejbContainer, attrs)
        writemsg("Updated general properties of EJB Container " + ejbContainer)
    #endIf
#endDef

#--------------------------------------------------------------------
# Procedure:   manageJvmCustomProperty
# Description: 
# Parameters:
#  		jvm         configuration id of a server jvm    
#       propertyname
#		propertyvalue
#       propertydescript
#       create
# Returns:	custProp
#--------------------------------------------------------------------  
def manageJvmCustomProperty( jvm, propertyname, propertyvalue, propertydescript, create ):
    custProp = findObjects("Property", jvm, "name", propertyname)  
    
    attrs = []
    # Use the java.lang.String to resolve the backslash issue
    attrs.append(java.lang.String("['value','" + propertyvalue + "']"))
       
    if (propertydescript != None):
        attrs.append(["description", propertydescript])
    #endIf
    
    if (custProp):      
        if (create):
            AdminConfig.modify(custProp, attrs)
            writemsg("Updated custom property " + propertyname + " for JVM " + jvm)
        else:
            AdminConfig.remove(custProp)
            writemsg("Deleted custom property " + propertyname + " from JVM " + jvm)
        #endIf
    else:                     
        if (create):
            attrs.append(["name", propertyname])
            custProp = AdminConfig.create("Property", jvm, attrs)
            writemsg("Created custom property " + propertyname + " for JVM " + jvm)
        else:
            writemsg("Custom property " + propertyname + " of JVM " + jvm + "doesn't exist")
        #endif         
    #endIf
    
    return custProp
#endDef 

#--------------------------------------------------------------------
# Procedure:   findConfigTargetWithScope
# Description: 
# Parameters:						
#		scope 		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		nameSearch	(name string to search)
#		type 		(Cell, Node Server ...)
# Returns:	targetName 	(success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def findConfigTargetWithScope ( scope, scopeName, nameSearch, typename ):
        global AdminConfig
        traceEntry("findConfigTargetWithScope: " + typename )
        trace("info","-----> Start findConfigTargetWithScope "+scope+" "+scopeName+" "+nameSearch+" "+ typename)
        scopeToUse = 0
        newWAS5NamePattern = nameSearch
        newWAS6NamePattern = nameSearch
        stringToFindWAS6 = ""
        stringToFindWAS5 = ""

        # 08/02/2006 update
        if (scope == "Cell" ):
                scopeToUse = findCell(scopeName )
                stringToFindWAS6 = "cells/"+scopeName+"|"
                stringToFindWAS5 = "cells/"+scopeName+":"
                newWAS5NamePattern += "\(cells/"+scopeName+":*?"
                newWAS6NamePattern += "\(cells/"+scopeName+"\|*?"
        elif (scope == "ServerCluster" ):
                scopeToUse = findServerCluster(scopeName)
                stringToFindWAS6 = "clusters/"+scopeName+"|"
                stringToFindWAS5 = "clusters/"+scopeName+":"
                newWAS5NamePattern += "\(cells/*?/clusters/"+scopeName+":*?"
                newWAS6NamePattern += "\(cells/*?/clusters/"+scopeName+"\|*?"
        elif (scope == "Node" ):
                scopeToUse = findNode(scopeName )
                stringToFindWAS6 = "nodes/"+scopeName+"|"
                stringToFindWAS5 = "nodes/"+scopeName+":"
                newWAS5NamePattern += "\(cells/*?/nodes/"+scopeName+":*?"
                newWAS6NamePattern += "\(cells/*?/nodes/"+scopeName+"\|*?"
        elif (scope == "Server" ):
                scopeToUse = findServer(scopeName )
                stringToFindWAS6 = "servers/"+scopeName+"|"
                stringToFindWAS5 = "servers/"+scopeName+":"
                newWAS5NamePattern += "\(cells/*?/nodes/*?/servers/"+scopeName+":*?"
                newWAS6NamePattern += "\(cells/*?/nodes/*?/servers/"+scopeName+"\|*?"
        #endIf 
	writemsg("scopeToUse=" + scopeToUse )
        if (scopeToUse == 0):
                return 0
        #endIf
        writemsg("newWAS5NameSearch=" + newWAS5NamePattern )
        writemsg("newWAS6NameSearch=" + newWAS6NamePattern )
        oList = AdminConfig.list(typename,scopeToUse).split(java.lang.System.getProperty("line.separator"))
        for oI in oList :
	        if (regexp(newWAS6NamePattern, oI, 0) == 1 ) :
	                if (oI.find(stringToFindWAS6)!=-1):
	                        writemsg("Found target:" + nameSearch + " for type:" + typename )	              
	                        writemsg(oI)
	                        oNameToUse = oI
	                        break
	                else:
	                        # writemsg("Target:" + nameSearch + " for type:" + typename + " not found" )
	                        oNameToUse = 0
	        else:
	                if (regexp(newWAS5NamePattern, oI, 0) == 1 ):
	                        if (oI.find(stringToFindWAS5)!=-1):
	                               writemsg("Found target:" + nameSearch + " for type:" + typename )	              
	                               writemsg(oI)
	                               oNameToUse = oI
	                               break
	                        else:
	                               # writemsg("Target:" + nameSearch + " for type:" + typename + " not found" )
	                               oNameToUse = 0
	                else:
	                        # writemsg("Target:" + nameSearch + " for type:" + typename + " not found" )
	                        oNameToUse = 0
	                #endElse 
	        #endElse 
	#endFor	
        traceExit("findConfigTargetWithScope: " + typename )
        return oNameToUse
#endDef

#--------------------------------------------------------------------
# Procedure:	manageMQQueueDestination
# Description:  Creates/Deletes a WebSphere MQ Queue Destination
# Parameters:
#		providerName	(name of this JMS provider)
#		scope		(Cell, Node, Server)
#		scopeName	(Actual name of cell, node, or server)
#		name		(name of MQ Queue Destination)
#		jndiName	(JNDI used to bind connection factory to server)
#		baseQueueName	(name of base queue where messages are sent)
#		baseQueueMgr	(name of base queue manager)
#		targetClient	(receiving application is JMS-compliant or WebSphere MQ)
#		CCSID		( code page ) - added YKL 02/02/2006
#		create		(Boolean if true, create queue dest., else delete it)
#
# Returns:	1 (could not create)
# History:
# 2004/08/20	pty	Created
#--------------------------------------------------------------------
def manageMQQueueDestination ( providerName, scope, scopeName, name, jndiName, baseQueueName, baseQueueMgr, targetClient, CCSID, create ):
        global AdminConfig

        traceEntry("manageMQQueueDestination" )

        # does the queue destination exist?
        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
        #endIf 
        queueDest = findMQQueueDestinationWithScope(scope, scopeName, name )
        if (queueDest != 0):
                if ((  not create ) ):
                        AdminConfig.remove(queueDest )
                        trace("info", "Deleted MQ Queue Destination "+name )
                else:
                        trace("info", "MQ Queue Destination "+name+" already exists" )
                #endElse 

        else:
                if (( create ) ):
                        trace("info", "Creating MQ Queue Destination "+name )
                        jmsProvider = findJMSProviderWithScope(scope, scopeName, providerName )

                        nameAttr = ["name", name]
                        jndiAttr = ["jndiName", jndiName]
                        baseQueueNameAttr = ["baseQueueName", baseQueueName]
                        baseQueueMgrAttr = ["baseQueueManagerName", baseQueueMgr]
                        targetClientAttr = ["targetClient", targetClient]
                        CCSID = ["CCSID", CCSID]

                        mqQDestAttrs = [nameAttr, jndiAttr, baseQueueNameAttr, baseQueueMgrAttr, targetClientAttr, CCSID]

                        template = AdminConfig.listTemplates("MQQueue" )
                        template = template.split(java.lang.System.getProperty("line.separator"))
                        #?PROBLEM? (jacl 1030) previous LINDEX may need variable.split("xx")

                        AdminConfig.createUsingTemplate("MQQueue", jmsProvider, mqQDestAttrs, template[0] )
                else:
                        trace("info", "MQ Queue Destination "+name+" doesn't exist" )
                #endElse 
        #endElse 

        traceExit("manageMQQueueDestination" )
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:	findMQQueueDestinationWithScope
# Description:  Finds the MQ Queue Destination at the specified scope, if it exists.
# Parameters:
#		scope		(Cell, Node, Server)
#		scopeName	(Actual name of cell, node, or server)
#		name		(name of MQ Queue Destination to find)
#
# Returns:	qDest		(success), 0 (could not find)
# History:
# 2004/10/24	fwr	Created
#--------------------------------------------------------------------
def findMQQueueDestinationWithScope ( scope, scopeName, name ):
        qDest = findConfigTargetWithScope(scope, scopeName, name, "MQQueue" )
        return qDest
#endDef 

#--------------------------------------------------------------------
# Procedure:	findJMSProviderWithScope
# Description:  Finds the JMS provider at the specified scope, if it exists.
#               Note: This proc also works for the built-in WMQ providers.
# Parameters:
#		scope		(Cell, Node, Server)
#		scopeName	(Actual name of cell, node, or server)
#		jmsProviderName	(name of JMS provider to find)
#
# Returns:	provider (success), 1 (could not create)
# History:
# 2004/01/07	wdj	Created
#--------------------------------------------------------------------
def findJMSProviderWithScope ( scope, scopeName, jmsProviderName ):
        jmsProvider = findConfigTargetWithScope(scope, scopeName, jmsProviderName, "JMSProvider" )
        return jmsProvider
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageMessageListener
# Description: 
# Parameters:
#		serverName 	(name of server on which LP will exist)
#		lpName		(name of listener port to create)
#		cfJndiName	(connection factory JNDI name)
#		destJndiName	(JMS destination JNDI name)
#		maxSessions	(maximum concurrent sessions)
#		maxRetries	(max number of tries to deliver message)
#		maxMessages	(max messages processed in one session)
#		create		(Boolean if true, create listener, else delete it) 
# Returns:	serverName=success; 0 =server does not exist
# History:
# 2003/03/15	ccd	Created
# 2004/09/09	pty	Modified to include max sessions, retries and messages
#--------------------------------------------------------------------
def manageMessageListener ( serverName, lpName, cfJndiName, destJndiName, maxSessions, maxRetries, maxMessages, create ):
        global AdminConfig

        traceEntry("manageMessageListener" )
        # 09/14/2006 fix for cluster installation
        listenPort = findConfigTargetWithScope("Server", serverName, lpName, "ListenerPort" )
        trace("info", "lpName port "+lpName )
        trace("info", "Listener port " + str(listenPort) )
        if (listenPort != 0):
                if ((  not create ) ):
                        trace("info", "Deleting Listener Port" + lpName )
                        AdminConfig.remove(listenPort )
                else:
                        trace("info", "Listener Port "+lpName+" already exists" )
                #endElse 

        else:
                if (( create ) ):
                        trace("info", "Creating Listener port "+lpName )
                        nameAttr = ["name", lpName]
                        destJndiAttr = ["destinationJNDIName", destJndiName]
                        cfJndiAttr = ["connectionFactoryJNDIName", cfJndiName]
                        maxSessAttr = ["maxSessions", maxSessions]
                        maxRetrAttr = ["maxRetries", maxRetries]
                        maxMessAttr = ["maxMessages", maxMessages]
                        lpAttrs = [nameAttr, destJndiAttr, cfJndiAttr, maxSessAttr, maxRetrAttr, maxMessAttr]

                        appserver = findServer(serverName )
                        mls = AdminConfig.list("MessageListenerService", appserver )

                        lp = AdminConfig.create("ListenerPort", mls, lpAttrs )

                        # Apparently we have to create this object to make the listener
                        # port start automatically when the server starts.
                        AdminConfig.create("StateManageable", lp, [["initialState", "START"]] )
                else:
                        trace("info", "Listener port "+lpName+" doesn't exist" )
                #endElse 
        #endElse 
        traceExit("manageMessageListener" )
        return 0
#endDef 

#--------------------------------------------------------------------
# Procedure:   findJAASConfigurationEntry
# Description:
# Parameters:
#               configName
# Returns:      targetName (success) 0 (does not exist)
# History:
# 2005/04/14    SPR     Created
#--------------------------------------------------------------------
def findJAASConfigurationEntry ( aliasName ):
        global AdminApp, AdminConfig

        writemsg(aliasName )
        configName = 0
        jaasConfigList = AdminConfig.list("JAASConfigurationEntry" ).split(java.lang.System.getProperty("line.separator"))

        for configEntry in jaasConfigList:
                #?PROBLEM? (jacl 232) previous FOREACH_ITEM_IN_LIST may need variable.split("xx")
                if (AdminConfig.showAttribute(configEntry, "alias" ) == aliasName):
                        print aliasName+" found"
                        configName = configEntry
                        break 
                #endIf 
        #endFor 

        if (configName != 0):
                return configName
        else:
                return 0
        #endElse 
#endDef 

#--------------------------------------------------------------------
# Procedure:   getConfigObject
# Description: Gets the config ID for the specified object name and
#		   object type.  Provided as a convenience.
# Parameters:  objectName	The name of the config object
#		   objectType	The type of object
# Returns:	   Config ID of the specified object.
#--------------------------------------------------------------------
def getConfigObject ( objectName, objectType ):
        global AdminConfig

        try:
                _excp_ = 0
                objID = AdminConfig.getid("/"+objectType+":"+objectName+"/" )
                _excp_ = 0 #reset (in case of nested exceptions)
        except:
                _type_, _value_, _tbck_ = sys.exc_info()
                objID = str(_value_)
                _excp_ = 1
        #endTry 
        if (_excp_ ):
                print "Error getting object ID of Config Object"
                print "Error Message = "+objID
                return 
        #endIf 
        return objID
#endDef 

#--------------------------------------------------------------------
# Procedure:   assignSharedLibToEAR
# Description: Assigns a Shared Library to an Enterprise Application.
# Parameters:  eaName		Name of the enterprise application.
#		   sharedLib	Name of the shared library
# Returns:	   Config ID of the specified object.
#--------------------------------------------------------------------
def assignSharedLibToEAR ( eaName, sharedLib ):
        global AdminConfig
        deployment = AdminConfig.getid("/Deployment:"+eaName+"/" )
        appDeploy = AdminConfig.showAttribute(deployment, "deployedObject" )
        classLoader = AdminConfig.showAttribute(appDeploy, "classloader" )

        libattr = ["libraryName", sharedLib]
        sclattr = ["sharedClassloader", "true"]
        attrs = [libattr, sclattr]

        AdminConfig.create("LibraryRef", classLoader, attrs )
#endDef 

#--------------------------------------------------------------------
# Procedure:	manageMQQueueConnectionFactory
# Description:  Creates/Deletes a WebSphere MQ Queue Connection Factory
# Parameters:
#		providerName	(name of this JMS provider)
#		scope		(Cell, Node, Server)
#		scopeName	(Actual name of cell, node, or server)
#		name		(name of Connection Factory)
#		jndiName	(JNDI used to bind connection factory to server)
#		queueManager	(name of queue manager for connection factory)
#		transportType   (specifies client connection or JNI bindings)
#		clientID	(JMS client identifier)
#		persistence	(message retention)
#		enableXA	(XA or non-XA QCF/TCF and coordination of messages)
#		CCSID		(code page )  - added YKL 02/02/2006
#		conPool		(connection pool settings) - added ktchu 10/7/2004
#		sessPool	(session pool settings) - added ktchu 10/7/2004
#		create		(Boolean, if true, create the QueueConnFactory, else
#				delete it)
#
#
# Returns:	1 (could not create)
# History:
# 2004/08/20	pty	Created
#--------------------------------------------------------------------
def manageMQQueueConnectionFactory ( providerName, scope, scopeName, name, jndiName, queueManager, transportType, clientID, msgRetention, XAEnabled, CCSID, conPool, sessPool, create ):
        global AdminConfig

        traceEntry("manageMQQueueConnectionFactory" )

        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
        #endIf 
        connFac = findMQQueueConnectionFactoryWithScope(scope, scopeName, name )
        if (connFac != 0):
                if ((  not create ) ):
                        trace("info", "Deleting MQ Queue Connection Factory "+name )
                        AdminConfig.remove(connFac )
                else:
                        trace("info", "MQ Queue Connection Factory "+name+" already exists" )
                #endElse 

        else:
                if (( create ) ):
                        trace("info", "MQ Queue Connection Factory "+name+" not found, creating it now." )
                        print "%%%scope"+scope
                        jmsProvider = findJMSProviderWithScope(scope, scopeName, providerName )

                        nameAttr = ["name", name]
                        jndiAttr = ["jndiName", jndiName]
                        queueMgrAttr = ["queueManager", queueManager]
                        transTypeAttr = ["transportType", transportType]
                        clientIDAttr = ["clientID", clientID]
                        msgRetentionAttr = ["msgRetention", msgRetention]
                        XAAttr = ["XAEnabled", XAEnabled]
                        CCSID = ["CCSID", CCSID]
                        conPoolAttr = ["connectionPool", conPool]
                        sessPoolAttr = ["sessionPool", sessPool]
                        authAttr = ["authMechanismPreference","BASIC_PASSWORD"]
			descAttr = ["description",""]
			
                        mqConnFacAttrs = [nameAttr, jndiAttr, queueMgrAttr, transTypeAttr, clientIDAttr, msgRetentionAttr, XAAttr, CCSID, conPoolAttr, sessPoolAttr, authAttr , descAttr ]

                        template = AdminConfig.listTemplates("MQQueueConnectionFactory" ).split(java.lang.System.getProperty("line.separator"))[0]
                        #?PROBLEM? (jacl 1170) previous LINDEX may need variable.split("xx")
                        print "jmsProvider=" + str(jmsProvider)
                        print "mqConnFacAttrs=" + str(mqConnFacAttrs)
                        print "template=" + str(template)

                        qmfactory=AdminConfig.createUsingTemplate("MQQueueConnectionFactory", jmsProvider, mqConnFacAttrs, template )
                        mappingAttrs=[["mappingConfigAlias","DefaultPrincipalMapping"],["authDataAlias",""]]
                        AdminConfig.create("MappingModule",qmfactory,mappingAttrs)
                        return 0
                else:
                        trace("info", "MQ Queue Connection Factory "+name+" doesn't exist" )
                #endElse 
                traceExit("manageMQQueueConnectionFactory" )
        #endElse 
#endDef 

#--------------------------------------------------------------------
# Procedure:	manageRemoteMQQueueConnectionFactory
# Description:  Creates/Deletes a WebSphere MQ Queue Connection Factory
# Parameters:
#		providerName	(name of this JMS provider)
#		scope		(Cell, Node, Server)
#		scopeName	(Actual name of cell, node, or server)
#		name		(name of Connection Factory)
#		jndiName	(JNDI used to bind connection factory to server)
#		queueManager	(name of queue manager for connection factory)
#		host		(specifies client connection host)
#		port		(specifies client connection port)
#		channel		(specifies client connection channel)
#		transportType   (specifies client connection or JNI bindings)
#		clientID	(JMS client identifier)
#		persistence	(message retention)
#		enableXA	(XA or non-XA QCF/TCF and coordination of messages)
#		CCSID		(code page )  - added YKL 02/02/2006
#		conPool		(connection pool settings) - added ktchu 10/7/2004
#		sessPool	(session pool settings) - added ktchu 10/7/2004
#		create		(Boolean, if true, create the QueueConnFactory, else
#				delete it)
#
#
# Returns:	1 (could not create)
# History:
# 2004/08/20	pty	Created
#--------------------------------------------------------------------
def manageRemoteMQQueueConnectionFactory ( providerName, scope, scopeName, name, jndiName, queueManager, host, port, channel, transportType, clientID, msgRetention, XAEnabled, CCSID, conPool, sessPool, create ):
        global AdminConfig

        traceEntry("manageMQQueueConnectionFactory" )

        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
        #endIf 
        connFac = findMQQueueConnectionFactoryWithScope(scope, scopeName, name )
        if (connFac != 0):
                if ((  not create ) ):
                        trace("info", "Deleting MQ Queue Connection Factory "+name )
                        AdminConfig.remove(connFac )
                else:
                        trace("info", "MQ Queue Connection Factory "+name+" already exists" )
                #endElse 

        else:
                if (( create ) ):
                        trace("info", "MQ Queue Connection Factory "+name+" not found, creating it now." )
                        print "%%%scope"+scope
                        jmsProvider = findJMSProviderWithScope(scope, scopeName, providerName )

                        nameAttr = ["name", name]
                        jndiAttr = ["jndiName", jndiName]
                        queueMgrAttr = ["queueManager", queueManager]
                        hostAttr = ["host", host]
                        portAttr = ["port", port]
                        channelAttr = ["channel", channel]                        
                        transTypeAttr = ["transportType", transportType]
                        clientIDAttr = ["clientID", clientID]
                        msgRetentionAttr = ["msgRetention", msgRetention]
                        XAAttr = ["XAEnabled", XAEnabled]
                        CCSID = ["CCSID", CCSID]
                        conPoolAttr = ["connectionPool", conPool]
                        sessPoolAttr = ["sessionPool", sessPool]
                        authAttr = ["authMechanismPreference","BASIC_PASSWORD"]
                        descAttr = ["description",""]
                        
                        mqConnFacAttrs = [nameAttr, jndiAttr, queueMgrAttr, hostAttr, portAttr, channelAttr, transTypeAttr, clientIDAttr, msgRetentionAttr, XAAttr, CCSID, conPoolAttr, sessPoolAttr, authAttr , descAttr ]

                        template = AdminConfig.listTemplates("MQQueueConnectionFactory" ).split(java.lang.System.getProperty("line.separator"))[0]
                        #?PROBLEM? (jacl 1170) previous LINDEX may need variable.split("xx")
                        print "jmsProvider=" + str(jmsProvider)
                        print "mqConnFacAttrs=" + str(mqConnFacAttrs)
                        print "template=" + str(template)

                        qmfactory=AdminConfig.createUsingTemplate("MQQueueConnectionFactory", jmsProvider, mqConnFacAttrs, template )
                        mappingAttrs=[["mappingConfigAlias","DefaultPrincipalMapping"],["authDataAlias",""]]
                        AdminConfig.create("MappingModule",qmfactory,mappingAttrs)
                        return 0
                else:
                        trace("info", "MQ Queue Connection Factory "+name+" doesn't exist" )
                #endElse 
                traceExit("manageMQQueueConnectionFactory" )
        #endElse 
#endDef 

#--------------------------------------------------------------------
# Procedure:	findMQQueueConnectionFactoryWithScope
# Description:  Finds the MQ Queue Connection Factory at the specified scope, if it exists.
# Parameters:
#		scope		(Cell, Node, Server)
#		scopeName	(Actual name of cell, node, or server)
#		name		(name of MQ Queue Connection Factory to find)
#
# Returns:	connFac (success), 0 (could not find)
# History:
# 2004/10/24	fwr	Created
#--------------------------------------------------------------------
def findMQQueueConnectionFactoryWithScope ( scope, scopeName, name ):
        connFac = findConfigTargetWithScope(scope, scopeName, name, "MQQueueConnectionFactory" )
        return connFac
#endDef 

#--------------------------------------------------------------------
# Procedure:   assignSharedLibToApplication
# Description: Assigns a Shared Library to a WebModule.
# Parameters:  	appName		Name of the enterprise application.
#		sharedLib	Name of the shared library
# Returns:	None.
#--------------------------------------------------------------------
def assignSharedLibToApplication ( appName, sharedLib ):
        global AdminApp 
        option = "[ -MapSharedLibForMod [[ " + appName + " " + "META-INF/application.xml" + " " + sharedLib + " ]]]"
        print "assignSharedLibToApplication option = " + option
	AdminApp.edit( appName , option )
#endDef

#--------------------------------------------------------------------
# Procedure:   assignSharedLibToWebModule
# Description: Assigns a Shared Library to a WebModule.
# Parameters:  	appName		Name of the enterprise application.
#		moduleName	Name of the Web Module.
#		warfile		Name of the warfile.
#		descriptor	Name of descriptor.
#		sharedLib	Name of the shared library
# Returns:	None.
#--------------------------------------------------------------------
def assignSharedLibToWebModule ( appName, moduleName, warfile, descriptor, sharedLib ):
        global AdminApp
        warfiledescriptor =  warfile + "," + descriptor 
        option = "[ -MapSharedLibForMod [[ " + moduleName + " " + warfiledescriptor + " " + sharedLib + " ]]]"
        print "assignSharedLibToWebModule option = " + option
	AdminApp.edit( appName , option )
#endDef 

#--------------------------------------------------------------------
# Procedure:   assignClassloaderToWebModule
# Description: Assigns a classloader to a WebModule.
# Parameters:  	appName		Name of the enterprise application.
#		warfile		Name of the warfile.
#		mode		classloader mode. ( PARENT_LAST or PARENT_FIRST )
# Returns:	None.
#--------------------------------------------------------------------
def assignClassloaderToWebModule ( appName, warfile, mode ):
        global AdminConfig
        deployments  = AdminConfig.getid("/Deployment:"+appName+"/" )
        depObject = AdminConfig.showAttribute(deployments, "deployedObject" )
        modules = AdminConfig.showAttribute(depObject, "modules")
        arrayModules = modules[1:len(modules)-1].split(" ")
        for module in arrayModules:
        	uri = AdminConfig.showAttribute(module, "uri")        	
        	if (cmp(uri, warfile) == 0):
        		print "WebModule " + warfile + " found :" + module
        		currentloaderMode = AdminConfig.showAttribute(module, "classloaderMode")
        		print "Current classloaderMode = " + currentloaderMode
        		AdminConfig.modify(module, [["classloaderMode", mode]])
        		print "WebModule classloaderMode set to " + mode
        		break

#endDef 


#--------------------------------------------------------------------
# Procedure:   manageSIBus
# Description: Creates/Deletes SI bus
#		If the bus already exists the bus will not be replaced
#
# Parameters:
#		busName	(Name of Bus)
#		busDescription	(Description of bus) 
#		busSecurity	(User ID)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	
#--------------------------------------------------------------------
def manageSIBus(busName, busDescription, busSecurity, create):
    traceEntry("manageSIBus" )

    cell = findCell(getCellName())
    attrs = [["description", busDescription], ["secure", busSecurity]]
    bus = findObjects("SIBus", cell, "name", busName)
    if (bus):      
        if (create):
            AdminConfig.modify(bus, attrs)
            writemsg("Updated SIBus " + busName)
        else:
            AdminConfig.remove(bus)
            writemsg("Deleted SIBus " + busName)
        #endIf
    else:                     
        if (create):
            attrs.append(["name", busName])
            bus = AdminConfig.create("SIBus", cell, attrs)
            writemsg("Created SIBus " + busName)
        else:
            writemsg("SIBus " + busName + " doesn't exist")
        #endif         
    #endIf
    
    traceExit("manageSIBus")
    
    return bus
#endDef

#--------------------------------------------------------------------
# Procedure:   ensureSIBusExistence
# Description: If no specified SIBus, throw error
#
# Parameters:
#		busName	(Name of Bus)
# Returns:	
#--------------------------------------------------------------------
def ensureSIBusExistence(busName):
    cell = findCell(getCellName())
    bus = findObjects("SIBus", cell, "name", busName)
    if (not bus):
        raise ValueError, "No such SIBus " + busName
    #endIf
#endDef

#--------------------------------------------------------------------
# Procedure:   manageSIBusMember
# Description: Creates/Deletes SI bus member
#		If the bus member already exists the bus member will not be replaced
#
# Parameters:
#		busName	(Name of Bus)
#       clusterName (Name of cluster)
#       nodeName	(Name of node)
#		severName	(Name of server) 
#		create		(Boolean if true, create entry, else delete it)
# Returns:	busmember
#--------------------------------------------------------------------
def manageSIBusMember(busName, clusterName, nodeName, serverName, datasourceJndiName, authAlias, createTables, schemaName, create):
    traceEntry("manageSIBusMember" )
    
    # Check if bus member already exists    
    cell = findCell(getCellName())
    bus = findObjects("SIBus", cell, "name", busName)
    busmember = None
    if (clusterName != "" and clusterName != None):
        if (bus):
            busmember = findObjects("SIBusMember", bus, "cluster", clusterName)
        #endIf
        memberName = clusterName
        # not applicable attrs = [["bus", busName], ["cluster", clusterName]]
        attrs = "-bus %s -cluster %s" % (busName, clusterName)
    else:
        if (bus):
            busmember = findObjects("SIBusMember", bus, "node", nodeName, "server", serverName)
        #endIf
        memberName = nodeName + ":" + serverName
        # not applicable attrs = [["bus", busName], ["node", nodeName], ["server", serverName]]
        attrs = "-bus %s -node %s -server %s" % (busName, nodeName, serverName)
    #endIf
    
    if (busmember):
        if (create):
            writemsg("SI bus " + busName + "'s member " + memberName + " already exists")
        else:
            attrs = "[" + attrs + "]"
            AdminTask.removeSIBusMember(attrs)
            writemsg("Deleted member " + memberName + " from SI bus " + busName)
        #endElse 
    else:
        if (create):
            ensureSIBusExistence(busName)
            attrs = "[" + attrs + " -datasourceJndiName %s -authAlias %s -createTables %s -schemaName %s]" % (datasourceJndiName, authAlias, createTables, schemaName)
            AdminTask.addSIBusMember(attrs)
            writemsg("Added member " + memberName + " to SI bus " + busName)
        else:
            writemsg("SI bus " + busName + "'s member " + memberName + " doesn't exist" )
	    #endElse
    #endElse 
    
    traceExit("manageSIBusMember" )
    
    return busmember
#endDef

#--------------------------------------------------------------------
# Procedure:   manageSIBusDestination
# Description: Creates/Deletes SI bus destination
#		If the bus destination already exists the bus destination will not be replaced
#
# Parameters:
#		destName	(Name of destination)
#		busName		(Name of Bus)
#		destinationType	( Queue, TopicSpace, Alias, Foreign )
#       description
#       reliability
#		create		(Boolean if true, create entry, else delete it)
# Returns:	busDestination
#--------------------------------------------------------------------
def manageSIBusDestination(destName, busName, destinationType, description, cluster, reliability, create):
    traceEntry("manageSIBusDestination" )
        
    # Check if bus destination already exists. Destination name (identifier) is unique in a bus.
    cell = findCell(getCellName())
    bus = findObjects("SIBus", cell, "name", busName)
    busDestination = None
    if (bus):
        busDestination = findObjects("SIBDestination", bus, "identifier", destName)
    #endIf
    if (busDestination):
        if (create):
            writemsg("Destination " + destName + " of SI Bus " + busName + " already exists")
        else:
            # Can not use AdminConfig.remove(busDestination), otherwise the queue points and other related resources are not deleted.
            AdminTask.deleteSIBDestination("-bus " + busName + " -name " + destName)
            writemsg("Deleted destination " + destName + " from SI Bus " + busName)
        #endIf 
    else:
        if (create):
            ensureSIBusExistence(busName)
            attrs = "[-bus %s -name %s -type %s -description %s -cluster %s -reliability %s]" % (busName, destName, destinationType, description, cluster, reliability)
            AdminTask.createSIBDestination(attrs)
            writemsg("Created destination " + destName + " on SI Bus " + busName)
        else:
            writemsg("Destination " + destName + " of SI Bus " + busName + " doesn't exist")
        #endIf 
    #endElse 
    
    traceExit("manageSIBusDestination" )

    return busDestination
#endDef


#--------------------------------------------------------------------
# Procedure:   findSIBus
# Description:
# Parameters:
#               busName	(Name of bus)
#		
# Returns:      targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------

def findSIBus ( busName):
        global AdminConfig
        siBusEntries = AdminConfig.list("SIBus" ).split(java.lang.System.getProperty("line.separator"))
        for siBusEntry in siBusEntries :
        	if ( len(siBusEntry)>0 ): 
	                busNameEntry = AdminConfig.showAttribute(siBusEntry,"name")
	                if (busName == busNameEntry ):
	                	return siBusEntry
	                        break 
	        	#endIf
		#endIf     
        #endFor 
        return 0
#endDef

#--------------------------------------------------------------------
# Procedure:   findJMSConnectionFactory
# Description:  Invoke AdminTask.listSIBJMSConnectionFactories(scope, "-type ...")
#    You can also use AdminConfig.list("J2CConnectionFactory", scope). Filter the result with provider name "SIB JMS Resource Adapter" and jndiName. But this is relatively more complex.
#
#    All following methods return ""
#        AdminConfig.list("JMSConnectionFactory", scope)
#        AdminConfig.list("SIBJMSConnectionFactory", scope)
#        AdminConfig.list("GenericJMSConnectionFactory", scope)
#        AdminConfig.list("SIBJMSQueueConnectionFactory", scope)
#        AdminConfig.list("WASQueueConnectionFactory", scope)
#        AdminConfig.list("SIBJMSTopicConnectionFactory", scope)
#        AdminConfig.list("WASTopicConnectionFactory", scope)
# Parameters:
#		scope 
#		jndiName Any of the JMS resources has unique JNDI name, no matter what scope it belongs to.
# Returns:   
#--------------------------------------------------------------------
def findJMSConnectionFactory(scope, jndiName):
    cfEntries = AdminTask.listSIBJMSConnectionFactories(scope, "-type all").split(lineSeparator) 
    for cfEntry in cfEntries:
        if (cfEntry==""):
                continue
        elif (AdminConfig.showAttribute(cfEntry, "jndiName") == jndiName):
                return cfEntry
	      #endIf     
    #endFor
    
    return None
#endDef

#--------------------------------------------------------------------
# Procedure:   manageJMSConnectionFactory
# Description: Creates/Deletes JMS connection factory
#		If the connection factory already exists the connection factory will not be replaced
#
# Parameters:
#		cfName			(Name of connection factory)
#		jndiName		(JNDI name of CF)
#		busName	(Name of Bus)
#		cfType	(Type of CF) 
#		targetType	(target type)
#		authDataAlias	(authentication alias)
#		xaAuthAlias	(xa authentication alias)
#       endpoints
#       nonPersistentMapping
#       persistentMapping
#       readAhead
#       targetSignificance
#       connectionProximity
#       shareDataSourceWithCMP
#		create		(Boolean if true, create entry, else delete it)
# Returns:	connectionFactory
#--------------------------------------------------------------------
def manageJMSConnectionFactory(targetContPath, cfName, jndiName, busName, cfType, targetType, authDataAlias, xaAuthAlias, providerEndPoints, nonPersistentMapping, persistentMapping, readAhead, targetSignificance, connectionProximity, shareDataSourceWithCMP, create):
    traceEntry("manageJMSConnectionFactory" )
		
    # Check if connection factory already exists. JNDI name is unique for all JMS resources.
    scope = AdminConfig.getid(targetContPath)
    connectionFactory = findJMSConnectionFactory(scope, jndiName)
    
    if (connectionFactory):
        if (create):
            if (providerEndPoints == None):
                providerEndPoints = ""
            #endIf
            attrs = "[-name %s -jndiName %s -busName %s -targetType %s -authDataAlias %s -xaRecoveryAuthAlias %s -providerEndPoints %s -nonPersistentMapping %s -persistentMapping %s -readAhead %s -targetSignificance %s -connectionProximity %s -shareDataSourceWithCMP %s]" % (cfName, jndiName, busName, targetType, authDataAlias, xaAuthAlias, providerEndPoints, nonPersistentMapping, persistentMapping, readAhead, targetSignificance, connectionProximity, shareDataSourceWithCMP)
            AdminTask.modifySIBJMSConnectionFactory(connectionFactory, attrs)
            writemsg("Updated JMS connection factory " + cfName)
        else:
            AdminTask.deleteSIBJMSConnectionFactory(connectionFactory)
            writemsg("Deleted JMS connection factory " + cfName)
        #endIf 
    else:
        if (create):
            ensureSIBusExistence(busName)
            if (providerEndPoints == None):
                providerEndPoints = ""
            #endIf
            
            if ((cfType == "" )or(cfType == None)):
            	attrs = "[-name %s -jndiName %s -busName %s -targetType %s -authDataAlias %s -xaRecoveryAuthAlias %s -providerEndPoints %s -nonPersistentMapping %s -persistentMapping %s -readAhead %s -targetSignificance %s -connectionProximity %s -shareDataSourceWithCMP %s]" % (cfName, jndiName, busName, targetType, authDataAlias, xaAuthAlias, providerEndPoints, nonPersistentMapping, persistentMapping, readAhead, targetSignificance, connectionProximity, shareDataSourceWithCMP)
            #endIf
            else:
       	    	attrs = "[-name %s -jndiName %s -busName %s -type %s -targetType %s -authDataAlias %s -xaRecoveryAuthAlias %s -providerEndPoints %s -nonPersistentMapping %s -persistentMapping %s -readAhead %s -targetSignificance %s -connectionProximity %s -shareDataSourceWithCMP %s]" % (cfName, jndiName, busName, cfType, targetType, authDataAlias, xaAuthAlias, providerEndPoints, nonPersistentMapping, persistentMapping, readAhead, targetSignificance, connectionProximity, shareDataSourceWithCMP)
       	    #endElse
       	    print attrs
       	    connectionFactory = AdminTask.createSIBJMSConnectionFactory(scope, attrs)
       	    writemsg("Created JMS connection factory " + cfName)
        else:
            writemsg("JMS connection factory " + cfName + " doesn't exist")
        #endIf 
    #endElse 

    traceExit("manageJMSConnectionFactory" )
    return connectionFactory
#endDef


#--------------------------------------------------------------------
# Procedure:   findJMSQueue
# Description:   The configuration type of the JMS Queue is J2CAdminObject
# Parameters:
#		scope
#       jndiName  Any of the JMS resources has unique JNDI name, no matter what scope it belongs to.
# Returns:     jmsQueue
#--------------------------------------------------------------------
def findJMSQueue(scope, jndiName):
    jmsQueues = AdminTask.listSIBJMSQueues(scope).split(lineSeparator) 
    for jmsQueue in jmsQueues :
    	if (jmsQueue==""):
    		continue
	elif (AdminConfig.showAttribute(jmsQueue, "jndiName") == jndiName):
		return jmsQueue
        #endIf  
    #endFor 
    
    return None
#endDef

#--------------------------------------------------------------------
# Procedure:   manageJMSQueue
# Description: Creates/Deletes JMS queue
#		If the JMS queue already exists the JMS queue will not be replaced
#
# Parameters:
#       targetContPath
#		jmsQName			(Name of JMS queue)
#		jndiName		(JNDI name of JMS queue)
#		busName	(Name of Bus)
#		queueName	(Name of destination queue) 
#       deliveryMode
#       readAhead
#		create		(Boolean if true, create entry, else delete it)
# Returns:	jmsqueue
#--------------------------------------------------------------------
def manageJMSQueue(targetContPath, jmsQName, jndiName, busName, queueName, deliveryMode, readAhead, create):
    traceEntry("manageJMSQueue" )
        
    # Check if jms queue already exists
    scope = AdminConfig.getid(targetContPath)
    jmsqueue = findJMSQueue(scope, jndiName)

    if (jmsqueue):
        if (create):
            writemsg("JMS queue " + jmsQName + " already exists" )
        else:
            AdminTask.deleteSIBJMSQueue(jmsqueue)
            writemsg("Deleted JMS queue " + jmsQName)
        #endIf 
    else:
        if (create):
            ensureSIBusExistence(busName)
            attrs = "[-name %s -jndiName %s -queueName %s -busName %s -deliveryMode %s -readAhead %s]" % (jmsQName,jndiName,queueName,busName, deliveryMode, readAhead)
            jmsqueue = AdminTask.createSIBJMSQueue(scope, attrs)
            writemsg("Created JMS queue " + jmsQName)
        else:
            writemsg("JMS queue " + jmsQName + " doesn't exist" )
        #endIf 
    #endIf

    traceExit("manageJMSQueue" )
    return jmsqueue
#endDef

#--------------------------------------------------------------------
# Procedure:   findJMSTopic
# Description: The configuration type of the JMS Topic is J2CAdminObject
# Parameters:
#		scope 
#		jndiName Any of the JMS resources has unique JNDI name, no matter what scope it belongs to.
# Returns:      jmstopic 
#--------------------------------------------------------------------
def findJMSTopic (scope, jndiName):
    jmsTopics = AdminTask.listSIBJMSTopics(scope).split(lineSeparator) 
    for jmsTopic in jmsTopics :
    	if (jmsTopic==""):
            	continue
        elif (AdminConfig.showAttribute(jmsTopic, "jndiName") == jndiName):
          	return jmsTopic
        #endIf  
    #endFor 

    return None
#endDef

#--------------------------------------------------------------------
# Procedure:   manageJMSTopic
# Description: Creates/Deletes JMS Topic
#		If the JMS Topic already exists the JMS Topic will not be replaced
#
# Parameters:
#       targetContPath
#		name		(Name of JMS Topic)
#		jndiName	(JNDI name of JMS Topic)
#		busName		(Name of Bus)
#		topicName	(Name of destination topic) 
#       topicSpace 
#       deliveryMode
#       readAhead
#		create		(Boolean if true, create entry, else delete it)
# Returns:	jmstopic
#--------------------------------------------------------------------
def manageJMSTopic(targetContPath, name, jndiName, busName, topicName, topicSpace, deliveryMode, readAhead, create):
    traceEntry("manageJMSTopic" )

    # Check if jms topic already exists
    scope = AdminConfig.getid(targetContPath)
    jmstopic = findJMSTopic(scope, jndiName)

    if (jmstopic):
        if (create):
            writemsg("JMS topic " + name + " already exists" )
        else:
            jmstopic = AdminTask.deleteSIBJMSTopic(jmstopic)
            writemsg("Deleted JMS topic " + name)
        #endElse 
    else:
        if (create):
            ensureSIBusExistence(busName)
       	    attrs = "[-name %s -jndiName %s -topicName %s -topicSpace %s -busName %s -deliveryMode %s -readAhead %s]" % (name, jndiName, topicName, topicSpace, busName, deliveryMode, readAhead)
       	    jmstopic = AdminTask.createSIBJMSTopic(scope, attrs)
       	    writemsg("Created JMS topic " + name)
        else:
            writemsg("JMS topic " + name + " doesn't exist")
        #endIf 
    #endElse 

    traceExit("manageJMSTopic" )
    return jmstopic
#endDef

#--------------------------------------------------------------------
# Procedure:   manageJMSActivationSpec
# Description: Creates/Deletes JMS activation specification
#		If the JMS activation spec already exists the JMS activation spec will not be replaced
#
# Parameters:
#       targetContPath
#		jmsActSpecName	(Name of JMS activation specification)
#		jndiName	(JNDI name of JMS queueor topic)
#		busName		(Name of Bus)
#		jndiDest	(JNDI Name of destination queue) 
#		authAlias	(Authentication alias)
#		messageSelector (Message selector)
#		durability 	(durability )
#		subscriptionName(subscriptionname)
#		clientID 	(clientid)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	jmsactspec
#--------------------------------------------------------------------
def manageJMSActivationSpec(targetContPath, jmsActSpecName, jndiName, busName, jndiDest, destType, description, authAlias, messageSelector, durability, subscriptionName, clientID, acknowledgeMode, durableSubscriptionHome, readAhead, targetType, targetSignificance, shareDurableSubscriptions, shareDataSourceWithCMP, create):
    traceEntry("manageJMSActivationSpec" )
        
    # Check if jms activation specification already exists
    jmsactspec = findObjectsByTarget("J2CActivationSpec", targetContPath, "jndiName", jndiName)

    if (jmsactspec):
        if ( not create ):
			AdminTask.deleteSIBJMSActivationSpec(jmsactspec)
			writemsg("Deleted JMS activation spec " + jmsActSpecName)
        else:
            writemsg("JMS activation spec " + jmsActSpecName + " already exists" )
        #endElse 
    else:
        if (create):
            ensureSIBusExistence(busName)
            scope = AdminConfig.getid(targetContPath)
            if (messageSelector == None):
                messageSelector = ""
            #endIf
            attrs = "[-name %s -jndiName %s -destinationJndiName %s -busName %s -destinationType %s -description %s -messageSelector %s -acknowledgeMode %s -readAhead %s -targetType %s -targetSignificance %s -shareDurableSubscriptions %s -shareDataSourceWithCMP %s" % (jmsActSpecName, jndiName, jndiDest, busName, destType, description, messageSelector, acknowledgeMode, readAhead, targetType, targetSignificance, shareDurableSubscriptions, shareDataSourceWithCMP)
            if (authAlias != None):
                attrs += " -authenticationAlias %s" % (authAlias)
            #endIf
            if (durability.lower() == "durable") :
       	        attrs += " -subscriptionDurability Durable -subscriptionName %s -clientId %s -durableSubscriptionHome %s" % (subscriptionName, clientID, durableSubscriptionHome)
       	    #endIf	  
       	    attrs += "]"
            AdminTask.createSIBJMSActivationSpec(scope, attrs)
            writemsg("Created JMS activation spec " + jmsActSpecName)
        else:
            writemsg("JMS activation spec " + jmsActSpecName + " doesn't exist" )
        #endIf 
    #endIf

    traceExit("manageJMSActivationSpec" )
    return jmsactspec
#endDef

#--------------------------------------------------------------------
# Procedure:   manageMQActivationSpec
# Description: Creates/Deletes MQ activation specification
#		If the MQ activation spec already exists the MQ activation spec will not be replaced
#
# Parameters:
#		scope 		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		jmsActSpecName	(Name of JMS activation specification)
#		jndiName	(JNDI name of JMS queueor topic)
#		busName		(Name of Bus)
#		jndiDest	(JNDI Name of destination queue) 
#		authAlias	(Authentication alias)
#		messageSelector (Message selector)
#		durability 	(durability )
#		subscriptionName(subscriptionname)
#		clientID 	(clientid)
#		queueManager 	(queueManager)
#		hostName 	(hostName)
#		port	 	(port)
#		channel 	(channel)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageMQActivationSpec(scope, scopeName, mqActSpecName, jndiName, jndiDest, destType, authAlias, messageSelector, queueManager, hostName, port, channel, durability, subscriptionName, clientID, create):
        global AdminApp, AdminConfig
        
        traceEntry("manageMQActivationSpec" )
        
        scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
                
       	trace("info", "scopeEntry= " + scopeEntry)
                
        # Check if jms activation specification already exists
        IsActivationSpecExist = 0
        activationSpecs=AdminTask.listWMQActivationSpecs(scopeEntry).split(java.lang.System.getProperty("line.separator")) 
     	for activationSpec in activationSpecs :
    		trace("info", "activationSpec= " + activationSpec)
    		if( activationSpec != "" ) : 
        		nameEntry = AdminConfig.showAttribute(activationSpec,"name")
			trace("info", "nameEntry= " + nameEntry)
        		if (mqActSpecName==nameEntry):
        			IsActivationSpecExist = 1
	                	break
	                #endIf
	        #endIf	
	#endFor 
        #trace("info", "IsActivationSpecExist= " + IsActivationSpecExist)       
        if (IsActivationSpecExist != 0):
                if ( not create ):
                        trace("info", "Deleting JMS activation spec "+mqActSpecName+"  - scopeName:" + scopeName)
                        
			#remove  JMS activation spec
			AdminTask.deleteWMQActivationSpec(activationSpec)
                else:
                        trace("info", "JMS activation spec "+mqActSpecName+"  - scopeName:" + scopeName+" already exists" )
                #endElse 

        else:
 		if (create):
			#create MQ activation spec
			if ((durability == "Durable") or (durability == "durable") ) :
	        		attrs = "[-name %s -jndiName %s -destinationJndiName %s -destinationType %s  -messageSelector %s -qmgrName %s -qmgrHostname %s -qmgrPortNumber %s -qmgrSvrconnChannel %s -subscriptionDurability Durable -subscriptionName %s -clientId %s -durableSubscriptionHome %s]" % (mqActSpecName,jndiName,jndiDest,destType,messageSelector,queueManager,hostName,port,channel,durability, subscriptionName, clientID)        				
        		else :
	        		attrs = "[-name %s -jndiName %s -destinationJndiName %s -destinationType %s  -messageSelector %s -qmgrName %s -qmgrHostname %s -qmgrPortNumber %s -qmgrSvrconnChannel %s ]" % (mqActSpecName,jndiName,jndiDest,destType,messageSelector,queueManager,hostName,port,channel)        				
	        	trace("info", "attribute= " + attrs )
        		activationSpec = AdminTask.createWMQActivationSpec(scopeEntry,attrs)
		#endIf 
		else:
	        	trace("info", "JMS activation spec "+mqActSpecName+"  - scopeName:" + scopeName +" doesn't exist" )
	        #endElse 

        #endElse
        trace("info", "activationSpec= " + activationSpec)
        traceExit("manageMQActivationSpec" )
        return activationSpec
#endDef

#--------------------------------------------------------------------
# Procedure:   manageStartupBeanService
# Description: Enable/Disable Startup bean services

# Parameters:
#		serverName 			(Server Name )
# Returns:	targetName (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageStartupBeanService(serverName, create):
        global AdminApp, AdminConfig
        
        traceEntry("manageStartupBeanService" )
        
        
        server = findServer(serverName)
	startupbean = 0;	
        if (server != 0):
        	startupbean = AdminConfig.list('StartupBeansService', server)
                if ( not create ):
                        trace("info", "Disabling startup bean services on server "+serverName)
			#disable  
			AdminConfig.modify(startupbean, [['enable', 'false']])
                else:
                        trace("info", "Enabling startup bean services on server "+serverName)
                        #eanble
                        AdminConfig.modify(startupbean, [['enable', 'true']])
                #endElse 

        else:
	       	trace("info", "Server "+serverName+" doesn't exist" )
	       	#endElse
        #endElse 
        traceExit("manageStartupBeanService" )
        return startupbean
#endDef

#--------------------------------------------------------------------
# Procedure:   manageObjectCacheInstance
# Description: Creates/Deletes Object cache instance
# Parameters:
#       targetContPath
#		objCacheName			(Name of Object cache instance)
#		jndiName		(JNDI name of Object cache instance)
#		cacheSize	(Cache size)
#		defaultPriority	(Default priority) 
#		disableDependencyId	(Disable dipendency ID support)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	objectCacheInstance
#--------------------------------------------------------------------
def manageObjectCacheInstance(targetContPath,objCacheName,jndiName,cacheSize,defaultPriority,category,enableDiskOffload,flushToDiskOnStop,diskCacheSizeInGB,diskCacheEntrySizeInMB,diskCacheCleanupFrequency,diskCacheSizeInEntries,pushFrequency,disableDependencyId,replicationType,enableCacheReplication,replicationDomain, create):
    traceEntry("manageObjectCacheInstance" )

    # There is one and only one CacheProvider in any target (cell, node, cluster or server). 
    cacheProvider = findObjectsByTarget("CacheProvider", targetContPath, "name", "CacheProvider");
    
    # Check if Object cache instance already exists
    objectCacheInstance = findObjects("ObjectCacheInstance", cacheProvider, "name", objCacheName)

    attrs = [["jndiName", jndiName],["cacheSize", cacheSize],["category", category],["enableDiskOffload", enableDiskOffload],["flushToDiskOnStop", flushToDiskOnStop],["diskCacheSizeInGB", diskCacheSizeInGB],["diskCacheEntrySizeInMB", diskCacheEntrySizeInMB],["diskCacheCleanupFrequency", diskCacheCleanupFrequency],["diskCacheSizeInEntries", diskCacheSizeInEntries],["pushFrequency", pushFrequency],["defaultPriority", defaultPriority],["disableDependencyId", disableDependencyId],["replicationType", replicationType],["enableCacheReplication", enableCacheReplication]]
    if (objectCacheInstance):
        if (create):
            AdminConfig.modify(objectCacheInstance, attrs)
            drsSettings=AdminConfig.list("DRSSettings", objectCacheInstance)
            attrs4 = [["messageBrokerDomainName", replicationDomain]]
            AdminConfig.modify(drsSettings, attrs4)
            writemsg("Updated object cache instance" + objCacheName)
        else:
            AdminConfig.remove(objectCacheInstance)
            writemsg("Deleted object cache instance " + objCacheName)
        #endIf 
    else:
        if (create):
            attrs.append(["name", objCacheName])
            objectCacheInstance = AdminConfig.create("ObjectCacheInstance", cacheProvider, attrs)
            attrs1 = [["algorithm", "NONE"],["lowThreshold", 70],["highThreshold", 80]]
            diskPolicy=AdminConfig.create("DiskCacheEvictionPolicy", objectCacheInstance, attrs1)
            attrs2 = [["messageBrokerDomainName", replicationDomain]]
            drsSettings=AdminConfig.create("DRSSettings", objectCacheInstance, [])
            if( (replicationDomain!=None) or (replicationDomain!="")):
            	AdminConfig.modify(drsSettings, attrs2)
            #endIf
            attrs3 = [["maxBufferedTemplates", 100],["maxBufferedDependencyIds", 10000],["maxBufferedCacheIdsPerMetaEntry", 1000]]
            performanceSettings=AdminConfig.create("DiskCacheCustomPerformanceSettings", objectCacheInstance, attrs3)
        else:
            writemsg("Object cache instance " + objCacheName + " doesn't exist" )
      	#endIf 
    #endIf 

    traceExit("manageObjectCacheResource" )
    return objectCacheInstance
#endDef


#--------------------------------------------------------------------
# Procedure:   manageResourceEnvironmentProvider
# Description: Creates/Deletes Resource Environment Provider
#
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#		providerName		(Name of provider Name)
#		description		(description)
#		factoryClassname	(factoryClassname)
#		classname		(classname)
#		create			(Boolean if true, create entry, else delete it)
# Returns:	Provider (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageResourceEnvironmentProvider ( scope, scopeName, providerName, description, factoryClassname, classname, create ):
	global AdminApp, AdminConfig
        traceEntry("manageResourceEnvironmentProvider" )
	scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
        reProvider = findConfigTargetWithScope(scope, scopeName, providerName, "ResourceEnvironmentProvider" )
	if (reProvider != 0):
		if ( not create ):
                        trace("info", "Deleting Resource Environment Provider "+ providerName +" - scope:" + scope)
			#remove resource provider
			AdminConfig.remove(reProvider)
                else:
                        trace("info", "Resource Environment Provider "+ providerName +" - scope:" + scope +" already exists" )
                #endElse 

        else:
                if (create):
			#create resource provider
			attributes = [["name", providerName],['description', description]]
			reProvider = AdminConfig.create("ResourceEnvironmentProvider", scopeEntry, attributes) 
     			if( reProvider != 0): 
     				attributes = [['factoryClassname', factoryClassname],['classname', classname]]
     				AdminConfig.create("Referenceable", reProvider, attributes)				
                #endIf 
                else:
                        trace("info", "Resource Environment Provider "+ providerName +" - scope:" + scope+" doesn't exist" )
                #endElse 
        #endElse 
        traceExit("manageResourceEnvironmentProvider" )	
	return reProvider	
#endDef


#--------------------------------------------------------------------
# Procedure:   manageResourceEnvironmentProviderProperty
# Description: Creates/Deletes Resource Environment Provider Property
#
# Parameters:
#		scope 			(Cell or Node)
#		scopeName		(cellName or nodeName)
#		providerName		(Name of provider Name)
#		name			(name)
#		value			(value)
#		create			(Boolean if true, create entry, else delete it)
# Returns:	Provider (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageResourceEnvironmentProviderProperty ( scope, scopeName, providerName, propName, propValue, create ):
	global AdminApp, AdminConfig
        traceEntry("manageResourceEnvironmentProviderProperty" )
	scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
        reProvider = findConfigTargetWithScope(scope, scopeName, providerName, "ResourceEnvironmentProvider" )
	propSet = AdminConfig.showAttribute(reProvider, 'propertySet') 
	attributes = [['name', propName],['value', propValue]]
	if (create): 
		if(propSet == None or propSet == ""):
	               	trace("info", "Create Resource Environment Provider propertySet and property"+ propName +" - scope:" + scope)
	        	propSet = AdminConfig.create('J2EEResourcePropertySet',reProvider,[])
			AdminConfig.create('J2EEResourceProperty', propSet, attributes)
		else :
			
			for ob in AdminConfig.list('J2EEResourceProperty',propSet).split(java.lang.System.getProperty("line.separator")) :
				name = AdminConfig.showAttribute(ob, "name")
	        		if (name == propName):
	                		trace("info", "Modify Resource Environment Provider Property "+ propName +" - scope:" + scope)
					AdminConfig.modify(ob, attributes)
					return
		
                	trace("info", "Create Resource Environment Provider Property "+ propName +" - scope:" + scope)
			AdminConfig.create('J2EEResourceProperty', propSet, attributes)
			return
					
				
	else :
		for ob in AdminConfig.list('J2EEResourceProperty',propSet).split(java.lang.System.getProperty("line.separator")) :
			name = AdminConfig.showAttribute(ob, "name")
	        	if (name == propName):
	                	trace("info", "Delete Resource Environment Provider Property"+ propName +" - scope:" + scope)
				AdminConfig.remove(ob)
        #endElse 
        traceExit("manageResourceEnvironmentProviderProperty" )	
	return 0	
#endDef

#--------------------------------------------------------------------
# Procedure:   manageResourceEnvironmentEntry
# Description: Creates/Deletes Resource Environment Provider Entry
#
# Parameters:
#		scope 			(Cell or Node or Server)
#		scopeName		(cellName or nodeName)
#		entryName		(Name of Entry Name)
#		jndiName		(jndi name)
#		providerName		(factoryClassname)
#		referenceId		(reference id)
#		create			(Boolean if true, create entry, else delete it)
# Returns:	Environment Entry (success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageResourceEnvironmentEntry ( scope, scopeName, entryName, jndiName, providerName, referenceId, create ):
	global AdminApp, AdminConfig
        traceEntry("manageResourceEnvironmentEntry" )
	scopeEntry = findScopeEntry(scope, scopeName )
        if (scopeEntry == 0):
                return 1
        reProvider = findConfigTargetWithScope(scope, scopeName, providerName, "ResourceEnvironmentProvider" )
    	for ob in AdminConfig.list('Referenceable',reProvider).split(java.lang.System.getProperty("line.separator")) :
        	name = AdminConfig.showAttribute(ob, "factoryClassname")
        	if (name == referenceId):
 	           	refid=ob
		else :
			return 0
	if (create): 
		envEntry= findConfigTargetWithScope(scope, scopeName, entryName, "ResourceEnvEntry" )
		if ( envEntry != 0 ) :
			trace("info", "Resource Environment entry "+ entryName +" already existed")		
		else :
			trace("info", "Create Resource Environment entry "+ entryName +" - scope:" + scope)
			attributes = [['name', entryName],['jndiName', jndiName], ['referenceable', refid]]
			envEntry=AdminConfig.create('ResourceEnvEntry', reProvider, attributes)
                	trace("info", "create EnvironmentEntry J2EEResourceProperty" )
               		propSet = AdminConfig.create("J2EEResourcePropertySet", envEntry, [] )
			
	else :
		trace("info", "Delete Resource Environment entry"+ entryName +" - scope:" + scope)
		envEntry=findConfigTargetWithScope(scope, scopeName, entryName, "ResourceEnvEntry" )
		AdminConfig.remove(envEntry)
        #endElse 
        traceExit("manageResourceEnvironmentEntry" )	
	return envEntry	
#endDef

#--------------------------------------------------------------------
# Procedure:   addCustomPropertyToEnvironmentEntryWithScope
# Description: 
# Parameters:
#		scope		(Cell or Node)
#		scopeName	(cellName or nodeName)
#		entryName	(DataSource name)
#		custName	(Custom Property Name)
#		custValue	(Custom Property Value)
# return :
# 		None
#--------------------------------------------------------------------
def addCustomPropertyToEnvironmentEntryWithScope ( scope, scopeName, entryName, custName, custValue ):
        global AdminConfig
        traceEntry ("addCustomPropertyToEnvironmentEntryWithScope")
        envEntry= findConfigTargetWithScope(scope, scopeName, entryName, "ResourceEnvEntry" )
        propSet = AdminConfig.showAttribute(envEntry, "propertySet" )
        name = ["name", custName]
        value = ["value", custValue]
        attr = [name, value]
        AdminConfig.create("J2EEResourceProperty", propSet, attr )
        traceExit ("addCustomPropertyToEnvironmentEntryWithScope") 
#endDef 


#--------------------------------------------------------------------
# Procedure:   manageWorkManager
# Description: Creates/Deletes Work manager
# Parameters:
#       targetContPath
#		workManagerName			(Name of work manager)
#		jndiName		(JNDI name of work manager)
#       description
#       workTimeout
#       workReqQFullAction
#       isGrowable
#       maxThreads
#       minThreads
#       numAlarmThreads
#       serviceNames
#       threadPriority
#       category
#		create		(Boolean if true, create entry, else delete it)
# Returns:	workManager
#--------------------------------------------------------------------
def manageWorkManager(targetContPath, workManagerName, jndiName, description, workTimeout, workReqQFullAction, isGrowable, maxThreads, minThreads, numAlarmThreads, serviceNames, threadPriority, category, create):
    traceEntry("manageWorkManager" )

    # There is one and only one work manager provider in any target (cell, node, cluster or server).
    workManagerProvider = findObjectsByTarget("WorkManagerProvider", targetContPath, "name", "WorkManagerProvider")

    # Check if work manager already exists
    workManager = findObjects("WorkManagerInfo", workManagerProvider, "name", workManagerName)
    attrs = [["jndiName", jndiName], ["description", description], ["workTimeout", workTimeout], ["workReqQFullAction", workReqQFullAction], ["isGrowable", isGrowable], ["maxThreads", maxThreads], ["minThreads", minThreads], ["numAlarmThreads", numAlarmThreads], ["serviceNames", serviceNames], ["threadPriority", threadPriority], ["category", category]]
    if (workManager):
        if (create):
            AdminConfig.modify(workManager, attrs)
            writemsg("Updated work manager " + workManagerName)
        else:
            AdminConfig.remove(workManager)
            writemsg("Deleted work manager " + workManagerName)
        #endIf 
    else:
        if (create):
            attrs.append(["name", workManagerName])
            workManager = AdminConfig.create("WorkManagerInfo", workManagerProvider, attrs)
            writemsg("Created work manager " + workManagerName) 
        else:
            writemsg("Work manager " + workManagerName + " doesn't exist" )
        #endIf 
    #endIf 

    traceExit("manageWorkManager" )
    return workManager
#endDef

#--------------------------------------------------------------------
# Procedure:   manageServerProperty
# Description: Creates/Deletes server property
#
# Parameters:
#		serverName		(cellName or nodeName)
#		serverpropName		(name of property)
#		value			(value of property)
#		description		(descrption of property)
#		required		(true or false) 
#		create			(Boolean if true, create entry, else delete it)
# Returns:	serverpropName 		(success) 
#		0 (does not exist)
#--------------------------------------------------------------------
def manageServerProperty(serverName,serverpropName,value,description,required,create):
        global AdminTask, AdminConfig      
        traceEntry("manageServerProperty" )
        for server in AdminTask.listServers('[-serverType APPLICATION_SERVER ]').split(java.lang.System.getProperty("line.separator")) :
        	trace("info", "server = " + server )
        	sname = AdminConfig.showAttribute(server, "name")
	        if (serverName == sname):
	        	appServer=AdminConfig.list("ApplicationServer", server ) 
			nameAttr = ["name", serverpropName]
	 		valueAttr = ["value", value]
	       		descAttr = ["description",description]
			requiredAttr = ["required", required]              
			attrs = [nameAttr, valueAttr, descAttr, requiredAttr]
	        	found = 0
	        	for serverProperty in AdminConfig.list('Property', appServer).split(java.lang.System.getProperty("line.separator")) :
	                	if(serverProperty == None or serverProperty == ""):
	                		trace("info", "No server property")
	                		break
	                	else :
		                	name = AdminConfig.showAttribute(serverProperty, "name")
		                	if ( serverpropName == name ):
		                		trace("info", "property name" + name + " existed")
		                		found = 1
		                		if( create ) :
		                			trace("info", "Modify property =" + name )
		                			AdminConfig.modify(serverProperty, attrs)
		                		else : 
		                			trace("info", "Delete property =" + name )
		                			AdminConfig.remove(serverProperty)
	                #endFor
	                if (found == 0):
	                	if( create ) :
	                		trace("info", "Create property =" + serverpropName )
	                		AdminConfig.create('Property', appServer, attrs )
	                		
	                	else:		
	 				trace("info", "property =" + name + " not found, No need to delete.")
	        #endIf
	#endFor
        traceExit("manageServerProperty" )
        return serverpropName
#endDef

#--------------------------------------------------------------------
# Procedure:   manageEventServer
# Description: 
# Parameters:
#		targetContPath	
#       name
#       jndiName
#       enableDataStore
#		create		
# Returns:	N/A
#--------------------------------------------------------------------
def manageEventServer(targetContPath, name, jndiName, enableDataStore, create):
    traceEntry("manageEventServer")

    eventServer = findObjectsByTarget("EventServerProfile", targetContPath, "jndiName", jndiName)
    attrs = [["enableDataStore", enableDataStore]]
    if (eventServer):      
        if (create):
            AdminConfig.modify(eventServer, attrs)
            writemsg("Updated Event Server " + name)
        #endIf
    #endIf
     
    traceExit("manageEventServer")
#endDef

#--------------------------------------------------------------------
# Procedure:   manageEventsGroup
# Description: Creates/Deletes an Event Group
# Parameters:
#		targetContPath	containment path which identifies the target in/from which to create/delete the WAS resource
#		eventgroupname		name of the eventgroup
#       eventSelectorString
#		persistEvents	
#		eventGroupProfileId
#		DistributeQ
#		CompatibilityMode_attrs
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	N/A
#--------------------------------------------------------------------
def manageEventsGroup(targetContPath,eventgroupname, eventSelectorString, persistEvents,eventGroupProfileId,DistributeQ,CompatibilityMode_attrs,create):
    
    eventsgroup = findObjectsByTarget("EventGroupProfile", targetContPath, "eventGroupName", eventgroupname)
    
    distributeQueues = ['distributionQueues',DistributeQ]
    
    resourceProperties = []
    resourceProperties.append(['resourceProperties',CompatibilityMode_attrs])
    customProperties = ['propertySet', resourceProperties]
    
    properties_attrs=[['eventGroupName',eventgroupname],['eventSelectorString',eventSelectorString],['persistEvents',persistEvents],distributeQueues,customProperties]
     
    if (eventsgroup):      # Existing eventsgroup found. Either update it or delete it according to the 'create' argument value.
        if (create):
            AdminConfig.modify(eventsgroup, properties_attrs)
            writemsg("Updated event group " + eventgroupname)
        else:
            AdminConfig.remove(eventsgroup)
            writemsg("Deleted event group " + eventgroupname)
        #endIf
    else:                     # Existing eventsgroup not found. Create it if requested, otherwise warn that it wasn't found.
        if (create):
            AdminConfig.create('EventGroupProfile', eventGroupProfileId, properties_attrs)
            writemsg("Created event group " + eventgroupname)
        else:
            writemsg("event group " + eventgroupname + " doesn't exist")
        #endif         
    #endIf
#endDef

#--------------------------------------------------------------------
# Procedure:   manageAsyncEmitter
# Description: Creates/Deletes an AsyncEmitter
# Parameters:
#		targetContPath	containment path which identifies the target in/from which to create/delete the WAS resource
#		asyncEmitterName		name of the AsyncEmitter
#       jndiName
#		description	
#		category	
#       transactionMode	
#		synchronizationMode
#		asyncJNDIName
#		syncJNDIName
#		filteringEnabled
#		filterJNDIName
#		providerid
#		CompatibilityMode_attrs
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	N/A
#--------------------------------------------------------------------
def manageAsyncEmitter(targetContPath,asyncEmitterName, jndiName,description,category,transactionMode,synchronizationMode,asyncJNDIName,syncJNDIName,filteringEnabled, filterJNDIName,providerid, CompatibilityMode_attrs,create):
    
    asyncEmitter = findObjectsByTarget("EmitterFactoryProfile", targetContPath, "jndiName", jndiName)
    print asyncEmitter
    customProperties = ['propertySet', [['resourceProperties', [CompatibilityMode_attrs]]]]
    properties_attrs = [['name',asyncEmitterName],['jndiName', jndiName],['description', description],['category', category],['preferredTransactionMode', transactionMode],['preferredSynchronizationMode', synchronizationMode],['asynchronousTransmissionProfileJNDIName', asyncJNDIName],['synchronousTransmissionProfileJNDIName', syncJNDIName],['filteringEnabled', filteringEnabled],['filterFactoryJNDIName', filterJNDIName],customProperties]
    
    if (asyncEmitter):      # Existing asyncEmitter found. Either update it or delete it according to the 'create' argument value.
        if (create):
            AdminConfig.modify(asyncEmitter, properties_attrs)
            writemsg("Updated event group " + asyncEmitterName)
        else:
            AdminConfig.remove(asyncEmitter)
            writemsg("Deleted event group " + asyncEmitterName)
            
        #endIf
    else:                     # Existing asyncEmitter not found. Create it if requested, otherwise warn that it wasn't found.
        if (create):
            AdminConfig.create('EmitterFactoryProfile', providerid, properties_attrs)
            writemsg("Created event group " + asyncEmitterName)
        else:
            writemsg("event group " + asyncEmitterName + " doesn't exist")
        #endif         
    #endIf
    
#endDef
#--------------------------------------------------------------------
# Procedure:   manageApplication
# Description: Creates/Deletes an application
# Parameters:
#		targetContPath	containment path which identifies the target in/from which to create/delete the WAS resource
#		appName		name of the application
#       earFile
#		opps	
#		mapModules_attrs	
#		mapVHs_attrs
#		mapCtxRoot_attrs
#		app_cluster
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	N/A
#--------------------------------------------------------------------
def manageApplication(targetContPath,appName, earFile, opps, mapModules_attrs, mapVHs_attrs,mapCtxRoot_attrs, mapEjbRef_attrs, mapRscRef_attrs, mapEnvEntry_attrs ,app_cluster, create):
    global AdminApp
    
    appNametrim=appName.replace('"',"") # Remove special character " from the application name
    
    listOfApps=AdminApp.list().split('\r\n')
    exist = appNametrim in listOfApps
    
    if (mapModules_attrs == ""):	# if the application has map modules
        if (opps == ""):
            if (app_cluster == None):
                app_cluster = ""
            #endIf
    	    deploymentInstruction_attrs =  "[-cluster " + app_cluster + "]"
    	else :
            deploymentInstruction_attrs =  "[" + opps + "-cluster " + app_cluster + "]"
        #endIf
    else:
        deploymentInstruction_attrs = "-appname " + appName + " " + opps + " -MapModulesToServers [" + mapModules_attrs + "]"
        
    if (mapVHs_attrs != ""):
        deploymentInstruction_attrs = deploymentInstruction_attrs + " -MapWebModToVH [" + mapVHs_attrs + "]"
    #endElse
    
    if(mapCtxRoot_attrs !=""):
        deploymentInstruction_attrs = deploymentInstruction_attrs + " -CtxRootForWebMod  [" + mapCtxRoot_attrs + "]"
        
    if(mapEjbRef_attrs !=""):
        deploymentInstruction_attrs = deploymentInstruction_attrs + " -MapEJBRefToEJB  [" + mapEjbRef_attrs + "]"
        
    if(mapRscRef_attrs !=""):
        deploymentInstruction_attrs = deploymentInstruction_attrs + " -MapResRefToEJB  [" + mapRscRef_attrs + "]"
    
    if(mapEnvEntry_attrs !=""):
        deploymentInstruction_attrs = deploymentInstruction_attrs + " -MapEnvEntryForWebMod  [" + mapEnvEntry_attrs + "]"
    
    if (exist):      # Existing application found. Either update it or delete it according to the 'create' argument value.
        if (create):
            AdminApp.uninstall(appNametrim)
            AdminApp.install(earFile,deploymentInstruction_attrs)
            writemsg("Updated Application " + appNametrim)
        else:
            AdminApp.uninstall(appNametrim)
            writemsg("Deleted Application " + appNametrim)
        #endIf
    else:                     # Existing applicaion not found. Create it if requested, otherwise warn that it wasn't found.
        if (create):
            AdminApp.install(earFile,deploymentInstruction_attrs)
            writemsg("Install Application " + appName)
        else:
            writemsg("Application " + appName + " doesn't exist")
        #endif         
    #endIf
#endDef

#--------------------------------------------------------------------
# Procedure:   manageResEnvProvider
# Description: Creates/Deletes a Resource Environment Providers
# Parameters:
#		targetContPath	containment path which identifies the target in/from which to create/delete the WAS resource
#		name		name of the Resource Environment Providers
#		isolatedClassLoader
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	Resource Environment Providers
#--------------------------------------------------------------------
def manageResEnvProvider(targetContPath, name, isolatedClassLoader, create):
    ResEnvProvider = findObjectsByTarget("ResourceEnvironmentProvider", targetContPath, "name", name)
    attrs = [['isolatedClassLoader', isolatedClassLoader]]
    if (ResEnvProvider):    # Existing provider found. Either update it or delete it according to the 'create' argument value.
        if (create):
            AdminConfig.modify(ResEnvProvider, attrs)
            writemsg("Updated Resource Environment Providers " + name)
        else:
            AdminConfig.remove(ResEnvProvider)
            writemsg("Deleted Resource Environment Providers" + name)
            return []
        #endIf
    else:                     # Existing Resource Environment Providers not found. Create it if requested, otherwise warn that it wasn't found.
        if (create):
            scope = AdminConfig.getid(targetContPath)
            attrs.append(['name', name])
            ResEnvProvider = AdminConfig.create("ResourceEnvironmentProvider", scope, attrs)
            writemsg("Created Resource Environment Providers " + name)
        else:
            writemsg("Resource Environment Providers " + name + " doesn't exist")
        #endif         
    #endIf
    
    return ResEnvProvider
#endDef

#--------------------------------------------------------------------
# Procedure:   manageProvidersRef
# Description: Creates/Deletes a Resource Environment Providers Referenceables
# Parameters:
#		targetContPath	containment path which identifies the target in/from which to create/delete the WAS resource
#		privider		name of the Resource Environment Providers
#		factoryClassname
#		classname
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	N/A
#--------------------------------------------------------------------
def manageProvidersRef(targetContPath, provider, factoryClassname,classname, create):
    providersRef = findObjectsByTarget("Referenceable", targetContPath, "factoryClassname", factoryClassname)
    
    attrs = [['factoryClassname', factoryClassname], ['classname', classname]]
    print attrs
    if (providersRef):    # Existing providersRef found. Either update it or delete it according to the 'create' argument value.
        if (create):
            AdminConfig.modify(providersRef, attrs)
            writemsg("Updated Resource Environment Providers Referenceables" + factoryClassname)
        else:
            AdminConfig.remove(providersRef)
            writemsg("Deleted Resource Environment Providers Referenceables" + factoryClassname)
        #endIf
    else:                     # Existing Resource Environment Providers not found. Create it if requested, otherwise warn that it wasn't found.
        if (create):
            ResEnvProvider = findObjectsByTarget("ResourceEnvironmentProvider", targetContPath, "name", provider)
            providersRef = AdminConfig.create("Referenceable", ResEnvProvider, attrs)
            writemsg("Created Resource Environment Providers Referenceable " + factoryClassname)
        else:
            writemsg("Resource Environment Providers Referenceable" + factoryClassname + " doesn't exist")
        #endif         
    #endIf
    
    return 
#endDef


#--------------------------------------------------------------------
# Procedure:   manageResEnvEntry
# Description: Creates/Deletes a Resource Environment Entries
# Parameters:
#		targetContPath	containment path which identifies the target in/from which to create/delete the WAS resource
#		name		 
#		jndiName
#		provider
#		factoryClassname
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	Resource Environment Providers
#--------------------------------------------------------------------
def manageResEnvEntry(targetContPath, name, jndiName, provider,factoryClassname, create):
    entry = findObjectsByTarget("ResourceEnvEntry", targetContPath, "name", name)
    
    providersRef = findObjectsByTarget("Referenceable", targetContPath, "factoryClassname", factoryClassname)
    attrs = [['jndiName', jndiName],['referenceable',providersRef]]
    
    if (entry):    # Existing entry found. Either update it or delete it according to the 'create' argument value.
        if (create):
            
            AdminConfig.remove(entry)
            ResEnvProvider = findObjectsByTarget("ResourceEnvironmentProvider", targetContPath, "name", provider)
            attrs.append(['name', name])
            entry = AdminConfig.create("ResourceEnvEntry", ResEnvProvider, attrs)
            AdminConfig.create("J2EEResourcePropertySet", entry, [] )
            
            writemsg("Updated Resource Environment Entries" + name)
        else:
            AdminConfig.remove(entry)
            writemsg("Deleted Resource Environment Entries " + name)
            return []
        #endIf
    else:           # Existing Resource Environment Entries not found. Create it if requested, otherwise warn that it wasn't found.
        if (create):
            ResEnvProvider = findObjectsByTarget("ResourceEnvironmentProvider", targetContPath, "name", provider)
            attrs.append(['name', name])
            entry = AdminConfig.create("ResourceEnvEntry", ResEnvProvider, attrs)
            AdminConfig.create("J2EEResourcePropertySet", entry, [] )
            
            writemsg("Created  Resource Environment Entries " + name)
        else:
            writemsg("Resource Resource Environment Entries " + name + " doesn't exist")
        #endif         
    #endIf
    
    return entry
#endDef

#--------------------------------------------------------------------
# Procedure:   updateWorkAreaService
# Description: Creates/Deletes a Work area service
# Parameters:
#		workareaservice	
#		enable
#		enableWebServicePropagation
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	Resource Environment Providers
#--------------------------------------------------------------------
def updateWorkAreaService(workareaservice, enable, enableWebServicePropagation, create):
    attrs = []
    if ((enable != None) and (enable !="")):
        attrs.append(["enable", enable])
    #endIf
    if ((enableWebServicePropagation != None) and (enableWebServicePropagation !="")):
        attrs.append(["enableWebServicePropagation", enableWebServicePropagation])
    #endIf
    if (len(attrs) == 0):
        writemsg("Nothing to be updated for general properties of Work area service " + workareaservice)
    else:
        AdminConfig.modify(workareaservice, attrs )
        writemsg("Updated general properties of  Work area service" + workareaservice)
    #endIf
#endDef 


#--------------------------------------------------------------------
# Procedure:   manageNSBinding
# Description: Creates/Deletes a Name Space Binding
# Parameters:
#		targetContPath
#		identifier	
#		name
#		providerurl
#		jndiname
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	Name Space Binding
#--------------------------------------------------------------------
def manageNSBinding(targetContPath, identifier, name, providerurl, jndiname, create):
    attrs = [['providerURL',providerurl],['jndiName', jndiname],['nameInNameSpace',name],['initialContextFactory', ""]]
    
    nsBinding = findObjectsByTarget("IndirectLookupNameSpaceBinding", targetContPath, "name", identifier)
    if (nsBinding):    # Existing provider found. Either update it or delete it according to the 'create' argument value.
        if (create):
            AdminConfig.modify(nsBinding, attrs)
            writemsg("Updated Name Space Binding " + identifier)
        else:
            AdminConfig.remove(nsBinding)
            writemsg("Deleted Name Space Binding " + identifier)
        #endIf
    else:                     # Existing Name Space Binding not found. Create it if requested, otherwise warn that it wasn't found.
        if (create):
            scope = AdminConfig.getid(targetContPath)
            attrs.append(['name', identifier])
            nsBinding = AdminConfig.create("IndirectLookupNameSpaceBinding", scope, attrs)
            writemsg("Created Name Space Binding " + identifier)
        else:
            writemsg("Name Space Binding " + identifier + " doesn't exist")
        #endif         
    #endIf
    return nsBinding
#endDef


#--------------------------------------------------------------------
# Procedure:   manageForeCellBinding
# Description: Creates/Deletes a Foreign Cell binding
# Parameters:
#		targetContPath
#		name
#		create		if true, create the specified WAS resource, otherwise, delete it
# Returns:	Foreign Cell binding
#--------------------------------------------------------------------
def manageForeCellBinding(targetContPath, name, host, port, create):
    foreCell = findObjectsByTarget("ForeignCell", targetContPath, "name", name)
    endPointAttrs = [[['host', host],['port', port]]]
    attrs = [['bootstrapAddresses',endPointAttrs ]]

    if (foreCell):    # Existing  ForeCellBinding found. Either update it or delete it according to the 'create' argument value.
        if (create):
            AdminConfig.remove(foreCell)
            scope = AdminConfig.getid(targetContPath)
            attrs.append(['name', name])
            foreCell = AdminConfig.create("ForeignCell", scope, attrs)
            writemsg("Updated Foreign Cell binding " + name)
        else:
            AdminConfig.remove(foreCell)
            writemsg("Deleted Foreign Cell Binding " + name)
        #endIf
    else:                     # Existing Name Space Binding not found. Create it if requested, otherwise warn that it wasn't found.
        if (create):
            scope = AdminConfig.getid(targetContPath)
            attrs.append(['name', name])
            foreCell = AdminConfig.create("ForeignCell", scope, attrs)
            writemsg("Created Foreign Cell Binding " + name)
        else:
            writemsg("Foreign Cell Binding " + name + " doesn't exist")
        #endif         
    #endIf
    return foreCell
#endDef

#--------------------------------------------------------------------
# Procedure:   manageTimerManager
# Description: Creates/Deletes timer manager
# Parameters:
#       targetContPath
#		name			(Name of timer manager)
#		jndiName		(JNDI name of timer manager)
#		description
#		category
#		numalarmthreads	(number of alarm threads) 
#		serviceNames	(service Names)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	timer manager
#--------------------------------------------------------------------
def  manageTimerManager(targetContPath, timerName, jndiName, description, category, numalarmthreads, defTranClass, serviceNames, create):
    traceEntry("manageTimerManager" )

    # There is one and only one TimerManagerProvider in any target (cell, node, cluster or server). 
    timerProvider = findObjectsByTarget("TimerManagerProvider", targetContPath, "name", "TimerManagerProvider");
    
    # Check if Timer manager already exists
    timerManager = findObjects("TimerManagerInfo", timerProvider, "name", timerName)

    attrs = [["jndiName", jndiName],["description", description],["numAlarmThreads", numalarmthreads],["category", category],["defTranClass", defTranClass],["serviceNames", serviceNames+" "]]
    if (timerManager):
        if (create):
            AdminConfig.modify(timerManager, attrs)
            writemsg("Updated timer manager " + timerManager)
        else:
            AdminConfig.remove(timerManager)
            writemsg("Deleted  timer manager " + timerManager)
        #endIf 
    else:
        if (create):
            attrs.append(["name", timerName])
            timerManager = AdminConfig.create("TimerManagerInfo", timerProvider, attrs) 
        else:
            writemsg(" timer manager " + timerName + " doesn't exist" )
      	#endIf 
    #endIf 

    traceExit("manageTimerManager" )
    return timerManager
#endDef

#--------------------------------------------------------------------
# Procedure:   manageReplicationDomain
# Description: Creates/Deletes replication domain
# Parameters:
#       targetContPath
#		name
#		requestTimeout
#		numberOfReplicas
#		encryptionType
#		create		(Boolean if true, create entry, else delete it)
# Returns:	replication domain
#--------------------------------------------------------------------
def  manageReplicationDomain(targetContPath, name, requestTimeout, numberOfReplicas, encryptionType, create):
    traceEntry("manageReplicationDomain" )
    # There is one and only one TimerManagerProvider in cell.
    objectList= AdminConfig.list("DataReplicationDomain", AdminConfig.getid(targetContPath)).split(lineSeparator);
    replicationDomain=0
    for entry in objectList:
        if (entry.find(name) > -1):
            replicationDomain = entry;
        #endIf
    #endFor 
    attrs = [["name", name]]
    attrs1 = [["requestTimeout", requestTimeout],["numberOfReplicas", numberOfReplicas],["encryptionType", encryptionType]]
    if (replicationDomain):
        if (create):
			# Check if replication domain already exists
			dataReplication = AdminConfig.list("DataReplication", replicationDomain)
			#writemsg("Updated data replication " + dataReplication)
			AdminConfig.modify(dataReplication, attrs1)
			writemsg("Updated data replication " + dataReplication)
        else:
            AdminConfig.remove(replicationDomain)
            writemsg("Deleted  replication domain " + replicationDomain)
        #endIf 
    else:
        if (create):
            replicationDomain = AdminConfig.create("DataReplicationDomain", AdminConfig.getid(targetContPath), attrs)
            dataReplication = AdminConfig.create("DataReplication", replicationDomain, attrs1) 
            
        else:
            writemsg(" replication domain " + name + " doesn't exist" )
      	#endIf 
    #endIf 

    traceExit("manageReplicationDomain" )
    return replicationDomain
#endDef

#--------------------------------------------------------------------
# Procedure:   manageScheduler
# Description: Creates/Deletes scheduler
# Parameters:
#       targetContPath
#		name			(Name of timer manager)
#		jndiName		(JNDI name of timer manager)
#		description
#		category
#		datasourceJNDIName	
#		datasourceAlias	
#		workManagerInfoJNDIName
#		tablePrefix
#		pollInterval
#		useAdminRoles
#		createTables
#		verifyTables
#		create		(Boolean if true, create entry, else delete it)
# Returns:	timer manager
#--------------------------------------------------------------------
def  manageScheduler(targetContPath, name, jndiname, description, category, datasourceJNDIName, datasourceAlias, workManagerInfoJNDIName,tablePrefix,pollInterval,useAdminRoles,createTables,verifyTables, create):
    global AdminConfig, AdminControl
    traceEntry("manageScheduler" )
    # There is one and only one TimerManagerProvider in any target (cell, node, cluster or server). 
    schedulerProvider = findObjectsByTarget("SchedulerProvider", targetContPath, "name", "SchedulerProvider")
    
    # Check if Scheduler already exists
    scheduler = findObjectsByTarget("SchedulerConfiguration", targetContPath, "name", name)
    
    attrs = [["jndiName", jndiname],["description", description],["category", category],["pollInterval", pollInterval],["tablePrefix", tablePrefix], ["datasourceJNDIName" ,datasourceJNDIName],
    ["workManagerInfoJNDIName" ,workManagerInfoJNDIName], ["useAdminRoles", useAdminRoles], ["datasourceAlias", datasourceAlias]]
    
    if (scheduler):
        if (create):
            AdminConfig.modify(scheduler, attrs)
            writemsg("Updated scheduler " + scheduler)
        else:
            AdminConfig.remove(scheduler)
            writemsg("Deleted  scheduler " + scheduler)
        #endIf 
    else:
        if (create):
            attrs.append(["name", name])
            scheduler = AdminConfig.create("SchedulerConfiguration", schedulerProvider, attrs)
        else:
            writemsg(" Scheduler " + name + " doesn't exist" )
      	#endIf 
    #endIf 
    
    if (create):
    	# Before creating tables for the scheduler, it is required that to save the new scheduler to the configuration
    	saveChanges("save")
    
    	cellName = getCellName()
    	cellNameStr = 'cell=' + cellName
    	nodeName = getNodeName()
    	nodeNameStr = 'node=' + nodeName
    	Scheduler_Config_Helper_str = 'WebSphere:name=Scheduler_Config_Helper,process=dmgr,platform=dynamicproxy,' + nodeNameStr + ',version=7.0.0.19,type=WASSchedulerCfgHelper,mbeanIdentifier=Scheduler_Config_Helper,' + cellNameStr + ',spec=1.0'
    
    	if (createTables):
        	AdminControl.invoke(Scheduler_Config_Helper_str, 'createTables', scheduler, '[java.lang.String]')
        	writemsg (" Created Tables for scheduler " + name)
   		#endIf 
    
    	if (verifyTables):
       		AdminControl.invoke(Scheduler_Config_Helper_str, 'verifyTables', scheduler, '[java.lang.String]')
        	writemsg (" Verified Tables for scheduler " + name)
    	#endIf 
    #endIf 
    
    traceExit("manageScheduler" )
    return scheduler
#endDef

