# IBM Confidential
#
# OCO Source Material
#
# IBM Integrated Information Core PID 5724-Y59
#
# (C) Copyright IBM Corp. 2011
#
# The source code for this program is not published or otherwise divested
# of its trade secrets, irrespective of what has been deposited with the
# U. S. Copyright Office.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

#******************************************************************************
# File Name:   propsfileprocs.py
# Description: This file contains a number of useful wsadmin procs that help
#              isolate the tedious coding for common operations.
# Fix record:
#******************************************************************************

#--------------------------------------------------------------------
# Procedure:   manageWASVariables
# Description: Creates/Deletes WebSphere variables based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create WAS resources, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageWASVariables ( props, create ):
    propPrefix = "wasvariable"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        wasvarname = props[prefixi + ".name"]
        if (wasvarname == None):    # No more related properties
            break
        #endIf
        
        value = props[prefixi + ".value"]
        description = props[prefixi + ".description"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        
        manageWASVariable(targetContPath, wasvarname, value, description, create)
        
        i = i+1
    #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageURLs
# Description: Creates/Deletes URLs based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create WAS resources, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageURLs(props, create):
    propPrefix = "url"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        name = props[prefixi + ".name"]
        if (name == None):    # No more related properties
            break
        #endIf
        
        providername = props[prefixi + ".providername"]
        jndiName = props[prefixi + ".jndiName"]
        spec = props[prefixi + ".spec"]
        description = props[prefixi + ".description"]
        category = props[prefixi + ".category"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        
        manageURL(targetContPath, name, providername, jndiName, spec, description, category, create)
        
        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageAppServerClassloaders
# Description: Creates/Deletes WebSphere application server classloaders based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create Properties, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageAppServerClassloaders(props, create):
    propPrefix = "appserverclassloader"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        serverName = props[prefixi + ".target.server"]
        if (serverName == None):    # No more related properties
            break
        #endIf
        
        mode = props[prefixi + ".mode"]
        librefs = props[prefixi + ".libraryrefs"].split(" ")        
            
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        
        manageAppServerClassloader(targetContPath, mode, librefs, create)
        
        i = i+1
    #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJ2CAliases
# Description: Creates/Deletes J2C aliases based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, aliases will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageJ2CAliases ( props, create ):
    propPrefix = "jaasAuthAlias"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        alias = props[prefixi + ".alias"]
        if (alias == None):    # No more related properties
            break
        #endIf

        userId = props[prefixi + ".userId"]
        password = props[prefixi + ".password"]
        description = props[prefixi + ".description"]

        manageJ2CAuthEntry(alias, userId, password, description, create)
        
        i = i+1
    #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJDBCProviders
# Description: Creates/Deletes JDBC Providers based on the 
#              values in the Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true create JDBC providers, else delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageJDBCProviders ( props, create ):
    i = 0
    propPrefix = "jdbcprovider"

    while (1):
        prefixi = propPrefix + "." + str(i)
        jdbcprovidername = props[prefixi + ".name"]
        if (jdbcprovidername == None):    # No more entries
            break
        #endIf
        
        description = props[prefixi + ".description"]
        classpath = props[prefixi + ".classpath"]
        nativepath = props[prefixi + ".nativepath"]
        xa = props[prefixi + ".xa"]
        implementationClassName = props[prefixi + ".implementationClassName"]
        providerType = props[prefixi + ".providerType"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        
        manageJDBCProvider(targetContPath, jdbcprovidername, description, classpath, nativepath, xa, implementationClassName, providerType, create)
        
        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageJDBCDataSources
# Description: Creates/Deletes JDBC data sources based on the 
#              values in the Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true create JDBC data sources, else delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageJDBCDataSources(props, create):
    propPrefix = "datasource"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        dsname = props[prefixi + ".name"]
        if (dsname == None):    # No more entries
            break
        #endIf
        
        jdbcprovidername = props[prefixi + ".jdbcprovidername"]
        jndiname = props[prefixi + ".jndiname"]
        desc = props[prefixi + ".description"]
        stmtcachesize = props[prefixi + ".statementCacheSize"]
        helperclass = props[prefixi + ".helperclass"]
        componentAuthAlias = props[prefixi + ".componentAuthAlias"]
        xaRecoveryAuthAlias = props[prefixi + ".xaRecoveryAuthAlias"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
 
        # Do not bother to update data source. Just delete it first, then create it again.
        dataSource = findObjectsByTarget("DataSource", targetContPath, "name", dsname)
        # delete the data source if already exists
        if (dataSource):
            AdminConfig.remove(dataSource)
            writemsg("Deleted data source " + dsname + " from scope " + targetContPath)
        #endIf
            
        if (not create):
            if (not dataSource):
                writemsg("Data source " + dsname + " does not exist in scope " + targetContPath)
            #endIf 
        else:
            # create datasource 
            datasource = createDataSource(targetContPath, jdbcprovidername, dsname, desc, jndiname, helperclass, stmtcachesize, componentAuthAlias, xaRecoveryAuthAlias)
            
            # add custom properties (one by one) to newly created datasource
            m = 0
            while (1):
                prefixm = prefixi + ".custom." + str(m)
                custName = props[prefixm + ".name"]
                if (custName == None):    # No more entries
                    break
                #endIf
                custValue = props[prefixm + ".value"]
                addCustomPropertyToDataSource(datasource, custName, custValue)
                m = m + 1
            #endWhile 
            
            # set the data source connection pool properties
            connectionPoolMax = props[prefixi + ".connectionPoolMax"]
            connectionPoolMin = props[prefixi + ".connectionPoolMin"]
            createConnectionPoolForDatasource(datasource, connectionPoolMax, connectionPoolMin )

            # set mapping configuration of the data source
            mappingConfigAlias = props[prefixi + ".mappingConfigAlias"]
            containerAuthAlias = props[prefixi + ".containerAuthAlias"]
            setMappingModuleForDatasource(datasource, mappingConfigAlias, containerAuthAlias)
            
            # set data source CMP
            cmpon = props[prefixi + ".cmpon"]
            if (cmpon != None):
                setDataSourceCMP(datasource, containerAuthAlias)
            #endIf 
        #endElse 
        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   addJDBCDataSourceProps
# Description: Add JDBC data sources custom properties based on the 
#              values in the Properties object.
#
# Parameters:  props	Properties object
#		
# Returns:     N/A
#--------------------------------------------------------------------
def addJDBCDataSourceProps(props):
    propPrefix = "datasource"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        dsname = props[prefixi + ".name"]
        if (dsname == None):    # No more entries
            break
        #endIf
        
        
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
 
        
        
        dataSource = findObjectsByTarget("DataSource", targetContPath, "name", dsname)
        if (len(dataSource) == 0):
        	writemsg("Can not find datasource: " + dsname)
        	break
         

        propSet = AdminConfig.showAttribute(dataSource, "propertySet" )
                 
        # add custom properties (one by one) to newly created datasource
        m = 0
        while (1):
            prefixm = prefixi + ".custom." + str(m)
            custName = props[prefixm + ".name"]
            if (custName == None):    # No more entries
                break
            #endIf
            custDesc = props[prefixm + ".description"]
            custRequired = props[prefixm + ".required"]
            custType = props[prefixm + ".type"]
            custValue = props[prefixm + ".value"]
            #addCustomPropertyToDataSource(datasource, custName, custValue)
            
            
            
            customResourceProperty1 = []
            
	    customResourceProperty1.append(['name', custName])
	    customResourceProperty1.append(['description', custDesc])
	    customResourceProperty1.append(['required', custRequired])
	    customResourceProperty1.append(['type', custType])
	    customResourceProperty1.append(['value', custValue])	
	    rpAttrs = customResourceProperty1
			
	    print AdminConfig.create('J2EEResourceProperty', propSet, rpAttrs)
			
            m = m + 1
        #endWhile            
       
        i = i+1
    #endWhile 
#endDef


#--------------------------------------------------------------------
# Procedure:   manageQueueDestination
# Description: Creates/Deletes Queue Destination on the 
#              values in the Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true create Queue Destination resources, else delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageQueueDestination ( props, create ):
        i = 0
        destinationprefix = "mqqueuedestination"

        while (1):
                #name = props[destinationprefix+"."+i+".name"]
                name = props["%s.%i.name" % (destinationprefix,i)]

                # If the next name entry didn't exist, break out of this loop
                if ((name == "") or (name==None)):
                        break 
                #endIf 
                #providerName = props[destinationprefix+"."+i+".providerName"]
                providerName = props["%s.%i.providerName" % (destinationprefix,i)]
                #scope = props[destinationprefix+"."+i+".scope"]
                scope = props["%s.%i.scope" % (destinationprefix,i)]
                targetName = props["%s.%i.target" % (destinationprefix,i)]
                

                # Create the queue destination on the target
                
                #?PROBLEM? (jacl 546) previous FOREACH_ITEM_IN_LIST may need variable.split("xx")
                #jndiName = props[destinationprefix+"."+i+".jndiName"]
                jndiName = props["%s.%i.jndiName" % (destinationprefix,i)]
                #baseQueueName = props[destinationprefix+"."+i+".baseQueueName"]
                baseQueueName = props["%s.%i.baseQueueName" % (destinationprefix,i)]
                #baseQueueMgr = props[destinationprefix+"."+i+".baseQueueMgr"]
                baseQueueMgr = props["%s.%i.baseQueueMgr" % (destinationprefix,i)]
                #targetClient = props[destinationprefix+"."+i+".targetClient"]
                targetClient = props["%s.%i.targetClient" % (destinationprefix,i)]
                #CCSID = props[destinationprefix+"."+i+".CCSID"]
                CCSID = props["%s.%i.CCSID" % (destinationprefix,i)]

                if (( create ) ):
                        writemsg("  Creating MQ Queue Destination "+name )
                else:
                        writemsg("  Deleting MQ Queue Destination "+name )
                #endElse 
                manageMQQueueDestination(providerName, scope, targetName, name, jndiName, baseQueueName, baseQueueMgr, targetClient, CCSID, create )
                
                i = i+1
        #endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageListenerPorts
# Description: Create/Delete Message Listener Ports based on the 
#              values in the Properties object.
#
# Parameters:  props	Properties object
#		create	(Boolean if true, create listener ports, else delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageListenerPorts ( props, create ):
        # Create/Delete listener ports
        i = 0
        while (1):
                lpprefix = "listenerport.%i" % (i)
                lpname = props[lpprefix+".name"]
                
                if ((lpname == None) or (lpname == "")):
                        # no more listener ports to create, so break out
                        break 
                #endIf 

                servername = props[lpprefix+".servername"]
                cfjndiname = props[lpprefix+".cfjndiname"]
                destjndiname = props[lpprefix+".destjndiname"]
                maxsessions = props[lpprefix+".maxsessions"]
                maxretries = props[lpprefix+".maxretries"]
                maxmessages = props[lpprefix+".maxmessages"]

                if (( create ) ):
                        writemsg("    Creating listener port: " + lpname )
                else:
                        writemsg("    Deleting listener port: " + lpname )
                #endElse 
                manageMessageListener(servername, lpname, cfjndiname, destjndiname, maxsessions, maxretries, maxmessages, create )

                i = i+1
        #endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJAASAppLogins
# Description: Creates/Deletes JAAS Application Logins based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create Properties, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageJAASAppLogins ( props, create ):
        global AdminConfig

        i = 0
        varprefix = "jaasconfig"

        while (1):
                writemsg("reading variable" )
                wasvarname = props[varprefix+"."+str(i)+".name"]

                if ((wasvarname == "") or (wasvarname == None)):
                        break 
                #endIf 

                # set cellName	[$props getProperty $varprefix.$i.scope]
                cellName = "discovery"

                appLoginAlias = props[varprefix+"."+str(i)+".name"]
                proxyClassName = props[varprefix+"."+str(i)+".proxyclass"]
                loginModuleClassName = props[varprefix+"."+str(i)+".loginmoduleclass"]
                authStrategy = props[varprefix+"."+str(i)+".authstrategy"]

                manageJAASAppLogin(cellName, appLoginAlias, proxyClassName, loginModuleClassName, authStrategy, create )

                if (create):
                        writemsg("AppLogin created" )
                else:
                        writemsg("AppLogin deleted" )
                #endElse 

                i = i+1
        #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJAASAppLogin
# Description: Creates/Deletes a JAAS Application Login
#		?? If the alias already exists the auth Alias will not be replaced ??
#
# Parameters:
#		cellName	(Name of Cell)
#		loginName	(Login name) 
#		className	(LoginModule className)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 0 (does not exist)
#--------------------------------------------------------------------
def manageJAASAppLogin ( cellName, appLoginAlias, proxyClassName, loginModuleClassName, loginStrategy, create ):
        global AdminApp, AdminConfig
        module_attrib = []
        prop_attrib = []

        #	puts stdout "The value of cellName is $cellName"
        #	puts stdout "The value of appLoginAlias is $appLoginAlias"
        #	puts stdout "The value of proxyClassName is $proxyClassName"
        #	puts stdout "The value of loginModuleClassName is $loginModuleClassName"
        #	puts stdout "The value of loginStrategy is $loginStrategy"
        #	puts stdout "The value of create is $create"

        #get the configuration id
        appLoginConfigId = AdminConfig.list("JAASConfiguration" ).split(java.lang.System.getProperty("line.separator"))[0]
        #?PROBLEM? (jacl 119) previous LINDEX may need variable.split("xx")

        # create the alias
        if (create):
                attrs0 = [["alias", appLoginAlias]]
                AdminConfig.create("JAASConfigurationEntry", appLoginConfigId, attrs0 )

                # list the configuration entries and loop until we find the one we just made
                JAASConfigEntryId = AdminConfig.list("JAASConfigurationEntry" ).split(java.lang.System.getProperty("line.separator"))
                for jaasConfigEntryId in JAASConfigEntryId:
                        #?PROBLEM? (jacl 128) previous FOREACH_ITEM_IN_LIST may need variable.split("xx")
                        alias = AdminConfig.showAttribute(jaasConfigEntryId, "alias" )
                        if (cmp(alias, appLoginAlias) == 0):
                                break 
                        #endIf 
                #endFor 

                module_attrib.append(["moduleClassName", proxyClassName ])
                module_attrib.append(["authenticationStrategy", loginStrategy ])


                # create the loginModule 
                newJAASLoginModuleId = AdminConfig.create("JAASLoginModule", jaasConfigEntryId, module_attrib )

                prop_attrib.append(["name", "delegate"])
                prop_attrib.append(["value", loginModuleClassName ])

                newPropertyId = AdminConfig.create("Property", newJAASLoginModuleId, prop_attrib )
        else:

                # remove the JAASConfigurationEnry
                objid = findJAASConfigurationEntry(appLoginAlias )
                AdminConfig.remove(objid )
        #endElse 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageSharedLibraries
# Description: Creates shared libraries based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#        create	Boolean if true create shared libraries, else delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageSharedLibraries(props, create):
    i = 0
    propPrefix = "sharedlib"
    while (1):
        prefixi = propPrefix + "." + str(i)
        sharedlibname = props[prefixi + ".name"]
        if (sharedlibname == None):   # No more related properties
            break
        elif (sharedlibname == ""):
            raise ValueError, "Shared library name can not be empty"
        #endIf
        
        classpath = props[prefixi + ".classpath"]
        nativepath = props[prefixi + ".nativepath"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        
        manageSharedLibrary(targetContPath, sharedlibname, classpath, nativepath, create)
        
        i = i+1
    #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageQueueConnectionFactory
# Description: Creates/Deletes MQ Queue Connection Factory based on the 
#              values in the Properties object.
#
# Parameters:  props	Properties object
#		create	(Boolean if true, create the factory objects, otherwise, delete them)
# Returns:     N/A
#--------------------------------------------------------------------
def manageQueueConnectionFactory ( props, create ):
        i = 0
        # Create/Delete MQ Queue Connection Factory
        connfactprefix = "mqqueueconnfac"

        while (1):
                #queueConnFactName 
                name = props[connfactprefix+"."+str(i)+".name"]

                # If the next name entry didn't exist, break out of this loop
                #queueConnFactName
                if ((name == "") or (name == None)):

                        break 
                #endIf 
                providerName = props[connfactprefix+"."+str(i)+".providerName"]
                scope = props[connfactprefix+"."+str(i)+".scope"]
                targetList = buildTargetList(scope )

                # For each target node, create the connection factory
                for targetName in targetList:
                        #?PROBLEM? (jacl 441) previous FOREACH_ITEM_IN_LIST may need variable.split("xx")
                        jndiName = props[connfactprefix+"."+str(i)+".jndiName"]
                        queueManager = props[connfactprefix+"."+str(i)+".queueManager"]
                        transportType = props[connfactprefix+"."+str(i)+".transportType"]
                        clientID = props[connfactprefix+"."+str(i)+".clientID"]
                        msgRetention = props[connfactprefix+"."+str(i)+".msgRetention"]
                        XAEnabled = props[connfactprefix+"."+str(i)+".XAEnabled"]
                        CCSID = props[connfactprefix+"."+str(i)+".CCSID"]

                        #set connection pool settings
                        conTimeoutValue = props[connfactprefix+"."+str(i)+".connectionpool.timeout"]
                        conTimeout = ["connectionTimeout", conTimeoutValue]
                        conMaxValue = props[connfactprefix+"."+str(i)+".connectionpool.maxconns"]
                        conMax = ["maxConnections", conMaxValue]
                        conMinValue = props[connfactprefix+"."+str(i)+".connectionpool.minconns"]
                        conMin = ["minConnections", conMinValue]
                        conAttrs = [conTimeout, conMax, conMin]

                        #set session pool settings
                        sessTimeoutValue = props[connfactprefix+"."+str(i)+".sessionpool.timeout"]
                        sessTimeout = ["connectionTimeout", sessTimeoutValue]
                        sessMaxValue = props[connfactprefix+"."+str(i)+".sessionpool.maxconns"]
                        sessMax = ["maxConnections", sessMaxValue]
                        sessMinValue = props[connfactprefix+"."+str(i)+".sessionpool.minconns"]
                        sessMin = ["minConnections", sessMinValue]
                        sessAttrs = [sessTimeout, sessMax, sessMin]

                        if (( create ) ):
                                writemsg("  Creating MQ Queue Connection Factory" )
                        else:
                                writemsg("  Deleting MQ Queue Connection Factory" )
                        #endElse 

                        manageMQQueueConnectionFactory(providerName, scope, targetName, name, jndiName, queueManager, transportType, clientID, msgRetention, XAEnabled, CCSID, conAttrs, sessAttrs, create )
                #endFor 
                i = i+1
        #endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageRemoteQueueConnectionFactory
# Description: Creates/Deletes MQ Queue Connection Factory based on the 
#              values in the Properties object.
#
# Parameters:  props	Properties object
#		create	(Boolean if true, create the factory objects, otherwise, delete them)
# Returns:     N/A
#--------------------------------------------------------------------
def manageRemoteQueueConnectionFactory ( props, create ):
        i = 0
        # Create/Delete MQ Queue Connection Factory
        connfactprefix = "mqqueueconnfac"

        while (1):
                #queueConnFactName 
                name = props[connfactprefix+"."+str(i)+".name"]

                # If the next name entry didn't exist, break out of this loop
                #queueConnFactName
                if ((name == "") or (name == None)):

                        break 
                #endIf 
                providerName = props[connfactprefix+"."+str(i)+".providerName"]
                scope = props[connfactprefix+"."+str(i)+".scope"]
                targetName = props[connfactprefix+"."+str(i)+".target"]
                

                # Create the connection factory on target
                
                #?PROBLEM? (jacl 441) previous FOREACH_ITEM_IN_LIST may need variable.split("xx")
                jndiName = props[connfactprefix+"."+str(i)+".jndiName"]
                queueManager = props[connfactprefix+"."+str(i)+".queueManager"]
                host = props[connfactprefix+"."+str(i)+".host"]
                port = props[connfactprefix+"."+str(i)+".port"]
                channel = props[connfactprefix+"."+str(i)+".channel"]
                transportType = props[connfactprefix+"."+str(i)+".transportType"]
                clientID = props[connfactprefix+"."+str(i)+".clientID"]
                msgRetention = props[connfactprefix+"."+str(i)+".msgRetention"]
                XAEnabled = props[connfactprefix+"."+str(i)+".XAEnabled"]
                CCSID = props[connfactprefix+"."+str(i)+".CCSID"]

                #set connection pool settings
                conTimeoutValue = props[connfactprefix+"."+str(i)+".connectionpool.timeout"]
                conTimeout = ["connectionTimeout", conTimeoutValue]
                conMaxValue = props[connfactprefix+"."+str(i)+".connectionpool.maxconns"]
                conMax = ["maxConnections", conMaxValue]
                conMinValue = props[connfactprefix+"."+str(i)+".connectionpool.minconns"]
                conMin = ["minConnections", conMinValue]
                conAttrs = [conTimeout, conMax, conMin]

                #set session pool settings
                sessTimeoutValue = props[connfactprefix+"."+str(i)+".sessionpool.timeout"]
                sessTimeout = ["connectionTimeout", sessTimeoutValue]
                sessMaxValue = props[connfactprefix+"."+str(i)+".sessionpool.maxconns"]
                sessMax = ["maxConnections", sessMaxValue]
                sessMinValue = props[connfactprefix+"."+str(i)+".sessionpool.minconns"]
                sessMin = ["minConnections", sessMinValue]
                sessAttrs = [sessTimeout, sessMax, sessMin]

                if (( create ) ):
                        writemsg("  Creating MQ Queue Connection Factory" )
                else:
                        writemsg("  Deleting MQ Queue Connection Factory" )
                #endElse 

                manageRemoteMQQueueConnectionFactory(providerName, scope, targetName, name, jndiName, queueManager, host, port, channel, transportType, clientID, msgRetention, XAEnabled, CCSID, conAttrs, sessAttrs, create )
                 
                i = i+1
        #endWhile 

#endDef
#--------------------------------------------------------------------
# Procedure:   mamageJVMSettings
# Description: Configure JVM settings based on a
#              properties file.
# Parameters:  props	Properties object
#		create (Boolean if true, add JVM property setting, else delete it)
# Returns:     N/A
#--------------------------------------------------------------------
def manageJVMSettings ( props, create ):
    i = 0
    propPrefix = "jvm"
    while (1):
        prefixi = propPrefix + "." + str(i)
        servername = props[prefixi + ".target.server"]
        if (servername == None):    # No more entries
            break
        #endIf
        
        maxheapsize = props[prefixi + ".maxheapsize"]
        initheapsize = props[prefixi + ".initheapsize"]
        genericJvmArguments = props[prefixi + ".genericJvmArguments"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        jvm = findObjectsByTarget("JavaVirtualMachine", targetContPath)

        updateJvmGeneralProperties(jvm, maxheapsize, initheapsize, genericJvmArguments)

        # Create/Delete JVM Custom Properties
        j = 0
        while (1):
            prefixj = prefixi + ".property." + str(j)
            propertyname = props[prefixj + ".name"]
            if (propertyname == None):    # No more entries
                break
            #endIf
        
            propertyvalue = props[prefixj + ".value"]
            propertydescript = props[prefixj + ".description"]
            
            manageJvmCustomProperty(jvm, propertyname, propertyvalue, propertydescript, create )

            j = j+1
        #endWhile 

        i = i+1
    #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageAppServerContainerTransactionServiceSettings
# Description: Configure application server container transaction Service settings based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create Properties, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageAppServerContainerTransactionServiceSettings(props, create):
    i = 0
    propPrefix = "transaction"
    while (1):
        prefixi = propPrefix + "." + str(i)
        servername = props[prefixi + ".target.server"]
        if (servername == None):    # No more entries
            break
        #endIf
        
        totalTranLifetimeTimeout = props[prefixi + ".totalTranLifetimeTimeout"]
        propogatedOrBMTTranLifetimeTimeout = props[prefixi + ".propogatedOrBMTTranLifetimeTimeout"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        transService = findObjectsByTarget("TransactionService", targetContPath)
        
        updateTransactionServiceGeneralProperties(transService, totalTranLifetimeTimeout,propogatedOrBMTTranLifetimeTimeout)

        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageAppServerTraceServiceSettings
# Description: 
#
# Parameters:  props	Properties object
#		create	Boolean 
# Returns:     N/A
#--------------------------------------------------------------------
def manageAppServerTraceServiceSettings(props, create):
    i = 0
    propPrefix = "traceservice"
    while (1):
        prefixi = propPrefix + "." + str(i)
        servername = props[prefixi + ".target.server"]
        if (servername == None):    # No more entries
            break
        #endIf
        
        startupTraceSpecificationPart = props[prefixi + ".startupTraceSpecificationPart"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        traceService = findObjectsByTarget("TraceService", targetContPath)
    
        updateAppServerTraceServiceSettings(traceService, startupTraceSpecificationPart, create)

        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageAppServerEJBContainerSettings
# Description: Configure application server ejb container settings based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
# Returns:     N/A
#--------------------------------------------------------------------
def manageAppServerEJBContainerSettings(props):
    i = 0
    propPrefix = "ejbcontainer"
    while (1):
        prefixi = propPrefix + "." + str(i)
        servername = props[prefixi + ".target.server"]
        if (servername == None):    # No more entries
            break
        #endIf
        
        inactivePoolCleanupInterval = props[prefixi + ".inactivePoolCleanupInterval"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        ejbContainer = findObjectsByTarget("EJBContainer", targetContPath)
    
        updateEJBContainerGeneralProperties(ejbContainer, inactivePoolCleanupInterval)

        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageSIBuses
# Description: Creates/Deletes SI buses based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, buses will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSIBuses ( props, create ):
    i = 0
    propPrefix = "bus"
    while (1):
        prefixi = propPrefix + "." + str(i)
        busName = props[prefixi + ".name"]
        if (busName == None):    # No more entries
            break
        #endIf

        busDescription = props[prefixi + ".description"]
        busSecurity = props[prefixi + ".security"]

        manageSIBus(busName, busDescription, busSecurity, create)
        
        i = i+1
    #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageSIBusMembers
# Description: Creates/Deletes SI buses members based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, bus members will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSIBusMembers ( props, create ):
    i = 0
    propPrefix = "busmember"
    while (1):
        prefixi = propPrefix + "." + str(i)
        busName = props[prefixi + ".busname"]
        if (busName == None):    # No more entries
            break
        #endIf

        nodeName = props[prefixi + ".node"]
        serverName = props[prefixi + ".server"]
        clusterName = props[prefixi + ".cluster"]
        datasourceJndiName = props[prefixi + ".datasourceJndiName"]
        authAlias = props[prefixi + ".authAlias"]
        createTables = props[prefixi + ".createTables"]
        schemaName = props[prefixi + ".schemaName"]
		
        manageSIBusMember(busName, clusterName, nodeName, serverName, datasourceJndiName, authAlias, createTables, schemaName, create)
        
        i = i+1
    #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageSIBusDestinations
# Description: Creates/Deletes SI buses destinations based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, bus destinations will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSIBusDestinations ( props, create ):
    i = 0
    propPrefix = "busdestination"
    while (1):
        prefixi = propPrefix + "." + str(i)
        destName = props[prefixi + ".name"]
        if (destName == None):    # No more entries
            break
        #endIf

        busName = props[prefixi + ".busname"]
        destinationType = props[prefixi + ".type"]
        description = props[prefixi + ".description"]
        cluster = props[prefixi + ".cluster"]
        reliability = props[prefixi + ".reliability"]
		
        manageSIBusDestination(destName, busName, destinationType, description, cluster, reliability, create)
 
        i = i+1
    #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJMSConnectionFactories
# Description: Creates/Deletes JMS Connection factories based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, connection factories will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageJMSConnectionFactories ( props, create ):
    i = 0
    propPrefix = "confactory"
    while (1):
        prefixi = propPrefix + "." + str(i)
        cfName = props[prefixi + ".name"]
        if (cfName == None):    # No more entries
            break
        #endIf

        jndiName = props[prefixi + ".jndiname"]
        if (jndiName == None or jndiName == ""):
            raise ValueError, "jndiName is required"
        #endIf
        busName = props[prefixi + ".busname"]
        cfType = props[prefixi + ".type"]
        targetType = props[prefixi + ".targetType"]
        authDataAlias = props[prefixi + ".authdataalias"]		
        xaAuthAlias = props[prefixi + ".xaauthalias"]
        providerEndPoints = props[prefixi + ".providerEndPoints"]
        nonPersistentMapping = props[prefixi + ".nonPersistentMapping"]
        persistentMapping = props[prefixi + ".persistentMapping"]
        readAhead = props[prefixi + ".readAhead"]
        targetSignificance = props[prefixi + ".targetSignificance"]
        connectionProximity = props[prefixi + ".connectionProximity"]
        shareDataSourceWithCMP = props[prefixi + ".shareDataSourceWithCMP"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
		
        connectionFactory = manageJMSConnectionFactory(targetContPath, cfName, jndiName, busName, cfType, targetType, authDataAlias, xaAuthAlias, providerEndPoints, nonPersistentMapping, persistentMapping, readAhead, targetSignificance, connectionProximity, shareDataSourceWithCMP, create)
        
        # Update connection pool properties
        maxConnections = props[prefixi + ".connpool.maxConnections"]
        connectionTimeout = props[prefixi + ".connpool.connectionTimeout"]
        attrs = []
        if (maxConnections != None):
            attrs.append(['maxConnections', maxConnections])
        #endIf
        if (connectionTimeout != None):
            attrs.append(['connectionTimeout', connectionTimeout])
        #endIf
        if (create and len(attrs) > 0):
            connpool = AdminConfig.showAttribute(connectionFactory, 'connectionPool')
            AdminConfig.modify(connpool, attrs)
            writemsg("Updated connection pool properties of the connectionFactory " + connectionFactory)
        #endIf        
        i = i+1
    #endWhile 
#endDef 


#--------------------------------------------------------------------
# Procedure:   manageJMSQueues
# Description: Creates/Deletes JMS queues based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, queue will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageJMSQueues ( props, create ):
    i = 0
    propPrefix = "jmsqueue"
    while (1):
        prefixi = propPrefix + "." + str(i)
        jmsQName = props[prefixi + ".name"]
        if (jmsQName == None):    # No more entries
            break
        #endIf

        jndiName = props[prefixi + ".jndiname"]
        busName = props[prefixi + ".busname"]
        queueName = props[prefixi + ".queuename"]
        deliveryMode = props[prefixi + ".deliveryMode"]
        readAhead = props[prefixi + ".readAhead"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        
        manageJMSQueue(targetContPath, jmsQName,jndiName,busName, queueName, deliveryMode, readAhead, create)
        
        i = i+1
    #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJMSTopics
# Description: Creates/Deletes JMS topics based on the values in the
#              Properties object.
#
# Parameters:  	props	Properties object
#		create	Boolean if true, topic will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageJMSTopics ( props, create ):
    i = 0
    propPrefix = "jmstopic"
    while (1):
        prefixi = propPrefix + "." + str(i)
        name = props[prefixi + ".name"]
        if (name == None):    # No more entries
            break
        #endIf

        jndiName = props[prefixi + ".jndiname"]
        busName = props[prefixi + ".busname"]
        topicName = props[prefixi + ".topicname"]
        topicSpace = props[prefixi + ".topicspace"]
        deliveryMode = props[prefixi + ".deliveryMode"]
        readAhead = props[prefixi + ".readAhead"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        
        manageJMSTopic(targetContPath, name, jndiName, busName, topicName, topicSpace, deliveryMode, readAhead, create)

        i = i+1
    #endWhile 
#endDef 


#--------------------------------------------------------------------
# Procedure:   manageJMSActivationSpecs
# Description: Creates/Deletes JMS activation specifications based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, JMS activation specification will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageJMSActivationSpecs ( props, create ):
    i = 0
    propPrefix = "jmsactspec"
    while (1):
        prefixi = propPrefix + "." + str(i)
        jmsActSpecName = props[prefixi + ".name"]
        if (jmsActSpecName == None):    # No more entries
            break
        #endIf

        jndiName = props[prefixi + ".jndiname"]
        busName = props[prefixi + ".busname"]
        jndiDest = props[prefixi + ".destinationjndiname"]
        destType = props[prefixi + ".destinationtype"]
        description = props[prefixi + ".description"]
        authAlias = props[prefixi + ".authalias"]
        messageSelector = props[prefixi + ".messageselector"]
        durability = props[prefixi + ".subscriptiondurability"]
        subscriptionName = props[prefixi + ".subscriptionname"]
        clientID = props[prefixi + ".clientid"]
        acknowledgeMode = props[prefixi + ".acknowledgeMode"]
        durableSubscriptionHome = props[prefixi + ".durableSubscriptionHome"]
        readAhead = props[prefixi + ".readAhead"]
        targetType = props[prefixi + ".targetType"]
        targetSignificance = props[prefixi + ".targetSignificance"]
        shareDurableSubscriptions = props[prefixi + ".shareDurableSubscriptions"]
        shareDataSourceWithCMP = props[prefixi + ".shareDataSourceWithCMP"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
		
        manageJMSActivationSpec(targetContPath, jmsActSpecName, jndiName, busName, jndiDest, destType, description, authAlias, messageSelector, durability, subscriptionName, clientID, acknowledgeMode, durableSubscriptionHome, readAhead, targetType, targetSignificance, shareDurableSubscriptions, shareDataSourceWithCMP, create)

        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageMQActivationSpecs
# Description: Creates/Deletes MQ activation specifications based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, JMS activation specification will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageMQActivationSpecs ( props, create ):
        i = 0
        done = "false"
        mqactspecprefix = "mqactspec"

        while (1):
		propName = mqactspecprefix + "." + str(i) + ".name"
		mqActSpecName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (mqActSpecName == ""):
                        break 
                #endIf 

		scope = props[mqactspecprefix+"."+ str(i) + ".scope"]
                jndiName = props[mqactspecprefix+"."+ str(i) + ".jndiname"]
                busName = props[mqactspecprefix+"."+ str(i) + ".busname"]
		jndiDest = props[mqactspecprefix+"."+ str(i) + ".destinationjndiname"]
		destType = props[mqactspecprefix+"."+ str(i) + ".destinationtype"]
		authAlias = props[mqactspecprefix+"."+ str(i) + ".authalias"]
		messageSelector = props[mqactspecprefix+"."+ str(i) + ".messageselector"]
                queueManager = props[mqactspecprefix+"."+ str(i) + ".queueManager"]
                hostName = props[mqactspecprefix+"."+ str(i) + ".hostName"]
                port = props[mqactspecprefix+"."+ str(i) + ".port"]
                channel = props[mqactspecprefix+"."+ str(i) + ".channel"]
                durability = props[mqactspecprefix+"."+ str(i) + ".subscriptiondurability"]
		subscriptionName = props[mqactspecprefix+"."+ str(i) + ".subscriptionname"]
		clientID = props[mqactspecprefix+"."+ str(i) + ".clientid"]
		
		
		targetList = buildTargetList(scope)
                # For each target node, create/delete MQ activation specification
                for targetName in targetList:
			writemsg("scope=" + scope + " target is " + targetName)
                        writemsg("MQ activation spec="+ mqActSpecName )
                        manageMQActivationSpec(scope, targetName, mqActSpecName, jndiName, jndiDest, destType, authAlias, messageSelector, queueManager, hostName, port, channel, durability, subscriptionName, clientID, create)
                 #endFor 
                i = i+1
        #endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageStartupBeanServices
# Description: Enable/Disable startup bean services based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, startup bean service will be enabled, else disabled
# Returns:     N/A
#--------------------------------------------------------------------
def manageStartupBeanServices ( props, create ):
        i = 0
        done = "false"
        startupprefix = "startupbean"

        while (1):
		propName = startupprefix + "." + str(i) + ".server"
		serverName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (serverName == ""):
                        break 
                #endIf 

		
                writemsg("Startup bean on server="+ serverName)
                manageStartupBeanService(serverName, create)
                i = i+1
        #endWhile 

#endDef 


#--------------------------------------------------------------------
# Procedure:   manageObjectCacheInstances
# Description: Creates/Deletes Object cache instances based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, Object cache resources will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageObjectCacheInstances ( props, create ):
    i = 0
    propPrefix = "objcacheinst"
    while (1):
        prefixi = propPrefix + "." + str(i)
        objCacheName = props[prefixi + ".name"]
        if (objCacheName == None):    # No more entries
            break
        #endIf

        jndiName = props[prefixi + ".jndiname"]
        cacheSize = props[prefixi + ".cachesize"]
        category = props[prefixi + ".category"]
        defaultPriority = props[prefixi + ".defaultPriority"]
        replicationType = props[prefixi + ".replicationType"]
        disableDependencyId = props[prefixi + ".disableDependencyId"]
        enableCacheReplication = props[prefixi + ".enableCacheReplication"]
        replicationDomain = props[prefixi + ".replicationDomain"]
        enableDiskOffload = props[prefixi + ".enableDiskOffload"]
        flushToDiskOnStop = props[prefixi + ".flushToDiskOnStop"]
        diskCacheSizeInGB = props[prefixi + ".diskCacheSizeInGB"]
        if ( (diskCacheSizeInGB=="") or (diskCacheSizeInGB==None) ):
        	diskCacheSizeInGB=0
		#endIf
        diskCacheEntrySizeInMB = props[prefixi + ".diskCacheEntrySizeInMB"]
        if ( (diskCacheEntrySizeInMB=="") or (diskCacheEntrySizeInMB==None) ):
        	diskCacheEntrySizeInMB=0
		#endIf
        diskCacheCleanupFrequency = props[prefixi + ".diskCacheCleanupFrequency"]
        if ( (diskCacheCleanupFrequency=="") or (diskCacheCleanupFrequency==None) ):
        	diskCacheCleanupFrequency=0
		#endIf
        diskCacheSizeInEntries = props[prefixi + ".diskCacheSizeInEntries"]
        if ( (diskCacheSizeInEntries=="") or (diskCacheSizeInEntries==None) ):
        	diskCacheSizeInEntries=0
		#endIf
        pushFrequency = props[prefixi + ".pushFrequency"]
        if ( (pushFrequency=="") or (pushFrequency==None) ):
        	pushFrequency=0
		#endIf
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        
        manageObjectCacheInstance(targetContPath,objCacheName, jndiName, cacheSize, defaultPriority, category, enableDiskOffload, flushToDiskOnStop, diskCacheSizeInGB, diskCacheEntrySizeInMB, diskCacheCleanupFrequency, diskCacheSizeInEntries, pushFrequency, disableDependencyId, replicationType, enableCacheReplication, replicationDomain, create)

        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageResourceEnvironmentProviders
# Description: Creates/Deletes Resource Environment Providers based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, esource Environment Providers will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageResourceEnvironmentProviders ( props, create ):
        i = 0
        done = "false"
        providerprefix = "resource.provider"

        while (1):
		propName = providerprefix + "." + str(i) + ".name"
		providerName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (providerName == ""):
                        break 
                #endIf 

		scope = props[providerprefix+"."+ str(i) + ".scope"]
                description = props[providerprefix+"."+ str(i) + ".description"]
                factoryClassname = props[providerprefix+"."+ str(i) + ".factoryClassname"]
		classname = props[providerprefix+"."+ str(i) + ".classname"]
	
		targetName = props["serverName"]
		writemsg("scope=" + scope + " target is " + targetName )
                writemsg("Resource environment Provider="+ providerName)
                manageResourceEnvironmentProvider(scope, targetName, providerName, description, factoryClassname, classname, create)
                i = i+1
        #endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageResourceEnvironmentProviderProperties
# Description: Creates/Deletes Resource Environment Provider Properties based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, Resource Environment Provider properties will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageResourceEnvironmentProviderProperties ( props, create ):
        i = 0
        done = "false"
        propertyprefix = "resource.provider.property"

        while (1):
		propName = propertyprefix + "." + str(i) + ".name"
		name = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (name == ""):
                        break 
                #endIf 

		scope = props[propertyprefix+"."+ str(i) + ".scope"]
                value = props[propertyprefix+"."+ str(i) + ".value"]
                providerName = props[propertyprefix+"."+ str(i) + ".providername"]
	
		targetName = props["serverName"]
		writemsg("scope=" + scope + " target is " + targetName )
                writemsg("Resource environment properties="+ name)
                manageResourceEnvironmentProviderProperty(scope, targetName, providerName, name, value, create)
                i = i+1
        #endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageResourceEnvironmentEntries
# Description: Creates/Deletes Resource Environment Entries based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#	       create	Boolean if true, Resource Environment Entry will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageResourceEnvironmentEntries ( props, create ):
        i = 0
        done = "false"
        entryprefix = "resource.provider.entry"

        while (1):
		propName = entryprefix + "." + str(i) + ".name"
		entryName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (entryName == ""):
                        break 
                #endIf 

		scope = props[entryprefix+"."+ str(i) + ".scope"]
                jndiname = props[entryprefix+"."+ str(i) + ".jndiname"]
                providerName = props[entryprefix+"."+ str(i) + ".providername"]
                factoryClassname = props[entryprefix+"."+ str(i) + ".factoryClassname"]
	
		targetName = props["serverName"]
		writemsg("scope=" + scope + " target is " + targetName )
                writemsg("Resource environment entry ="+ entryName)
                
                manageResourceEnvironmentEntry(scope, targetName, entryName, jndiname, providerName, factoryClassname, create)
                if (create): 
	                j = 0
	                while (1):
	                        custPropPre = entryprefix + "." + str(i) + ".custom." + str(j)
	                        custName = props.getProperty(custPropPre + ".name" , "" )
	                        if ((custName == "") or (custName == None)):
	                                # No more custom properties
	                                break 
	                        #endIf 
	                        custValue = props[custPropPre + ".value"]
	                        writemsg("Resource environment entry custom property name ="+ custName)
	                        writemsg("Resource environment entry custom property value ="+ custValue)
	                        addCustomPropertyToEnvironmentEntryWithScope(scope, targetName, entryName, custName, custValue )
	                        j = j + 1
	                #endWhile 
                #endif
                
                
                i = i+1
        #endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageSIBusWMQServers
# Description: Creates/Deletes SI Bus WMQ Servers
#
# Parameters:  props	Properties object
#	       create	Boolean if true, SI Bus WMQ Servers will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSIBusWMQServers ( props, create ):
	global AdminTask
        i = 0
        done = "false"
        entryprefix = "busmqserver"

        while (1):
		propName = entryprefix + "." + str(i) + ".name"
		entryName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (entryName == ""):
                        break 
                #endIf 

                name = props[entryprefix+"."+ str(i) + ".name"]
		serverName = props[entryprefix+"."+ str(i) + ".serverName"]
                description = props[entryprefix+"."+ str(i) + ".description"]
                mqtype = props[entryprefix+"."+ str(i) + ".type"]
                bindingsMode = props[entryprefix+"."+ str(i) + ".bindingsMode"]
                host = props[entryprefix+"."+ str(i) + ".host"]
                port = props[entryprefix+"."+ str(i) + ".port"]
                channel = props[entryprefix+"."+ str(i) + ".channel"]
                securityAuthAlias = props[entryprefix+"."+ str(i) + ".securityAuthAlias"]
                transportChain = props[entryprefix+"."+ str(i) + ".transportChain"]
                allowDiscovery = props[entryprefix+"."+ str(i) + ".allowDiscovery"]
                replyToQueue = props[entryprefix+"."+ str(i) + ".replyToQueue"]
                  
                sibusMqServers=AdminTask.listSIBWMQServers().split(java.lang.System.getProperty("line.separator"))
                 
     		for sibusMqServer in sibusMqServers :
     			if( sibusMqServer != "" ) :
     				nameEntry = AdminConfig.showAttribute(sibusMqServer,"name")
     				if ( nameEntry == name ) :
     					if ( create ) :
     						writemsg("MQ Server for SiBus entry = "+ name + " existed ")
     						writemsg("Update MQ Server for SiBus entry = "+ name)
     						attrs = "[-name %s -serverName %s -description %s -type %s -bindingsMode %s -host %s -port %s -channel %s -securityAuthAlias %s -transportChain %s -allowDiscovery %s -replyToQueue %s -trustUserIds true -discoveryAuthAlias ]" % (name, serverName, description, mqtype, bindingsMode, host, port, channel, securityAuthAlias, transportChain, allowDiscovery, replyToQueue )
				                trace("info", "attribute= " + attrs )
						AdminTask.modifySIBWMQServer(attrs)
						done = "true"					
					else :
						attrs = "[-name %s ]" %(name)
						AdminTask.deleteSIBWMQServer(attrs)
						done = "true"
					#endElse 
				#endIf
			#endIf
		#endFor
		if ( done == "false"):
			attrs = "[-name %s -serverName %s -description %s -type %s -bindingsMode %s -host %s -port %s -channel %s -securityAuthAlias %s -transportChain %s -allowDiscovery %s -replyToQueue %s -trustUserIds true -discoveryAuthAlias ]" % (name, serverName, description, mqtype, bindingsMode, host, port, channel, securityAuthAlias, transportChain, allowDiscovery, replyToQueue )
	                writemsg("Create MQ Server for SiBus entry = "+ name)
	                trace("info", "attribute= " + attrs )
			AdminTask.createSIBWMQServer(attrs) 
			writemsg("Created MQ Server for SiBus entry = "+ name)
                i = i+1
        #endWhile       

#endDef


#--------------------------------------------------------------------
# Procedure:   manageSIBusWMQMembers
# Description: Creates/Deletes SI Bus WMQ Members
#
# Parameters:  props	Properties object
#	       create	Boolean if true, SI Bus WMQ Members will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSIBusWMQMembers ( props, create ):
	global AdminTask
        i = 0
        done = "false"
        entryprefix = "busmqmember"
        
        while (1):
		propName = entryprefix + "." + str(i) + ".busname"
		entryName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (entryName == ""):
                        break 
                #endIf 

                busname = props[entryprefix+"."+ str(i) + ".busname"]
		wmqServer = props[entryprefix+"."+ str(i) + ".wmqServer"]
                virtualQueueManagerName = props[entryprefix+"."+ str(i) + ".virtualQueueManagerName"]
                if ( create ) :	
			attrs = "[-bus %s -wmqServer %s -virtualQueueManagerName %s ]" % (busname, wmqServer, virtualQueueManagerName )
	                writemsg("Add MQ Server for SiBus member ="+ wmqServer)
	                trace("info", "attribute= " + attrs )
			AdminTask.addSIBusMember(attrs) 
		else :
			attrs = "[-bus %s -wmqServer %s ]" % (busname, wmqServer)
			writemsg("Remove MQ Server for SiBus member ="+ wmqServer)
			AdminTask.removeSIBusMember(attrs)
                i = i+1
        #endWhile         
        

#endDef

#--------------------------------------------------------------------
# Procedure:   manageSIBusWMQDestinations
# Description: Creates/Deletes SI Bus WMQ Destinations
#
# Parameters:  props	Properties object
#	       create	Boolean if true, SI Bus WMQ Destinations will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSIBusWMQDestinations(props, create ):
	global AdminTask
        i = 0
        done = "false"
        entryprefix = "busmqdestination"
        
        while (1):
		propName = entryprefix + "." + str(i) + ".name"
		entryName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (entryName == ""):
                        break 
                #endIf 

                name = props[entryprefix+"."+ str(i) + ".name"]
                busname = props[entryprefix+"."+ str(i) + ".busname"]
                destinationtype = props[entryprefix+"."+ str(i) + ".type"]
		wmqServer = props[entryprefix+"."+ str(i) + ".wmqserver"]
		wmqqueuename = props[entryprefix+"."+ str(i) + ".wmqqueuename"]
                if ( create ) :	
                	if ( destinationtype == "Queue" ) : 
				attrs = "[-bus %s -name %s -type %s -wmqServer %s -wmqQueueName %s ]" % (busname, name, destinationtype, wmqServer, wmqqueuename )
	                else :
	                	attrs = "[-bus %s -name %s -type %s ]" % (busname, name, destinationtype)
	                writemsg("Create SI Bus MQ Server destination ="+ name)
	                trace("info", "attribute= " + attrs )
			AdminTask.createSIBDestination(attrs) 
		else :
			attrs = "[-bus %s -name %s ]" % (busname, name )
			writemsg("Delete SI Bus MQ Server destination ="+ name)
			AdminTask.deleteSIBDestination(attrs)
                i = i+1
        #endWhile         
        

#endDef

#--------------------------------------------------------------------
# Procedure:   manageWorkManagers
# Description: Creates/Deletes Work managers based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, Work manager resources will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageWorkManagers ( props, create ):
    i = 0
    propPrefix = "workmanager"
    while (1):
        prefixi = propPrefix + "." + str(i)
        workManagerName = props[prefixi + ".name"]
        if (workManagerName == None):    # No more entries
            break
        #endIf

        jndiName = props[prefixi + ".jndiname"]
        description = props[prefixi + ".description"]
        workTimeout = props[prefixi + ".workTimeout"]
        workReqQFullAction = props[prefixi + ".workReqQFullAction"]
        isGrowable = props[prefixi + ".isGrowable"]
        maxThreads = props[prefixi + ".maxThreads"]
        minThreads = props[prefixi + ".minThreads"]
        numAlarmThreads = props[prefixi + ".numAlarmThreads"]
        serviceNames = props[prefixi + ".serviceNames"]
        threadPriority = props[prefixi + ".threadPriority"]
        category = props[prefixi + ".category"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        
        manageWorkManager(targetContPath, workManagerName, jndiName, description, workTimeout, workReqQFullAction, isGrowable, maxThreads, minThreads, numAlarmThreads, serviceNames, threadPriority, category, create)

        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageServerProperties
# Description: Creates/Deletes server properties based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, Work manager resources will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageServerProperties ( props, create ):
        i = 0
        done = "false"
        serverpropprefix = "serverproperties"

        while (1):
		propName = serverpropprefix + "." + str(i) + ".name"
		serverpropName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (serverpropName == ""):
                        break 
                #endIf 

		servername = props[serverpropprefix+"."+ str(i) + ".servername"]
		value = props[serverpropprefix+"."+ str(i) + ".value"]
                description = props[serverpropprefix+"."+ str(i) + ".description"]
		required = props[serverpropprefix+"."+ str(i) + ".required"]
		
		writemsg("servername=" + servername)
                writemsg("server properties Name="+ serverpropName)
                manageServerProperty(servername,serverpropName,value,description,required,create)
                
                i = i+1
        #endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageEventServers
# Description: 
#
# Parameters:  props	Properties object
# Returns:     N/A
#--------------------------------------------------------------------
def manageEventServers(props, create):
    i = 0
    propPrefix = "eventserver"
    while (1):
        prefixi = propPrefix + "." + str(i)
        jndiName = props[prefixi + ".jndiName"]
        if (jndiName == None):    # No more entries
            break
        #endIf

        name = props[prefixi + ".name"]
        enableDataStore = props[prefixi + ".enableDataStore"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        
        manageEventServer(targetContPath, name, jndiName, enableDataStore, create)
        
        i = i+1
    #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageEventsGroups
# Description: Creates/Deletes  CEI EventGroups based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, CEI EventGroup resources will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageEventsGroups(props, create):
    global  AdminConfig
    propPrefix = "eventgroup"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        eventgroupname = props[prefixi + ".name"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        
        if (eventgroupname == None):    # No more related properties
            break
        #endIf
        
       	eventSelectorString = props[prefixi + ".eventSelectorString"]
       	persistEvents = props[prefixi + ".persistEvents"]
               
        providerid = AdminConfig.list("EventInfrastructureProvider",AdminConfig.getid(targetContPath))
		
        eventGroupProfileId = AdminConfig.list('EventGroupProfileList',providerid)

       	# add custom properties (one by one) to newly created event group
        m = 0
        CompatibilityMode_attrs = []
	DistributeQ = []
        while (1):
            
            prefixm = prefixi + ".custom." + str(m)
	    #print "perfixm="+prefixm
            custName = props[prefixm + ".name"]
	    #print "custName="+custName
            if (custName == None):    # No more entries
            	
		break
            #endIf
            
            custValue = props[prefixm + ".value"]
            cusDes = props[prefixm + ".description"]
            cusReq = props[prefixm + ".required"]
            cusType = props[prefixm + ".type"]
            
            CompatibilityMode_attrs.append([['name', custName], ['description', cusDes], ['required',cusReq],['type',cusType],['value',custValue]])
            m = m + 1
       	#endWhile
	#print CompatibilityMode_attrs

	n = 0
	while (1):
		
	    prefixn = prefixi + ".distq." + str(n)
	   
            qjndiName = props[prefixn + ".queueJNDIName"]
 	    #print "qjndiName="+qjndiName
	    if (qjndiName == None):    # No more entries
                #print "can not find "+prefixn+".queueJNDIName"
		break
	    #endIf
	    #print "find "+qjndiName

	    qcfjndiName = props[prefixn + ".queueConnectionFactoryJNDIName"]
	    if (qcfjndiName == None):    # No more entries
                break
            #endIf
	    #print "find "+qcfjndiName
	    DistributeQ.append([['queueJNDIName',qjndiName],['queueConnectionFactoryJNDIName',qcfjndiName]])
	    n = n + 1
	#endWhile
	#print DistributeQ
       	#create CEI event group
       	manageEventsGroup(targetContPath,eventgroupname, eventSelectorString, persistEvents,eventGroupProfileId,DistributeQ,CompatibilityMode_attrs,create)
        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageAsyncEmitters
# Description: Creates/Deletes AsyncEmitter based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, AsyncEmitter resources will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageAsyncEmitters(props, create):
    global  AdminConfig
    propPrefix = "asyncEmitter"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        asyncEmitterName = props[prefixi + ".name"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        if (asyncEmitterName == None):    # No more related properties
            break
        #endIf
        
       	jndiName = props[prefixi + ".jndiName"]
       	description = props[prefixi + ".description"]
       	category = props[prefixi + ".category"]
       	transactionMode = props[prefixi + ".TransactionMode"]
       	synchronizationMode = props[prefixi + ".SynchronizationMode"]
       	syncJNDIName = props[prefixi + ".SyncJNDIName"]
       	asyncJNDIName = props[prefixi + ".AsyncJNDIName"]
       	filteringEnabled = props[prefixi + ".FilteringEnabled"]
       	filterJNDIName = props[prefixi + ".FilterJNDIName"]

        supportCluster = props[prefixi + ".target.cluster"]
        cluster=AdminConfig.getid('/ServerCluster:'+ supportCluster + '/')
        
        providerid = AdminConfig.list("EventInfrastructureProvider",cluster)
        
       	# add custom properties (one by one) to newly created datasource
        m = 0
        CompatibilityMode_attrs = []
        while (1):
            prefixm = prefixi + ".custom." + str(m)
            custName = props[prefixm + ".name"]
            if (custName == None):    # No more entries
            	break
            #endIf
            
            custValue = props[prefixm + ".value"]
            cusDes = props[prefixm + ".description"]
            cusReq = props[prefixm + ".required"]
            cusType = props[prefixm + ".type"]
            
            CompatibilityMode_attrs = [['name',custName], ['description',cusDes], ['required', cusReq],['type',cusType], ['value',custValue]]
            m = m + 1
       	#endWhile
       	#create CEI event group
        manageAsyncEmitter(targetContPath,asyncEmitterName, jndiName,description,category,transactionMode,synchronizationMode,asyncJNDIName,syncJNDIName,filteringEnabled, filterJNDIName,providerid, CompatibilityMode_attrs,create)
        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageApplications
# Description: Creates/Deletes Applications based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, Applications will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageApplications(props, create):
    propPrefix = "application"
    i = 0
    
    while (1):
        prefixi = propPrefix + "." + str(i)
        appName = props[prefixi + ".name"]
        if ((appName == "") or (appName == None)):
        	break 
        #endIf 
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        earFile = props[prefixi + ".earfile"]
        cellName = getCellName()
        deployejb = props[prefixi + ".deployejb"]
        deployws = props[prefixi + ".deployws"]
        useDefaultBindings = props[prefixi + ".useDefaultBindings"]
        opps = ""
        
        if (deployejb == "yes"):
            deployejbopp = "-deployejb "
        else:
            deployejbopp = ""
        #endElse 
        opps +=  deployejbopp
        
        if (deployws == "yes"):
            deploywsopp = "-deployws "
        else:
            deploywsopp = ""
        #endElse 
        opps += deploywsopp
        
        if (useDefaultBindings == "yes"):
            useDefaultBindings = "-usedefaultbindings "
        else:
            useDefaultBindings = ""
        #endElse 
        opps += useDefaultBindings
        
		# add Maping Modules to application
        m = 0
        
        mapModules_attrs = ""
        mapVHs_attrs = ""
        mapCtxRoot_attrs = ""
        mapEjbRef_attrs = ""
        mapRscRef_attrs = ""
        mapEnvEntry_attrs=""
        
        app_cluster = ""
        while (1):
            prefixm = prefixi + ".module." + str(m)
            moduleName = props[prefixm + ".name"]
            if (moduleName == None):    # No more entries
            	break
            #endIf
            
            moduleFile = props[prefixm + ".moduleFile"]
            deployment = props[prefixm + ".deployment"]
            
            # add clusters to mapping modules
            j = 0
            clusters_attrs = ""
            
            while (1):
            	prefixj = prefixi + ".module." + str(m) + ".cluster." + str(j)
            	clusterName = props[prefixj + ".name"]
            	if (clusterName == None):    # No more entries
            		break
            	#endIf
            	
            	if (clusters_attrs == ""):
            		clusters_attrs += "WebSphere:cell=" + cellName + ",cluster=" + clusterName
            	else :
            		clusters_attrs += "+" + "WebSphere:cell=" + cellName + ",cluster=" + clusterName
            	#endElse
            	j = j + 1
            #endWhile
            
            
           	# add application servers to mapping modules
            j = 0
            while (1):
                prefixj = prefixi + ".module." + str(m) + ".server." + str(j)
                serverName = props[prefixj + ".name"]
                if (serverName == None):    # No more entries
                    break
                #endIf
            	
                if (clusters_attrs == ""):
                    serverId = findServer(serverName)
                    nodeName = serverId[serverId.find("/nodes/") + 7 : serverId.find("/servers/")]
                    clusters_attrs += "WebSphere:cell=" + cellName + ",node=" + nodeName + ",server=" + serverName
                else :
                    clusters_attrs += "+" + "WebSphere:cell=" + cellName + ",node=" + nodeName + ",server=" + serverName
                #endElse
                j = j + 1
            #endWhile
            
            webServer = props[prefixm + ".webServer"]
            if ((webServer != "") and (webServer != None)):
				serverId = findServer(webServer)
				nodeName = serverId[serverId.find("/nodes/") + 7 : serverId.find("/servers/")]
				clusters_attrs += "+" + "WebSphere:cell=" + cellName + ",node=" + nodeName + ",server=" + webServer
        	#endIf
        	
            virtualHost_attrs = ""
            virtualHost = props[prefixm + ".virtualHost"]
            if ((virtualHost != "") and (virtualHost != None)):
                virtualHost_attrs = moduleName+ " " + moduleFile + "," + deployment + " " + virtualHost
                mapVHs_attrs = mapVHs_attrs + "[" + virtualHost_attrs + "]"
            #endIf
            
            # add context root for web modules
            ctxRoots_attrs = ""
            ctxRoot = props[prefixm + ".ctxRoot"]
            if ((ctxRoot != "") and (ctxRoot != None)):
                ctxRoots_attrs = moduleName+ " " + moduleFile + "," + deployment + " " + ctxRoot
                mapCtxRoot_attrs = mapCtxRoot_attrs + "[" + ctxRoots_attrs + "]"
            #endIf
            
            # add EJB Reference
            j = 0
            ejbRefs_attrs = ""
            
            while (1):
            	prefixj = prefixi + ".module." + str(m) + ".ejb." + str(j)
            	ejbName = props[prefixj + ".name"]
            	rscRef = props[prefixj + ".rscRef"]
            	refClass = props[prefixj + ".refClass"]
            	ejbRef = props[prefixj + ".ejbRef"]
            	if (ejbRef == None):    # No more entries
            		break
            	#endIf
            	
            	if ((ejbRef != "") and (ejbRef != None)):
            		ejbRefs_attrs = moduleName+ " " + ejbName + " " + moduleFile + "," + deployment + " " + rscRef + " " + refClass + " " + ejbRef
            		mapEjbRef_attrs = mapEjbRef_attrs + "[" + ejbRefs_attrs + "]"
            	#endIf
            	j = j + 1
            #endWhile
            
            # add Resource Reference
            j = 0
            rscRefs_attrs = ""
            
            while (1):
            	prefixj = prefixi + ".module." + str(m) + ".ejb." + str(j)
            	ejbName = props[prefixj + ".name"]
            	rscRef = props[prefixj + ".rscRef"]
            	rscType = props[prefixj + ".rscType"]
            	rscJNDI = props[prefixj + ".rscJNDI"]
            	if (ejbName == None):    # No more entries
            		break
            	#endIf
            	
            	if ((rscJNDI != "") and (rscJNDI != None)):
            		rscRefs_attrs = moduleName+ " " + ejbName + " " + moduleFile + "," + deployment + " " + rscRef + " " +rscType + " " + rscJNDI
            		mapRscRef_attrs = mapRscRef_attrs + "[" + rscRefs_attrs + "]"
            	#endIf
            	j = j + 1
            #endWhile
            
            # add Environment Entries for Web Module
            j = 0
            envEntry_attrs = ""
            
            while (1):
            	prefixj = prefixi + ".module." + str(m) + ".envEntry." + str(j)
            	envEntryName = props[prefixj + ".name"]
            	envEntryValue = props[prefixj + ".value"]
            	envEntryType = props[prefixj + ".type"]
            	envEntryDescription = props[prefixj + ".description"]
            	if (envEntryName == None):    # No more entries
            		break
            	#endIf
            	
            	envEntry_attrs = moduleName + " " + moduleFile + "," + deployment + " " + envEntryName + " " + envEntryType + " " + envEntryDescription + " " + envEntryValue
            	mapEnvEntry_attrs = mapEnvEntry_attrs + "[" + envEntry_attrs + "]"
            	
            	j = j + 1
            #endWhile
            
            module_attrs = moduleName+ " " + moduleFile + "," + deployment + " " + clusters_attrs
            mapModules_attrs =mapModules_attrs + "[" + module_attrs + "]"
            m = m + 1
        #endWhile 
        
        if (mapModules_attrs== ""):
            app_cluster = props[prefixi + ".cluster"]
            
        
        #Install application
        manageApplication(targetContPath, appName, earFile, opps, mapModules_attrs, mapVHs_attrs,mapCtxRoot_attrs, mapEjbRef_attrs, mapRscRef_attrs, mapEnvEntry_attrs ,app_cluster, create)
        i = i+1
    #endWhile 
#endDef


#--------------------------------------------------------------------
# Procedure:   manageResEnvProviders
# Description: Creates/Deletes Resource Environment Providers based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create WAS resources, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageResEnvProviders(props, create):
    propPrefix = "resenvProviders"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        name = props[prefixi + ".name"]
        if (name == None):    # No more related properties
            break
        #endIf
        
        isolatedClassLoader = props[prefixi + ".isolatedClassLoader"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        
        manageResEnvProvider(targetContPath, name, isolatedClassLoader, create)
        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageProvidersRefs
# Description: Creates/Deletes Resource Environment Providers Referenceables based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create WAS resources, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageProvidersRefs(props, create):
    propPrefix = "reference"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        provider = props[prefixi + ".provider"]
        if (provider == None):    # No more related properties
            break
        #endIf
        
        factoryClassname = props[prefixi + ".factoryClassname"]
        classname = props[prefixi + ".classname"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        manageProvidersRef(targetContPath, provider, factoryClassname, classname, create)
        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageResEnvEntries
# Description: Creates/Deletes Resource Environment Entries based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create WAS resources, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageResEnvEntries(props, create):
    propPrefix = "entry"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        name = props[prefixi + ".name"]
        if (name == None):    # No more related properties
            break
        #endIf
        
        jndiName = props[prefixi + ".jndiName"]
        provider = props[prefixi + ".provider"]
        
        factoryClassname = props[prefixi + ".referenceable.factoryClassname"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        
        entry = manageResEnvEntry(targetContPath, name, jndiName,provider,factoryClassname,create)
        if ((entry == None) or (entry == [])):
        	i=i+1
        	continue
        #endIf
        
        # add custom properties (one by one) to newly created entry
        m = 0
        while (1):
        	prefixm = prefixi + ".custom." + str(m)
        	custName = props[prefixm + ".name"]
        	if (custName == None):    # No more entries
        		break
        	#endIf
        	custValue = props[prefixm + ".value"]
        	addCustomPropertyToEntry(entry, custName, custValue)
        	m = m + 1
        #endWhile
        i = i+1
    #endWhile 
#endDef


#--------------------------------------------------------------------
# Procedure:   manageWorkAreaServices
# Description: Creates/Deletes Server Work Area Services based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create WAS resources, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageWorkAreaServices(props, create):
    propPrefix = "workareaservice"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        servername = props[prefixi + ".target.server"]
        if ((servername == "") or (servername == None)):    # No more related properties
            break
        #endIf
        
        enable = props[prefixi + ".enable"]
        enableWebServicePropagation = props[prefixi + ".enableWebServicePropagation"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        workareaservice = findObjectsByTarget("WorkAreaService", targetContPath)
        updateWorkAreaService(workareaservice, enable,enableWebServicePropagation, create)
        i = i+1
    #endWhile 
#endDef


#--------------------------------------------------------------------
# Procedure:   manageNSBinding
# Description: Creates/Deletes Name Space Binding based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create WAS resources, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageNSBindings(props, create):
    propPrefix = "nsbinding"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        identifier = props[prefixi + ".identifier"]
        if ((identifier == "") or (identifier == None)):    # No more related properties
            break
        #endIf
        
        name = props[prefixi + ".name"]
        providerurl = props[prefixi + ".providerurl"]
        jndiname = props[prefixi + ".jndiname"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        nsbinding = manageNSBinding(targetContPath, identifier, name, providerurl, jndiname, create)
        i = i+1
    #endWhile 
#endDef


#--------------------------------------------------------------------
# Procedure:   manageForeCellBindings
# Description: Creates/Deletes Foreign Cell binding based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create Foreign Cell bindings, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageForeCellBindings(props, create):
    propPrefix = "foreigncell"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        name = props[prefixi + ".name"]
        if ((name == "") or (name == None)):    # No more related properties
            break
        #endIf
        
        name = props[prefixi + ".name"]
        host = props[prefixi + ".host"]
        port = props[prefixi + ".port"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        cellBinding = manageForeCellBinding(targetContPath, name, host, port, create)
        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageTimerManagers
# Description: Creates/Deletes timer manager based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#			   create	Boolean if true, create timer manager, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageTimerManagers(props, create):
    propPrefix = "timermanager"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        name = props[prefixi + ".name"]
        if ((name == "") or (name == None)):    # No more related properties
            break
        #endIf
        
        name = props[prefixi + ".name"]
        jndiname = props[prefixi + ".jndiname"]
        description = props[prefixi + ".description"]
        category = props[prefixi + ".category"]
        numalarmthreads = props[prefixi + ".numalarmthreads"]
        defTranClass = props[prefixi + ".defTranClass"]
        serviceNames = props[prefixi + ".serviceNames"]       
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        manageTimerManager(targetContPath, name, jndiname, description, category, numalarmthreads, defTranClass, serviceNames, create)
        i = i+1
    #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageReplicationDomains
# Description: Creates/Deletes replication domain based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#			   create	Boolean if true, create replication domain, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageReplicationDomains(props, create):
    propPrefix = "replicationdomain"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        name = props[prefixi + ".name"]
        if ((name == "") or (name == None)):    # No more related properties
            break
        #endIf
        
        name = props[prefixi + ".name"]
        requestTimeout = props[prefixi + ".requestTimeout"]
        numberOfReplicas = props[prefixi + ".numberOfReplicas"]
        encryptionType = props[prefixi + ".encryptionType"]
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        manageReplicationDomain(targetContPath, name, requestTimeout, numberOfReplicas, encryptionType, create)
        i = i+1
    #endWhile 
#endDef


#--------------------------------------------------------------------
# Procedure:   manageSchedulers
# Description: Creates/Deletes schedulers based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#			   create	Boolean if true, create schedulers, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageSchedulers(props, create):
    propPrefix = "scheduler"
    i = 0
    while (1):
        prefixi = propPrefix + "." + str(i)
        name = props[prefixi + ".name"]
        if ((name == "") or (name == None)):    # No more related properties
            break
        #endIf
        
        name = props[prefixi + ".name"]
        jndiname = props[prefixi + ".jndiname"]
        description = props[prefixi + ".description"]
        category = props[prefixi + ".category"]
        datasourceJNDIName = props[prefixi + ".datasourceJNDIName"]
        datasourceAlias = props[prefixi + ".datasourceAlias"]
        workManagerInfoJNDIName = props[prefixi + ".workManagerInfoJNDIName"]
        tablePrefix = props[prefixi + ".tablePrefix"]
        pollInterval = props[prefixi + ".pollInterval"]
        useAdminRoles = props[prefixi + ".useAdminRoles"]
        
        createTables = props[prefixi + ".createTables"]
        verifyTables = props[prefixi + ".verifyTables"]
        
        targetContPath = getTargetContainmentPathFromProperties(props, prefixi)
        writemsg("Target: " + targetContPath)
        scheduler = manageScheduler(targetContPath, name, jndiname, description, category, datasourceJNDIName, datasourceAlias, workManagerInfoJNDIName,tablePrefix,pollInterval,useAdminRoles,createTables,verifyTables, create)
        
        i = i+1
    #endWhile 
#endDef
