# /*
#  * IBM Confidential OCO Source Material
#  *
#  * Licensed Materials - Property of IBM
#  * 
#  * (C) Copyright IBM Corp. 2011, 2013.  All rights reserved.
#  *
#  * The source code for this program is not published or otherwise divested
#  * of its trade secrets, irrespective of what has been deposited with the
#  * U. S. Copyright Office.
#  *
#  * US Government Users Restricted Rights - Use, duplication or
#  * disclosure restricted by GSA ADP Schedule Contract with IBM
#  * Corp.
#  */

#******************************************************************************
# File Name:   application.py
# Description: This file contains a number of useful wsadmin procs that help
#              isolate the tedious coding for common operations.
# Fix record:
#******************************************************************************
import sys
def listSearch(lst, target):
        if(lst.count(target)<=0): return -1
        return lst.index(target)
def wsadminToList(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split(" ")
        else:
                tmpList = inStr.split("\n")  #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();        #removes any Windows "\r"
                if (len(item)>0):
                        outList.append(item)
        return outList
#endDef
errorCode = ""
errorInfo = ""


#--------------------------------------------------------------------
# Procedure:   installEarToCluster
# Description: Installs an EAR file to the specified cluster.
# Parameters:
#			appName 	"TestEnterpriseApp" 
#			earFile		"/projects/projX/testapp.ear"
#			secRoles	[list of security role mappings]
#			clusterName	"TestCluster"
#			bindingsFile	(opt) file for default bindings
#
# Returns:	None
#--------------------------------------------------------------------
def installEarToCluster ( appName, earFile, secRoles, clusterName, otheropps, bindingsFile = "" ):
        global AdminApp

        traceEntry("installEarToCluster" )
        # Build the AdminApp arguments to install to a cluster.
        opps = "-cluster " + clusterName
	#print "otheropps="+str(otheropps)
        opps += " " + otheropps
	#print "opps="+str(opps)

        # Call the common ear install proc
        installEarToTarget(appName, earFile, secRoles, opps, bindingsFile )
        trace("info", "Installed application "+appName+" to ear file "+earFile+" on cluster "+clusterName+" " )
        traceExit("installEarToCluster" )
#endDef 

#--------------------------------------------------------------------
# Procedure:   installEarToServer
# Description: Installs an EAR file to the specified server on the specified node
# Parameters:
#			appName 	"TestEnterpriseApp" 
#			earFile		"/projects/projX/testapp.ear"
#			secRoles	[list of security role mappings]
#			nodeName	"Node1"
#			serverName	"server1OnNode1"
#			otheropps    other options for installing the ear file
#			bindingsFile	(opt) file for default bindings
#
# Returns:	None
#--------------------------------------------------------------------
def installEarToServer ( appName, earFile, secRoles, nodeName, serverName, otheropps, bindingsFile = "" ):
        global AdminApp

        traceEntry("installEarToServer " )
        # Build the AdminApp arguments to install to an application
        # server.
        opps = "-node" + " " + nodeName
        serverOpps = "-server" + " " + serverName

        opps += " " + serverOpps	
        opps += " " + otheropps

        # Call the common ear install proc
        installEarToTarget(appName, earFile, secRoles, opps, bindingsFile )
        trace("info", "Installed application "+appName+" to ear file "+earFile+" on server "+serverName+" " )
        traceExit("installEarToServer " )
#endDef 


#--------------------------------------------------------------------
# Procedure:   installWarToServer
# Description: Installs an EAR file to the specified server on the specified node
# Parameters:
#			appName 	"TestEnterpriseApp" 
#			earFile		"/projects/projX/testapp.ear"
#			secRoles	[list of security role mappings]
#			nodeName	"Node1"
#			serverName	"server1OnNode1"
#			otheropps    other options for installing the ear file
#			bindingsFile	(opt) file for default bindings
#
# Returns:	None
#--------------------------------------------------------------------
def installWarToServer ( appName, earFile, secRoles, nodeName, serverName, contextRoot, otheropps, bindingsFile = "" ):
        global AdminApp

        traceEntry("installWarToServer" )
        # Build the AdminApp arguments to install to an application
        # server.
        opps = "-node" + " " + nodeName
        serverOpps = "-server" + " " + serverName
        cRoot  = "-contextroot" + " " + contextRoot

        opps += " " + serverOpps
        opps += " " + cRoot
        opps += " " + "-defaultbinding.virtual.host default_host"
        opps += " " + "-usedefaultbindings"
        opps += " " + otheropps

        # Call the common ear install proc
        installEarToTarget(appName, earFile, secRoles, opps, bindingsFile )
        trace("info", "Installed application "+appName+" to ear file "+earFile+" on server "+serverName+" " )
        traceExit("installEarToServer " )
#endDef 

#--------------------------------------------------------------------
# Procedure:   installWarToCluster
# Description: Installs an EAR file to the specified cluster
# Parameters:
#			appName 	"TestEnterpriseApp" 
#			earFile		"/projects/projX/testapp.ear"
#			secRoles	[list of security role mappings]
#			clusterName	"appCluster"
#			otheropps    other options for installing the ear file
#			bindingsFile	(opt) file for default bindings
#
# Returns:	None
#--------------------------------------------------------------------
#def installWarToCluster ( appName, earFile, secRoles, clusterName, contextRoot, otheropps, bindingsFile = "" ):
def installWarToCluster ( appName, earFile, secRoles, clusterName, otheropps, bindingsFile = "" ):
        global AdminApp

        traceEntry("installWarToServer" )
        # Build the AdminApp arguments to install to an application
        # server.
       
        #opps = "-cluster " +  clusterName
        #cRoot  = "-contextroot" + " " + contextRoot

        #opps += " " + cRoot
        #opps += " " + "-defaultbinding.virtual.host default_host"
        #opps += " " + "-usedefaultbindings"
        opps = otheropps

        # Call the common ear install proc
        installEarToTarget(appName, earFile, secRoles, opps, bindingsFile )
        trace("info", "Installed application "+appName+" to ear file "+earFile+" on cluster "+clusterName+" " )
        traceExit("installEarToServer " )
#endDef

#--------------------------------------------------------------------
# Procedure:   installEarToTarget
# Description: Installs an EAR file to the specified deployment target.
#              This usually isn't called directly, but it is used by
#              other procedures to minimize duplicate code.
#
# Parameters:
#			appName 	"TestEnterpriseApp" 
#			earFile		"/projects/projX/testapp.ear"
#			secRoles		[list of security role mappings]
#			deplTargetOpps	"-node Node1 -server server1"
#							or
#						"-cluster TestCluster"
#			bindingsFile	(opt) file for default bindings
#
# Returns:	None
#--------------------------------------------------------------------
def installEarToTarget ( appName, earFile, secRoles, deplTargetOpps, bindingsFile = "" ):
        global AdminApp

        traceEntry("installEarToTarget " )

        # Uninstall the application
	if (str(earFile).endswith("ear")):
        	uninstallApplication(appName)

        # Create each of the options
        appNameOpps = "-appname" + " " + appName
        print "appNameOpps="+appNameOpps

        # Append all of the options together to prepare to pass to the
        # install command.
        opps=""

        if (len(secRoles) != 0):
                secRoleOpps = "-MapRolesToUsers" + " " + secRoles
                opps += secRoleOpps
                print "secRoleOpps="+secRoleOpps
        #endIf 

        opps += appNameOpps + " " + deplTargetOpps
        print "deplTargetOpps="+deplTargetOpps

        # If a bindings file was specified, pass that in as well.
        if (len(bindingsFile) > 0):
                trace("debug", "  Using Bindings file: "+bindingsFile )
                bndOpps = "-defaultbinding.strategy.file" + " " + bindingsFile
                forcebndOpps = "-usedefaultbindings"
                opps += " " + bndOpps
                print "bndOpps="+bndOpps
                opps += " " + forcebndOpps
                print "forcebndOpps="+forcebndOpps
        #endIf 

        opps = "[" + opps + "]"
        trace("debug", "  Installing EAR: " +earFile+ " using opps: " +opps)
        
        AdminApp.install(earFile, opps )
#endDef 

#--------------------------------------------------------------------
# Procedure:   uninstallApplication
# Description: Uninstalls an enterprise application
# Parameters:
#			appName 	"TestEnterpriseApp" 
#
# Returns:	None
#--------------------------------------------------------------------
def uninstallApplication ( appName ):
        global AdminApp

        traceEntry("uninstallApplication" )
        # If the app is still installed, uninstall it
        
	applist = AdminApp.list().split(java.lang.System.getProperty("line.separator"))
	for app in applist:
		#print app
		if (app == appName):
			trace("debug", "Uninstalling "+appName )
			AdminApp.uninstall(appName)
			break
		#endIf 
        
        #appIndex = listSearch(AdminApp.list( ),appName)
        #if (appIndex != -1):
        #        trace("debug", "Uninstalling "+appName )
        #        AdminApp.uninstall(appName )
        #endIf 
        traceExit("uninstallApplication" )
#endDef 



#--------------------------------------------------------------------
# Procedure:   changeWARDelegationMode
# Description: Changes the WAR classloading delegation mode for an
#              enterprise application.
#              This is the same property that is available in the 
#              WAS Console through:
#
#              Enterprise Applications > [EARName]
#              "Classloader Mode" property
#
# Parameters:
#		earName		(Name of the enterprise application)
#		newMode		(Delegation mode (PARENT_FIRST, PARENT_LAST)
#
# Returns:	None
#--------------------------------------------------------------------
def changeWARDelegationMode ( earName, newMode ):
        global AdminConfig

        traceEntry("changeWARDelegationMode" )
        # There has to be a better way to get the web module 
        # from the deployed ear
        deployment = AdminConfig.getid("/Deployment:"+earName+"/" )
        deployedObj = AdminConfig.showAttribute(deployment, "deployedObject" )
        classloader = AdminConfig.showAttribute(deployedObj, "classloader" )

        modeAttrs = ["mode", newMode]
        attrs = [modeAttrs]
        AdminConfig.modify(classloader, attrs )
        trace("info", "Set WAR delegation mode to "+newMode+" on "+earName )
        traceExit("changeWARDelegationMode" )
#endDef 

#--------------------------------------------------------------------
# Procedure:   installApplications
# Description: Installs applications based on the setting in the properties files
#
# Parameters:  props	Properties object
# Returns:     N/A
#--------------------------------------------------------------------
def installApplications ( props ):
        global AdminConfig

        traceEntry("installApplications" )

        i = 0
        appprefix = "application"

        cellName = props["cellName"]
        nodeName = props["nodeName"]
        serverName = props["serverName"]
        clusterName = props["clusterName"]

        while (1):

                #appName = props[appprefix+"."+i+".name"]
                appName = props["%s.%i.name" % (appprefix,i)]

                if ((appName == "") or (appName == None)):
                        break 
                #endIf 

                #earFile = props[appprefix+"."+i+".earFile"]
                earFile = props["%s.%i.earFile" % (appprefix,i)]               
                
                #scope = props[appprefix+"."+i+".scope"]
                scope = props["%s.%i.scope" % (appprefix,i)]
                #bindingsFile = props[appprefix+"."+i+".bindingsFile"]
                bindingsFile = props["%s.%i.bindingsFile" % (appprefix,i)]
                #secRoles = props[appprefix+"."+i+".securityRoles"]
                secRoles = props["%s.%i.securityRoles" % (appprefix,i)]
                trace("debug", "Installing application "+appName+" " )

                if ( scope == "Server" ):
                        installEarToServer(appName, earFile, secRoles, nodeName, serverName, bindingsFile )
                elif ( scope == "Cluster" ):
                        installEarToCluster(appName, earFile, secRoles, clusterName, bindingsFile )
                elif ( scope == "default" ):
                        global errorInfo
                        errorInfo = str(_tbck_.dumpStack())
                        global errorCode
                        errorCode = None
                        raise RuntimeError, "Unknown install scope: "+scope+" for application:"+appName
                #endIf
                trace("info", "Installed application "+appName )

                i = i+1
        #endWhile 
        #?PROBLEM? (jacl 308) COMMAND_UNKNOWN?  traceExit("installApplications" )
#endDef 

#--------------------------------------------------------------------
# Procedure:   simpleAppInstall
# Description: Installs simple applications based on the
# settings in the properties files
#
# Parameters:  props	Properties object
# Returns:     N/A
#--------------------------------------------------------------------
def simpleAppInstall ( props ):
        global AdminConfig

        traceEntry("simpleAppInstall" )

        i = 0
        appprefix = "application"

        cellName = props["cellName"]
        nodeName = props["nodeName"]
        clusterName = props["clusterName"]
        serverName = props["serverName"]
        scope = props["scope"]
        secRoles = ""
        bindingsFile = ""

        while (1):
                #appName = props[appprefix+"."+i+".name"]
                appName = props["%s.%i.name" % (appprefix,i)]

                if ((appName == "") or (appName == None)):
                        break 
                #endIf 
                trace("debug", "Installing application "+appName+" " )
				
				
		
				#if (len(node) > 0 ):
					#trace("info", "Need to install WAR file .. ...  "+warFile )
                #earFile = props[appprefix+"."+i+".earfile"]
                earFile = props["%s.%i.earfile" % (appprefix,i)]
                trace("info", "earFile  .. ...  "+earFile ) 
                contextRoot = props["%s.%i.contextroot" % (appprefix,i)] 
    
                
                if (str(earFile).endswith("war")):               
                	trace("info", "We need to process WAR file" + earFile)               
              
                #deployejb = props[appprefix+"."+i+".deployejb"]
                deployejb = props["%s.%i.deployejb" % (appprefix,i)]
                if ((deployejb =="") or (deployejb == None)):
                        deployejb = "false"
                
                if (deployejb == "true"):
                        deployejbopp = "-deployejb"
                else:
                        deployejbopp = "-nodeployejb"
                #endElse 

                opps = " " + deployejbopp
                
                #deployws = props[appprefix+"."+i+".deployws"]
                deployws = props["%s.%i.deployws" % (appprefix,i)]
                if ((deployws =="") or (deployws == None)):
                        deployws = "false"
                if (deployws == "true"):
                        deploywsopp = "-deployws"
                else:
                        deploywsopp = "-nodeployws"
                #endElse 
                opps += " " + deploywsopp

		if (deployejb == "true"):
	                deployejbdbtype = props["%s.%i.deployejb.dbtype" % (appprefix,i)]
	                deployejbdbtypeopp = "-deployejb.dbtype" + " " + deployejbdbtype
	                opps += " " + deployejbdbtypeopp
	
	                deployejbdbschema = props["%s.%i.deployejb.dbschema" % (appprefix,i)]
	                deployejbdbschemaopp = "-deployejb.dbschema" + " " +deployejbdbschema
	                opps += " " + deployejbdbschemaopp


		sharedLibrary = props["%s.%i.sharedLibrary" % (appprefix,i)]
            	sharedlibopp = sharedLibrary
                dbtype=props["%s.%i.dbtype" % (appprefix,i)]
                if ( (dbtype == "oracle") ):
		        sharedlibopp += "+OracleXMLAccess"
                #endIf  

                mapModuleToVirtualHost = props["%s.%i.mapmoduletovirtualhost" % (appprefix,i)]
                if ( (mapModuleToVirtualHost != "") and (mapModuleToVirtualHost != None) ):
                        mapModuleFileToVirtualHost = props["%s.%i.mapmodulefiletovirtualhost" % (appprefix,i)]
                        modToVHs = mapModuleToVirtualHost.split("+")
                        modFileToVHs = mapModuleFileToVirtualHost.split("+")
                        if (len(modToVHs) != len(modFileToVHs)):
                                global errorInfo
                                errorInfo = str(_tbck_.dumpStack())
                                global errorCode
                                errorCode = None
                                raise RuntimeError, "modToVHs and modFileToVHs mismatched for application: "+appName
						#endIf
					
                        mapWebModToVH_opps = ""
                        for host, file in zip(modToVHs,modFileToVHs):
                                mapWebModToVH_opps += "[" +host+ " " +file+ ",WEB-INF/web.xml default_host] "
                        #endFor
					
                        opps += " "  + "-MapWebModToVH [" +mapWebModToVH_opps+ "]"
                #endIf
                if (sharedlibopp != ""):
                        opps += " " + "-MapSharedLibForMod [[ " + appName + " META-INF/application.xml " + sharedlibopp + " ]]"
                #endIf

                if ( (contextRoot != "") and (contextRoot != None) ):
                        contextRootAppName = props["%s.%i.contextrootappname" % (appprefix,i)]
                        contextRootAppFileName = props["%s.%i.contextrootappfilename" % (appprefix,i)]
                        contextRootForWebMod_opps = "[" +contextRootAppName+ " " +contextRootAppFileName+ ",WEB-INF/web.xml " +contextRoot+ " ]"
                        opps += " "  + "-CtxRootForWebMod [" +contextRootForWebMod_opps+ "]"
                #endIf


                moduleMappings = ""
                j=0
                while (1):
                        moduleName = props["%s.%i.module.%i.name" % (appprefix,i,j)]

                        if ((moduleName == "") or (moduleName == None)):
                                break 
                        #endIf 
                        trace("debug", "Mapping module "+moduleName+" " )

                        moduleFile = props["%s.%i.module.%i.modulefile" % (appprefix,i,j)]
                        deployment = props["%s.%i.module.%i.deployment" % (appprefix,i,j)]
                        cell = props["%s.%i.module.%i.cell" % (appprefix,i,j)]
                        cluster = props["%s.%i.module.%i.cluster" % (appprefix,i,j)]

                        ihs_mapping = ""						
                        k=0
                        while (1):
                                ihsCell = props["%s.%i.module.%i.ihs.%i.ihscell" % (appprefix,i,j,k)]
                                if ((ihsCell == "") or (ihsCell == None)):
                                        break 
                                #endIf

                                ihsServer = props["%s.%i.module.%i.ihs.%i.ihsserver" % (appprefix,i,j,k)]
                                ihsNode = props["%s.%i.module.%i.ihs.%i.ihsnode" % (appprefix,i,j,k)]

                                ihs_mapping += "+WebSphere:cell=" +str(ihsCell)+ ",node=" +str(ihsNode)+ ",server=" +str(ihsServer)

                                k=k+1
                        #endWhile

                        mapping = " [ " +str(moduleName)+ " " +str(moduleFile)+ "," +str(deployment)+ " WebSphere:cell=" +str(cell)+ ",cluster=" +str(cluster) +ihs_mapping+ " ] "
                        moduleMappings += str(mapping)

                        j = j+1
                #endWhile

                if (moduleMappings != ""):
                        opps += " " + "-MapModulesToServers" +  " [" + moduleMappings + "]"
                #endIf

                j = 0
                #output = "blank"
		backendidopps =""
                while (1):
                	output = []
                        #ejbName = props[appprefix+"."+i+".ejb."+j+".name"]
                        ejbNameKey = "%s.%i.ejb.%i.name" % (appprefix,i,j)
                        ejbName = props[ejbNameKey]
                        #ejbFile = props[appprefix+"."+i+".ejb."+j+".jarfile"]
                        ejbFileKey = "%s.%i.ejb.%i.jarfile" % (appprefix,i,j)
                        ejbFile = props[ejbFileKey]
                        #ejbDesc = props[appprefix+"."+i+".ejb."+j+".descriptor"]
                        ejbDescKey = "%s.%i.ejb.%i.descriptor" % (appprefix,i,j)
                        ejbDesc = props[ejbDescKey]
                        #ejbBackendId = props[appprefix+"."+i+".ejb."+j+".backendid"]
                        ejbidKey = "%s.%i.ejb.%i.backendid" % (appprefix,i,j)
                        ejbBackendId = props[ejbidKey]

                        if ((ejbName == None) or (ejbName=="")):
                                break 
                        #endIf 

                        ejbList = ejbName
                        ejbList += " " + ejbFile + "," + ejbDesc + " " + ejbBackendId
                        output = "[" + ejbList + "]"
                        #if (j == 0):
                        #        output = [ejbList]
                        #else:
                        #        output += [" "] + [ejbList]
                        #endElse 
                        if (output != "blank"):
                                backendidopps += " " + output
				
       
                        else:
                                writemsg("No ejb backend id's to configure for application: "+appName )
                        #endElse 
                        j = j+1
                #endWhile
		if ( backendidopps != "") :
                        opps += " " + "-BackendIdSelection" +  " [" + backendidopps + "]"

                trace("debug", "Attributes for this application are " + deployejb + " " + deployws  )
                trace("debug", "Parms for this application are "+appName+" "+earFile+" "+nodeName+" "+str(opps)+" "+serverName+" "+secRoles )
                
                if (str(earFile).endswith("war")):
                        if ( scope == "Server" ):
                                installWarToServer ( appName, earFile, secRoles, nodeName, serverName, contextRoot, bindingsFile ) 
                	
                	elif ( scope == "ServerCluster" ):
                                print "opps="+str(opps)
                                installWarToCluster ( appName, earFile, secRoles, clusterName, opps, bindingsFile )
                        #endIf        
                elif ( scope == "Server" ):
                        installEarToServer(appName, earFile, secRoles, nodeName, serverName, opps, bindingsFile )
                elif ( scope == "ServerCluster" ):
                        print "opps="+str(opps)
                        installEarToCluster(appName, earFile, secRoles, clusterName, opps, bindingsFile )
                elif ( scope == "default" ):
                        global errorInfo
                        errorInfo = str(_tbck_.dumpStack())
                        global errorCode
                        errorCode = None
                        raise RuntimeError, "Unknown install scope: "+scope+" for application:"+appName
                #endIf

                # should classloader mode be set?
                #classLoaderMode = props[appprefix+"."+i+".classLoaderMode"]
                classLoaderMode = props["%s.%i.classLoaderMode" % (appprefix,i)]
                
                if ((classLoaderMode != "") and (classLoaderMode != None)):
                        trace("debug", "Setting class loader mode to "+classLoaderMode )
                        changeWARDelegationMode(appName, classLoaderMode )
                #endIf 

                # defect 145643
                # should application starting weight be set?
                #weight = props[appprefix+"."+i+".startingWeight"]
                weight = props["%s.%i.startingWeight" % (appprefix,i)]
                dep = AdminConfig.getid("/Deployment:"+appName+"/" )
                deployedObject = AdminConfig.showAttribute(dep, "deployedObject" )
                AdminConfig.modify(deployedObject, [["startingWeight", weight]] )
                appWeight = AdminConfig.show(deployedObject, "startingWeight" )
                writemsg("Application "+appName+" : "+appWeight )
                
                trace("info", "Installed application "+appName )

                i = i+1
        #endWhile 
        traceExit("simpleAppinstall" )
#endDef 

#--------------------------------------------------------------------
# Procedure:   simpleAppUninstall
# Description: Uninstalls simple applications based on the
# settings in the properties files
#
# Parameters:  props	Properties object
# Returns:     N/A
#--------------------------------------------------------------------
def simpleAppUninstall ( props ):
        global AdminConfig
	global AdminControl
        traceEntry("simpleAppUninstall" )

        i = 0
        appprefix = "application"

        while (1):
                #appName = props[appprefix+"."+i+".name"]
                appName = props["%s.%i.name" % (appprefix,i)]
		serverName = props["serverName"]
                if ((appName == "") or (appName == None)):
                        break 
                #endIf 
                
		appState= AdminControl.completeObjectName("type=Application,name=" + appName + ",*" )
		trace("debug", "Application state : "+ appName )
		if ((appState != "") ):
			appManager=AdminControl.queryNames("type=ApplicationManager,process=" + serverName + ",*")
			trace("debug", "stopping application "+appName )
			AdminControl.invoke(appManager,"stopApplication",appName)
		trace("debug", "Uninstalling application "+appName ) 
                uninstallApplication(appName )

                i = i+1
        #endWhile 
        traceExit("simpleAppUninstall" )
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageWebModuleClassloader
# Description: Assigns a classloader to a WebModule.
# Parameters:  	eaName		Name of the enterprise application.
#		webModule	Name of the Web Module .
#		sharedLib	Name of the shared library
# Returns:	   Config ID of the specified object.
#--------------------------------------------------------------------
def manageWebModuleClassLoaderAndSharedLib ( props ) :
	global AdminConfig
        i = 0	
	while (1):
		appprefix = "application"
                appname = props["%s.%i.name" % (appprefix,i)]	
                if ((appname == "") or (appname == None)):
			# no more jvm settings to change
                        break
                j = 0
                while (1):
			
			webname = props["%s.%i.web.%i.name" % (appprefix,i,j)]
			
			if ((webname == "") or (webname == None)):
				# no more jvm settings to change
                        	break
			warfile = props["%s.%i.web.%i.warfile" % (appprefix,i,j)]
			classloadermode = props["%s.%i.web.%i.classloadermode" % (appprefix,i,j)]
			print "Custom manageWebModuleClassLoaderAndSharedLib appname =" +appname+ " warfile= " + warfile + " classloadermode =" + classloadermode
			if (( warfile != "") and ( classloadermode !="") ) :
				assignClassloaderToWebModule ( appName, warfile, classloadermode )

			descriptor = props["%s.%i.web.%i.descriptor" % (appprefix,i,j)]
			sharedlib = props["%s.%i.web.%i.sharedlib" % (appprefix,i,j)]
			print "Custom manageWebModuleClassLoaderAndSharedLib appname =" +appname+ " descriptor= " + descriptor + " sharedlib =" + sharedlib 
			if( ( descriptor != "" ) and ( sharedlib != "") ) :
				assignSharedLibToWebModule ( appName, webname, warfile, descriptor, sharedlib )
			
			j = j+1
		
		#endWhile
		i = i+1
	#endWhile
#endDef
