# /*
#  * IBM Confidential OCO Source Material
#  *
#  * Licensed Materials - Property of IBM
#  * 
#  * (C) Copyright IBM Corp. 2011, 2013  All rights reserved.
#  *
#  * The source code for this program is not published or otherwise divested
#  * of its trade secrets, irrespective of what has been deposited with the
#  * U. S. Copyright Office.
#  *
#  * US Government Users Restricted Rights - Use, duplication or
#  * disclosure restricted by GSA ADP Schedule Contract with IBM
#  * Corp.
#  */

#******************************************************************************
# File Name:   propsfileprocs.py
# Description: This file contains a number of useful wsadmin procs that help
#              isolate the tedious coding for common operations.
# Fix record:
#******************************************************************************

#--------------------------------------------------------------------
# Procedure:   manageWASVariables
# Description: Creates/Deletes WebSphere variables based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create Properties, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageWASVariables ( props, create ):
        global AdminConfig
        i = 0
        varprefix = "wasvariable"
        while (1):
                propName = varprefix + "." + str(i) + ".name"
                wasvarname = props.getProperty(propName,"")
                if ((wasvarname == "") or (wasvarname == None)):
                        break 
                #endIf 
                scope = props[varprefix+"."+ str(i) + ".scope"]
                value = props[varprefix+"."+ str(i) + ".value"]
                description = props[varprefix+"."+ str(i)+ ".description"]
                scopename = props[varprefix+"."+ str(i) + ".scopename"]	
                writemsg("buildTargetList scope =" + scope )
                targetList = buildTargetList(scope,scopename)
                # For each target node, create/delete the WAS variable
                for targetName in targetList:
			writemsg("scope=" + scope + " target is " + targetName)
                        writemsg("wasvarname=" + wasvarname + " value=" + value )
                        #if ( (scope == "ServerCluster") and !(scopename == targetName) ) :
                        #	writemsg("only crate in cluster name =" + scopename )
                        #	continue
                        if (create):
                                manageWASVariable(scope, targetName, wasvarname, value, description, create )
                                writemsg("wasvarname=" + wasvarname + " created" )
                        else:
                                if ( wasvarname == "ISS_INSTALL_ROOT" ):
                                	manageWASVariable(scope, targetName, wasvarname, value, description, create )
                                	writemsg("wasvarname=" + wasvarname + " deleted" )	
                                else:
                                	writemsg("variable reserved for other applications" )
                        #endElse 
                #endFor 
                i = i+1
        #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJ2CAliases
# Description: Creates/Deletes J2C aliases based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, aliases will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageJ2CAliases ( props, create ):
        i = 0
        done = "false"
        jaasprefix = "jaasAuthAlias"

        # Create an authentication alias for each set of entries in the 
        # properties file that begin with 'jaasAuthAlias'.
        #
        # Entries in the properties file should look like this, and start 
        # with 0:
        #
        #	jaasAuthAlias.0.name=LDAP_ALIAS
        #	jaasAuthAlias.0.user=cn=root
        #	jaasAuthAlias.0.password=passw0rd

        while (1):
                propName = jaasprefix + "." + str(i) + ".name"
		authAliasName = props.getProperty(propName,"")
                # If the next numbered entry didn't exist, break out of this loop
                if ((authAliasName == "") or (authAliasName == None)):
                        break 
                #endIf 

                user = props[jaasprefix + "." + str(i) + ".user"]
                password = props[jaasprefix + "." + str(i) + ".password"]

                if (create):
                        writemsg("Creating J2C alias: "+authAliasName )
                else:
                        writemsg("Deleting J2C alias: "+authAliasName )
                #endElse
                cellName = getCellName()
                manageJ2CAuthEntry(cellName, authAliasName, user, password, create )
                i = i+1
        #endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJDBCResources
# Description: Creates/Deletes JDBC Providers and Data Sources based on the 
#              values in the Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true create JDBC resources, else delete them
# Returns:     N/A
# 2004/10/09   senegal   Fix AuthAlias
# 2004/10/28   ktchu     Added db2 possibility 
#--------------------------------------------------------------------
def manageJDBCResources ( props, create ):
        # Create JDBC Providers
        i = 0
        dbType = props["jdbcprovider.type"]
        if ((dbType == None) or (dbType == "")):
        	return;
        #endIf
        jdbcprefix = dbType + ".jdbcprovider"

        while (1):
                propName = jdbcprefix + "." + str(i) + ".name"
		jdbcprovidername = props.getProperty(propName,"")
                if ((jdbcprovidername == "") or (jdbcprovidername == None)):
                        break 
                #endIf 

                scope = props[jdbcprefix + "." + str(i) + ".scope"]
                scopename = props[jdbcprefix+"."+ str(i) + ".scopename"]
                description = props[jdbcprefix + "." + str(i) + ".description"]
                driverClassPath = props[jdbcprefix + "." + str(i) + ".driverClassPath"]
                classname = props[jdbcprefix + "." + str(i) + ".classname"]
		providerType = props[jdbcprefix + "." + str(i) + ".providerType"]
		
                targetList = buildTargetList(scope,scopename)
                nodeName = props["nodeName"]
                # For each target node, create/delete the JDBC provider
                for targetName in targetList:
                         #if (targetName == nodeName):
                                if (create):
                                        writemsg("  Creating "+jdbcprovidername+" on "+targetName+" of type "+dbType )
                                        manageJDBCProviderWithScope(scope, targetName, jdbcprovidername, description, driverClassPath, classname, providerType, create )
                                else:
                                        writemsg("  Retaining "+jdbcprovidername+" on "+targetName )
                                #endElse 
                                j = 0
                                while (1):
                                        dsprefix = jdbcprefix + "." + str(i) + ".datasource." + str(j)
                                        propName = dsprefix + ".name"
                                        dsname = props.getProperty(propName,"")
                                        if ((dsname == "") or (dsname == None)):
                                                # No more data sources to process
                                                break 
                                        #endIf 

                                        jndiname = props[dsprefix + ".jndiname"]
                                        desc = props[dsprefix + ".description"]
                                        stmtcachesize = props[dsprefix + ".statementCacheSize"]
                                        helperclass = props[dsprefix + ".helperclass"]
                                        authAlias = props[dsprefix + ".componentAuthAlias"]

                                        #get first custom property
                                        nameProp = props[dsprefix + ".custom.0.name" ]
                                        name = ["name", nameProp]
                                        valueProp = props[dsprefix+ ".custom.0.value"]
                                        value = ["value", valueProp]
                                        attr = [name, value]

                                        if (not create):
                                                # delete datasource
                                                ds = findDataSourceWithScope(scope, targetName, dsname )
                                                if (ds != 0):
                                                        trace("info", "Datasource "+dsname+" already exists" )
                                                        global AdminConfig
                                                        AdminConfig.remove(ds)
                                                #endIf 
                                        #endIf 
                                        else:
                                                # create datasource 
                                                createDataSourceWithScope(scope, targetName, jdbcprovidername, dsname, desc, jndiname, helperclass, stmtcachesize, authAlias, attr )
                                                #add custom properties (one by one) to newly created datasource
                                                m = 1
                                                while (1):
                                                        custPropPre = dsprefix + ".custom." + str(m)
                                                        custName = props.getProperty(custPropPre + ".name" , "" )
                                                        if ((custName == "") or (custName == None)):
                                                                # No more custom properties
                                                                break 
                                                        #endIf 
                                                        custValue = props[custPropPre + ".value"]
                                                        addCustomPropertyToDataSourceWithScope(scope, targetName, dsname, custName, custValue )
                                                        m = m + 1
                                                #endWhile 

                                                # set the data sdurce connection pool properties
                                                connectionPoolMax = props[dsprefix + ".connectionPoolMax"]
                                                connectionPoolMin = props[dsprefix + ".connectionPoolMin"]

                                                if (connectionPoolMax != "" or connectionPoolMin != ""  ):
                                                        writemsg("Creating connection pool on "+targetName )
                                                        setConnectionPool(scope, targetName, dsname, connectionPoolMax, connectionPoolMin )                 
                                                #endIf 

                                                mappingConfigAlias = props[dsprefix + ".mappingConfigAlias"]
                                                authDataAlias = props[dsprefix + ".containerAuthAlias"]
                                                if (( mappingConfigAlias != "")  or ( authDataAlias != "") ):
                                                        writemsg("Creating Mapping module on "+targetName )
                                                        setMappingModule(scope, targetName, dsname, authDataAlias, mappingConfigAlias )
                                                #endIf 

                                                cmpon = props.getProperty(dsprefix + ".cmpon" , "")
                                                if (( cmpon != "") and (cmpon != None)):
                                                        writemsg("Creating DataSourceCMP on "+targetName )
                                                        setDataSourceCMP(scope, targetName, dsname, authDataAlias )
                                                #endIf 
                                        #endElse 
                                        j = j+1
                                #endWhile 
                        #endIf 
                #endFor 
                i = i+1
        #endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageQueueDestinations
# Description: Creates/Deletes Queue Destination on the 
#              values in the Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true create Queue Destination resources, else delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageQueueDestinations ( props, create ):
        i = 0
        destinationprefix = "mqqueuedestination"

        while (1):
                #name = props[destinationprefix+"."+i+".name"]
                name = props["%s.%i.name" % (destinationprefix,i)]

                # If the next name entry didn't exist, break out of this loop
                if ((name == "") or (name==None)):
                        break 
                #endIf 
                #providerName = props[destinationprefix+"."+i+".providerName"]
                providerName = props["%s.%i.providerName" % (destinationprefix,i)]
                #scope = props[destinationprefix+"."+i+".scope"]
                scope = props["%s.%i.scope" % (destinationprefix,i)]
                scopename = props["%s.%i.scopename" % (destinationprefix,i)]
                targetList = buildTargetList(scope,scopename)

                # For each target node, create the queue destination
                for targetName in targetList:
                        #?PROBLEM? (jacl 546) previous FOREACH_ITEM_IN_LIST may need variable.split("xx")
                        #jndiName = props[destinationprefix+"."+i+".jndiName"]
                        jndiName = props["%s.%i.jndiName" % (destinationprefix,i)]
                        #baseQueueName = props[destinationprefix+"."+i+".baseQueueName"]
                        baseQueueName = props["%s.%i.baseQueueName" % (destinationprefix,i)]
                        #baseQueueMgr = props[destinationprefix+"."+i+".baseQueueMgr"]
                        baseQueueMgr = props["%s.%i.baseQueueMgr" % (destinationprefix,i)]
                        #targetClient = props[destinationprefix+"."+i+".targetClient"]
                        targetClient = props["%s.%i.targetClient" % (destinationprefix,i)]
                        #CCSID = props[destinationprefix+"."+i+".CCSID"]
                        CCSID = props["%s.%i.CCSID" % (destinationprefix,i)]

                        if (( create ) ):
                                writemsg("  Creating MQ Queue Destination "+name )
                        else:
                                writemsg("  Deleting MQ Queue Destination "+name )
                        #endElse 
                        manageMQQueueDestination(providerName, scope, targetName, name, jndiName, baseQueueName, baseQueueMgr, targetClient, CCSID, create )
                #endFor 
                i = i+1
        #endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageListenerPorts
# Description: Create/Delete Message Listener Ports based on the 
#              values in the Properties object.
#
# Parameters:  props	Properties object
#		create	(Boolean if true, create listener ports, else delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageListenerPorts ( props, create ):
        # Create/Delete listener ports
        i = 0
        while (1):
                lpprefix = "listenerport.%i" % (i)
                lpname = props[lpprefix+".name"]
                
                if ((lpname == None) or (lpname == "")):
                        # no more listener ports to create, so break out
                        break 
                #endIf 

                servername = props[lpprefix+".servername"]
                cfjndiname = props[lpprefix+".cfjndiname"]
                destjndiname = props[lpprefix+".destjndiname"]
                maxsessions = props[lpprefix+".maxsessions"]
                maxretries = props[lpprefix+".maxretries"]
                maxmessages = props[lpprefix+".maxmessages"]

                if (( create ) ):
                        writemsg("    Creating listener port: " + lpname )
                else:
                        writemsg("    Deleting listener port: " + lpname )
                #endElse 
                manageMessageListener(servername, lpname, cfjndiname, destjndiname, maxsessions, maxretries, maxmessages, create )

                i = i+1
        #endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJAASAppLogins
# Description: Creates/Deletes JAAS Application Logins based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, create Properties, otherwise delete them
# Returns:     N/A
#--------------------------------------------------------------------
def manageJAASAppLogins ( props, create ):
        global AdminConfig

        i = 0
        varprefix = "jaasconfig"

        while (1):
                writemsg("reading variable" )
                wasvarname = props[varprefix+"."+str(i)+".name"]

                if ((wasvarname == "") or (wasvarname == None)):
                        break 
                #endIf 

                # set cellName	[$props getProperty $varprefix.$i.scope]
                cellName = "discovery"

                appLoginAlias = props[varprefix+"."+str(i)+".name"]
                proxyClassName = props[varprefix+"."+str(i)+".proxyclass"]
                loginModuleClassName = props[varprefix+"."+str(i)+".loginmoduleclass"]
                authStrategy = props[varprefix+"."+str(i)+".authstrategy"]

                manageJAASAppLogin(cellName, appLoginAlias, proxyClassName, loginModuleClassName, authStrategy, create )

                if (create):
                        writemsg("AppLogin created" )
                else:
                        writemsg("AppLogin deleted" )
                #endElse 

                i = i+1
        #endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJAASAppLogin
# Description: Creates/Deletes a JAAS Application Login
#		?? If the alias already exists the auth Alias will not be replaced ??
#
# Parameters:
#		cellName	(Name of Cell)
#		loginName	(Login name) 
#		className	(LoginModule className)
#		create		(Boolean if true, create entry, else delete it)
# Returns:	targetName (success) 0 (does not exist)
#--------------------------------------------------------------------
def manageJAASAppLogin ( cellName, appLoginAlias, proxyClassName, loginModuleClassName, loginStrategy, create ):
        global AdminApp, AdminConfig
        module_attrib = []
        prop_attrib = []

        #	puts stdout "The value of cellName is $cellName"
        #	puts stdout "The value of appLoginAlias is $appLoginAlias"
        #	puts stdout "The value of proxyClassName is $proxyClassName"
        #	puts stdout "The value of loginModuleClassName is $loginModuleClassName"
        #	puts stdout "The value of loginStrategy is $loginStrategy"
        #	puts stdout "The value of create is $create"

        #get the configuration id
        appLoginConfigId = AdminConfig.list("JAASConfiguration" ).split(java.lang.System.getProperty("line.separator"))[0]
        #?PROBLEM? (jacl 119) previous LINDEX may need variable.split("xx")

        # create the alias
        if (create):
                attrs0 = [["alias", appLoginAlias]]
                AdminConfig.create("JAASConfigurationEntry", appLoginConfigId, attrs0 )

                # list the configuration entries and loop until we find the one we just made
                JAASConfigEntryId = AdminConfig.list("JAASConfigurationEntry" ).split(java.lang.System.getProperty("line.separator"))
                for jaasConfigEntryId in JAASConfigEntryId:
                        #?PROBLEM? (jacl 128) previous FOREACH_ITEM_IN_LIST may need variable.split("xx")
                        alias = AdminConfig.showAttribute(jaasConfigEntryId, "alias" )
                        if (cmp(alias, appLoginAlias) == 0):
                                break 
                        #endIf 
                #endFor 

                module_attrib.append(["moduleClassName", proxyClassName ])
                module_attrib.append(["authenticationStrategy", loginStrategy ])


                # create the loginModule 
                newJAASLoginModuleId = AdminConfig.create("JAASLoginModule", jaasConfigEntryId, module_attrib )

                prop_attrib.append(["name", "delegate"])
                prop_attrib.append(["value", loginModuleClassName ])

                newPropertyId = AdminConfig.create("Property", newJAASLoginModuleId, prop_attrib )
        else:

                # remove the JAASConfigurationEnry
                objid = findJAASConfigurationEntry(appLoginAlias )
                AdminConfig.remove(objid )
        #endElse 
#endDef 

#--------------------------------------------------------------------
# Procedure:   createSharedLibraries
# Description: Creates shared libraries based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
# Returns:     N/A
#--------------------------------------------------------------------
def createSharedLibraries ( props ):
	global AdminConfig

	# Note: The classpath value should be:
	#	- contained in double quotes, and
	#	- have curly braces escaped with backslash
	i = 0
	slprefix = "sharedlib"

	while (1):
		sharedlibname = props[slprefix+"."+str(i)+".name"]
		if ((sharedlibname == "") or (sharedlibname == None)):
			break 
		#endIf 

		# defect 152065
		dbType = props["jdbcprovider.type"]
		if ((sharedlibname == "OracleXMLAccess") and (dbType != "oracle" )):
			print "skip shared library for oracle only" 
		# defect 152065
		else:
			scope = props[slprefix+"."+str(i)+".scope"]
			scopename = props[slprefix+"."+str(i)+".scopename"]
			classpath = props[slprefix+"."+str(i)+".classpath"]
			nameAttr = ["name", sharedlibname]
			classPathAttr = ["classPath", classpath]
			attrs = [nameAttr, classPathAttr]
			
			targetList = buildTargetList(scope,scopename)
			for targetName in targetList:
				# Create the shared library on each of the specified targets.
				targetId = getConfigObject(targetName, scope )
				existingSharedLibs = AdminConfig.list('Library', targetId)
				
				existingSharedLibsList = []
				if ( existingSharedLibs ):
					existingSharedLibsList = existingSharedLibs.split(java.lang.System.getProperty("line.separator"))
				#endIf
				
				isLibExists = 0
				for lib in existingSharedLibsList:
					libName = AdminConfig.showAttribute(lib, "name")
					if ( AdminConfig.showAttribute(lib, "name") == sharedlibname):
						isLibExists = 1
						print " Modifying shared library '" + str(lib) + "' with attributes '" + str(attrs) + "'" 
						AdminConfig.modify(lib, attrs)
					#endIf
				#endFor
				
				if (isLibExists == 0):
					print " Creating shared library with attributes '" + str(attrs) + "'"
					AdminConfig.create("Library", targetId, attrs)
				#endIf
				
			#endFor
		#endElse
		i = i+1
	#endWhile 
#endDef 

#--------------------------------------------------------------------
# Procedure:   removeSharedLibraries
# Description: removes shared libraries based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		
# Returns:     N/A
#--------------------------------------------------------------------

def removeSharedLibraries ( props ):
        global AdminConfig

        i = 0
        slprefix = "sharedlib"

        while (1):
                sharedlibname = props[slprefix+"."+str(i)+".name"]

                if ((sharedlibname == "") or (sharedlibname == None)):
                        break 
                #endIf 

                libID = findSharedLibrary(sharedlibname )

                if (libID != 0):
                        AdminConfig.remove(libID )
                else:
                        writemsg("library "+sharedlibname+" does not exist" )
                #endElse 

                i = i+1
        #endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageQueueConnectionFactories
# Description: Creates/Deletes MQ Queue Connection Factory based on the 
#              values in the Properties object.
#
# Parameters:  props	Properties object
#		create	(Boolean if true, create the factory objects, otherwise, delete them)
# Returns:     N/A
#--------------------------------------------------------------------
def manageQueueConnectionFactories ( props, create ):
        i = 0
        # Create/Delete MQ Queue Connection Factory
        connfactprefix = "mqqueueconnfac"

        while (1):
                #queueConnFactName 
                name = props[connfactprefix+"."+str(i)+".name"]

                # If the next name entry didn't exist, break out of this loop
                #queueConnFactName
                if ((name == "") or (name == None)):

                        break 
                #endIf 
                providerName = props[connfactprefix+"."+str(i)+".providerName"]
                scope = props[connfactprefix+"."+str(i)+".scope"]
                scopename = props[connfactprefix+"."+str(i)+".scopename"]
                targetList = buildTargetList(scope,scopename)

                # For each target node, create the connection factory
                for targetName in targetList:
                        #?PROBLEM? (jacl 441) previous FOREACH_ITEM_IN_LIST may need variable.split("xx")
                        jndiName = props[connfactprefix+"."+str(i)+".jndiName"]
                        queueManager = props[connfactprefix+"."+str(i)+".queueManager"]
                        transportType = props[connfactprefix+"."+str(i)+".transportType"]
                        clientID = props[connfactprefix+"."+str(i)+".clientID"]
                        msgRetention = props[connfactprefix+"."+str(i)+".msgRetention"]
                        XAEnabled = props[connfactprefix+"."+str(i)+".XAEnabled"]
                        CCSID = props[connfactprefix+"."+str(i)+".CCSID"]

                        #set connection pool settings
                        conTimeoutValue = props[connfactprefix+"."+str(i)+".connectionpool.timeout"]
                        conTimeout = ["connectionTimeout", conTimeoutValue]
                        conMaxValue = props[connfactprefix+"."+str(i)+".connectionpool.maxconns"]
                        conMax = ["maxConnections", conMaxValue]
                        conMinValue = props[connfactprefix+"."+str(i)+".connectionpool.minconns"]
                        conMin = ["minConnections", conMinValue]
                        conAttrs = [conTimeout, conMax, conMin]

                        #set session pool settings
                        sessTimeoutValue = props[connfactprefix+"."+str(i)+".sessionpool.timeout"]
                        sessTimeout = ["connectionTimeout", sessTimeoutValue]
                        sessMaxValue = props[connfactprefix+"."+str(i)+".sessionpool.maxconns"]
                        sessMax = ["maxConnections", sessMaxValue]
                        sessMinValue = props[connfactprefix+"."+str(i)+".sessionpool.minconns"]
                        sessMin = ["minConnections", sessMinValue]
                        sessAttrs = [sessTimeout, sessMax, sessMin]

                        if (( create ) ):
                                writemsg("  Creating MQ Queue Connection Factory" )
                        else:
                                writemsg("  Deleting MQ Queue Connection Factory" )
                        #endElse 

                        manageMQQueueConnectionFactory(providerName, scope, targetName, name, jndiName, queueManager, transportType, clientID, msgRetention, XAEnabled, CCSID, conAttrs, sessAttrs, create )
                #endFor 
                i = i+1
        #endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageRemoteQueueConnectionFactory
# Description: Creates/Deletes MQ Queue Connection Factory based on the 
#              values in the Properties object.
#
# Parameters:  props	Properties object
#		create	(Boolean if true, create the factory objects, otherwise, delete them)
# Returns:     N/A
#--------------------------------------------------------------------
def manageRemoteQueueConnectionFactory ( props, create ):
        i = 0
        # Create/Delete MQ Queue Connection Factory
        connfactprefix = "mqqueueconnfac"

        while (1):
                #queueConnFactName 
                name = props[connfactprefix+"."+str(i)+".name"]

                # If the next name entry didn't exist, break out of this loop
                #queueConnFactName
                if ((name == "") or (name == None)):

                        break 
                #endIf 
                providerName = props[connfactprefix+"."+str(i)+".providerName"]
                scope = props[connfactprefix+"."+str(i)+".scope"]
                scopename = props[connfactprefix+"."+str(i)+".scopename"]
                targetList = buildTargetList(scope,scopename)

                # For each target node, create the connection factory
                for targetName in targetList:
                        #?PROBLEM? (jacl 441) previous FOREACH_ITEM_IN_LIST may need variable.split("xx")
                        jndiName = props[connfactprefix+"."+str(i)+".jndiName"]
                        queueManager = props[connfactprefix+"."+str(i)+".queueManager"]
                        host = props[connfactprefix+"."+str(i)+".host"]
                        port = props[connfactprefix+"."+str(i)+".port"]
                        channel = props[connfactprefix+"."+str(i)+".channel"]
                        transportType = props[connfactprefix+"."+str(i)+".transportType"]
                        clientID = props[connfactprefix+"."+str(i)+".clientID"]
                        msgRetention = props[connfactprefix+"."+str(i)+".msgRetention"]
                        XAEnabled = props[connfactprefix+"."+str(i)+".XAEnabled"]
                        CCSID = props[connfactprefix+"."+str(i)+".CCSID"]
                        componentAuthAlias = props[connfactprefix+"."+str(i)+".componentAuthAlias"]

                        #set connection pool settings
                        conTimeoutValue = props[connfactprefix+"."+str(i)+".connectionpool.timeout"]
                        conTimeout = ["connectionTimeout", conTimeoutValue]
                        conMaxValue = props[connfactprefix+"."+str(i)+".connectionpool.maxconns"]
                        conMax = ["maxConnections", conMaxValue]
                        conMinValue = props[connfactprefix+"."+str(i)+".connectionpool.minconns"]
                        conMin = ["minConnections", conMinValue]
                        conAttrs = [conTimeout, conMax, conMin]

                        #set session pool settings
                        sessTimeoutValue = props[connfactprefix+"."+str(i)+".sessionpool.timeout"]
                        sessTimeout = ["connectionTimeout", sessTimeoutValue]
                        sessMaxValue = props[connfactprefix+"."+str(i)+".sessionpool.maxconns"]
                        sessMax = ["maxConnections", sessMaxValue]
                        sessMinValue = props[connfactprefix+"."+str(i)+".sessionpool.minconns"]
                        sessMin = ["minConnections", sessMinValue]
                        sessAttrs = [sessTimeout, sessMax, sessMin]

                        if (( create ) ):
                                writemsg("  Creating MQ Queue Connection Factory" )
                        else:
                                writemsg("  Deleting MQ Queue Connection Factory" )
                        #endElse 

                        manageRemoteMQQueueConnectionFactory(providerName, scope, targetName, name, jndiName, queueManager, host, port, channel, transportType, clientID, msgRetention, XAEnabled, CCSID, conAttrs, sessAttrs, componentAuthAlias, create )
                #endFor 
                i = i+1
        #endWhile 

#endDef
#--------------------------------------------------------------------
# Procedure:   mamageJVMSettings
# Description: Creates/Deletes custom JVM properties based on a
#              properties file.
# Parameters:  props	Properties object
#		create (Boolean if true, add JVM property setting, else delete it)
# Returns:     N/A
#--------------------------------------------------------------------

def manageJVMSettings ( props, create ):
        global AdminConfig
        i = 0

        while (1):
                key = "jvm."+str(i)
                servername = props[key+".servername"]
                nodename = props[key+".nodename"]
                #maxheapsize = props[key+".maxheapsize"]
                #initheapsize = props[key+".initheapsize"]

                if ((servername == "") or (servername == None)):
                        # no more jvm settings to change
                        break 
                #endIf 

                #updateJvmSettings(servername, maxheapsize, initheapsize, create )

                j = 0
                while (1):
                        key = "jvm."+str(i)+".property."+str(j)
                        propertyname = props[key+".name"]
                        propertyvalue = props[key+".value"]
                        propertydescript = props[key+".description"]

                        if ((propertyname == "") or (propertyname == None)):
                                # no more jvm propertiessettings to change
                                break 
                        #endIf 
                        # Change JVM Properties Settings
                        writemsg("Changing JVM settings for "+servername +" at node:"+nodename )

                        updateJvmProperties(nodename, servername, propertyname, propertyvalue, propertydescript, create )

                        j = j+1
                #endWhile 
                i = i+1
        #endWhile 

#endDef 

def updateJvmSettings ( servername, maxheapsize, initheapsize, create ):
        global AdminConfig
        global AdminControl

        maxheapattr = ["maximumHeapSize", maxheapsize]
        initheapattr = ["initialHeapSize", initheapsize]
        attrs = [maxheapattr, initheapattr]
        node = AdminControl.getNode().split(java.lang.System.getProperty("line.separator"))[0]
        server = AdminConfig.getid("/Node:" + node + "/Server:"+servername+"/" )
        processDef = AdminConfig.list("JavaProcessDef", server )
        jvmEntries = AdminConfig.list("JavaVirtualMachine", processDef )

        AdminConfig.modify(jvmEntries, attrs )
#endDef 

def updateJvmProperties ( nodename, servername, propertyname, propertyvalue, propertydescript, create ):
        global AdminConfig
        global AdminControl

        # if prop values were specified, use them to create a custom jvm prop
        if (propertyname == "" or propertyvalue == ""):
                return 
        #endIf 
        #node = AdminControl.getNode().split(java.lang.System.getProperty("line.separator"))[0]
        server = AdminConfig.getid("/Node:" + nodename + "/Server:"+servername+"/" )
        processDef = AdminConfig.list("JavaProcessDef", server )
        jvmEntries = AdminConfig.list("JavaVirtualMachine", processDef )
        #jvmCustomProp = findConfigTarget(propertyname, "Property" )

        jvmCustomProp = findConfigTargetWithNodeName(propertyname, nodename, "Property" )
        print "jvmCustomProp: "+str(jvmCustomProp)

        if (jvmCustomProp != 0):
                if (( create ) ):
                        trace("info", "Custom JVM property named '"+propertyname+"' already exists" )
                else:
                        trace("info", "Deleting custom JVM property named '"+propertyname+"'" )
                        AdminConfig.remove(jvmCustomProp )
                #endElse 

        else:
                if (( create ) ):
                        writemsg("Creating custom JVM property named '"+propertyname+"' with value '"+propertyvalue+"' and description '"+propertydescript+"'" )
                        namelist = ["name", propertyname]
                        valuelist = ["value", propertyvalue]
                        descriptlist = ["description", propertydescript]
                        attrs = [namelist, valuelist, descriptlist]
                        AdminConfig.create("Property", jvmEntries, attrs )
                else:
                        trace("info", "Custom JVM property named '"+propertyname+"' doesn't exist" )
                #endElse 
        #endElse 
#endDef 


#--------------------------------------------------------------------
# Procedure:   manageSIBuses
# Description: Creates/Deletes SI buses based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, buses will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSIBuses ( props, create ):
        i = 0
        done = "false"
        busprefix = "bus"

        while (1):
		propName = busprefix + "." + str(i) + ".name"
		busName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (busName == ""):
                        break 
                #endIf 

                busDescription = props[busprefix+"."+ str(i) + ".description"]
                busSecurity = props[busprefix+"."+ str(i) + ".security"]

                manageSIBus(busName, busDescription, busSecurity, create)
                i = i+1
        #endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageSIBusMembers
# Description: Creates/Deletes SI bus members based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, bus members will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSIBusMembers ( props, create ):
	i = 0
	done = "false"
	busmemberprefix = "busmember"

	while (1):
		propName = busmemberprefix + "." + str(i) + ".busname"
		busName = props.getProperty(propName,"")

		# If the next numbered entry didn't exist, break out of this loop
		if (busName == ""):
			break 
		#endIf 

		scope = props[busmemberprefix+"."+ str(i) + ".scope"]
		scopename = props[busmemberprefix+"."+ str(i) + ".scopename"]
		serverName = props[busmemberprefix+"."+ str(i) + ".servername"]
		clusterName = props[busmemberprefix+"."+ str(i) + ".clustername"]
		createDefaultDatasource = props[busmemberprefix+"."+ str(i) + ".createDefaultDatasource"]
		datasourceJndiName = props[busmemberprefix+"."+ str(i) + ".datasourceJndiName"]
		authAlias = props[busmemberprefix+"."+ str(i) + ".authAlias"]
		createTables = props[busmemberprefix+"."+ str(i) + ".createTables"]
		schemaName = props[busmemberprefix+"."+ str(i) + ".schemaName"]
               
		targetList = buildTargetList(scope,scopename)
		# For each target node, create/delete SI bus member
		for targetName in targetList:
			writemsg("scope=" + scope + " target is " + targetName)
			writemsg("bus member busName=" + busName + " serverName="+serverName+" clusterName="+clusterName)
			#manageSIBusMember(scope,targetName,busName, serverName, clusterName, create)
			manageSIBusMember(scope,targetName,busName, serverName, clusterName, createDefaultDatasource, datasourceJndiName, authAlias, createTables, schemaName, create)
		#endFor 
		i = i+1
	#endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageSIBusDestinations
# Description: Creates/Deletes SI buses destinations based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, bus destinations will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSIBusDestinations ( props, create ):
	i = 0
	done = "false"
	busdestprefix = "busdestination"

	while (1):
		propName = busdestprefix + "." + str(i) + ".name"
		destName = props.getProperty(propName,"")

		# If the next numbered entry didn't exist, break out of this loop
		if (destName == ""):
			break 
		#endIf 

		scope = props[busdestprefix+"."+ str(i) + ".scope"]
		serverName = props[busdestprefix+"."+ str(i) + ".servername"]
		busName = props[busdestprefix+"."+ str(i) + ".busname"]
		destinationType = props[busdestprefix+"."+ str(i) + ".type"]
		scopename = props[busdestprefix+"."+ str(i) + ".scopename"]
		clustername = props[busdestprefix+"."+ str(i) + ".clustername"]
		
		targetList = buildTargetList(scope,scopename)
		# For each target node, create/delete SI bus destination
		for targetName in targetList:
			writemsg("scope=" + scope + " target is " + targetName)
			writemsg("bus destination destName="+ destName+" busName=" + busName + " serverName="+ serverName + " destinationType="+ destinationType )
			#manageSIBusDestination(scope,targetName,destName,busName, serverName, destinationType, create)
			manageSIBusDestination(scope,targetName,destName,busName,serverName,destinationType,clustername,create)
		#endFor 
		i = i+1
	#endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJMSConnectionFactories
# Description: Creates/Deletes JMS Connection factories based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, connection factories will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageJMSConnectionFactories ( props, create ):
        i = 0
        done = "false"
        confactoryprefix = "confactory"

        while (1):
		propName = confactoryprefix + "." + str(i) + ".name"
		cfName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (cfName == ""):
                        break 
                #endIf 

		scope = props[confactoryprefix+"."+ str(i) + ".scope"]
                jndiName = props[confactoryprefix+"."+ str(i) + ".jndiname"]
                busName = props[confactoryprefix+"."+ str(i) + ".busname"]
		cfType = props[confactoryprefix+"."+ str(i) + ".type"]
		authDataAlias = props[confactoryprefix+"."+ str(i) + ".authdataalias"]
		targetType = props[confactoryprefix+"."+ str(i) + ".targettype"]
		xaAuthAlias = props[confactoryprefix+"."+ str(i) + ".xaauthalias"]
		endpoints = props[confactoryprefix+"."+ str(i) + ".endpoints"]
		scopename = props[confactoryprefix+"."+ str(i) + ".scopename"]
		
		targetList = buildTargetList(scope,scopename)
                # For each target node, create/delete JMS connection factory
                for targetName in targetList:
			writemsg("scope=" + scope + " target is " + targetName)
                        writemsg("JMS connection="+ cfName+" busName=" + busName)
                        manageJMSConnectionFactory(scope,targetName,cfName,jndiName,busName,cfType,authDataAlias,targetType,xaAuthAlias,endpoints,create)
                 #endFor 
                i = i+1
        #endWhile 

#endDef 


#--------------------------------------------------------------------
# Procedure:   manageJMSQueues
# Description: Creates/Deletes JMS queues based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, queue will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageJMSQueues ( props, create ):
        i = 0
        done = "false"
        jmsqueueprefix = "jmsqueue"

        while (1):
		propName = jmsqueueprefix + "." + str(i) + ".name"
		jmsQName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (jmsQName == ""):
                        break 
                #endIf 

                scope = props[jmsqueueprefix+"."+ str(i) + ".scope"]
                jndiName = props[jmsqueueprefix+"."+ str(i) + ".jndiname"]
                busName = props[jmsqueueprefix+"."+ str(i) + ".busname"]
                queueName = props[jmsqueueprefix+"."+ str(i) + ".queuename"]
                queueMgr = props[jmsqueueprefix+"."+ str(i) + ".queuemanager"]
                scopename = props[jmsqueueprefix+"."+ str(i) + ".scopename"]

                targetList = buildTargetList(scope,scopename)
                # For each target node, create/delete JMS queue
                for targetName in targetList:
                        writemsg("scope=" + scope + " target is " + targetName)
                        if (busName == "" or busName == None):
                                writemsg("JMS WMQ queue="+ jmsQName+" queueManager=" + queueMgr)
                                manageWMQQueue(scope,targetName,jmsQName,jndiName,queueMgr,queueName,create)
                        else:
                                writemsg("JMS SIB queue="+ jmsQName+" busName=" + busName)
                                manageSIBJMSQueue(scope,targetName,jmsQName,jndiName,busName,queueName,create)
                        #endElse
                 #endFor 
                i = i+1
        #endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageJMSTopics
# Description: Creates/Deletes JMS topics based on the values in the
#              Properties object.
#
# Parameters:  	props	Properties object
#		create	Boolean if true, topic will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageJMSTopics ( props, create ):
        i = 0
        done = "false"
        jmstopicprefix = "jmstopic"

        while (1):
		propName = jmstopicprefix + "." + str(i) + ".name"
		name = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (name == ""):
                        break 
                #endIf 

		scope = props[jmstopicprefix+"."+ str(i) + ".scope"]
                jndiName = props[jmstopicprefix+"."+ str(i) + ".jndiname"]
                busName = props[jmstopicprefix+"."+ str(i) + ".busname"]
                topicName = props[jmstopicprefix+"."+ str(i) + ".topicname"]
		topicSpace = props[jmstopicprefix+"."+ str(i) + ".topicspace"]
		scopename = props[jmstopicprefix+"."+ str(i) + ".scopename"]

		targetList = buildTargetList(scope,scopename)
                # For each target node, create/delete JMS topic
                for targetName in targetList:
			writemsg("scope=" + scope + " target is " + targetName)
                        writemsg("JMS Topic="+ name +" busName=" + busName)
                        manageJMSTopic(scope,targetName,name,jndiName,busName,topicName,topicSpace,create)
                 #endFor 
                i = i+1
        #endWhile 

#endDef 


#--------------------------------------------------------------------
# Procedure:   manageJMSActivationSpecs
# Description: Creates/Deletes JMS activation specifications based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, JMS activation specification will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageJMSActivationSpecs ( props, create ):
        i = 0
        done = "false"
        jmsactspecprefix = "jmsactspec"

        while (1):
		propName = jmsactspecprefix + "." + str(i) + ".name"
		jmsActSpecName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (jmsActSpecName == ""):
                        break 
                #endIf 

		scope = props[jmsactspecprefix+"."+ str(i) + ".scope"]
                jndiName = props[jmsactspecprefix+"."+ str(i) + ".jndiname"]
                busName = props[jmsactspecprefix+"."+ str(i) + ".busname"]
		jndiDest = props[jmsactspecprefix+"."+ str(i) + ".destinationjndiname"]
		destType = props[jmsactspecprefix+"."+ str(i) + ".destinationtype"]
		authAlias = props[jmsactspecprefix+"."+ str(i) + ".authalias"]
		messageSelector = props[jmsactspecprefix+"."+ str(i) + ".messageselector"]
		durability = props[jmsactspecprefix+"."+ str(i) + ".subscriptiondurability"]
		subscriptionName = props[jmsactspecprefix+"."+ str(i) + ".subscriptionname"]
		clientID = props[jmsactspecprefix+"."+ str(i) + ".clientid"]
		scopename = props[jmsactspecprefix+"."+ str(i) + ".scopename"]
		
		targetList = buildTargetList(scope,scopename)
                # For each target node, create/delete JMS activation specification
                for targetName in targetList:
			writemsg("scope=" + scope + " target is " + targetName)
                        writemsg("JMS activation spec="+ jmsActSpecName+" busName=" + busName)
                        manageJMSActivationSpec(scope, targetName, jmsActSpecName, jndiName, busName, jndiDest, destType, authAlias, messageSelector, durability, subscriptionName, clientID, create)
                 #endFor 
                i = i+1
        #endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageMQActivationSpecs
# Description: Creates/Deletes MQ activation specifications based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, JMS activation specification will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageMQActivationSpecs ( props, create ):
        i = 0
        done = "false"
        mqactspecprefix = "mqactspec"

        while (1):
		propName = mqactspecprefix + "." + str(i) + ".name"
		mqActSpecName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (mqActSpecName == ""):
                        break 
                #endIf 

		scope = props[mqactspecprefix+"."+ str(i) + ".scope"]
                jndiName = props[mqactspecprefix+"."+ str(i) + ".jndiname"]
                busName = props[mqactspecprefix+"."+ str(i) + ".busname"]
		jndiDest = props[mqactspecprefix+"."+ str(i) + ".destinationjndiname"]
		destType = props[mqactspecprefix+"."+ str(i) + ".destinationtype"]
		authAlias = props[mqactspecprefix+"."+ str(i) + ".authalias"]
		messageSelector = props[mqactspecprefix+"."+ str(i) + ".messageselector"]
                queueManager = props[mqactspecprefix+"."+ str(i) + ".queueManager"]
                hostName = props[mqactspecprefix+"."+ str(i) + ".hostName"]
                port = props[mqactspecprefix+"."+ str(i) + ".port"]
                channel = props[mqactspecprefix+"."+ str(i) + ".channel"]
                durability = props[mqactspecprefix+"."+ str(i) + ".subscriptiondurability"]
		subscriptionName = props[mqactspecprefix+"."+ str(i) + ".subscriptionname"]
		clientID = props[mqactspecprefix+"."+ str(i) + ".clientid"]
		scopename = props[mqactspecprefix+"."+ str(i) + ".scopename"]
		
		targetList = buildTargetList(scope,scopename)
                # For each target node, create/delete MQ activation specification
                for targetName in targetList:
			writemsg("scope=" + scope + " target is " + targetName)
                        writemsg("MQ activation spec="+ mqActSpecName )
                        manageMQActivationSpec(scope, targetName, mqActSpecName, jndiName, jndiDest, destType, authAlias, messageSelector, queueManager, hostName, port, channel, durability, subscriptionName, clientID, create)
                 #endFor 
                i = i+1
        #endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageStartupBeanServices
# Description: Enable/Disable startup bean services based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, startup bean service will be enabled, else disabled
# Returns:     N/A
#--------------------------------------------------------------------
def manageStartupBeanServices ( props, create ):
        i = 0
        done = "false"
        startupprefix = "startupbean"

        while (1):
		propName = startupprefix + "." + str(i) + ".server"
		serverName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (serverName == ""):
                        break 
                #endIf 

		
                writemsg("Startup bean on server="+ serverName)
                manageStartupBeanService(serverName, create)
                i = i+1
        #endWhile 

#endDef 


#--------------------------------------------------------------------
# Procedure:   manageObjectCacheInstances
# Description: Creates/Deletes Object cache instances based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, Object cache resources will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageObjectCacheInstances ( props, create ):
	i = 0
	done = "false"
	ociprefix = "objcacheinst"

	while (1):
		propName = ociprefix + "." + str(i) + ".name"
		ociName = props.getProperty(propName,"")

		# If the next numbered entry didn't exist, break out of this loop
		if (ociName == ""):
			break 
		#endIf

		scope = props[ociprefix + "."+ str(i) + ".scope"]
		scopename = props[ociprefix+"."+ str(i) + ".scopename"]
		jndiName = props[ociprefix + "."+ str(i) + ".jndiname"]
		cacheSize = props[ociprefix + "."+ str(i) + ".cachesize"]
		category = props[ociprefix + ".category"]
		if ( category==None ):
			category=""
		#endIf
		defaultPriority = props[ociprefix + "."+ str(i) + ".defaultpriority"]
		disableDependencyId = props[ociprefix + "."+ str(i) + ".disabledependencyid"]
		enableCacheReplication = props[ociprefix + "."+ str(i) + ".enablecachereplication"]
		replicationDomain = props[ociprefix + "."+ str(i) + ".replicationdomain"]
		replicationType = props[ociprefix+"."+ str(i) + ".replicationtype"]
		enableDiskOffload = props[ociprefix + "."+ str(i) + ".enablediskoffload"]
		flushToDiskOnStop = props[ociprefix + "."+ str(i) + ".flushtodiskonstop"]
		diskCacheSizeInGB = props[ociprefix + "."+ str(i) + ".diskcachesizeingb"]
		if ( (diskCacheSizeInGB=="") or (diskCacheSizeInGB==None) ):
			diskCacheSizeInGB=0
		#endIf
		diskCacheEntrySizeInMB = props[ociprefix + "."+ str(i) + ".diskcacheentrysizeinmb"]
		if ( (diskCacheEntrySizeInMB=="") or (diskCacheEntrySizeInMB==None) ):
			diskCacheEntrySizeInMB=0
		#endIf
		diskCacheCleanupFrequency = props[ociprefix + "."+ str(i) + ".diskcachecleanupfrequency"]
		if ( (diskCacheCleanupFrequency=="") or (diskCacheCleanupFrequency==None) ):
			diskCacheCleanupFrequency=0
		#endIf
		diskCacheSizeInEntries = props[ociprefix + "."+ str(i) + ".diskcachesizeinentries"]
		if ( (diskCacheSizeInEntries=="") or (diskCacheSizeInEntries==None) ):
			diskCacheSizeInEntries=0
		#endIf
		pushFrequency = props[ociprefix + "."+ str(i) + ".pushfrequency"]
		if ( (pushFrequency=="") or (pushFrequency==None) ):
			pushFrequency=0
		#endIf

		targetList = buildTargetList(scope,scopename)
		# For each target node, create/delete Object Cache Instance
		for targetName in targetList:
			writemsg("scope = " + scope + "; target = " + targetName)
			writemsg("Object Cache Instance = "+ ociName)
			manageObjectCacheInstance(scope,targetName,ociName,jndiName,cacheSize,category,defaultPriority,disableDependencyId,enableCacheReplication,replicationDomain,replicationType,enableDiskOffload,flushToDiskOnStop,diskCacheSizeInGB,diskCacheEntrySizeInMB,diskCacheCleanupFrequency,diskCacheSizeInEntries,pushFrequency,create)
		#endFor 
		i = i+1
	#endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageResourceEnvironmentProviders
# Description: Creates/Deletes Resource Environment Providers based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, esource Environment Providers will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageResourceEnvironmentProviders ( props, create ):
        i = 0
        done = "false"
        providerprefix = "resource.provider"

        while (1):
		propName = providerprefix + "." + str(i) + ".name"
		providerName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (providerName == ""):
                        break 
                #endIf 

		scope = props[providerprefix+"."+ str(i) + ".scope"]
                description = props[providerprefix+"."+ str(i) + ".description"]
                factoryClassname = props[providerprefix+"."+ str(i) + ".factoryClassname"]
		classname = props[providerprefix+"."+ str(i) + ".classname"]
	
		targetName = props["serverName"]
		writemsg("scope=" + scope + " target is " + targetName )
                writemsg("Resource environment Provider="+ providerName)
                manageResourceEnvironmentProvider(scope, targetName, providerName, description, factoryClassname, classname, create)
                i = i+1
        #endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:	manageResourceEnvironmentProviderProperties
# Description:	Creates/Deletes Resource Environment Provider Properties based on the values in the
# 		Properties object.
#
# Parameters:	props	Properties object
#		create	Boolean if true, Resource Environment Provider properties will be created, else deleted
# Returns:	N/A
#--------------------------------------------------------------------
def manageResourceEnvironmentProviderProperties ( props, create ):
	i = 0
	done = "false"
	propertyprefix = "resource.provider.property"

	while (1):
		propName = propertyprefix + "." + str(i) + ".name"
		name = props.getProperty(propName,"")
		
		# If the next numbered entry didn't exist, break out of this loop
		if (name == ""):
			break 
		#endIf 
		
		scope = props[propertyprefix+"."+ str(i) + ".scope"]
		writemsg("scope:"+scope)
		value = props[propertyprefix+"."+ str(i) + ".value"]
		writemsg("value:"+value)
		providerName = props[propertyprefix+"."+ str(i) + ".providername"]
		writemsg("providerName:"+providerName)
	
		#targetName = props["serverName"]
		targetName = props[propertyprefix+"."+ str(i) + ".scopename"]
		writemsg("scope=" + scope + " target is " + targetName )
		writemsg("Resource environment properties="+ name)
		manageResourceEnvironmentProviderProperty(scope, targetName, providerName, name, value, create)
		i = i+1
	#endWhile
#endDef

#--------------------------------------------------------------------
# Procedure:   manageResourceEnvironmentEntries
# Description: Creates/Deletes Resource Environment Entries based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#	       create	Boolean if true, Resource Environment Entry will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageResourceEnvironmentEntries ( props, create ):
        i = 0
        done = "false"
        entryprefix = "resource.provider.entry"

        while (1):
		propName = entryprefix + "." + str(i) + ".name"
		entryName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (entryName == ""):
                        break 
                #endIf 

		scope = props[entryprefix+"."+ str(i) + ".scope"]
                jndiname = props[entryprefix+"."+ str(i) + ".jndiname"]
                providerName = props[entryprefix+"."+ str(i) + ".providername"]
                factoryClassname = props[entryprefix+"."+ str(i) + ".factoryClassname"]
	
		targetName = props["serverName"]
		writemsg("scope=" + scope + " target is " + targetName )
                writemsg("Resource environment entry ="+ entryName)
                
                manageResourceEnvironmentEntry(scope, targetName, entryName, jndiname, providerName, factoryClassname, create)
                if (create): 
	                j = 0
	                while (1):
	                        custPropPre = entryprefix + "." + str(i) + ".custom." + str(j)
	                        custName = props.getProperty(custPropPre + ".name" , "" )
	                        if ((custName == "") or (custName == None)):
	                                # No more custom properties
	                                break 
	                        #endIf 
	                        custValue = props[custPropPre + ".value"]
	                        writemsg("Resource environment entry custom property name ="+ custName)
	                        writemsg("Resource environment entry custom property value ="+ custValue)
	                        addCustomPropertyToEnvironmentEntryWithScope(scope, targetName, entryName, custName, custValue )
	                        j = j + 1
	                #endWhile 
                #endif
                
                
                i = i+1
        #endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageSIBusWMQServers
# Description: Creates/Deletes SI Bus WMQ Servers
#
# Parameters:  props	Properties object
#	       create	Boolean if true, SI Bus WMQ Servers will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSIBusWMQServers ( props, create ):
	global AdminTask
        i = 0
        done = "false"
        entryprefix = "busmqserver"

        while (1):
		propName = entryprefix + "." + str(i) + ".name"
		entryName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (entryName == ""):
                        break 
                #endIf 

                name = props[entryprefix+"."+ str(i) + ".name"]
		serverName = props[entryprefix+"."+ str(i) + ".serverName"]
                description = props[entryprefix+"."+ str(i) + ".description"]
                mqtype = props[entryprefix+"."+ str(i) + ".type"]
                bindingsMode = props[entryprefix+"."+ str(i) + ".bindingsMode"]
                host = props[entryprefix+"."+ str(i) + ".host"]
                port = props[entryprefix+"."+ str(i) + ".port"]
                channel = props[entryprefix+"."+ str(i) + ".channel"]
                securityAuthAlias = props[entryprefix+"."+ str(i) + ".securityAuthAlias"]
                transportChain = props[entryprefix+"."+ str(i) + ".transportChain"]
                allowDiscovery = props[entryprefix+"."+ str(i) + ".allowDiscovery"]
                replyToQueue = props[entryprefix+"."+ str(i) + ".replyToQueue"]
                  
                sibusMqServers=AdminTask.listSIBWMQServers().split(java.lang.System.getProperty("line.separator"))
                 
     		for sibusMqServer in sibusMqServers :
     			if( sibusMqServer != "" ) :
     				nameEntry = AdminConfig.showAttribute(sibusMqServer,"name")
     				if ( nameEntry == name ) :
     					if ( create ) :
     						writemsg("MQ Server for SiBus entry = "+ name + " existed ")
     						writemsg("Update MQ Server for SiBus entry = "+ name)
     						attrs = "[-name %s -serverName %s -description %s -type %s -bindingsMode %s -host %s -port %s -channel %s -securityAuthAlias %s -transportChain %s -allowDiscovery %s -replyToQueue %s -trustUserIds true -discoveryAuthAlias ]" % (name, serverName, description, mqtype, bindingsMode, host, port, channel, securityAuthAlias, transportChain, allowDiscovery, replyToQueue )
				                trace("info", "attribute= " + attrs )
						AdminTask.modifySIBWMQServer(attrs)
						done = "true"					
					else :
						attrs = "[-name %s ]" %(name)
						AdminTask.deleteSIBWMQServer(attrs)
						done = "true"
					#endElse 
				#endIf
			#endIf
		#endFor
		if ( done == "false"):
			attrs = "[-name %s -serverName %s -description %s -type %s -bindingsMode %s -host %s -port %s -channel %s -securityAuthAlias %s -transportChain %s -allowDiscovery %s -replyToQueue %s -trustUserIds true -discoveryAuthAlias ]" % (name, serverName, description, mqtype, bindingsMode, host, port, channel, securityAuthAlias, transportChain, allowDiscovery, replyToQueue )
	                writemsg("Create MQ Server for SiBus entry = "+ name)
	                trace("info", "attribute= " + attrs )
			AdminTask.createSIBWMQServer(attrs) 
			writemsg("Created MQ Server for SiBus entry = "+ name)
                i = i+1
        #endWhile       

#endDef


#--------------------------------------------------------------------
# Procedure:   manageSIBusWMQMembers
# Description: Creates/Deletes SI Bus WMQ Members
#
# Parameters:  props	Properties object
#	       create	Boolean if true, SI Bus WMQ Members will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSIBusWMQMembers ( props, create ):
	global AdminTask
        i = 0
        done = "false"
        entryprefix = "busmqmember"
        
        while (1):
		propName = entryprefix + "." + str(i) + ".busname"
		entryName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (entryName == ""):
                        break 
                #endIf 

                busname = props[entryprefix+"."+ str(i) + ".busname"]
		wmqServer = props[entryprefix+"."+ str(i) + ".wmqServer"]
                virtualQueueManagerName = props[entryprefix+"."+ str(i) + ".virtualQueueManagerName"]
                if ( create ) :	
			attrs = "[-bus %s -wmqServer %s -virtualQueueManagerName %s ]" % (busname, wmqServer, virtualQueueManagerName )
	                writemsg("Add MQ Server for SiBus member ="+ wmqServer)
	                trace("info", "attribute= " + attrs )
			AdminTask.addSIBusMember(attrs) 
		else :
			attrs = "[-bus %s -wmqServer %s ]" % (busname, wmqServer)
			writemsg("Remove MQ Server for SiBus member ="+ wmqServer)
			AdminTask.removeSIBusMember(attrs)
                i = i+1
        #endWhile         
        

#endDef

#--------------------------------------------------------------------
# Procedure:   manageSIBusWMQDestinations
# Description: Creates/Deletes SI Bus WMQ Destinations
#
# Parameters:  props	Properties object
#	       create	Boolean if true, SI Bus WMQ Destinations will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSIBusWMQDestinations(props, create ):
	global AdminTask
        i = 0
        done = "false"
        entryprefix = "busmqdestination"
        
        while (1):
		propName = entryprefix + "." + str(i) + ".name"
		entryName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (entryName == ""):
                        break 
                #endIf 

                name = props[entryprefix+"."+ str(i) + ".name"]
                busname = props[entryprefix+"."+ str(i) + ".busname"]
                destinationtype = props[entryprefix+"."+ str(i) + ".type"]
		wmqServer = props[entryprefix+"."+ str(i) + ".wmqserver"]
		wmqqueuename = props[entryprefix+"."+ str(i) + ".wmqqueuename"]
                if ( create ) :	
                	if ( destinationtype == "Queue" ) : 
				attrs = "[-bus %s -name %s -type %s -wmqServer %s -wmqQueueName %s ]" % (busname, name, destinationtype, wmqServer, wmqqueuename )
	                else :
	                	attrs = "[-bus %s -name %s -type %s ]" % (busname, name, destinationtype)
	                writemsg("Create SI Bus MQ Server destination ="+ name)
	                trace("info", "attribute= " + attrs )
			AdminTask.createSIBDestination(attrs) 
		else :
			attrs = "[-bus %s -name %s ]" % (busname, name )
			writemsg("Delete SI Bus MQ Server destination ="+ name)
			AdminTask.deleteSIBDestination(attrs)
                i = i+1
        #endWhile         
        

#endDef

#--------------------------------------------------------------------
# Procedure:   manageWorkManagers
# Description: Creates/Deletes Work manager based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, Work manager resources will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageWorkManagers ( props, create ):
        i = 0
        done = "false"
        workmprefix = "workmanager"

        while (1):
		propName = workmprefix + "." + str(i) + ".name"
		workMName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (workMName == ""):
                        break 
                #endIf 

		scope = props[workmprefix+"."+ str(i) + ".scope"]
                jndiName = props[workmprefix+"."+ str(i) + ".jndiname"]
                scopename = props[workmprefix+"."+ str(i) + ".scopename"]

		targetList = buildTargetList(scope,scopename)
                # For each target node, create/delete Work manager
                for targetName in targetList:
			writemsg("scope=" + scope + " target is " + targetName)
                        writemsg("Work manager="+ workMName)
                        manageWorkManager(scope,targetName,workMName,jndiName, create)
                 #endFor 
                i = i+1
        #endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageTimerManagers
# Description: Creates/Deletes Timer manager based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, Work manager resources will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------

def manageTimerManagers ( props, create ):
	i = 0
	done = "false"
	timermprefix = "timermanager"

	while (1):
		propName = timermprefix + "." + str(i) + ".name"
		timerMName = props.getProperty(propName,"")

		# If the next numbered entry didn't exist, break out of this loop
		if (timerMName == ""):
				break 
		#endIf 

		scope = props[timermprefix+"."+ str(i) + ".scope"]
		jndiName = props[timermprefix+"."+ str(i) + ".jndiname"]
		scopename = props[timermprefix+"."+ str(i) + ".scopename"]
		numAlarmThread = props[timermprefix+"."+ str(i) + ".numAlarmThread"]

		targetList = buildTargetList(scope,scopename)
		# For each target node, create/delete Work manager
		for targetName in targetList:
			writemsg("scope=" + scope + " target is " + targetName)
			writemsg("Timer manager="+ timerMName)
			#manageWorkManager(scope,targetName,timerMName,jndiName, create)
			manageTimerManager(scope,targetName,timerMName,jndiName,numAlarmThread,create)
		#endFor 

		i = i+1
	#endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageSchedulers
# Description: Creates/Deletes Scheduler  based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, Work manager resources will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSchedulers ( props, create ):
	i = 0
	done = "false"
	schedulerprefix = "scheduler"

	while (1):
		propName = schedulerprefix + "." + str(i) + ".name"
		schName = props.getProperty(propName,"")

		# If the next numbered entry didn't exist, break out of this loop
		if (schName == ""):
				break 
		#endIf

		name = props[schedulerprefix+"."+ str(i) + ".name"]
		jndiName = props[schedulerprefix+"."+ str(i) + ".jndiname"]
		description = props[schedulerprefix+"."+ str(i) + ".description"]
		category =  props[schedulerprefix+"."+ str(i) + ".category"]
		datasourceJNDIName = props[schedulerprefix+"."+ str(i) + ".datasourceJNDIName"]
		datasourceAlias =  props[schedulerprefix+"."+ str(i) + ".datasourceAlias"]
		workManagerInfoJNDIName = props[schedulerprefix+"."+ str(i) + ".workManagerInfoJNDIName"]
		tablePrefix =  props[schedulerprefix+"."+ str(i) + ".tablePrefix"]
		pollInterval =  props[schedulerprefix+"."+ str(i) + ".pollInterval"]
		useAdminRoles  =  props[schedulerprefix+"."+ str(i) + ".useAdminRoles"]
		createTables  =  props[schedulerprefix+"."+ str(i) + ".createTables"]
		verifyTables  =  props[schedulerprefix+"."+ str(i) + ".verifyTables"]
		targetContPath =  props[schedulerprefix+"."+ str(i) + ".target"]		
		manageScheduler(targetContPath, name, jndiName, description, category, datasourceJNDIName, datasourceAlias, workManagerInfoJNDIName,tablePrefix,pollInterval,useAdminRoles,createTables,verifyTables, create)
		i = i+1
	#endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageSessionTimeOut
# Description: Update/Deletes Session Timeout for the given server 
#
# Parameters:  props	Properties object
#		create	Boolean if true, essionTimeOut resources will be update, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def modifySessionTimeOut ( props, create ):	
	i = 0	
	sessionTimeOutrefix = "sessionTimeOut"  
	writemsg("sessionTimeOutrefix :" + sessionTimeOutrefix)
	timeOutValue = props[sessionTimeOutrefix+"."+ str(i) + ".invalidationTimeout"] 
	writemsg("timeOutValue :" + timeOutValue)
	serverName = props[sessionTimeOutrefix+"."+ str(i) + ".server"] 
	writemsg("serverName :" + serverName)
	nodeName = props[sessionTimeOutrefix+"."+ str(i) + ".node"] 
	writemsg("nodeName :" + nodeName)
	server = AdminConfig.getid("/Node:"+nodeName+"/Server:"+serverName+"/")
	sms = AdminConfig.list("TuningParams", server)
	print AdminConfig.showall(sms)
	para = "[[invalidationTimeout " + timeOutValue + "]]"
	writemsg("para :" + para)	
	AdminConfig.modify(sms, para) 
	AdminConfig.save()
#endDef

#--------------------------------------------------------------------
# Procedure:   manageServerProperties
# Description: Creates/Deletes server properties based on the values in the
#              Properties object.
#
# Parameters:  props	Properties object
#		create	Boolean if true, Work manager resources will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageServerProperties ( props, create ):
        i = 0
        done = "false"
        serverpropprefix = "serverproperties"

        while (1):
		propName = serverpropprefix + "." + str(i) + ".name"
		serverpropName = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (serverpropName == ""):
                        break 
                #endIf 

		servername = props[serverpropprefix+"."+ str(i) + ".servername"]
		value = props[serverpropprefix+"."+ str(i) + ".value"]
                description = props[serverpropprefix+"."+ str(i) + ".description"]
		required = props[serverpropprefix+"."+ str(i) + ".required"]
		
		writemsg("servername=" + servername)
                writemsg("server properties Name="+ serverpropName)
                manageServerProperty(servername,serverpropName,value,description,required,create)
                
                i = i+1
        #endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageMQTopics
# Description: Creates/Deletes MQ topics based on the values in the
#              Properties object.
#
# Parameters:  	props	Properties object
#		create	Boolean if true, topic will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageMQTopics ( props, create ):
        i = 0
        done = "false"
        jmstopicprefix = "mqtopic"

        while (1):
		propName = jmstopicprefix + "." + str(i) + ".name"
		name = props.getProperty(propName,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (name == ""):
                        break 
                #endIf 

		scope = props[jmstopicprefix+"."+ str(i) + ".scope"]
                jndiName = props[jmstopicprefix+"."+ str(i) + ".jndiname"]                
                topicName = props[jmstopicprefix+"."+ str(i) + ".topicname"]
                scopename = props[jmstopicprefix+"."+ str(i) + ".scopename"]

		targetList = buildTargetList(scope,scopename)
                # For each target node, create/delete JMS topic
                for targetName in targetList:
			writemsg("scope=" + scope + " target is " + targetName)
                        writemsg("MQ Topic="+ name )
                        manageMQTopic(scope,targetName,name,jndiName,topicName,create)
                 #endFor 
                i = i+1
        #endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageSSLCertificateProperties
# Description: Creates/Deletes SSL Signer Certificate based on the values in the
#              Properties object.
#
# Parameters:  	props	Properties object
#		create	Boolean if true, topic will be created, else deleted
# Returns:     N/A
#--------------------------------------------------------------------
def manageSSLTrustStoreSignerCertificates ( props, create ):
        i = 0
        done = "false"
        sslcertprefix = "sslsignercert"

        while (1):
                certificateAlias = sslcertprefix + "." + str(i) + ".certificatealias"
                name = props.getProperty(certificateAlias,"")
		
                # If the next numbered entry didn't exist, break out of this loop
                if (name == ""):
                        break 
                #endIf 

                host = props[sslcertprefix+"."+ str(i) + ".host"]
                # Check if DMZ_2 is defined in the IOC topology file
                if (host.startswith('~{')):
                        i = i+1
                        continue
                #endIf

                port = props[sslcertprefix+"."+ str(i) + ".port"]
                keyStoreName = props[sslcertprefix+"."+ str(i) + ".keystorename"]
                sslConfigName = props[sslcertprefix+"."+ str(i) + ".sslconfigname"]
                keyStoreScope = props[sslcertprefix+"."+ str(i) + ".keystorescope"]
                sslConfigScopeName = props[sslcertprefix+"."+ str(i) + ".sslconfigscopename"]

                writemsg("SSL Trustore Signer Certificate Alias = "+ name)

                attrs = "[-keyStoreName %s -keyStoreScope %s ]" % (keyStoreName,keyStoreScope)
                certs = AdminTask.listSignerCertificates(attrs) 

                isCertOnList = 0
                certsList = certs.split(lineSeparator)
                for cert in certsList:
                        # There are cases that "" entry is returned in the certsList
                        if (cert == ""):
                                continue
                        #endIf

                        aliasElement = cert[cert.index("alias"): cert.index("]", cert.index("alias"))]
                        if (aliasElement == ""):
                                continue
                        #endIf

                        aliasName = aliasElement.split(" ")[1]
                        if (aliasName == name):
                                writemsg("SSL Trustore Signer Certificate with alias "+ name + " already exists, so it will not be created")
                                isCertOnList = 1
                                break
                        #endIf
                #endFor

                if (isCertOnList == 0):
                       manageSSLTrustStoreSignerCertificate(name,host,port,keyStoreName,sslConfigName,keyStoreScope,sslConfigScopeName,create)
                i = i+1
        #endWhile 

#endDef 

#--------------------------------------------------------------------
# Procedure:   manageDynamicCacheServices
# Description: Configure Dynamic Cache Services for Websphere Application Servers
#
# Parameters:  props	Properties object
#			   create	
# Returns:     N/A
#--------------------------------------------------------------------
def manageDynamicCacheServices(props, create):
	dcsprefix = "dynamiccachesvc"

	i = 0
	while (1):
		propName = dcsprefix + "." + str(i) + ".enablecachereplication"
		enableCacheReplication = props.getProperty(propName,"")

		# If the next numbered entry didn't exist, break out of this loop
		if ((enableCacheReplication == "") or (enableCacheReplication == None)):
			break
		#endIf

		replicationType = props[dcsprefix + "." + str(i) + ".replicationtype"]
		pushFrequency = props[dcsprefix + "." + str(i) + ".pushfrequency"]
		messageBrokerDomainName = props[dcsprefix + "." + str(i) + ".messagebrokerdomainname"]
		cluster = props[dcsprefix + "." + str(i) + ".cluster"]
		serverlist = AdminConfig.list("Server").split(java.lang.System.getProperty("line.separator"))

		clusterID = AdminConfig.getid("/ServerCluster:"+cluster+"/")
		clusterMemberList = AdminConfig.list("ClusterMember",clusterID).split(lineSeparator) 

		# In HA environment, will do same configuration for each cluster member 
		for member in clusterMemberList:
			member_name = AdminConfig.showAttribute(member,"memberName")
			for server in serverlist:
				servername = AdminConfig.showAttribute(server, "name" )
				if (member_name == servername):
					dynacache = AdminConfig.list("DynamicCache", server)
					manageDynamicCacheService(dynacache, enableCacheReplication, messageBrokerDomainName, replicationType, pushFrequency, create)
					break
				#endIf
			#endFor
		#endFor
		
		i = i+1
	#endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageServerCookies
# Description: Configure cookies for application server based on a
#              properties file.
# Parameters:  props	Properties object
#		create (Boolean if true, add environment entries setting, else delete it)
# Returns:     N/A
#--------------------------------------------------------------------
def manageServerCookies(props, create):
	global AdminConfig

	i = 0
	while (1):
		key = "server.cookie."+str(i)
		servername = props[key+".servername"]
		nodename = props[key+".nodename"]
		name = props[key+".name"]
		httpOnly = props[key+".httponly"]

		if ((name == "") or (name == None)):
			# no more environment entry settings to change
			break 
		#endIf 

		# Change Application Server Cookies Settings
		writemsg("Changing Cookies for "+servername +" at node:"+nodename )
		manageServerCookie(nodename, servername, name, httpOnly, create )
		i = i+1
	#endWhile 

#endDef

#--------------------------------------------------------------------
# Procedure:   manageSessionManagementProperties
# Description: Configure Web Container Session Management settings for 
#              application server based on a properties file.
# Parameters:  props	Properties object
#		create (Boolean if true, add environment entries setting, else delete it)
# Returns:     N/A
#--------------------------------------------------------------------
def manageSessionManagementProperties(props, create):
	global AdminConfig
	global AdminControl
	
	key = "session.mgmt"
	servername = props[key+".servername"]
	if (servername == None):    # No more entries
		return
	#endIf
	
	traceEntry("manageSessionManagementProperties")

	nodename = props[key+".nodename"]		
	enableSecurityIntegration = props[key+".enablesecurityintegration"]
	maxWaitTime = props[key+".maxwaittime"]
	allowSerializedSessionAccess = props[key+".allowserializedsessionaccess"]
	enableProtocolSwitchRewriting = props[key+".enableprotocolswitchrewriting"]
	enableUrlRewriting = props[key+".enableurlrewriting"]
	enable = props[key+".enable"]
	accessSessionOnTimeout = props[key+".accesssessionontimeout"]
	enableSSLTracking = props[key+".enablessltracking"]
	enableCookies = props[key+".enablecookies"]
	allowOverflow = props[key+".allowoverflow"]
	invalidationTimeout = props[key+".invalidationtimeout"]
	maxInMemorySessionCount = props[key+".maxinmemorysessioncount"]
	
	server = AdminConfig.getid('/Node:' + nodename + '/Server:' + servername + '/')
	sessionManager = AdminConfig.list('SessionManager', server)
	tuningParams = AdminConfig.list('TuningParams', server)

	enableSecurityIntegration = ["enableSecurityIntegration", enableSecurityIntegration]
	maxWaitTime = ["maxWaitTime", maxWaitTime]
	allowSerializedSessionAccess = ["allowSerializedSessionAccess", allowSerializedSessionAccess]
	enableProtocolSwitchRewriting = ["enableProtocolSwitchRewriting", enableProtocolSwitchRewriting]
	enableUrlRewriting = ["enableUrlRewriting", enableUrlRewriting]
	enable = ["enable", enable]
	accessSessionOnTimeout = ["accessSessionOnTimeout", accessSessionOnTimeout]
	enableSSLTracking = ["enableSSLTracking", enableSSLTracking]
	enableCookies = ["enableCookies", enableCookies]
	allowOverflow = ["allowOverflow", allowOverflow]
	invalidationTimeout = ["invalidationTimeout", invalidationTimeout]
	maxInMemorySessionCount = ["maxInMemorySessionCount", maxInMemorySessionCount]
	
	smAttrs = [enableSecurityIntegration, maxWaitTime, allowSerializedSessionAccess, enableProtocolSwitchRewriting, enableUrlRewriting, enable, accessSessionOnTimeout, enableSSLTracking, enableCookies]
	tpAttrs = [allowOverflow, invalidationTimeout, maxInMemorySessionCount]
	
	writemsg("Modifing SessionManager '" +str(sessionManager)+ "' with attributes '" +str(smAttrs)+ "'")
	AdminConfig.modify(sessionManager, smAttrs)
	
	writemsg("Modifing TuningParams '" +str(tuningParams)+ "' with attributes '" +str(tpAttrs)+ "'")
	AdminConfig.modify(tuningParams, tpAttrs)
	
	traceExit("manageSessionManagementProperties")
#endDef

#--------------------------------------------------------------------
# Procedure:   manageTransactionServiceConfiguration
# Description: Configure Container Services Transaction Service settings
#              application server based on a properties file.
# Parameters:  props	Properties object
#		create (Boolean if true, add environment entries setting, else delete it)
# Returns:     N/A
#--------------------------------------------------------------------
def manageTransactionServiceConfiguration(props, create):
	global AdminConfig
	global AdminControl
	
	key = "transaction.service"
	servername = props[key+".servername"]
	if (servername == None):    # No more entries
		return
	#endIf
	
	traceEntry("manageTransactionServiceConfiguration")

	nodename = props[key+".nodename"]		
	totalTranLifetimeTimeout  = props[key+".totalTranLifetimeTimeout"]
	httpProxyPrefix = props[key+".httpProxyPrefix"]
	LPSHeuristicCompletion = props[key+".LPSHeuristicCompletion"]
	httpsProxyPrefix = props[key+".httpsProxyPrefix"]
	wstxURLPrefixSpecified = props[key+".wstxURLPrefixSpecified"]
	enableFileLocking = props[key+".enableFileLocking"]
	enable = props[key+".enable"]
	transactionLogDirectory = props[key+".transactionLogDirectory"]
	enableProtocolSecurity = props[key+".enableProtocolSecurity"]
	heuristicRetryWait = props[key+".heuristicRetryWait"]
	propogatedOrBMTTranLifetimeTimeout = props[key+".propogatedOrBMTTranLifetimeTimeout"]
	enableLoggingForHeuristicReporting = props[key+".enableLoggingForHeuristicReporting"]
	asyncResponseTimeout = props[key+".asyncResponseTimeout"]
	clientInactivityTimeout = props[key+".clientInactivityTimeout"]
	heuristicRetryLimit = props[key+".heuristicRetryLimit"]
	acceptHeuristicHazard = props[key+".acceptHeuristicHazard"]
	
	server = AdminConfig.getid('/Node:' + nodename + '/Server:' + servername + '/')
	transactionService = AdminConfig.list('TransactionService', server)

	totalTranLifetimeTimeout = ["totalTranLifetimeTimeout", totalTranLifetimeTimeout]
	httpProxyPrefix = ["httpProxyPrefix", httpProxyPrefix]
	LPSHeuristicCompletion = ["LPSHeuristicCompletion", LPSHeuristicCompletion]
	httpsProxyPrefix = ["httpsProxyPrefix", httpsProxyPrefix]
	wstxURLPrefixSpecified = ["wstxURLPrefixSpecified", wstxURLPrefixSpecified]
	enableFileLocking = ["enableFileLocking", enableFileLocking]
	enable = ["enable", enable]
	transactionLogDirectory = ["transactionLogDirectory", transactionLogDirectory]
	enableProtocolSecurity = ["enableProtocolSecurity", enableProtocolSecurity]
	heuristicRetryWait = ["heuristicRetryWait", heuristicRetryWait]
	propogatedOrBMTTranLifetimeTimeout = ["propogatedOrBMTTranLifetimeTimeout", propogatedOrBMTTranLifetimeTimeout]
	enableLoggingForHeuristicReporting = ["enableLoggingForHeuristicReporting", enableLoggingForHeuristicReporting]
	asyncResponseTimeout = ["asyncResponseTimeout", asyncResponseTimeout]
	clientInactivityTimeout = ["clientInactivityTimeout", clientInactivityTimeout]
	heuristicRetryLimit = ["heuristicRetryLimit", heuristicRetryLimit]
	acceptHeuristicHazard = ["acceptHeuristicHazard", acceptHeuristicHazard]
	
	tsAttrs = [totalTranLifetimeTimeout, httpProxyPrefix, LPSHeuristicCompletion, httpsProxyPrefix, wstxURLPrefixSpecified, enableFileLocking, enable, transactionLogDirectory, enableProtocolSecurity, heuristicRetryWait, propogatedOrBMTTranLifetimeTimeout, enableLoggingForHeuristicReporting, asyncResponseTimeout, clientInactivityTimeout, heuristicRetryLimit, acceptHeuristicHazard]
	
	writemsg("Modifing Transaction Service '" +str(transactionService)+ "' with attributes '" +str(tsAttrs)+ "'")
	AdminConfig.modify(transactionService, tsAttrs)
	
	traceExit("manageTransactionServiceConfiguration")
#endDef

#--------------------------------------------------------------------
# Procedure:   manageHadrDatasources
# Description: Configure HADR Datasources for Websphere Application Servers
#
# Parameters:  props	Properties object
#			   create	
# Returns:     N/A
#--------------------------------------------------------------------
def manageHadrDatasources(props, create):
	hadrdsprefix = "hadr.datasources"

	i = 0
	while (1):
		propName = hadrdsprefix + "." + str(i) + ".dsName"
		dsName = props.getProperty(propName,"")

		# If the next numbered entry didn't exist, break out of this loop
		if ((dsName == "") or (dsName == None)):
			break
		#endIf

		cell = props[hadrdsprefix + "." + str(i) + ".cell"]
		cluster = props[hadrdsprefix + "." + str(i) + ".cluster"]
		JDBCProviderName = props[hadrdsprefix + "." + str(i) + ".JDBCProviderName"]
		clientRerouteAlternateServerName = props[hadrdsprefix + "." + str(i) + ".clientRerouteAlternateServerName"]
		clientRerouteAlternatePortNumber = props[hadrdsprefix + "." + str(i) + ".clientRerouteAlternatePortNumber"]
		retryIntervalForClientReroute = props[hadrdsprefix + "." + str(i) + ".retryIntervalForClientReroute"]
		maxRetriesForClientReroute = props[hadrdsprefix + "." + str(i) + ".maxRetriesForClientReroute"]

		ds = ""
		if ((cluster == "") or (cluster == None)):
			ds = AdminConfig.getid('/Cell:'+cell+'/JDBCProvider:' +JDBCProviderName+ '/DataSource:'+dsName+ '/')
		else:
			ds = AdminConfig.getid('/Cell:'+cell+'/ServerCluster:'+cluster+ '/JDBCProvider:' +JDBCProviderName+ '/DataSource:'+dsName+ '/')
		#endIf
		
		#ds = AdminConfig.getid('/Cell:'+cell+'/ServerCluster:'+cluster+ '/JDBCProvider:' +JDBCProviderName+ '/DataSource:'+dsName+ '/')
		#ds = AdminConfig.getid('/Cell:'+cell+'/JDBCProvider:' +JDBCProviderName+ '/DataSource:'+dsName+ '/')
		if ((ds == "") or (ds == None)):
			writemsg("No datasource named \'" +dsName+ "\' found")
			break
		#endIf
		
		retryIntervalForClientRerouteAttr = [["name", "retryIntervalForClientReroute"],["type", "java.lang.Integer"],["description", "The amount of time (in secs)between automatic client reroute retries."], ["value", retryIntervalForClientReroute], ["required", "false"]]
		maxRetriesForClientRerouteAttr = [["name", "maxRetriesForClientReroute"],["type", " "],["description", " "], ["value", maxRetriesForClientReroute], ["required", "false"]]
		clientRerouteAlternateServerNameAttr = [["name", "clientRerouteAlternateServerName"],["type", " "],["description", " "], ["value", clientRerouteAlternateServerName], ["required", "false"]]
		clientRerouteAlternatePortNumberAttr = [["name", "clientRerouteAlternatePortNumber"],["type", " "],["description", " "], ["value", clientRerouteAlternatePortNumber], ["required", "false"]]
		
		manageHadrDatasource(ds,retryIntervalForClientRerouteAttr,maxRetriesForClientRerouteAttr,clientRerouteAlternateServerNameAttr,clientRerouteAlternatePortNumberAttr, create)
		
		i = i+1
	#endWhile
	
#endDef

#--------------------------------------------------------------------
# Procedure:   manageThreadPools
# Description: Configure Thread Pools for Websphere Application Servers
#
# Parameters:  props	Properties object
#			   create	
# Returns:     N/A
#--------------------------------------------------------------------
def manageThreadPools(props, create):
	tpprefix = "threadpool"

	i = 0
	while (1):
		propName = tpprefix + "." + str(i) + ".threadPoolName"
		threadPoolName = props.getProperty(propName,"")

		# If the next numbered entry didn't exist, break out of this loop
		if ((threadPoolName == "") or (threadPoolName == None)):
			break
		#endIf


		parentType = props[tpprefix + "." + str(i) + ".parentType"]
		nodeName = props[tpprefix + "." + str(i) + ".nodeName"]
		serverName = props[tpprefix + "." + str(i) + ".serverName"]
		maximumSize = props[tpprefix + "." + str(i) + ".maximumSize"]
		minimumSize = props[tpprefix + "." + str(i) + ".minimumSize"]
		inactivityTimeout = props[tpprefix + "." + str(i) + ".inactivityTimeout"]
		otherAttributeList = props[tpprefix + "." + str(i) + ".otherAttributeList"]

		manageThreadPool(nodeName, serverName, parentType, threadPoolName, maximumSize, minimumSize, inactivityTimeout, otherAttributeList, create)
		
		i = i+1
	#endWhile 
#endDef

#--------------------------------------------------------------------
# Procedure:   manageDatasourceCustomProperties
# Description: Configure Datasource's Custom Properties
#
# Parameters:  props	Properties object
#			   create	
# Returns:     N/A
#--------------------------------------------------------------------
def manageDatasourceCustomProperties(props, create):
	dscustpropprefix = "datasources.customproperty"

	i = 0
	while (1):
		propName = dscustpropprefix + "." + str(i) + ".name"
		name = props.getProperty(propName,"")

		# If the next numbered entry didn't exist, break out of this loop
		if ((name == "") or (name == None)):
			break
		#endIf

		name = props[dscustpropprefix + "." + str(i) + ".name"]
		value = props[dscustpropprefix + "." + str(i) + ".value"]
		type = props[dscustpropprefix + "." + str(i) + ".type"]
		description = props[dscustpropprefix + "." + str(i) + ".description"]
		dsName = props[dscustpropprefix + "." + str(i) + ".dsName"]
		JDBCProviderName = props[dscustpropprefix + "." + str(i) + ".JDBCProviderName"]
		cell = props[dscustpropprefix + "." + str(i) + ".cell"]
		cluster = props[dscustpropprefix + "." + str(i) + ".cluster"]

		ds = ""
		if ((cluster == "") or (cluster == None)):
			ds = AdminConfig.getid('/Cell:'+cell+'/JDBCProvider:' +JDBCProviderName+ '/DataSource:'+dsName+ '/')
		else:
			ds = AdminConfig.getid('/Cell:'+cell+'/ServerCluster:'+cluster+ '/JDBCProvider:' +JDBCProviderName+ '/DataSource:'+dsName+ '/')
		#endIf
		
		if ((ds == "") or (ds == None)):
			writemsg("No datasource named \'" +dsName+ "\' found")
			break
		#endIf
		
		manageDatasourceCustomProperty(ds, name, value, type, dsName, JDBCProviderName, create)
		
		i = i+1
	#endWhile
	
#endDef
