# /*
#  * IBM Confidential OCO Source Material
#  *
#  * Licensed Materials - Property of IBM
#  * 
#  * (C) Copyright IBM Corp. 2012, 2013.  All rights reserved.
#  *
#  * The source code for this program is not published or otherwise divested
#  * of its trade secrets, irrespective of what has been deposited with the
#  * U. S. Copyright Office.
#  *
#  * US Government Users Restricted Rights - Use, duplication or
#  * disclosure restricted by GSA ADP Schedule Contract with IBM
#  * Corp.
#  */

import sys
from time import sleep

execfile("lib/common.py" )
execfile("lib/propsfileprocs.py" )

# -----------------------------------------------------
# Ensure the user entered the correct argument(s)
# If not, display the command syntax and return
# -----------------------------------------------------
if (len(sys.argv) < 1):
	print "Syntax: GenericEvent_config.py propsFile [uninstall]"
	print "  where \"propsFile\" is the name of the properties file containing"
	print "  configuration information."
	sys.exit(0)
#endIf


# propsFile to be the first argument to this script.
propsFileName = sys.argv[0]
props = loadProperties(propsFileName)
cellName = props["cellName"]
nodeName = props["nodeName"]
serverName = props["serverName"]
create = 1

#is this an install or uninstall?
if (len(sys.argv) == 2):
	uninstall = sys.argv[1]
	if (uninstall == "uninstall"):
		create = 0
	#endIf
#endIf

trclevel = props["traceLevel"]
initTracelevel(trclevel)


#--------------------------------------------------------------------
# Create resources - order differs from Delete resources order
#--------------------------------------------------------------------
if (create):
	writemsg("Configuring resources for IBM Intelligent Operations Center using " + propsFileName )
	#--------------------------------------------------------------------
	# Create WAS Variables
	#--------------------------------------------------------------------
	writemsg("Creating WAS Variables" )
	manageWASVariables(props,create)
	#--------------------------------------------------------------------
	# Create Object Cache Instances
	#--------------------------------------------------------------------
	writemsg("Creating Object Cache Instances" )
	manageObjectCacheInstances(props, create )
	#--------------------------------------------------------------------
	# Create Dynamic Cache Services
	#--------------------------------------------------------------------
	writemsg("Creating Dynamic Cache Services" )
	manageDynamicCacheServices(props, create )
	#--------------------------------------------------------------------
	# Create J2C Aliases
	#--------------------------------------------------------------------
	writemsg("Creating J2C Aliases" )
	manageJ2CAliases(props, create )
	#--------------------------------------------------------------------
	# Create JDBC Providers and Datasources
	#--------------------------------------------------------------------
	writemsg("Creating JDBC Providers and DataSources" )
	manageJDBCResources(props, create )	
	#--------------------------------------------------------------------
	# Create JAAS Application Logins
	#--------------------------------------------------------------------
	writemsg("Creating JAAS Logins" )
	manageJAASAppLogins(props, create )
	#--------------------------------------------------------------------
	# Create Shared Libraries
	#--------------------------------------------------------------------
	writemsg("Creating Shared Libraries" )
	createSharedLibraries(props )	
	#--------------------------------------------------------------------
	# Change JVM Settings
	#--------------------------------------------------------------------
	writemsg("Adding custom properties to JVM" )
	manageJVMSettings(props, create )
	#--------------------------------------------------------------------
	# Create SI Buses
	#--------------------------------------------------------------------
	writemsg("Creating SI Buses" )
	manageSIBuses(props, create )
	#--------------------------------------------------------------------
	# Create SI Bus Members
	#--------------------------------------------------------------------
	writemsg("Creating SI Bus Members" )
	manageSIBusMembers(props, create )
	#--------------------------------------------------------------------
	# Create SI Bus Destinations
	#--------------------------------------------------------------------
	writemsg("Creating SI Bus Destinations" )
	manageSIBusDestinations(props, create )
	#--------------------------------------------------------------------
	# Create MQ Connection Destination(s)
	#--------------------------------------------------------------------
	writemsg("Creating MQ Connection Destination(s)" )
	manageQueueDestinations(props, create )
	#--------------------------------------------------------------------
	# Create MQ Connection Factories
	#--------------------------------------------------------------------
	writemsg("Creating MQ Connection Factories" )
	manageQueueConnectionFactories(props, create )
	#--------------------------------------------------------------------
	# Create MQ Activation Specification 
	#--------------------------------------------------------------------
	writemsg("Creating MQ Activation Specification" )
	manageMQActivationSpecs(props, create )
	#--------------------------------------------------------------------
	# Create JMS Activation Specification 
	#--------------------------------------------------------------------
	writemsg("Creating JMS Activation Specification" )
	manageJMSActivationSpecs(props, create )
	#--------------------------------------------------------------------
	# Create JMS Connection Factories
	#--------------------------------------------------------------------
	writemsg("Creating JMS Connection Factories" )
	manageJMSConnectionFactories(props, create )
	#--------------------------------------------------------------------
	# Create JMS Queues
	#--------------------------------------------------------------------
	writemsg("Creating JMS Queues" )
	manageJMSQueues(props, create )
	#--------------------------------------------------------------------
	# Create JMS Topic Destination(s) 
	#--------------------------------------------------------------------
	writemsg("Creating JMS Topic Destination(s)" )
	manageJMSTopics(props,create)
	#--------------------------------------------------------------------
	# Create Timer Manager(s)
	#--------------------------------------------------------------------
	writemsg("Creating Timer Manager(s)" )
	manageTimerManagers(props,create)
	#--------------------------------------------------------------------
	# Create Work Manager(s)
	#--------------------------------------------------------------------
	writemsg("Creating Work Manager(s)" )
	manageWorkManagers(props,create)
	#--------------------------------------------------------------------
	# Create Scheduler Manager(s)
	#--------------------------------------------------------------------
	writemsg("Creating Scheduler(s)" )
	manageSchedulers(props,create)
	#--------------------------------------------------------------------
	# Create Resource Environment Provider Properties
	#--------------------------------------------------------------------
	writemsg("Creating Resource Environment Provider Properties" )
	manageResourceEnvironmentProviderProperties(props, create )
	#--------------------------------------------------------------------
	# Create SSL Truststore Signer Certificate Properties
	#--------------------------------------------------------------------
	writemsg("Creating SSL Truststore Signer Certificate Properties" )
	manageSSLTrustStoreSignerCertificates(props, create )
	#--------------------------------------------------------------------
	# Create Server Cookie Configurations
	#--------------------------------------------------------------------
	writemsg("Creating Server Cookie Configurations" )
	manageServerCookies(props, create )
	#--------------------------------------------------------------------
	# Create Web Container Session Management Configuration
	#--------------------------------------------------------------------
	writemsg("Creating Web Container Session Management Configuration" )
	manageSessionManagementProperties(props, create )
	#--------------------------------------------------------------------
	# Create Container Services Transaction Service Configuration
	#--------------------------------------------------------------------
	writemsg("Creating Container Services Transaction Service Configuration" )
	manageTransactionServiceConfiguration(props, create )
	#--------------------------------------------------------------------
	# Create HADR Datasource Configuration
	#--------------------------------------------------------------------
	writemsg("Creating HADR Datasource Configuration" )
	manageHadrDatasources(props, create )
	#--------------------------------------------------------------------
	# Create ThreadPool Configuration
	#--------------------------------------------------------------------
	writemsg("Creating ThreadPool Configuration" )
	manageThreadPools(props, create )
	#--------------------------------------------------------------------
	# Create Datasource Custom Properties
	#--------------------------------------------------------------------
	writemsg("Creating Datasource Custom Properties" )
	manageDatasourceCustomProperties(props, create )
#endif
#--------------------------------------------------------------------
# Delete Resource - different order from create sequence
#--------------------------------------------------------------------
else:
	#--------------------------------------------------------------------
	# Delete Datasource Custom Properties
	#--------------------------------------------------------------------
	writemsg("Deleting Datasource Custom Properties" )
	manageDatasourceCustomProperties(props, create )
	#--------------------------------------------------------------------
	# Delete ThreadPool Configuration
	#--------------------------------------------------------------------
	writemsg("Deleting ThreadPool Configuration" )
	manageThreadPools(props, create )
	#--------------------------------------------------------------------
	# Delete HADR Datasource Configuration
	#--------------------------------------------------------------------
	writemsg("Deleting HADR Datasource Configuration" )
	manageHadrDatasources(props, create )
	#--------------------------------------------------------------------
	# Delete Container Services Transaction Service Configuration
	#--------------------------------------------------------------------
	writemsg("Deleting Container Services Transaction Service Configuration" )
	manageTransactionServiceConfiguration(props, create )
	#--------------------------------------------------------------------
	# Delete Web Container Session Management Configuration
	#--------------------------------------------------------------------
	writemsg("Deleting Web Container Session Management Configuration" )
	manageSessionManagementProperties(props, create )
	#--------------------------------------------------------------------
	# Delete Server Cookie Configurations
	#--------------------------------------------------------------------
	writemsg("Deleting Server Cookie Configurations" )
	manageServerCookies(props, create )
	#--------------------------------------------------------------------
	# Delete SSL Truststore Signer Certificate Properties
	#--------------------------------------------------------------------
	writemsg("Deleting SSL Truststore Signer Certificate Properties" )
	manageSSLTrustStoreSignerCertificates(props, create)
	#--------------------------------------------------------------------
	# Delete Resource Environment Provider Properties
	#--------------------------------------------------------------------
	writemsg("Deleting Resource Environment Provider Properties" )
	manageResourceEnvironmentProviderProperties(props, create )
	#--------------------------------------------------------------------
	# Create Scheduler Manager(s)
	#--------------------------------------------------------------------
	writemsg("Deleting Scheduler(s)" )
	manageSchedulerManagers(props,create)
	#--------------------------------------------------------------------
	# Delete Work Manager(s)
	#--------------------------------------------------------------------
	writemsg("Deleting Work Manager(s)" )
	manageWorkManagers(props,create)
	#--------------------------------------------------------------------
	# Delete Timer Manager(s)
	#--------------------------------------------------------------------
	writemsg("Deleting Timer Manager(s)" )
	manageTimerManagers(props,create)
	#--------------------------------------------------------------------
	# Delete JMS Topic Destination(s) 
	#--------------------------------------------------------------------
	writemsg("Deleting JMS Topic Destination(s)" )
	manageJMSTopics(props,create)
	#--------------------------------------------------------------------
	# Delete JMS Queues
	#--------------------------------------------------------------------
	writemsg("Deleting JMS Queues" )
	manageJMSQueues(props, create )
	#--------------------------------------------------------------------
	# Delete JMS Connection Factories
	#--------------------------------------------------------------------
	writemsg("Deleting JMS Connection Factories" )
	manageJMSConnectionFactories(props, create )
	#--------------------------------------------------------------------
	# Delete JMS Activation Specification 
	#--------------------------------------------------------------------
	writemsg("Deleting JMS Activation Specification" )
	manageJMSActivationSpecs(props, create )
	#--------------------------------------------------------------------
	# Delete MQ Activation Specification(s) 
	#--------------------------------------------------------------------
	writemsg("Deleting MQ Activation Specifications" )
	manageMQActivationSpecs(props, create )
	#--------------------------------------------------------------------
	# Delete MQ Connection Destination(s)
	#--------------------------------------------------------------------
	writemsg("Deleting MQ Connection Destination(s)" )
	manageQueueDestinations(props, create )
	#--------------------------------------------------------------------
	# Delete MQ Connection Factories
	#--------------------------------------------------------------------
	writemsg("Deleting MQ Connection Factories" )
	manageQueueConnectionFactories(props, create )
	#--------------------------------------------------------------------
	# Delete SI Bus Destinations
	#--------------------------------------------------------------------
	writemsg("Deleting SI Bus Destinations" )
	manageSIBusDestinations(props, create )
	#--------------------------------------------------------------------
	# Delete SI Bus Members
	#--------------------------------------------------------------------
	writemsg("Deleting SI Bus Members" )
	manageSIBusMembers(props, create )
	#--------------------------------------------------------------------
	# Delete SI Buses
	#--------------------------------------------------------------------
	writemsg("Deleting SI Buses" )
	manageSIBuses(props, create )
	#--------------------------------------------------------------------
	# Change JVM Settings
	#--------------------------------------------------------------------
   	writemsg("Removing custom properties from JVM")
	manageJVMSettings(props, create)
	#--------------------------------------------------------------------
	# Delete Shared Libraries
	#--------------------------------------------------------------------
	writemsg("Deleting Shared Libraries" )
	removeSharedLibraries(props )
	#--------------------------------------------------------------------
	# Delete JAAS Application Logins
	#--------------------------------------------------------------------
	writemsg("Deleting JAAS Application Login" )
	manageJAASAppLogins(props, create )	
	#--------------------------------------------------------------------
	# Delete JDBC Providers and Datasources
	#--------------------------------------------------------------------
	writemsg("Deleting JDBC Providers and DataSources" )
	manageJDBCResources(props, create )	
	#--------------------------------------------------------------------
	# Delete J2C Aliases
	#--------------------------------------------------------------------
	writemsg("Deleting J2C Aliases" )
	manageJ2CAliases(props, create )
	#--------------------------------------------------------------------
	# Delete Dynamic Cache Services
	#--------------------------------------------------------------------
	writemsg("Deleting Dynamic Cache Services" )
	manageDynamicCacheServices(props, create )
	#-------------------------------------------------------------------
	# Delete Object Cache Resources
	#--------------------------------------------------------------------
	writemsg("Deleting Object Cache Instances" )
	manageObjectCacheInstances(props, create )		
	#--------------------------------------------------------------------
	# Delete WAS Variables
	#--------------------------------------------------------------------
	writemsg("Deleting WAS Variables" )
	manageWASVariables(props,create)
#endElse

# -------------------------------------------------------------------------
# Save or reset changes
# -------------------------------------------------------------------------
saveChanges(props["finalOperation"] )

if ( create ):
        writemsg("Finished configuring IBM Intelligent Operations Center WAS resources" )
else:
        writemsg("Finished removal of IBM Intelligent Operations Center WAS resources" )
