# /*
#  * IBM Confidential OCO Source Material
#  *
#  * Licensed Materials - Property of IBM
#  * 
#  * (C) Copyright IBM Corp. 2012  All rights reserved.
#  *
#  * The source code for this program is not published or otherwise divested
#  * of its trade secrets, irrespective of what has been deposited with the
#  * U. S. Copyright Office.
#  *
#  * US Government Users Restricted Rights - Use, duplication or
#  * disclosure restricted by GSA ADP Schedule Contract with IBM
#  * Corp.
#  */


import javax.xml.parsers.DocumentBuilderFactory as DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder as DocumentBuilder
import sys

from java.io import FileInputStream
from java.io import ByteArrayOutputStream
from java.io import StringWriter
from javax.xml.transform import TransformerFactory
from javax.xml.transform.stream import StreamSource
from javax.xml.transform.stream import StreamResult 
from javax.xml.transform.dom import DOMSource



#--------------------------------------------------------------------
# Procedure:   loadProperties
# Description: Loads a properties file from disk and returns a Java
#              Properties object.
#
# Parameters:  propFileName	Name of the properties file
# Returns:     Properties object representing contents of the file
#--------------------------------------------------------------------
def loadProperties ( propFileName ):
	from java.util import Properties
	fileprop = Properties() 
	from java.io import FileInputStream
	fileStream = FileInputStream( propFileName )
	fileprop.load(fileStream)

	return fileprop
#endDef 

#--------------------------------------------------------------------
# Procedure:	manageSkins
# Description:	Uses the properties file to make some changes to a 
# 		Document Object Model's skin node(s). These changes
#		include modifying the values of certain elements'
#		attributes.
# Parameters:	dom	The Java Document object we wish to change.
# Returns:	
#--------------------------------------------------------------------
def manageSkins (props, dom):
	i = 0
	varprefix = "skin"

	while (1):
		propName = varprefix + "." + str(i) + ".uniquename"
		uniquename = props.getProperty(propName,"")
		if ((uniquename == "") or (uniquename == None)):
			break 
		#endIf 
		
		contextroot = props[varprefix+"."+ str(i) + ".contextroot"]
		resourceroot = props[varprefix+"."+ str(i) + ".resourceroot"]

		skinNodeFound = 0
		nodeList = dom.getElementsByTagName("skin")
		for j in range(nodeList.getLength()):
			node = nodeList.item(j)
			if (node.getNodeType() == node.ELEMENT_NODE):
				childNodeList = node.getChildNodes()
				for k in range(childNodeList.getLength()):
					nd = childNodeList.item(k)
					if ( (nd.getNodeType() == nd.ELEMENT_NODE) and (nd.getNodeName() == "parameter") ):
						text = nd.getTextContent()
						if ( (text != None) and (text.find(uniquename) != -1) ):
							skinNodeFound = 1
							n = node
							break
						#endIf
					#endIf
				else:
					continue
				#endElse
				break
				#endFor
			#endIf
		#endFor

		if (skinNodeFound == 1):
			n.setAttribute("context-root",contextroot)
			n.setAttribute("resourceroot",resourceroot)
			n.setAttribute("uniquename",uniquename)
			
			skinname_oid_dict[uniquename] = n.getAttribute("objectid")
		#endIf
		
		i = i+1
	#endWhile
#endDef

#---------------------------------------------------------------------
# Procedure:	manageThemes
# Description:	Uses the properties file to make some changes to a 
# 		Document Object Model "theme" node(s). These changes 
#		include modifying the values of certain elements' 
#		attributes, adding new elements which denote allowed
#		skins, and setting the default skin for that theme.
#
# Parameters:	dom	The Java Document object we wish to change.
# Returns:	
#---------------------------------------------------------------------
def manageThemes (props, dom):
	i = 0
	varprefix = "theme"

	while (1):
		propName = varprefix + "." + str(i) + ".uniquename"
		uniquename = props.getProperty(propName,"")
		if ((uniquename == "") or (uniquename == None)):
			break 
		#endIf 
		
		contextroot = props[varprefix+"."+ str(i) + ".contextroot"]
		defaultskin = props[varprefix+"."+ str(i) + ".defaultskin"]
			
		themeNodeFound = 0
		nodeList = dom.getElementsByTagName("theme")
		for j in range(nodeList.getLength()):
			node = nodeList.item(j)
			if (node.getNodeType() == node.ELEMENT_NODE):
				childNodeList = node.getChildNodes()
				for k in range(childNodeList.getLength()):
					nd = childNodeList.item(k)
					if ( (nd.getNodeType() == nd.ELEMENT_NODE) and (nd.getNodeName() == "parameter") ):
						text = nd.getTextContent()
						if ( (text != None) and (text.find(uniquename) != -1) ):
							themeNodeFound = 1
							n = node
							break
						#endIf
					#endIf
				else:
					continue
				#endElse
				break
				#endFor
			#endIf
		#endFor
		
		if (themeNodeFound == 1):
			n.setAttribute("context-root",contextroot)
			n.setAttribute("uniquename",uniquename)
			
			# Get a pre-existing allowed-skin node to use as a template for creating new ones.
			nodeList = dom.getElementsByTagName("allowed-skin")
			if (nodeList.getLength() > 0):
				allowedNode = nodeList.item(0)
			#endIf
			
			if (allowedNode):
				for k, v in skinname_oid_dict.items():
					clone = allowedNode.cloneNode(1)
					clone.setAttribute("skin",skinname_oid_dict[k])
					
					# Only add a node if it's not there already.
					isAllowedSkinNodeFound = 0
					themeChildNodeList = n.getChildNodes()
					for j in range(themeChildNodeList.getLength()):
						node = themeChildNodeList.item(j)
						if (node.getNodeName() == "allowed-skin"):
							if (node.getAttribute("skin") == clone.getAttribute("skin")):
								isAllowedSkinNodeFound = 1
							#endIf
						#endIf
					#endFor
					
					# Insert the new "allowed-skin" node before the first "parameter" node
					if not (isAllowedSkinNodeFound):
						for j in range(themeChildNodeList.getLength()):
							node = themeChildNodeList.item(j)
							if (node.getNodeName() == "parameter"):
								n.insertBefore(clone,node)
								break
							#endIf
						#endFor
					#endIf
				#endFor
				
				# set the defaultskinref for this theme
				n.setAttribute("defaultskinref",skinname_oid_dict[defaultskin])
			#endIf
		#endIf
		
		i = i+1	
	#endWhile
#endDef


if (len(sys.argv) < 1):
	print "Problem executing test.py..."
	print "  Syntax: test.py propsFile"
	print "  where \"propsFile\" is the name of the properties file containing"
	print "  configuration information."
	sys.exit(0)
#endIf

print "Reading properties file \"" +str(sys.argv[0])+ "\"..."
propsFileName = sys.argv[0]
props = loadProperties(propsFileName)

dbf = DocumentBuilderFactory.newInstance()
db = dbf.newDocumentBuilder()
print "Reading dom from file \"output.xml\"..."
dom = db.parse("output.xml")

skinname_oid_dict = {}
manageSkins(props, dom)
manageThemes(props, dom)

tf = TransformerFactory.newInstance()
writer = StringWriter()
result = StreamResult(writer)
transformer = tf.newTransformer()
transformer.transform(DOMSource(dom), result)
writer.flush()
#print writer.toString()

print "Writing new dom to file \"output.xml\"..."
f = open('output.xml','w')
f.write(writer.toString())
f.close()
