#!/bin/sh
###############################################################################
# Licensed Materials - Property of IBM
# Restricted Materials of IBM
# <PID>
# (C) Copyright IBM Corp. 2012, 2013
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp
###############################################################################

CUR_DIR=$(cd -P -- "$(dirname -- "$0")" && pwd -P)

# Set message variables
function iopmsg() {
	typeset -r msgkey=$1
	lang=`echo $LANG | awk -F"." '{print $1}'`
	lang=${lang:-en_US}
	MESSAGES_FILE="$CUR_DIR/${lang}/message.properties"
	[[ ! -f $MESSAGES_FILE ]] && lang=en_US
	line=$(grep "$msgkey" $MESSAGES_FILE)
	var=$(echo "$line" | sed -n -e 's/[^=]*=//p')
	echo $var
}

function menu() {
	echo ""
	iopmsg "messageid_CDIOW052I"
	echo ""
	iopmsg "messageid_CDIOW053I"
	iopmsg "messageid_CDIOW054I"
	iopmsg "messageid_CDIOW055I"
	iopmsg "messageid_CDIOW056I"
	iopmsg "messageid_CDIOW057I"
#	iopmsg "messageid_CDIOW058I"
#	iopmsg "messageid_CDIOW059I"
}

function run_precheck() { 		# 1.topology # 2.statement_message # 3.error_message
	iopmsg $2
	sed -i '/PRE_CHECK/s/Ready/New/' $CUR_DIR/topology/$1.xml
	install_topology $1 $3
}

function install_topology(){	# 1.topology # 2.error_message		
	$CUR_DIR/bin/ba.sh installTopology -t $1 -p $TOPOLOGY_PASSWORD
	if [ $? -ne 0 ]
	then
		iopmsg $2
		exit 1
	fi
}

function substitute_sunshine_component__new_to_ready(){ 	# 1. component
	sed -i "/$1/s/New/Ready/" $CUR_DIR/topology/water_sunshine_topo.xml
}

function substitute_sunshine_component__ready_to_new(){		# 1. component
	sed -i "/$1/s/Ready/New/" $CUR_DIR/topology/water_sunshine_topo.xml
}

while getopts 'p:t:' OPTION
do
	case $OPTION in
	p)	TOPOLOGY_PASSWORD="$OPTARG"
	    if [ ! -f $CUR_DIR/bin/ba.sh ]
	    	then
	    		iopmsg "messageid_CDIOW017I"
			exit 1
		fi	
	
		$CUR_DIR/bin/ba.sh listTopology -p $TOPOLOGY_PASSWORD > /dev/null
		if [ $? -ne 0 ]
			then
				echo "Keystore was tampered with, or password was incorrect,Password verification failed"
			exit 1
		fi
		;;
	t)  INSTALL_TYPE="$OPTARG"		
		;;
	?)
		iopmsg "messageid_CDIOW018I"
		exit 1 				
	esac	
done

shift $((OPTIND - 1))

if [ -z "$TOPOLOGY_PASSWORD" ]; then
	iopmsg "messageid_CDIOW012I"
	exit 1
fi

if [ -z "$INSTALL_TYPE" ]; then
	iopmsg "messageid_CDIOW051I"
	exit 1
fi

if [ "$INSTALL_TYPE" == "delta" ]
	then
		$CUR_DIR/prepare-iow-sample-solution-delta.sh
		if [ $? -ne 0 ]
		then
			iopmsg "messageid_CDIOW068I"
			exit 1
		fi

		############### check if Water has been installed	
		run_precheck water_precheck_topo "messageid_CDIOW032I" "messageid_CDIOW066I"

		run_precheck water_sunshine_precheck_topo "messageid_CDIOW070I" "messageid_CDIOW022E"

		iopmsg "messageid_CDIOW072I"
		install_topology water_sunshine_topo "messageid_CDIOW022E"
		iopmsg "messageid_CDIOW071I"
		exit 0
	else 	
		if [ "$INSTALL_TYPE" == "full" ]
		then 
			iopmsg "messageid_CDIOW069I"
		else
			iopmsg "messageid_CDIOW051I"
			exit 1
		fi			
fi

ACTION=$1

# Command line input for the 'action' ?
RESPONSE=
options="1 2 3 4 5 6 7 "
optionlist=($options)
if [[ "$ACTION" != "" ]]; then 
	for option in ${optionlist[@]}; do
		[[ "$option" == "$ACTION" ]] && RESPONSE=$ACTION && break
	done  
	if [[ "$RESPONSE" == "" ]]; then
		iopmsg "messageid_CDIOW016I"
	fi
fi

if [[ "$RESPONSE" == "" ]]; then
	menu
	read RESPONSE
fi


case $RESPONSE in
	1) 
		############### check if Water has been installed	
		run_precheck water_precheck_topo "messageid_CDIOW032I" "messageid_CDIOW066I"
	
		############### check if CSO has been installed
		run_precheck water_cso_precheck_topo "messageid_CDIOW062I" "messageid_CDIOW065I"
		
		############### check if Maximo has been installed
		run_precheck water_eam_precheck_topo "messageid_CDIOW060I" "messageid_CDIOW063I"
		
		############### check if WCP has been installed
		run_precheck water_wcp_precheck_topo "messageid_CDIOW061I" "messageid_CDIOW064I"
			
		iopmsg "messageid_CDIOW072I"	
		install_topology water_sunshine_topo "messageid_CDIOW022E"
		;;
		
	2)
		############### check if Water has been installed
		run_precheck water_precheck_topo "messageid_CDIOW033I"
		
		substitute_sunshine_component__new_to_ready SUNSHINE_EAM_DB
		substitute_sunshine_component__new_to_ready SUNSHINE_EAM_PORTAL
		substitute_sunshine_component__new_to_ready SUNSHINE_WCP_DB
		substitute_sunshine_component__new_to_ready SUNSHINE_WCP_PORTAL
		substitute_sunshine_component__new_to_ready SUNSHINE_CSO_DB
		substitute_sunshine_component__new_to_ready SUNSHINE_CSO_PORTAL
		substitute_sunshine_component__new_to_ready SUNSHINE_PO_DB
		substitute_sunshine_component__new_to_ready SUNSHINE_PO_PORTAL
		substitute_sunshine_component__new_to_ready SUNSHINE_PFP_DB
		substitute_sunshine_component__new_to_ready SUNSHINE_PFP_PORTAL
		
		install_topology water_sunshine_topo "messageid_CDIOW022E"
		;;

	3) 
		############### check if Water has been installed
		run_precheck water_precheck_topo "messageid_CDIOW032I" "messageid_CDIOW033I"

		############### check if Maximo has been installed
		run_precheck water_eam_precheck_topo "messageid_CDIOW060I" "messageid_CDIOW063I"
		
		############### check if Sunshine base portal has been installed
		## 	run_precheck water_sunshine_precheck_topo "messageid_CDIOW070I" "messageid_CDIOW022E"

		substitute_sunshine_component__ready_to_new SUNSHINE_EAM_DB
		substitute_sunshine_component__ready_to_new SUNSHINE_EAM_PORTAL
		substitute_sunshine_component__ready_to_new MAINTAIN_DB

		install_topology water_sunshine_topo "messageid_CDIOW022E"
		;;
		
	4) 
		############### check if Water has been installed	
		run_precheck water_precheck_topo "messageid_CDIOW032I" "messageid_CDIOW033I"
		
		############### check if WCP has been installed
		run_precheck water_wcp_precheck_topo "messageid_CDIOW061I" "messageid_CDIOW064I"
		
		############### check if Sunshine base portal has been installed
		## 	run_precheck water_sunshine_precheck_topo "messageid_CDIOW070I" "messageid_CDIOW022E"
		
		substitute_sunshine_component__ready_to_new SUNSHINE_WCP_DB
		substitute_sunshine_component__ready_to_new SUNSHINE_WCP_PORTAL

		install_topology water_sunshine_topo "messageid_CDIOW022E"
		;;
		
	5) 
		############### check if Water has been installed
		run_precheck water_precheck_topo "messageid_CDIOW032I" "messageid_CDIOW033I"
		
		############### check if CSO has been installed
		run_precheck water_cso_precheck_topo "messageid_CDIOW062I" "messageid_CDIOW065I"
		
		############### check if Sunshine base portal has been installed
		## 	run_precheck water_sunshine_precheck_topo "messageid_CDIOW070I" "messageid_CDIOW022E"
		
		substitute_sunshine_component__ready_to_new SUNSHINE_CSO_DB
		substitute_sunshine_component__ready_to_new SUNSHINE_CSO_PORTAL
		substitute_sunshine_component__ready_to_new MAINTAIN_DB
		
		install_topology water_sunshine_topo "messageid_CDIOW022E"
		;;
		
	*)
		iopmsg "messageid_CDIOW016I"
		exit 1
		;;
		
		esac

			iopmsg "messageid_CDIOW023I"
		echo ""