#!/bin/sh
###############################################################################
# Licensed Materials - Property of IBM
# Restricted Materials of IBM
# <PID>
# (C) Copyright IBM Corp. 2012, 2013
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp
################################################################################

CUR_DIR=$(cd -P -- "$(dirname -- "$0")" && pwd -P)

# Set message variables
function iopmsg() {
	typeset -r msgkey=$1
	lang=`echo $LANG | awk -F"." '{print $1}'`
	lang=${lang:-en_US}
	MESSAGES_FILE="$CUR_DIR/${lang}/message.properties"
	[[ ! -f $MESSAGES_FILE ]] && lang=en_US
	line=$(grep "$msgkey" $MESSAGES_FILE)
	var=$(echo "$line" | sed -n -e 's/[^=]*=//p')
	echo $var
}

function menu() {
	echo ""
	iopmsg "messageid_CDIOW040I"
	echo ""
	iopmsg "messageid_CDIOW041I"
	iopmsg "messageid_CDIOW042I"
	iopmsg "messageid_CDIOW043I"
	iopmsg "messageid_CDIOW044I"
	iopmsg "messageid_CDIOW045I"
#	iopmsg "messageid_CDIOW046I"
#	iopmsg "messageid_CDIOW047I"
#	iopmsg "messageid_CDIOW048I"
}

cat en_US/Lic_en.txt
echo
echo "You can check the license file in /opt/IBM/water/lib on app server"
read -p "Do you accept the license?(y/n)"   result 
if [ "$result" != "y" ] && [ "$result" != "Y" ];then
 
      exit 0; 
fi

while getopts 'p:t:' OPTION
do
	case $OPTION in
	p)	TOPOLOGY_PASSWORD="$OPTARG"
	    if [ ! -f $CUR_DIR/bin/ba.sh ]
	    	then
	    		iopmsg "messageid_CDIOW017I"
			exit 1
		fi	
	
		$CUR_DIR/bin/ba.sh listTopology -p $TOPOLOGY_PASSWORD > /dev/null
		if [ $? -ne 0 ]
			then
				echo "Keystore was tampered with, or password was incorrect,Password verification failed"
			exit 1
		fi
		;;	
	?)
		iopmsg "messageid_CDIOW018I"
		exit 1 				
	esac	
done

shift $((OPTIND - 1))

if [ -z "$TOPOLOGY_PASSWORD" ]; then
	iopmsg "messageid_CDIOW012I"
	exit 1
fi

ACTION=$1

# Command line input for the 'action' ?
RESPONSE=
options="1 2 3 4 5 6 7 "
optionlist=($options)
if [[ "$ACTION" != "" ]]; then 
	for option in ${optionlist[@]}; do
		[[ "$option" == "$ACTION" ]] && RESPONSE=$ACTION && break
	done  
	if [[ "$RESPONSE" == "" ]]; then
		iopmsg "messageid_CDIOW016I"
	fi
fi

if [[ "$RESPONSE" == "" ]]; then
	menu
	read RESPONSE
fi

case $RESPONSE in
	1)
		iopmsg "messageid_CDIOW024I"
		$CUR_DIR/bin/ba.sh installSolution -s water_wih -p $TOPOLOGY_PASSWORD
		if [ $? -ne 0 ]
		then
			iopmsg "messageid_CDIOW022E"
			exit 1
		fi
		;;
	2)
		iopmsg "messageid_CDIOW025I"
		$CUR_DIR/bin/ba.sh installTopology -t water_wih_core_topo -p $TOPOLOGY_PASSWORD
		if [ $? -ne 0 ]
		then
			iopmsg "messageid_CDIOW022E"
			exit 1
		fi
		;;
		
	3)
	    ###############run precheck, check if Water 1.6 has been installed##################
		iopmsg "messageid_CDIOW032I"
		sed -i '/PRE_CHECK/s/Ready/New/' $CUR_DIR/topology/water_precheck_topo.xml		
		$CUR_DIR/bin/ba.sh installTopology -t water_precheck_topo -p $TOPOLOGY_PASSWORD
		if [ $? -ne 0 ]
		then
			iopmsg "messageid_CDIOW033I"
			exit 1
		fi	
		
		###############Install eam topology#################################################
		iopmsg "messageid_CDIOW026I"
		$CUR_DIR/bin/ba.sh installTopology -t water_content_eam_topo -p $TOPOLOGY_PASSWORD
		if [ $? -ne 0 ]
		then
			iopmsg "messageid_CDIOW022E"
			exit 1
		fi	
		;;
	
	4)
	    ###############run precheck, check if Water 1.6 has been installed##################
	    iopmsg "messageid_CDIOW032I"
		sed -i '/PRE_CHECK/s/Ready/New/' $CUR_DIR/topology/water_precheck_topo.xml		
		$CUR_DIR/bin/ba.sh installTopology -t water_precheck_topo -p $TOPOLOGY_PASSWORD
		if [ $? -ne 0 ]
		then
			iopmsg "messageid_CDIOW033I"
			exit 1
		fi		
		
		###############Install wcp topology#################################################
		iopmsg "messageid_CDIOW028I"
		$CUR_DIR/bin/ba.sh installTopology -t water_content_wcp_topo -p $TOPOLOGY_PASSWORD
		if [ $? -ne 0 ]
		then
			iopmsg "messageid_CDIOW022E"
			exit 1
		fi
		;;
	5)
	    ###############run precheck, check if Water 1.6 has been installed##################
	    iopmsg "messageid_CDIOW032I"
		sed -i '/PRE_CHECK/s/Ready/New/' $CUR_DIR/topology/water_precheck_topo.xml		
		$CUR_DIR/bin/ba.sh installTopology -t water_precheck_topo -p $TOPOLOGY_PASSWORD
		if [ $? -ne 0 ]
		then
			iopmsg "messageid_CDIOW033I"
			exit 1
		fi
				
		###############Install cso topology#################################################
		iopmsg "messageid_CDIOW030I"
		$CUR_DIR/bin/ba.sh installTopology -t water_content_cso_topo -p $TOPOLOGY_PASSWORD
		if [ $? -ne 0 ]
		then
			iopmsg "messageid_CDIOW022E"
			exit 1
		fi
		;;
	*)
		iopmsg "messageid_CDIOW016I"
		exit 1
		;;
		
		esac

			iopmsg "messageid_CDIOW023I"
		echo ""

	
