#!/bin/sh
###############################################################################
# Licensed Materials - Property of IBM
# Restricted Materials of IBM
# <PID>
# (C) Copyright IBM Corp. 2012, 2013
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp
###############################################################################

CUR_DIR=$(cd -P -- "$(dirname -- "$0")" && pwd -P)
prod_exists="false"

# Set message variables
function iowmsg() {
	typeset -r msgkey=$1
	lang=`echo $LANG | awk -F"." '{print $1}'`
	lang=${lang:-en_US}
	MESSAGES_FILE="$CUR_DIR/${lang}/message.properties"
	[[ ! -f $MESSAGES_FILE ]] && lang=en_US
	line=$(grep "$msgkey" $MESSAGES_FILE)
	var=$(echo "$line" | sed -n -e 's/[^=]*=//p')
	echo $var
}

while getopts 'p:t:b:d:' OPTION
do
	case $OPTION in
	p)	TOPOLOGY_PASSWORD="$OPTARG"
		
		$CUR_DIR/../ioc16/bin/ba.sh listTopology -p $TOPOLOGY_PASSWORD > /dev/null
		if [ $? -ne 0 ]
			then
				echo "Keystore was tampered with, or password was incorrect,Password verification failed"
			exit 1
		fi
		;;
	t)  INSTALL_TYPE="$OPTARG"		
		;;
	b)	IOC16_HOME="$OPTARG"
		;;
	d)	IOC_DELTA_HOME="$OPTARG"
		;;
	?)
		iowmsg "messageid_CDIOW001I"
		exit 1 				
	esac	
done

shift $((OPTIND - 1))

if [ -z "$TOPOLOGY_PASSWORD" ]; then
	iowmsg "messageid_CDIOW012I"
	exit 1
fi

if [ -z "$INSTALL_TYPE" ]; then
	iowmsg "messageid_CDIOW051I"
	exit 1
fi

if [ -z "$IOC16_HOME" ]; then
	iowmsg "messageid_CDIOW002I"
	exit 1
fi

if [ -z "$IOC_DELTA_HOME" ]; then
	iowmsg "messageid_CDIOW003I"
	exit 1
fi

#CUR_DIR_M1=$(cd ..; cd -P -- "$(dirname -- "$0")" && pwd -P)
dir=$(cd -P -- "$(dirname -- "$0")" && pwd -P)

# Check whether IOC16_HOME is correct
if [ ! -e "$IOC16_HOME/bin/ioc-env.sh" ]
then
	iowmsg "messageid_CDIOW005E"
	echo ""
	exit 1
fi

# Check whether IOC_DELTA_HOME is correct
if [ ! -e "$IOC_DELTA_HOME/bin/ba.sh" ]
then
	iowmsg "messageid_CDIOW006E"
	echo ""
	exit 1
fi

iowmsg "messageid_CDIOW007I"
if [ ! -d ./bin ]
then
	mkdir ./bin
fi
if [ ! -d ./topology ]
then
	mkdir ./topology
fi
if [ ! -d ./log ]
then
	mkdir ./log
fi
\cp -R $IOC16_HOME/ant .
\cp -R $IOC16_HOME/help .
\cp -R $IOC16_HOME/precheck .
\cp -R $IOC16_HOME/resource .
\cp -R $IOC16_HOME/samples .
\cp -R $IOC16_HOME/schema .
\cp -R $IOC16_HOME/script .
\cp -R $IOC16_HOME/tools .
\cp -R $IOC16_HOME/spec .
\cp -R $IOC16_HOME/bin/ba.sh ./bin
\cp $IOC16_HOME/topology/*.xml ./topology
\cp $IOC16_HOME/topology/queryAllPorts.properties ./topology
if [ $? -ne 0 ]
	then
	iowmsg "messageid_CDIOW008E"
	echo ""
	exit 1
fi

iowmsg "messageid_CDIOW009I"
\cp -R $IOC_DELTA_HOME/spec/SOLUTION ./spec
if [ $? -ne 0 ]
	then
	iowmsg "messageid_CDIOW010E"
	echo ""
	exit 1
fi

if [ "$INSTALL_TYPE" == "delta" ]
	then 
		$CUR_DIR/prepare-iow-std-delta.sh $TOPOLOGY_PASSWORD		
	else 	
		if [ "$INSTALL_TYPE" == "full" ]
		then 
			echo "Start full install"
			iowmsg "messageid_CDIOW011I"
			$CUR_DIR/bin/ba.sh importSolution -f $CUR_DIR/IOW_1611.zip -p $TOPOLOGY_PASSWORD
			if [ $? -ne 0 ]
			then
				iopmsg "messageid_CDIOW014E"
				echo ""
				exit 1
			fi
		else
			iowmsg "messageid_CDIOW051I"
			exit 1
		fi
					
fi 

iowmsg "messageid_CDIOW004I"
echo ""
